@@ -36,6 +36,19 @@ fi
 if [ $HOST_VAL != "x86_64-w64-mingw32" ] && [ $HOST_VAL != i686-w64-mingw32 ] ; then
 	echo "Unknown HOST_VAL"
 	exit 1
+else 
+	case $HOST_VAL in
+		x86_64-w64-mingw32)
+			BITS_VAL=64
+			;;
+		i686-w64-mingw32)
+			BITS_VAL=32
+			;;
+		*)
+			echo "Should not have got here - bug!"
+			exit 1
+			;;
+	esac
 fi
 
 #----
@@ -51,16 +64,15 @@ BASE=`pwd`
 PREFIX=/
 NUM_PROCS=4
 
-IS_RELEASE=1
+IS_RELEASE=0
 
 if [ `id -u` -eq 0 ]; then
 	echo "This script should not be run as root."
 	echo " If you know what you are doing, you can disable this check, but be aware you _can_ break your system by doing this."
 	exit 1;
 fi
-#1) Filezilla wxwidgets patch for 64 bit support under mingw
 #2) own patch for fixing wx-config's lack of sysroot support
-PATCHES_WXWIDGETS_PRE="wxWidgets-2.8.12-mingw64-1.patch configure-wxbool-patch"
+PATCHES_WXWIDGETS_PRE="wx_changeset_76890.diff"
 PATCHES_WXWIDGETS_POST="wx-config-sysroot.patch"
 #1) Zlib no longer needs to explicitly link libc, and will fail if it tries
 PATCHES_ZLIB="zlib-no-lc.patch"
@@ -71,13 +83,11 @@ PATCHES_FTGL_POSTCONF="ftgl-override-configure-2"
 #2) gettext fails to correctly determine windows function call prefix.
 #   should be fixed for gettext > 0.18.1.1 ?
 #   https://lists.gnu.org/archive/html/bug-gettext/2012-12/msg00071.html
-PATCHES_GETTEXT="gettext-disable-tools"    #gettext-win32-prefix
+PATCHES_GETTEXT="gettext-disable-tools gettext-fix-configure-versions"    #gettext-win32-prefix
 
 PATCHES_GLEW="glew-makefile.base"
 
-#Disable broken build for "widgets" directory, which we don't need
-PATCHES_MATHGL="mathgl-disable-widgets"
-
+PATCHES_MATHGL="mathgl-openmp-linker-flag"
 PATCH_LIST="$PATCHES_WXWIDGETS_PRE $PATCHES_WXWIDGETS_POST $PATCHES_GSL $PATCHES_ZLIB $PATCHES_LIBPNG $PATCHES_GETTEXT $PATCHES_FTGL $PATCHES_GLEW $PATCHES_MATHGL $PATCHES_FTGL_POSTCONF"
 
 BUILD_STATUS_FILE="$BASE/build-status"
@@ -123,7 +133,7 @@ function install_mingw()
 	GET_PACKAGES="";
 	for i in $MINGW_PACKAGES
 	do
-		if [ x`apt-cache pkgnames --installed $i` != x"$i" ] ; then
+		if [ x`dpkg --get-selections | grep ^$i | awk '{print $1}'  ` != x"$i" ] ; then
 			GET_PACKAGES="$GET_PACKAGES $i";
 		fi
 	done
@@ -139,12 +149,13 @@ function grabDeps()
 {
 	pushd deps 2>/dev/null
 
-	DEB_PACKAGES="expat freetype ftgl gettext gsl libpng libxml2 mathgl qhull tiff wxwidgets2.8 zlib glew"
-	if [ x$DIST_NAME == x"Ubuntu" ] || [ x$DIST_NAME == x"LinuxMint" ] ; then 
-		LIBJPEGNAME="libjpeg6b"
+	DEB_PACKAGES="expat freetype ftgl gettext gsl libpng libxml2 mathgl qhull tiff wxwidgets3.0 zlib glew libvigraimpex"
+	if [ x$DIST_NAME == x"Ubuntu" ] || [ x$DIST_NAME == x"LinuxMint" ]  ; then 
+       		LIBJPEGNAME="libjpeg6b"
 	else
-		LIBJPEGNAME="libjpeg8"
-	
+		#Libjpeg seems to be forked/renamed very frequently in debian
+		# Likely a new libjpeg will need to be picked each time this script is run
+		LIBJPEGNAME="libjpeg-turbo"
 	fi
 	DEB_PACKAGES="$DEB_PACKAGES $LIBJPEGNAME"
 
@@ -163,6 +174,15 @@ function grabDeps()
 	if [ x"$GET_PACKAGES" != x"" ] ; then
 		apt-get source $GET_PACKAGES
 
+		if [ $? -ne 0 ] ; then
+			echo "Package retrieval failed"
+			echo "apt-get source failed... Maybe check internet connection, then try updating package database, then re-run?"
+			echo " other possibilities could include, eg, that the required package is not available in the debian archive.."
+			exit 1
+		fi
+
+		#Strip patches from the build and patch status files, 
+		# if we are retriving new packages
 		for i in $GET_PACKAGES
 		do
 			grep -v $i ../build-status > tmp
@@ -174,11 +194,7 @@ function grabDeps()
 
 		done
 	fi
-	
-	if [ $? -ne 0 ] ; then
-		echo "apt-get source failed... Maybe check internet connection, then try updating package database, then re-run?"
-		exit 1
-	fi
+
 
 	#Move debian stuff into packages folder
 	if [ x"$GET_PACKAGES" != x"" ] ; then
@@ -350,7 +366,7 @@ function build_glew()
 	fi
 
 	#Perform dynamic modification of patch
-	if [ x`grep patches/glew-makefile.base HOST_VAL` == x""   ||  x`grep patches/glew-makefile.base BASEDIR` == x"" ] ; then
+	if [ x"`grep HOST_VAL patches/glew-makefile.base`" == x""  -o   x"`grep BASEDIR patches/glew-makefile.base`" == x"" ] ; then
 		echo "patches/glew-makefile did not contain replacement keywords"
 		exit 1
 	fi
@@ -383,6 +399,10 @@ function build_glew()
 	popd >/dev/null
 	popd >/dev/null
 
+	#remove static library, as this can be incorrectly caught by linker
+	# leading to confusing messages
+	rm ${BASE}/lib/libglew*.a
+
 	echo "glew" >> $BUILD_STATUS_FILE
 }
 
@@ -596,14 +616,18 @@ function build_qhull()
 
 	make clean
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "qhull configure failed"; exit 1; } 
+	sed -i "s/ gcc$/${HOST_VAL}-gcc/" Makefile
+	sed -i "s/ g++$/${HOST_VAL}-g++/" Makefile
 
-	make -j $NUM_PROCS || { echo "qhull build failed"; exit 1; } 
-	
+	make SO="dll" -j $NUM_PROCS 
+	find ./ -name \*dll -exec cp {} ${BASE}/bin/	
+	make SO="dll" -j $NUM_PROCS || { echo "qhull build failed"; exit 1; } 
 	make install DESTDIR="$BASE"|| { echo "qhull install failed"; exit 1; } 
 
 	popd >/dev/null
 	popd >/dev/null
+
+	ln -s ${BASE}/include/libqhull ${BASE}/include/qhull
 	
 	FIX_LA_FILE_ARG=libqhull
 	fix_la_file
@@ -626,11 +650,12 @@ function build_expat()
 		echo "expat dir missing, or duplicated?"
 		exit 1
 	fi
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "Libtiff configure failed"; exit 1; } 
 
-	make -j $NUM_PROCS || { echo "expat build failed"; exit 1; } 
+	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "$NAME configure failed"; exit 1; } 
+
+	make -j $NUM_PROCS || { echo "$NAME build failed"; exit 1; } 
 	
-	make install DESTDIR="$BASE"|| { echo "expat install failed"; exit 1; } 
+	make install DESTDIR="$BASE"|| { echo "$NAME install failed"; exit 1; } 
 
 	#DLL needs to be copied into lib manually
 	cp -p .libs/${NAME}-[0-9]*.dll $BASE/lib/ 
@@ -709,7 +734,7 @@ function build_wx()
 
 	APPLY_PATCH_ARG=$PATCHES_WXWIDGETS_PRE
 	applyPatches
-#WX_DISABLE="--disable-compat26 --disable-ole --disable-dataobj --disable-ipc --disable-apple_ieee --disable-zipstream --disable-protocol_ftp --disable-mshtmlhelp --disable-aui --disable-mdi --disable-postscript --disable-datepick --disable-splash --disable-wizarddlg --disable-joystick --disable-loggui --disable-debug --disable-logwin --disable-logdlg --disable-tarstream --disable-fs_archive --disable-fs_inet --disable-fs_zip --disable-snglinst --disable-sound --disable-variant --without-regex"
+	WX_DISABLE="--disable-compat26 --disable-compat28 --disable-ole --disable-dataobj --disable-ipc --disable-apple_ieee --disable-zipstream --disable-protocol_ftp --disable-mshtmlhelp --disable-aui --disable-mdi --disable-postscript --disable-datepick --disable-splash --disable-wizarddlg --disable-joystick --disable-loggui --disable-debug --disable-logwin --disable-logdlg --disable-tarstream --disable-fs_archive --disable-fs_inet --disable-fs_zip --disable-snglinst --disable-sound --disable-variant --without-regex"
 
 	./configure --host=$HOST_VAL --enable-shared --disable-static --with-opengl --enable-unicode --without-regex --prefix=/ || { echo "wxwidgets configure failed"; exit 1; } 
 
@@ -729,7 +754,21 @@ function build_wx()
 
 	pushd ./bin/
 	unlink wx-config
-	ln -s `find ${BASE}/lib/wx/config/ -name \*release-2.8` wx-config
+
+
+	#Search for the wx-config file. It get installed into /lib/wx, but has
+	# unusual naming conventions
+	WX_CONFIG_FILE=`find ${BASE}/lib/wx/config/ -type f -executable -name \*release-\*`
+	if [ x$WX_CONFIG_FILE == x"" ] ; then
+		WX_CONFIG_FILE=`find ${BASE}/lib/wx/config/ -type f  -executable -name \*-unicode-\*`
+	fi
+
+	if [ x$WX_CONFIG_FILE == x"" ] ; then
+		echo "Couldn't find the wx-config script."
+		exit 1
+	fi
+
+	cp $WX_CONFIG_FILE wx-config 
 	APPLY_PATCH_ARG=$PATCHES_WXWIDGETS_POST
 	PATCH_LEVEL=0
 	applyPatches
@@ -738,7 +777,7 @@ function build_wx()
 	popd
 
 	pushd ./lib/
-	ln -s wx-2.8/wx/ wx
+	ln -s wx-3.0/wx/ wx
 	popd
 
 
@@ -766,7 +805,7 @@ function build_freetype()
 
 	pushd freetype-[0-9]*
 	make clean
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "freetype configure failed"; exit 1; } 
+	./configure --host=$HOST_VAL --enable-shared --disable-static --without-png --prefix=/ || { echo "freetype configure failed"; exit 1; } 
 
 	make -j $NUM_PROCS || { echo "freetype build failed"; exit 1; } 
 	
@@ -835,7 +874,7 @@ function build_gettext()
 	pushd gettext-* >/dev/null
 		
 	if [ $? -ne 0 ] ; then
-		echo "gettext dir missing, or duplicated?"
+		echo "$NAME dir missing, or duplicated?"
 		exit 1
 	fi
 
@@ -845,12 +884,16 @@ function build_gettext()
 	applyPatches
 	automake
 
-	./configure --host=$HOST_VAL --disable-threads --enable-shared --disable-static --prefix=/ || { echo "gettext configure failed"; exit 1; } 
+	./configure --host=$HOST_VAL --disable-threads --enable-shared --disable-static --prefix=/ || { echo "$NAME configure failed"; exit 1; } 
 
-	make -j $NUM_PROCS || { echo "gettext build failed"; exit 1; } 
+	make -j $NUM_PROCS || { echo "$NAME build failed"; exit 1; } 
 	
-	make install DESTDIR="$BASE"|| { echo "gettext install failed"; exit 1; } 
+	make install DESTDIR="$BASE"|| { echo "$NAME install failed"; exit 1; } 
 
+	#FIXME: I had to copy the .lib, .la and .a files manually
+	# I don't know why the makefile does not do this.
+	cp gettext-runtime/intl/.libs/libintl.{la,lib,a} ${BASE}/lib/ || {  echo "semi-manual copy of libintl failed"; exit 1; } 
+	
 	popd >/dev/null
 	popd >/dev/null
 	
@@ -882,27 +925,58 @@ function build_mathgl()
 	APPLY_PATCH_ARG=$PATCHES_MATHGL
 	applyPatches
 
-	libtoolize --copy --force
-	aclocal
-	automake --add-missing
+	if [ -d $BASEDIR/include/mgl2 ] ; then
+		echo "there are mgl2 headers already installed. Abort abort!"\
+		exit 1
+	fi
 
-	autoreconf
-	LIBS="${LIBS} -lz" ./configure --host=$HOST_VAL --disable-gsl --disable-pthread --enable-shared --disable-static --prefix=/ || { echo "mathgl configure failed"; exit 1; } 
+	LIBS=-lpng cmake -Denable-gsl="yes" -Denable-mpi="no"  -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASEDIR}/include/
 
-	#RPATH disable hack
-	sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' libtool
-	sed -i 's|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' libtool
+	make -j $NUM_PROCS
 
-	#Hack to fix mathgl's libpng dll search
-	mkdir -p $BASE/lib/.libs/
-	ln -s $BASE/lib/libpng.dll $BASE/lib/.libs/libpng.dll.a 
-
-	make -j $NUM_PROCS || { echo "mathgl build failed"; exit 1; } 
+	if [ x"`find ./ -name \*dll`" == x"" ] ; then
+		echo "Did cmake fail to make a DLL? Cmake rarely builds cleanly, but I should be able to find the DLL..."
+		exit 1
+	fi
+		
+	make install
 	
-	make install DESTDIR="$BASE"|| { echo "mathgl install failed"; exit 1; } 
+	ln -s ${BASE}/include/mgl2 ${BASE}/include/mgl
+
+	popd >/dev/null
+	popd >/dev/null
+	FIX_LA_FILE_ARG=libmgl
+	fix_la_file
 	
-	#DLL needs to be copied into lib manually
-	cp -p .libs/${NAME}-[0-9]*.dll $BASE/lib/ 
+	echo ${NAME} >> $BUILD_STATUS_FILE
+}
+
+function build_libvigra()
+{
+	NAME="libvigra"
+	ISBUILT_ARG=${NAME}
+	isBuilt
+	if [ $ISBUILT -eq 1 ] ; then
+		return;
+	fi
+	pushd deps >/dev/null
+	pushd libvigraimpex-* >/dev/null
+	
+	if [ $? -ne 0 ] ; then
+		echo "$NAME dir missing, or duplicated?"
+		exit 1
+	fi
+	make clean
+
+	APPLY_PATCH_ARG=$PATCHES_MATHGL
+	applyPatches
+
+	cmake -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASEDIR}/include/
+
+	make -j $NUM_PROCS
+	
+	make install || { echo "$NAME install failed"; exit 1; } 
+
 
 	popd >/dev/null
 	popd >/dev/null
@@ -956,10 +1030,16 @@ function build_ftgl()
 
 	#MAkefile refers to ECHO variable for reporting completion, which does not exist
 	sed -i 's/ECHO_C =/ECHO=echo/' Makefile
+	sed -i "s@-I//@-I${BASE}/@" Makefile
+	sed -i 's/ECHO_C =/ECHO=echo/' Makefile
+	
+	#HACK - find all -I// and -L// and replace them with something sane
+	find ./ -name Makefile -exec sed -i "s@-I//@-I${BASE}/@" {} \;
+	find ./ -name Makefile -exec sed -i "s@-L//@-L${BASE}/@" {} \;
 
 	make -j $NUM_PROCS || { echo "ftgl build failed"; exit 1; } 
 	
-	make install DESTDIR="$BASE"|| { echo "ftgl install failed"; exit 1; } 
+	DESTDIR="$BASE" make install | { echo "ftgl install failed"; exit 1; } 
 
 	popd >/dev/null
 	popd >/dev/null
@@ -1056,38 +1136,68 @@ function build_3Depict()
 	make distclean
 
 
-	CONF_FLAG="--host=$HOST_VAL"
+	CONF_FLAG="--host=$HOST_VAL --with-libqhull-link=-lqhull_p"
 	if [ $IS_RELEASE -ne 0 ] ; then
-		CONF_FLAG="$CONF_FLAG --disable-debug-checks"
+		CONF_FLAG="$CONF_FLAG --disable-debug-checks --enable-openmp-parallel"
 	fi
 
-	CFLAGS="$CFLAGS -DUNICODE" CPPFLAGS="${CPPFLAGS} -DUNICODE" ./configure  $CONF_FLAG
+	FTGL_CFLAGS="-I${BASE}/include/freetype/" CFLAGS="$CFLAGS -DUNICODE" CPPFLAGS="${CPPFLAGS} -DUNICODE" ./configure  $CONF_FLAG
 
 	if [ $? -ne 0 ] ; then
 		echo "Failed 3Depict configure"
 		exit 1
 	fi
 
-	#HACK - strip all makefiles of -D_GLIBCXX_DEBUG
-	#	mingw & GLIBCXX_DEBG don't play nice
-	find ./ -name Makefile -exec sed -i 's/-D_GLIBCXX_DEBUG//g' {} \;
+	#sanity check that windres is activated
+	if [ x`grep HAVE_WINDRES_TRUE config.log | grep '#' ` != x"" ] ; then
+		echo "Windres appears to be commented out. Shouldn't be for windows builds"
+		exit 1
+	fi
 
+	#Check that wx's manifest matches our arch
+	MANIFEST=`find ../../include/ -name wx.manifest`
+	if [ x"$MANIFEST" == x"" ] ; then
+		echo "Didnt' find manifest!"
+		exit 1
+	fi
+	case $BITS_VAL in
+		32)
+			MANIFEST_TARG=x86
+			MANIFEST_NOT=amd64
+			;;
+		64)
+			MANIFEST_TARG=amd64
+			MANIFEST_NOT=x86
+			;;
+	esac
+
+	if [  x"`grep -i $MANIFEST_TARG $MANIFEST`" == x"" ] ; then
+		echo "Manifest arch does not match!"	
+		echo " file examined: $MANIFEST"
+		echo " Expected :" $MANIFEST_TARG 
+		exit 1
+	fi
+
+	if [  x"`grep -i $MANIFEST_NOT $MANIFEST`" != x"" ] ; then
+		echo "Manifest arch does not match!"	
+		echo " file examined: $MANIFEST"
+		echo " This should be missing, but isnt:" $MANIFEST_NOT 
+		exit 1
+	fi
+
+       #HACK - strip all makefiles of -D_GLIBCXX_DEBUG
+       #	mingw & GLIBCXX_DEBG don't play nice
+	find ./ -name Makefile -exec sed -i 's/-D_GLIBCXX_DEBUG//g' {} \;
+	#HACK - find all -I// and -L// and replace them with something sane
+	find ./ -name Makefile -exec sed -i "s@-I//@-I${BASE}/@" {} \;
+	find ./ -name Makefile -exec sed -i "s@-L//@-L${BASE}/@" {} \;
+	
 	make -j$NUM_PROCS
 	if [ $? -ne 0 ] ; then
 		echo "Failed 3Depict build"
 		exit 1
 	fi
 
-	if [ $IS_RELEASE -ne 0 ] ; then
-		#Sanity check that we are actually in debug mode
-		TEST_FLAG=`./3Depict --help  2>&1 | grep "\-\-test"`
-		if [ x"$TEST_FLAG" != x"" ] ; then
-			echo "3Depict\'s Unit tests available, but should not be, when in release mode" 
-			exit 1
-		fi
-	fi
-
-
 	#if the locales are missing, try to rebuild them
 	if [ x`find locales/ -name \*.mo` = x""  ] ; then
 		
@@ -1116,12 +1226,22 @@ function make_package()
 {
 	pushd ./code/3Depict 2> /dev/null
 
-	NSI_FILE=./windows-installer.nsi
-	if [ ! -f $NSI_FILE ] ; then
-		echo "NSI file missing whilst trying to build package"
-		exit 1;
+	#Check that the PDF manual has been built
+	if [ ! -f docs/manual-latex/manual.pdf ] ; then
+		echo "PDF Manual not built. Building"
+	
+		pushd docs/manual-latex
+		pdflatex manual.tex && bibtex manual && pdflatex manual.tex  || { echo " Manual not pre-built, and failed to build. Aborting" ; exit 1; }
+		popd
+		if [ ! -f docs/manual-latex/manual.pdf ] ; then 
+			echo "Failed to build manual, even though latex completed with no errors. Aborting " 
+			exit 1;
+		fi
 	fi
 
+
+	NSI_FILE=./windows-installer.nsi
+
 	#copy as needed
 	# Due to debian bug : #704828, makensis cannot correctly handle symlinks,
 	# so don't use symlinks
@@ -1129,9 +1249,28 @@ function make_package()
 		cp ./packaging/mingw-debian-cross/windows-installer.nsi .
 	fi
 
+	
+	if [ ! -f $NSI_FILE ] ; then
+		echo "NSI file missing whilst trying to build package"
+		exit 1;
+	fi
+
+	#Check NSI file has PROGRAMFILES / PROGRAMFILES64 set
+	if [ x"`grep PROGRAMFILES64 $NSI_FILE`" == x"" -a $BITS_VAL == 64 ] ; then
+		echo "NSI file should contain PROGRAMFILES64 output path."
+		exit 1;
+	else
+		if [ x"`grep PROGRAMFILES64 $NSI_FILE`" != x"" -a $BITS_VAL == 32 ] ; then
+			echo "NSI file contained 64 bit install dir, but this is 32"
+			exit 1;
+		fi
+	fi
+	
+	
+	 
 
 	echo -n " Copying dll files... "
-	SYSTEM_DLLS="(ADVAPI32.dll|COMCTL32.DLL|COMDLG32.DLL|GDI32.dll|KERNEL32.dll|ole32.dll|OLEAUT32.dll|RPCRT4.dll|SHELL32.DLL|USER32.dll|WINMM.DLL|WINSPOOL.DRV|WSOCK32.DLL|GLU32.dll|OPENGL32.dll|msvcrt.dll)"
+	SYSTEM_DLLS="(ADVAPI32.dll|COMCTL32.DLL|COMDLG32.DLL|GDI32.dll|KERNEL32.dll|ole32.dll|OLEAUT32.dll|RPCRT4.dll|SHELL32.DLL|USER32.dll|WINMM.DLL|WINSPOOL.DRV|WSOCK32.DLL|GLU32.dll|OPENGL32.dll|msvcrt.dll|WS2_32.dll)"
 
 	DLL_FILES=`${HOST_VAL}-objdump -x src/3Depict.exe | grep 'DLL Name:' | awk '{print $3}' | egrep -i -v ${SYSTEM_DLLS}`
 	FOUND_DLLS=""
@@ -1152,7 +1291,7 @@ function make_package()
 		for i in $DLL_FILES
 		do
 			HAVE_DLL=0
-			for j in ${BASE}/lib/ ${BASE}/bin/ $SYS_DIR
+			for j in ${BASE}/lib/ ${BASE}/bin/ $SYS_DIR /usr/${HOST_VAL}/lib/
 			do
 				FIND_RES=`find $j -name $i | head -n 1`
 				if [ x$FIND_RES != x"" ] ; then
@@ -1185,24 +1324,23 @@ function make_package()
 		popd > /dev/null
 	fi
 
-	if [ x"`cat windows-installer.nsi | grep INSERT_DLLS_HERE`" == x"" ]  ||  [ x"`cat windows-installer.nsi | grep INSERT_UNINST_DLLS_HERE`" == x"" ] ; then
-		echo "DLL insertion/removal tokens not found. Was looking for INSERT_DLLS_HERE and INSERT_UNINST_DLLS_HERE"
-		exit 1
-	fi
-
-	
-	#Check that each file in the data/textures/ dir is listed in the NSI file
-	FILE_MISSED=0
-	for i in  data/textures/*png
+	#Check for differeent DLL types (eg 32/64)
+	DLL_FILE_OUT="";
+	for i in $FOUND_DLLS
 	do
-		FILE_GREP=`grep "data\\textures\\$i" windows-installer.nsi`
-		if [ x${FILE_GREP} == x"" ] ; then
-			echo "MISSING FILE: " $i
-			FILE_MISSED=1
+		J=`file $i | awk -F: '{print $2}' | sed 's/\s*//'`
+		if [ x"$DLL_FILE_OUT" == x"" ] ; then	
+			DLL_FILE_OUT=$j
+		else
+			if [ x"$DLL_FILE_OUT" != x"$j" ] ; then
+				echo "DLL Mismatched file info. $i"
+				exit 1
+			fi
 		fi
 	done
 
-	if [ $FILE_MISSED -ne 0 ] ; then
+	if [ x"`cat windows-installer.nsi | grep INSERT_DLLS_HERE`" == x"" ]  ||  [ x"`cat windows-installer.nsi | grep INSERT_UNINST_DLLS_HERE`" == x"" ] ; then
+		echo "DLL insertion/removal tokens not found. Was looking for INSERT_DLLS_HERE and INSERT_UNINST_DLLS_HERE"
 		exit 1
 	fi
 
@@ -1224,15 +1362,20 @@ function make_package()
 
 	makensis `basename $NSI_FILE` ||  { echo "makensis failed" ; exit 1; }
 
+	echo "-------------------"
+	VERSION=`cat $NSI_FILE | grep "define PRODUCT_VERSION " | awk '{print $3}' | sed s/\"//g | sed s/\s*$//`
 	if [ $IS_RELEASE -ne 0 ] ; then
-		VERSION=`cat $NSI_FILE | grep "define PRODUCT_VERSION " | awk '{print $3}' | sed s/\"//g | sed s/\s*$//`
+		echo "Release mode enabled:"
 		TARGET_FILE=3Depict-$VERSION-$HOST_EXT.exe
-		mv Setup.exe  $TARGET_FILE
-		echo "-------------------"
-		echo "File written to : `pwd`/$TARGET_FILE"
-		echo "-------------------"
+	else
+		echo "Release mode disabled:"
+		TARGET_FILE=3Depict-${VERSION}-${HOST_EXT}-debug.exe
 	fi
 	
+	mv Setup.exe  $TARGET_FILE
+	echo "File written to : `pwd`/$TARGET_FILE"
+	echo "-------------------"
+	
 	popd > /dev/null
 }
 
@@ -1258,7 +1401,7 @@ case ${HOST_VAL}  in
 		HOST_EXT="win64"
 	;;
 	i686-w64-mingw32)
-		MINGW_PACKAGES="gcc-mingw32"
+		MINGW_PACKAGES="gcc-mingw-w64-i686 g++-mingw-w64-i686"
 		HOST_EXT="win32"
 	;;
 	*)
@@ -1303,10 +1446,12 @@ build_expat
 build_freetype
 build_libiconv
 build_gettext 
-build_mathgl 
 build_ftgl 
 build_glew
-build_wx	# I'm not sure I've done this 100% right. Check wx-config output 
+build_libvigra
+
+build_mathgl 
+build_wx
 
 build_3Depict
 
