@@ -1,6 +1,6 @@
 /*
  *	threeDepict.cpp - main program implementation
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -39,20 +39,27 @@ enum
 
 class threeDepictApp: public wxApp {
 private:
+#ifndef DEBUG
+	//instance of this class suppresses internal wx error dialogs.
+	// these are a nuisance in release code, as recovered errors often annoy the user
+	wxLogNull nullifyLogs;
+#endif
+
 	MainWindowFrame* MainFrame ;
 	wxArrayString commandLineFiles;
 	wxLocale* usrLocale;
-	//long language;
-
-	//void initLanguageSupport();
+	long language;
 
+	void initLanguageSupport();
+	//Don't load the main window, as debugging was in progress
+	bool dontLoad;
 
 public:
 
     threeDepictApp() ;
-    ~threeDepictApp() { if(usrLocale) delete usrLocale;}
     bool OnInit();
     virtual void OnInitCmdLine(wxCmdLineParser& parser);
+    virtual int OnExit();
     virtual bool OnCmdLineParsed(wxCmdLineParser& parser);
 		 
     int FilterEvent(wxEvent &event);
@@ -61,36 +68,20 @@ public:
     void MacReopenFile(const wxString & fileName);
 #endif
 
-#ifdef DEBUG
-    void setEventloggerFile(const char *file);
-#endif
 };
 
 //Check version is in place because wxT is deprecated for wx 2.9
 //Command line parameter table
 static const wxCmdLineEntryDesc g_cmdLineDesc [] =
 {
-#if wxCHECK_VERSION(2,9,0) 
 	{ wxCMD_LINE_SWITCH, ("h"), ("help"), ("displays this message"),
 		wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP },
 	{ wxCMD_LINE_PARAM,  NULL, NULL, ("inputfile"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE},
-#else
-	{ wxCMD_LINE_SWITCH, wxT("h"), wxT("help"), wxNTRANS("displays this message"),
-		wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP },
-	{ wxCMD_LINE_PARAM,  NULL, NULL, wxNTRANS("inputfile"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE},
-#endif
 	//Unit testing system
 #ifdef DEBUG
-#if wxCHECK_VERSION(2,9,0) 
 	{ wxCMD_LINE_SWITCH, ("t"), ("test"), ("Run debug unit tests, returns nonzero on test failure, zero on success.\n\t\t"
 		       "XML files may be passed to run , instead of default tests"), wxCMD_LINE_VAL_NONE, wxCMD_LINE_SWITCH},
-#else
-	{ wxCMD_LINE_SWITCH, wxT("t"), wxT("test"), 
-	wxNTRANS("Run debug unit tests, returns nonzero on test failure, zero on success.\n\t\t" 
-			"XML files may be passed to run, instead of default tests"), wxCMD_LINE_VAL_NONE, wxCMD_LINE_SWITCH},
 #endif
-#endif
-
   { wxCMD_LINE_NONE,NULL,NULL,NULL,wxCMD_LINE_VAL_NONE,0 }
 
 };
@@ -103,37 +94,37 @@ winconsole winC;
 #endif
 #endif
 
-//DEBUG NaN and INF
-#ifdef DEBUG
-#ifdef __linux__
-#include <fenv.h>
-void trapfpe () {
-//  feenableexcept(FE_INVALID|FE_DIVBYZERO|FE_OVERFLOW);
-}
-#endif
-#endif
 
 IMPLEMENT_APP(threeDepictApp)
 
 threeDepictApp::threeDepictApp()
 {
        	MainFrame=0;usrLocale=0;
+	dontLoad=false;
 #ifndef DEBUG
-#if wxCHECK_VERSION(2,9,0)
 	//Wx 2.9 and up now has assertions auto-enabled. 
 	//Disable for release builds
 	wxSetAssertHandler(NULL);
 #endif
-#endif
 }
 
+int threeDepictApp::OnExit()
+{
+	if(usrLocale) 	
+		delete usrLocale;
+	
+	//libxml2 by default seems to leak memory, unless you call this function
+	xmlCleanupParser();
 
-/*void threeDepictApp::initLanguageSupport()
+	return wxApp::OnExit();
+}
+
+void threeDepictApp::initLanguageSupport()
 {
 	language =  wxLANGUAGE_DEFAULT;
 
 	// load language if possible, fall back to English otherwise
-	if(false)// (wxLocale::IsAvailable(language))
+	if(wxLocale::IsAvailable(language))
 	{
 		//Wx 2.9 and above are now unicode, so locale encoding
 		//conversion is deprecated.
@@ -152,7 +143,7 @@ threeDepictApp::threeDepictApp()
 		usrLocale->AddCatalogLookupPathPrefix(paths->GetResourcesDir());
 		usrLocale->AddCatalogLookupPathPrefix ( wxT ( "locales" ) );
 #endif
-		usrLocale->AddCatalog(wxCStr(PROGRAM_NAME));
+		usrLocale->AddCatalog((PROGRAM_NAME));
 
 
 
@@ -171,7 +162,7 @@ threeDepictApp::threeDepictApp()
 #ifdef __WXMAC__
 			bindtextdomain( PROGRAM_NAME, paths->GetResourcesDir().mb_str(wxConvUTF8) );
 #elif defined(__WIN32) || defined(__WIN64)
-			cerr << paths->GetResourcesDir().mb_str(wxConvUTF8) << endl;
+			cerr << paths->GetResourcesDir().mb_str(wxConvUTF8) << std::endl;
 			std::string s;
 			s =  paths->GetResourcesDir().mb_str(wxConvUTF8);
 			s+="/locales/";
@@ -186,15 +177,15 @@ threeDepictApp::threeDepictApp()
 			switch(curPage)
 			{
 				case 1252:
-					cerr << "Bound cp1252" << endl;
+					std::cerr << "Bound cp1252" << std::endl;
 					bind_textdomain_codeset(PROGRAM_NAME, "CP1252");
 					break;
 				case 65001:
-					cerr << "Bound utf8"<< endl;
+					std::cerr << "Bound utf8"<< std::endl;
 					bind_textdomain_codeset(PROGRAM_NAME, "UTF-8");
 					break;
 				default:
-					cerr << "Unknown codepage " << curPage << endl;
+					std::cerr << "Unknown codepage " << curPage << std::endl;
 					break;
 			}			
 #else
@@ -205,12 +196,12 @@ threeDepictApp::threeDepictApp()
 	}
 	else
 	{
-		std::cout << "Language not supported, falling back to English" << endl;
+		std::cout << "Language not supported, falling back to English" << std::endl;
 		usrLocale = new wxLocale( wxLANGUAGE_ENGLISH );
 		language = wxLANGUAGE_ENGLISH;
 	}
 }
-*/
+
 //Catching key events globally.
 int threeDepictApp::FilterEvent(wxEvent& event)
 {
@@ -283,12 +274,12 @@ void threeDepictApp::OnInitCmdLine(wxCmdLineParser& parser)
 {
 	wxString name,version,preamble;
 
-	name=wxCStr(PROGRAM_NAME);
+	name=(PROGRAM_NAME);
 	name=name+ wxT(" ");
-	version=wxCStr(PROGRAM_VERSION);
+	version=(PROGRAM_VERSION);
 	version+=wxT("\n");
 
-	preamble=wxT("Copyright (C) 2013  3Depict team\n");
+	preamble=wxT("Copyright (C) 2015  3Depict team\n");
 	preamble+=wxT("This program comes with ABSOLUTELY NO WARRANTY; for details see LICENCE file.\n");
 	preamble+=wxT("This is free software, and you are welcome to redistribute it under certain conditions.\n");
 	preamble+=wxT("Source code is available under the terms of the GNU GPL v3.0 or any later version (http://www.gnu.org/licenses/gpl.txt)\n");
@@ -317,73 +308,74 @@ bool threeDepictApp::OnCmdLineParsed(wxCmdLineParser& parser)
 				strFile=stlStr(f.GetFullPath());
 				if( !f.FileExists() )
 				{
-					cerr << "Unable to locate file:" << strFile << endl;
-					continue;
+					std::cerr << "Unable to locate file:" << strFile << std::endl;
+					return false;
 				}
 
-				cerr << "Loading :" << strFile << endl ;
+				std::cerr << "Loading :" << strFile << std::endl ;
 
 				{
 				VisController visControl;
-				if(!visControl.loadState(strFile.c_str(),cerr,false,true))
+				if(!visControl.state.load(strFile.c_str(),true,std::cerr))
 				{
-					cerr << "Error loading state file:" << endl;
-					exit(1);
+					std::cerr << "Error loading state file:" << std::endl;
+					return false;
 				}
 
 				//Run a refresh over the filter tree as a test
 				FilterTree f;
-				visControl.cloneFilterTree(f);
+				visControl.state.treeState.cloneFilterTree(f);
 				if(f.hasHazardousContents())
 				{
 					f.stripHazardousContents();
-					cerr << "For security reasons, the tree was pruned prior to execution." << endl;
+					std::cerr << "For security reasons, the tree was pruned prior to execution." << std::endl;
 				}
 				
 				if(!testFilterTree(f))
 				{
-					cerr << "Failed loading :" << strFile << " , aborting" << endl;
-					exit(1);
+					std::cerr << "Failed loading :" << strFile << " , aborting" << std::endl;
+					return false;
 				}
 				}
 
-				cerr << "OK" << endl; 
+				std::cerr << "OK" << std::endl; 
 
 			}
 			
 			 
-			cerr << "Test XML File(s) Loaded OK" << endl;	
-			exit(0);
+			std::cerr << "Test XML File(s) Loaded OK" << std::endl;
+			dontLoad=true;	
 		}
 		else
 		{
 			//Unit tests failed
 			if(!runUnitTests()) 
 			{
-				cerr << "Unit tests failed" <<endl;
-				exit(1);
+				std::cerr << "Unit tests failed" <<std::endl;
+				return false;
 			}
 			else
 			{
-				cerr << "Unit tests succeeded!" <<endl;
-				exit(0);
+				std::cerr << "Unit tests succeeded!" <<std::endl;
+				dontLoad=true;
 			}
 		}
 	}
+	else
 #endif
-
-	for(unsigned int ui=0;ui<parser.GetParamCount();ui++)
 	{
-		wxFileName f;
-		f.Assign(parser.GetParam(ui));
+		for(unsigned int ui=0;ui<parser.GetParamCount();ui++)
+		{
+			wxFileName f;
+			f.Assign(parser.GetParam(ui));
 
-		if( f.FileExists() )
-			commandLineFiles.Add(f.GetFullPath());
-		else
-			std::cerr << TRANS("File : ") << stlStr(f.GetFullPath()) << TRANS(" does not exist. Skipping") << std::endl;
+			if( f.FileExists() )
+				commandLineFiles.Add(f.GetFullPath());
+			else
+				std::cerr << TRANS("File : ") << stlStr(f.GetFullPath()) << TRANS(" does not exist. Skipping") << std::endl;
 
+		}
 	}
-
 	return true;
 }
 
@@ -411,13 +403,29 @@ void threeDepictApp::MacReopenFile(const wxString &filename)
 bool threeDepictApp::OnInit()
 {
 
-    //initLanguageSupport();
+    initLanguageSupport();
 	
+#if defined(DEBUG) && defined(__linux__)
+	//Virtualbox has a bug, where the video driver generates FPEs
+//   trapfpe(); //Under Linux, enable  segfault on invalid floating point operations
+#endif
 
     //Set the gettext language
     //Register signal handler for backtraces
     if (!wxApp::OnInit())
-    	return false; 
+    	return false;
+
+    //if we ran the debug code, don't load the main window
+    if(dontLoad)
+    {
+	OnExit();
+	//FIXME: This causes wx to shutdown incorrectly, but gives us the return code
+	// - I can't seem to simultaneously set the return code and 
+	// abort the init.
+	exit(0);
+	
+	return false;
+    }
 
     //Need to seed random number generator for entire program
     srand (time(NULL));
@@ -429,15 +437,8 @@ bool threeDepictApp::OnInit()
     wxInitAllImageHandlers();
     MainFrame = new MainWindowFrame(NULL, ID_MAIN_WINDOW, wxEmptyString,wxDefaultPosition,wxDefaultSize);
 
-    if(!MainFrame->initOK())
-	    return false;
-  
- 
     SetTopWindow(MainFrame);
 
-#if defined(DEBUG) && defined(__linux__)
-   trapfpe(); //Under Linux, enable  segfault on invalid floating point operations
-#endif
 
 #ifdef __APPLE__    
    	//Switch the working directory into the .app bundle's resources
@@ -457,11 +458,17 @@ bool threeDepictApp::OnInit()
 
 
     MainFrame->Show();
-    
+   
+    MainFrame->checkShowTips();
+    MainFrame->checkReloadAutosave();
+
+
     if(commandLineFiles.GetCount())
     	MainFrame->SetCommandLineFiles(commandLineFiles);
 
     MainFrame->fixSplitterWindow();
+
+    MainFrame->finaliseStartup();
     return true;
 }
 
