@@ -1,6 +1,6 @@
 /* 
  * APTClasses.h - Generic APT components code
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,11 @@
 #include "ionhit.h"
 
 #include "../../common/stringFuncs.h"
+#include "../../common/basics.h"
 #include "../../common/translation.h"
 
+
+
 #include <cstring>
 #include <new>
 
@@ -35,7 +38,18 @@ using std::make_pair;
 
 const size_t PROGRESS_REDUCE=5000;
 
+
 //---------
+const char *TEXT_LOAD_ERR_STRINGS[] = { "",
+					NTRANS("Error opening file"),
+					NTRANS("Only found header, no data"),
+					NTRANS("Unable to reopen file after first scan"),
+					NTRANS("Error whilst reading file contents"),
+					NTRANS("Unexpected file format"),
+					NTRANS("Unexpected file format"),
+					NTRANS("Insufficient memory to continue"),
+					};
+
 const char *POS_ERR_STRINGS[] = { "",
        				NTRANS("Memory allocation failure on POS load"),
 				NTRANS("Error opening pos file"),
@@ -43,6 +57,7 @@ const char *POS_ERR_STRINGS[] = { "",
 				NTRANS("Pos file size appears to have non-integer number of entries"),
 				NTRANS("Error reading from pos file (after open)"),
 				NTRANS("Error - Found NaN in pos file"),
+				NTRANS("Error - Found Inf in pos file"),
 				NTRANS("Pos load aborted by interrupt.")
 };
 //---------
@@ -56,7 +71,6 @@ enum
 	TEXT_ERR_REOPEN,
 	TEXT_ERR_READ_CONTENTS,
 	TEXT_ERR_FORMAT,
-	TEXT_ERR_NUM_FIELDS,
 	TEXT_ERR_ALLOC_FAIL,
 	TEXT_ERR_ENUM_END //not an error, just end of enum
 };
@@ -72,7 +86,7 @@ const char *ION_TEXT_ERR_STRINGS[] = { "",
 					};
 //---------
 
-//ATO formatted files error codes and asscoiated strings
+//ATO formatted files error codes and associated strings
 //---------
 enum
 {
@@ -95,10 +109,11 @@ const char *LAWATAP_ATO_ERR_STRINGS[] = { "",
 				};
 //---------
 
-unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumcols, unsigned int index[], vector<IonHit> &posIons,const char *posFile, size_t limitCount,
-	       	unsigned int &progress, bool (*callback)(bool),bool strongSampling)
+unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumcols, const unsigned int index[], vector<IonHit> &posIons,const char *posFile, size_t limitCount,
+	       	unsigned int &progress, ATOMIC_BOOL &wantAbort,bool strongSampling)
 {
 
+
 	//Function is only defined for 4 columns here.
 	ASSERT(outputnumcols == 4);
 	//buffersize must be a power of two and at least outputnumcols*sizeof(float)
@@ -164,7 +179,7 @@ unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumc
 		delete[] buffer;
 		delete[] buffer2;
 		//Try opening it using the normal functions
-		return GenericLoadFloatFile(inputnumcols, outputnumcols, index, posIons,posFile,progress, callback);
+		return GenericLoadFloatFile(inputnumcols, outputnumcols, index, posIons,posFile,progress, wantAbort);
 	}
 
 	//Use a sampling method to load the pos file
@@ -177,7 +192,7 @@ unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumc
 		rng.initTimer();
 		unsigned int dummy;
 		randomDigitSelection(ionsToLoad,maxIons,rng,
-				limitCount,dummy,callback,strongSampling);
+				limitCount,dummy,strongSampling);
 	}
 	catch(std::bad_alloc)
 	{
@@ -187,9 +202,10 @@ unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumc
 	}
 
 
-	//sort again	
-	GreaterWithCallback<size_t> g(callback,PROGRESS_REDUCE);
-	std::sort(ionsToLoad.begin(),ionsToLoad.end(),g);
+	//sort again
+	//NOTE: I tried to use a functor here to get progress
+	// It was not stable with parallel sort	
+	std::sort(ionsToLoad.begin(),ionsToLoad.end());
 
 	unsigned int curProg = PROGRESS_REDUCE;	
 
@@ -228,14 +244,21 @@ unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumc
 			delete[] buffer2;
 			return POS_NAN_LOAD_ERROR;	
 		}
-			
+	
+		if(posIons[ui].hasInf())
+		{
+			delete[] buffer;
+			delete[] buffer2;
+			return POS_INF_LOAD_ERROR;	
+		}
+		
 		pointCount++;
 		if(!curProg--)
 		{
 
 			progress= (unsigned int)((float)(CFile.tellg())/((float)fileSize)*100.0f);
 			curProg=PROGRESS_REDUCE;
-			if(!(*callback)(false))
+			if(wantAbort)
 			{
 				delete[] buffer;
 				delete[] buffer2;
@@ -253,8 +276,8 @@ unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumc
 }
 
 unsigned int GenericLoadFloatFile(unsigned int inputnumcols, unsigned int outputnumcols, 
-		unsigned int index[], vector<IonHit> &posIons,const char *posFile, 
-			unsigned int &progress, bool (*callback)(bool))
+		const unsigned int index[], vector<IonHit> &posIons,const char *posFile, 
+			unsigned int &progress, ATOMIC_BOOL &wantAbort)
 {
 	ASSERT(outputnumcols==4); //Due to ionHit.setHit
 	//buffersize must be a power of two and at least sizeof(float)*outputnumCols
@@ -369,6 +392,14 @@ unsigned int GenericLoadFloatFile(unsigned int inputnumcols, unsigned int output
 					delete[] buffer2;
 					return POS_NAN_LOAD_ERROR;	
 				}
+
+				if(hit.hasInf())
+				{
+					delete[] buffer;
+					delete[] buffer2;
+					return POS_INF_LOAD_ERROR;	
+				}
+
 				posIons[ionP] = hit;
 				ionP++;
 				
@@ -379,7 +410,7 @@ unsigned int GenericLoadFloatFile(unsigned int inputnumcols, unsigned int output
 			{
 				progress= (unsigned int)((float)(CFile.tellg())/((float)fileSize)*100.0f);
 				curProg=PROGRESS_REDUCE;
-				if(!(*callback)(false))
+				if(wantAbort)
 				{
 					delete[] buffer;
 					delete[] buffer2;
@@ -406,9 +437,8 @@ unsigned int GenericLoadFloatFile(unsigned int inputnumcols, unsigned int output
 //TODO: Add progress
 unsigned int limitLoadTextFile(unsigned int maxCols, 
 			vector<vector<float> > &data,const char *textFile, const char *delim, const size_t limitCount,
-				unsigned int &progress, bool (*callback)(bool),bool strongRandom)
+				unsigned int &progress, ATOMIC_BOOL &wantAbort,bool strongRandom)
 {
-
 	ASSERT(maxCols);
 	ASSERT(textFile);
 
@@ -567,7 +597,7 @@ unsigned int limitLoadTextFile(unsigned int maxCols,
 		rng.initTimer();
 		unsigned int dummy;
 		randomDigitSelection(dataToLoad,newLinePositions.size(),rng,
-				limitCount,dummy,callback,strongRandom);
+				limitCount,dummy,strongRandom);
 	}
 	catch(std::bad_alloc)
 	{
@@ -575,11 +605,23 @@ unsigned int limitLoadTextFile(unsigned int maxCols,
 		return TEXT_ERR_ALLOC_FAIL;
 	}
 
-
+	//check for abort before/after sort, as this is a long process that we cannot
+	// safely abort
+	if(wantAbort)
+	{
+		delete[] buffer;
+		return POS_ABORT_FAIL;
+	}
 	//Sort the data such that we are going to
 	//always jump forwards in the file; better disk access and whatnot.
-	GreaterWithCallback<size_t> g(callback,PROGRESS_REDUCE);
-	std::sort(dataToLoad.begin(),dataToLoad.end(),g);
+	std::sort(dataToLoad.begin(),dataToLoad.end());
+	
+	//check again for abort
+	if(wantAbort)
+	{
+		delete[] buffer;
+		return POS_ABORT_FAIL;
+	}
 
 	//OK, so we have  a list of newlines
 	//that we can use as entry points for random seek.
@@ -648,7 +690,7 @@ unsigned int limitLoadTextFile(unsigned int maxCols,
 
 
 
-unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned int &progress, bool (*callback)(bool),unsigned int forceEndian)
+unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned int &progress, ATOMIC_BOOL &wantAbort,unsigned int forceEndian)
 {
 
 
@@ -727,8 +769,8 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	}
 
 
-	//Heuristic fo detect endianness.
-	// - Randomly sample 100 pts from file, and check to see if, when interpreted either wa
+	//Heuristic to detect endianness.
+	// - Randomly sample 100 pts from file, and check to see if, when interpreted either way,
 	// there are any NaN
 	//   
 
@@ -755,9 +797,9 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 		RandNumGen rng;
 		rng.initTimer();
 		randomDigitSelection(randomNumbers,pointCount,rng, 
-					numToCheck,dummy,dummyCallback);
+					numToCheck,dummy,wantAbort);
 
-		//Make the travese in ascending order
+		//Make the traverse in ascending order
 		std::sort(randomNumbers.begin(),randomNumbers.end());
 
 		//One for no endian-flip, one for flip
@@ -848,7 +890,7 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 
 	//File records consist of 14 fields, some of which may not be initialised.
 	// each being 4-byte IEEE little-endian float
-	// It is unknown how to detect initalisated state.
+	// It is unknown how to detect initialised state.
 	// Field 	Data	
 	// 0-3		x,y,z,m/c in Angstrom (x,yz) or Da (m/c)
 	// 4		clusterID, if set
@@ -861,7 +903,7 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	// 11		"Virtual voltage" for reconstruction.
 	//			- Ignore this field, as this information is redundant
 	// 12,13	Fourier intensity
-	//			- Ignore theese fields, as this information is redundant
+	//			- Ignore these fields, as this information is redundant
 	//Attempt to detect
 	CFile.seekg(8);
 
@@ -885,7 +927,7 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	}
 	else
 	{
-		//read wthout swapping
+		//read without swapping
 		while((size_t)CFile.tellg() < fileSize)
 		{	
 			CFile.read((char*)buffer,LAWATAP_ATO_RECORD_SIZE);
@@ -975,27 +1017,26 @@ bool testATOFormat()
 	unsigned int dummyProgress;
 
 
+	ATOMIC_BOOL wantAbort;
+	wantAbort=false;
 	vector<IonHit> ions;
 	//Load using auto-detection of endinanness
-	TEST(!LoadATOFile(filename.c_str(),ions,dummyProgress,dummyCallback),"ATO load test  (auto endianness)");
+	TEST(!LoadATOFile(filename.c_str(),ions,dummyProgress,wantAbort),"ATO load test  (auto endianness)");
 
 	TEST(ions.size() == 100,"ion size check");
 
-	TEST((ions[0].getPos().sqrDist(Point3D(1,1,0)) < sqrt(std::numeric_limits<float>::epsilon())),"Checking read/write OK");
+	TEST((ions[0].getPos().sqrDist(Point3D(1,1,0)) < sqrtf(std::numeric_limits<float>::epsilon())),"Checking read/write OK");
 	//Load using auto-detection of endinanness
 
 	//Load, forcing assuming cont4ents are little endianness as requried
-	TEST(!LoadATOFile(filename.c_str(),ions,dummyProgress,dummyCallback,1),"ATO load test (forced endianness)");
+	TEST(!LoadATOFile(filename.c_str(),ions,dummyProgress,wantAbort,1),"ATO load test (forced endianness)");
 	TEST(ions.size() == 100,"ion size check");
-	TEST((ions[0].getPos().sqrDist(Point3D(1,1,0)) < sqrt(std::numeric_limits<float>::epsilon())),"checking read/write OK");
+	TEST((ions[0].getPos().sqrDist(Point3D(1,1,0)) < sqrtf(std::numeric_limits<float>::epsilon())),"checking read/write OK");
 
 
 	
 
-	rmFile(filename.c_str());
-
-
-
+	rmFile(filename);
 
 	return true;
 
