@@ -1,6 +1,6 @@
 /*
  * APTClasses.h - Generic APT components header 
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,8 @@ extern const char *ION_TEXT_ERR_STRINGS[];
 
 extern const char *LAWATAP_ATO_ERR_STRINGS[];
 
+extern const char *TEXT_LOAD_ERR_STRINGS[];
+
 //!Errors that can be encountered when openning pos files
 enum posErrors
 {
@@ -48,6 +50,7 @@ enum posErrors
 	POS_SIZE_MODULUS_ERR,	
 	POS_READ_FAIL,
 	POS_NAN_LOAD_ERROR,
+	POS_INF_LOAD_ERROR,
 	POS_ABORT_FAIL,
 	POS_ERR_FINAL // Not actually an error, but tells us where the end of the num is.
 };
@@ -62,26 +65,26 @@ enum posErrors
  * */
 //!Load a pos file into a T of IonHits
 unsigned int GenericLoadFloatFile(unsigned int inputnumcols, unsigned int outputnumcols, 
-		unsigned int index[], vector<IonHit> &posIons,const char *posFile, 
-				unsigned int &progress, bool (*callback)(bool));
+		const unsigned int index[], vector<IonHit> &posIons,const char *posFile, 
+				unsigned int &progress, ATOMIC_BOOL &wantAbort);
 
 
-unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumcols, unsigned int index[], 
+unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumcols, const unsigned int index[], 
 			vector<IonHit> &posIons,const char *posFile, size_t limitCount,
-					       	unsigned int &progress, bool (*callback)(bool),bool strongRandom);
+					       	unsigned int &progress, ATOMIC_BOOL &wantAbort,bool strongRandom);
 
 
 
 unsigned int limitLoadTextFile(unsigned int numColsTotal, 
 			vector<vector<float> > &data,const char *posFile, const char *deliminator, const size_t limitCount,
-					       	unsigned int &progress, bool (*callback)(bool),bool strongRandom);
+					       	unsigned int &progress, ATOMIC_BOOL &wantAbort,bool strongRandom);
 
 
 //Load a CAMECA LAWATAP "ATO" formatted file.
 //	- This is a totally different format to the "FlexTAP" ato format
 //Supported versions are "version 3"
 //	Force endian : 0 - do not force, autodetect, 1 - force little, 2- force big
-unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned int &progressm, bool (*callback)(bool), unsigned int forceEndian=0);
+unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned int &progressm, ATOMIC_BOOL &wantAbort, unsigned int forceEndian=0);
 
 
 #ifdef DEBUG
