@@ -1,6 +1,6 @@
 /*
  * APTRanges.cpp - Atom probe rangefile class 
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,8 @@ using std::accumulate;
 
 //Arbitrary maximum range file line size
 const size_t MAX_LINE_SIZE = 16536;
-
+//Arbitrary maximum range file size, in bytes
+const size_t MAX_RANGEFILE_SIZE = 20*1024*1024;
 const char *rangeErrStrings[] = 
 {
 	"",
@@ -62,6 +63,7 @@ const char *rangeErrStrings[] =
 	NTRANS("Range file appears to be inconsistent (eg, overlapping ranges)"),
 	NTRANS("No ion name mapping found  for multiple ion."),
 	NTRANS("Polyatomic extension range matches multiple masses in first section"),
+	NTRANS("Range file is exceedingly large. Refusing to open"),
 };
 
 const char *RANGE_EXTS[] = { "rng",
@@ -94,6 +96,7 @@ const char *elementList[] = {
 };	
 
 
+//Known issues - will not decompose brackets, eg Fe(OH)2
 bool RangeFile::decomposeIonNames(const std::string &name,
 		std::vector<pair<string,size_t> > &fragments)
 {
@@ -197,17 +200,17 @@ bool RangeFile::decomposeIonNames(const std::string &name,
 	vector<bool> toKill(fragments.size(),false);
 	for(size_t ui=0;ui<fragments.size();ui++)
 	{
-		//skip empty framgnets
+		//skip empty fragments
 		if(fragments[ui].first.empty())
-			ui++;
+			continue;	
 
 		for(size_t uj=ui+1;uj<fragments.size();uj++)
 		{
-			//skip empty gragments
+			//skip empty fragments
 			if(fragments[uj].first.empty())
-				uj++;
+				continue;	
 
-			//Collect fragment multiplcities if they have the same name
+			//Collect fragment multiplicities if they have the same name
 			if(fragments[uj].first == fragments[ui].first)
 			{
 				fragments[ui].second+=fragments[uj].second;
@@ -470,17 +473,16 @@ unsigned int RangeFile::write(std::ostream &f, size_t format) const
 
 			for(size_t ui=0;ui<ranges.size();ui++)
 			{
+				ColourRGBA tmpRgba;
+				tmpRgba.fromRGBf(colours[ui]);
 				std::string colString;
-				genColString((unsigned char)colours[ui].red*255,
-					(unsigned char)colours[ui].green*255,
-					(unsigned char)colours[ui].blue*255,colString);
-
+				colString = tmpRgba.rgbString();
 				//strip leading #
 				colString=colString.substr(1);
 				ASSERT(colString.size() == 6);
 
 				//FIXME: This is incomplete. we need to break the species apart into its 
-				// compponents, then decide to use the Element:count notation, or the Name:species notation
+				// components, then decide to use the Element:count notation, or the Name:species notation
 				string strName;
 				strName=ionNames[ionIDs[ui]].first;
 				if(elementSet.find(strName) != elementSet.end())
@@ -492,6 +494,8 @@ unsigned int RangeFile::write(std::ostream &f, size_t format) const
 				}
 				else
 				{
+					//Difference is we have to use the "Name" parameter if not in set
+					
 					f << "Range" << ui+1 <<"=" 
 					<< ranges[ui].first << " " << ranges[ui].second <<
 					" " << "Name:" << strName << ":1" << 
@@ -531,6 +535,12 @@ void RangeFile::clear()
 
 unsigned int RangeFile::open(const char *rangeFilename, unsigned int fileFormat)
 {
+	size_t fileSize;
+	getFilesize(rangeFilename,fileSize);
+
+	if(fileSize > MAX_RANGEFILE_SIZE)
+		return RANGE_ERR_FILESIZE;
+
 	FILE *fpRange;
 	fpRange=fopen(rangeFilename,"r");
 	if (fpRange== NULL) 
@@ -594,10 +604,17 @@ unsigned int RangeFile::open(const char *rangeFilename, unsigned int fileFormat)
 
 bool RangeFile::openGuessFormat(const char *rangeFilename)
 {
-	unsigned int assumedFileFormat;
-
 
+	//Check the filesize before attempting to guess the format
+	size_t fileSize;
+	getFilesize(rangeFilename,fileSize);
+	if(fileSize > MAX_RANGEFILE_SIZE)
+	{
+		errState=RANGE_ERR_FILESIZE;
+		return false;
+	}
 	//Try to auto-detect the filetype
+	unsigned int assumedFileFormat;
 	assumedFileFormat=detectFileType(rangeFilename);
 
 	if(assumedFileFormat < RANGE_FORMAT_END_OF_ENUM)
@@ -651,7 +668,7 @@ unsigned int RangeFile::openDoubleRNG(FILE *fpRange)
 	if(errCode)
 		return errCode;
 
-	//Spin forwards ot the "polyatomic extension" line
+	//Spin forwards to the "polyatomic extension" line
 
 		
 	char *inBuffer = new char[MAX_LINE_SIZE];
@@ -681,7 +698,7 @@ unsigned int RangeFile::openDoubleRNG(FILE *fpRange)
 	//Now merge the two range files by using the mass pair data as a key
 
 	//Find the matching ranges
-	//range IDs from first and second file who have matching rnage values
+	//range IDs from first and second file who have matching range values
 	vector< pair<size_t,size_t> > rangeMatches;
 	//IonID from the first dataset that we will need to replace
 	vector<size_t> overrideIonID;
@@ -897,7 +914,7 @@ unsigned int RangeFile::openRNG( FILE *fpRange)
 			if(composeMap.empty())
 			{
 				//We have a multiple, but no way of composing it!
-				// we will need ot build our own table entry.
+				// we will need to build our own table entry.
 				// For now, just store the freq tableentry
 				unassignedMultiples.push_back(make_pair(ui,freqEntries));
 				ionIDs.push_back(-2);
@@ -1715,8 +1732,9 @@ unsigned int RangeFile::openENV(FILE *fpRange)
 	//There should be more data following the range information.
 	// if not, this is not really an env file
 	if(feof(fpRange))
+	{
 		return RANGE_ERR_FORMAT;
-
+	}	
 
 	return 0;
 }
@@ -1829,10 +1847,10 @@ unsigned int RangeFile::openRRNG(FILE *fpRange)
 					basicIonNames.push_back(split[1]);
 
 					if (basicIonNames.size()  > numBasicIons)
-				{
-					delete[] inBuffer;
-					return RANGE_ERR_FORMAT;
-				}
+					{
+						delete[] inBuffer;
+						return RANGE_ERR_FORMAT;
+					}
 				}
 				else
 				{
@@ -1996,13 +2014,16 @@ unsigned int RangeFile::openRRNG(FILE *fpRange)
 							//which requires a leading #,
 							//in lowercase
 							value = string("#") + lowercase(value);
-							unsigned char r,g,b,a;
+							ColourRGBA tmpRgba;
+							if(!tmpRgba.parse(value))
+							{
+								delete[] inBuffer;
+								return RANGE_ERR_FORMAT;
+							}
 
-							parseColString(value,r,g,b,a);
-
-							col.red = (float)r/255.0f;
-							col.green=(float)g/255.0f;
-							col.blue=(float)b/255.0f;
+							col.red = (float)tmpRgba.r()/255.0f;
+							col.green=(float)tmpRgba.g()/255.0f;
+							col.blue=(float)tmpRgba.b()/255.0f;
 						}
 						else
 						{
@@ -2850,3 +2871,80 @@ void RangeFile::eraseIon(size_t ionId)
 
 
 }
+
+bool RangeFile::decompose(RangeFile &rng) const
+{
+	//find the list of decomposables
+	rng=*this;	
+	for(size_t ui=0; ui<ionNames.size(); ui++)
+	{	
+		vector<pair<string,size_t > > fragments;
+		if(!decomposeIonNames(ionNames[ui].first,fragments))
+			return false;
+
+		for(size_t uj=0; uj<fragments.size();uj++)
+		{
+			if(rng.getIonID(fragments[uj].first) == -1)
+			{
+				//make a new random colour
+				RGBf col;
+				col.red=rand()/(float)std::numeric_limits<int>::max();
+				col.green=rand()/(float)std::numeric_limits<int>::max();
+				col.blue=rand()/(float)std::numeric_limits<int>::max();
+				//Create a new ion to support this fragment
+				rng.addIon(fragments[uj].first,fragments[uj].first,col);
+			}
+		}	
+	}	
+	ASSERT(rng.isSelfDecomposable());
+
+	return true;
+}
+
+bool RangeFile::isSelfDecomposable() const
+{
+	for(size_t ui=0; ui<ionNames.size(); ui++)
+	{	
+		vector<pair<string,size_t > > fragments;
+		if(!decomposeIonNames(ionNames[ui].first,fragments))
+			return false;
+		//ion cannot be decomposed into smaller fragments. 
+		//This is not a decomposable rangefile
+		if(getIonID(ionNames[ui].first) == -1)
+			return false;
+	}	
+
+	return true;
+}
+
+bool RangeFile::getDecomposition(std::map<unsigned int, vector< std::pair<unsigned int, unsigned int> > > &decomposition) const
+{
+	decomposition.clear();
+	//TODO: Convert to cached, statified map?
+	for(size_t ui=0;ui<ionNames.size(); ui++)
+	{
+		vector<pair<string,size_t> > thisFragment;
+		if(!decomposeIonNames(ionNames[ui].first,thisFragment))
+			return false;
+
+		//convert the name,count pairs to  ionID, count pairs
+		vector<pair<unsigned int, unsigned int> > fragmentAsRanges;
+		fragmentAsRanges.resize(thisFragment.size());
+		
+		for(size_t uj=0;uj<fragmentAsRanges.size();uj++)
+		{	
+			unsigned int ionID;
+			ionID = getIonID(thisFragment[uj].first);
+			
+			if(ionID ==(unsigned int)-1)
+				return false;
+	
+			fragmentAsRanges[uj] = make_pair(ionID,thisFragment[uj].second);
+		}
+		decomposition[ui] = fragmentAsRanges;
+	
+		fragmentAsRanges.clear();	
+	}
+
+	return true;
+}
