@@ -1,6 +1,6 @@
 /*
  * APTRanges.h - Atom probe rangefile class 
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,11 @@
 
 #include <vector>
 #include <string>
+#include <map>
 
 
 #include "backend/APT/ionhit.h"
+#include "common/basics.h"
 
 enum{	
 	RANGE_ERR_OPEN =1, 
@@ -44,16 +46,10 @@ enum{
 	RANGE_ERR_DATA_INCONSISTENT,
 	RANGE_ERR_DATA_NOMAPPED_IONNAME,
 	RANGE_ERR_NONUNIQUE_POLYATOMIC,
+	RANGE_ERR_FILESIZE,
 	RANGE_ERR_ENUM_END
 };
 
-//!Data holder for colour as float
-typedef struct RGBf
-{
-	float red;
-	float green;
-	float blue;
-} RGBf;
 
 //Number of elements stored in the table
 const unsigned int NUM_ELEMENTS=119;
@@ -64,6 +60,7 @@ enum{ RANGE_FORMAT_ORNL,
 	RANGE_FORMAT_RRNG,
 	RANGE_FORMAT_END_OF_ENUM //not a format, just end of enumueration.
 };
+
 //!Data storage and retrieval class for various range files
 class RangeFile
 {
@@ -132,7 +129,7 @@ class RangeFile
 		RangeFile();
 
 		const RangeFile& operator=(const RangeFile &other);
-		//!Open a specified range file
+		//!Open a specified range file, returns zero on success, nonzero on failure
 		unsigned int open(const char *rangeFile, unsigned int format=RANGE_FORMAT_ORNL);	
 		//!Open a specified range file - returns true on success
 		bool openGuessFormat(const char *rangeFile);
@@ -283,6 +280,18 @@ class RangeFile
 		//erase given ions and associated rnagefes)
 		void eraseIon(size_t ionId);
 
+		//can we decompose all composed ranges in this file into
+		// other components that already exist within this range?
+		bool isSelfDecomposable() const;
+		
+		//Generate a secondary rangefile with decomposable ranges as needed.	
+		bool decompose(RangeFile &rng) const;
+
+		//Obtain the decompsiition for this range. This maps composed ranges to their decomposed ones.
+		//  - This will perform a decompsition if needed, and return false if not self decomposable.
+		bool getDecomposition(
+			std::map<unsigned int, std::vector< std::pair< unsigned int, unsigned int > > > &decomposition) const;
+	
 		//Break a given string down into a series of substring-count pairs depicting basic ionic components
 		static bool decomposeIonNames(const std::string &name,
 
