@@ -1,6 +1,6 @@
 /*
  * ionhit.cpp - Ion event data class
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -176,6 +176,12 @@ bool IonHit::hasNaN()
 				std::isnan(pos[1]) || std::isnan(pos[2]));
 }
 
+bool IonHit::hasInf()
+{
+	return (std::isinf(massToCharge) || std::isinf(pos[0]) || 
+				std::isinf(pos[1]) || std::isinf(pos[2]));
+}
+
 void IonHit::getCentroid(const std::vector<IonHit> &points,Point3D &centroid)
 {
 	centroid=Point3D(0,0,0);
