@@ -1,6 +1,6 @@
 /*
  * ionhit.h - Ion event data class
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +50,8 @@ class IonHit
 		inline const Point3D &getPosRef() const {return pos;};
 		//returns true if any of the 4 data pts are NaN
 		bool hasNaN();
+		//returns true if any of the 4 data pts are +-inf
+		bool hasInf();
 
 #ifdef __LITTLE_ENDIAN__		
 		void switchEndian();
