@@ -1,6 +1,6 @@
 /*
- *	filter.h - modular data filter implementation 
- *	Copyright (C) 2013, D Haley 
+ *	filter.cpp - modular data filter implementation 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -17,13 +17,17 @@
 */
 
 #include "filter.h"
+#include "plot.h"
 
 #include "common/stringFuncs.h"
 #include "common/translation.h"
 
 #include "wx/wxcomponents.h"
 
+#include "common/voxels.h"
+
 #include <set>
+#include <deque>
 
 #ifdef _OPENMP
 #include <omp.h>
@@ -34,14 +38,18 @@ using std::vector;
 using std::string;
 using std::pair;
 using std::make_pair;
+using std::endl;
+using std::deque;
 
 
 
 
 bool Filter::strongRandom= false;
+ATOMIC_BOOL *Filter::wantAbort= 0;
 
 const char *STREAM_NAMES[] = { NTRANS("Ion"),
 				NTRANS("Plot"),
+				NTRANS("2D Plot"),
 				NTRANS("Draw"),
 				NTRANS("Range"),
 				NTRANS("Voxel")};
@@ -64,48 +72,6 @@ const char *FILTER_NAMES[] = { "posload",
 				"annotation"
 				};
 
-void updateFilterPropertyGrid(wxCustomPropGrid *g, const Filter *f)
-{
-
-	ASSERT(f);
-	ASSERT(g);
-	
-	FilterPropGroup p;
-	f->getProperties(p);
-#ifdef DEBUG
-	//If debugging, test self consistency
-	p.checkConsistent();
-#endif	
-	g->clearKeys();
-	g->setNumGroups(p.numGroups());
-
-	
-	//Create the keys to add to the grid
-	for(size_t ui=0;ui<p.numGroups();ui++)
-	{
-		vector<FilterProperty> propGrouping;
-		p.getGroup(ui,propGrouping);
-
-		for(size_t uj=0;uj<propGrouping.size();uj++)
-		{
-			g->addKey(propGrouping[uj].name,ui,
-				propGrouping[uj].key,
-				propGrouping[uj].type,
-				propGrouping[uj].data,
-				propGrouping[uj].helpText);
-		}
-
-		//Set the name that is to be displayed for this grouping
-		// of properties
-		std::string title;
-		p.getGroupTitle(ui,title);
-		g->setGroupName(ui,title);
-	}
-	
-	//Let the property grid layout what it needs to
-	g->propertyLayout();
-}
-
 size_t numElements(const vector<const FilterStreamData *> &v, unsigned int mask)
 {
 	size_t nE=0;
@@ -118,6 +84,96 @@ size_t numElements(const vector<const FilterStreamData *> &v, unsigned int mask)
 	return nE;
 }
 
+
+template<>
+bool Filter::applyPropertyNow(bool &prop, const std::string &val, bool &needUp)
+{
+	needUp=false;
+	bool tmp;
+	if(!boolStrDec(val,tmp))
+		return false;
+	
+	//return true, as technically, we did something OK
+	// but erasing the cache, and re-setting the value is pointless
+	if(tmp == prop)
+		return true;
+	
+	prop=tmp;
+	clearCache();
+
+	needUp=true;
+	return true;	
+}
+
+template<>
+bool Filter::applyPropertyNow(Point3D &prop, const std::string &val, bool &needUp)
+{
+	needUp=false;
+
+	Point3D newPt;	
+	if(!newPt.parse(val))
+		return false;
+	
+	//return true, as technically, we did something OK
+	// but erasing the cache, and re-setting the value is pointless
+	if(newPt== prop)
+		return true;
+	
+	prop=newPt;
+	clearCache();
+	needUp=true;
+	return true;	
+}
+
+template<>
+bool Filter::applyPropertyNow(std::string &prop, const std::string &val, bool &needUp)
+{
+	needUp=false;
+
+	//return true, as it is technically ok that we did this
+	if(val == prop)
+		return true;
+	
+	prop=val;
+	clearCache();
+	needUp=true;
+	return true;	
+}
+
+void Filter::cacheAsNeeded(FilterStreamData *stream)
+{
+	if(cache)
+	{
+		stream->cached=1;
+		filterOutputs.push_back(stream);
+		cacheOK=true;
+	}	
+	else
+	{
+		stream->cached=0;
+	}
+}
+
+std::string Filter::getErrString(unsigned int errCode) const
+{
+	//First see if we have a generic error code, before attempting to
+	// switch to a specific error code
+	std::string errString;
+	errString = getBaseErrString(errCode);
+	if(!errString.empty())
+		return errString;
+	return this->getSpecificErrString(errCode);
+}
+
+//If we recognise a base error string, check that
+std::string Filter::getBaseErrString(unsigned int errCode)
+{
+	if(errCode == FILTER_ERR_ABORT)
+		return TRANS("Aborted");
+
+	return string("");
+}
+
 #ifdef DEBUG
 bool FilterProperty::checkSelfConsistent() const
 {	
@@ -143,8 +199,8 @@ bool FilterProperty::checkSelfConsistent() const
 		}
 		case PROPERTY_TYPE_COLOUR:
 		{
-			unsigned char r,g,b,a;
-			if(!parseColString(data,r,g,b,a))
+			ColourRGBA rgba;
+			if(!rgba.parse(data))
 				return false;
 
 			break;
@@ -225,6 +281,16 @@ void FilterPropGroup::getGroup(size_t targetGroup, vector<FilterProperty> &vec)
 #endif
 }
 
+bool FilterPropGroup::hasGroup(size_t targetGroup) const
+{
+	for(size_t ui=0;ui<keyGroupings.size();ui++)
+	{
+		if(keyGroupings[ui].second==targetGroup)
+			return true;
+	}
+
+	return false;
+}
 void FilterPropGroup::getGroupTitle(size_t group, std::string &s) const
 {
 	ASSERT(group < groupNames.size());
@@ -282,14 +348,16 @@ void FilterPropGroup::checkConsistent() const
 	//Check that the group names are the same as the number of groups
 	ASSERT(groupNames.size() ==groupCount);
 
+
+	//check that each group has a name
+	for(size_t ui=0;ui<groupNames.size(); ui++)
+	{
+		ASSERT(!groupNames[ui].empty())
+	}
 }
 #endif
 
 
-void VoxelStreamData::clear()
-{
-	data.clear();
-}
 
 void DrawStreamData::clear()
 {
@@ -320,7 +388,7 @@ void DrawStreamData::checkSelfConsistent() const
 #endif
 
 PlotStreamData::PlotStreamData() :  r(1.0f),g(0.0f),b(0.0f),a(1.0f),
-	plotStyle(PLOT_TRACE_LINES), logarithmic(false) , useDataLabelAsYDescriptor(true), 
+	plotStyle(PLOT_LINE_LINES), logarithmic(false) , useDataLabelAsYDescriptor(true), 
 	index((unsigned int)-1)
 {
 	streamType=STREAM_TYPE_PLOT;
@@ -422,7 +490,7 @@ void PlotStreamData::checkSelfConsistent() const
 	ASSERT(!(regionID.size() && !regionParent));
 
 	//Must have valid trace style
-	ASSERT(plotStyle<PLOT_TRACE_ENDOFENUM);
+	ASSERT(plotStyle<PLOT_TYPE_ENUM_END);
 	//Must have valid error bar style
 	ASSERT(errDat.mode<PLOT_ERROR_ENDOFENUM);
 
@@ -433,7 +501,41 @@ void PlotStreamData::checkSelfConsistent() const
 }
 #endif
 
+Plot2DStreamData::Plot2DStreamData()
+{
+	streamType=STREAM_TYPE_PLOT2D;
+	r=g=0.0f;
+	b=a=1.0f;
+
+	scatterIntensityLog=false;
+}
+
+size_t Plot2DStreamData::getNumBasicObjects() const
+{
+	if(xyData.size())
+		return xyData.size();
+	else if (scatterData.size())
+		return scatterData.size();
+	else
+		ASSERT(false);
+
+	return 0;
+}
+
 #ifdef DEBUG
+
+void Plot2DStreamData::checkSelfConsistent() const
+{
+	//only using scatter or xy, not both
+	ASSERT(xorFunc(xyData.empty(), scatterData.empty()));
+
+	//no intensity without data
+	if(scatterData.empty())
+		ASSERT(scatterIntensity.empty());
+
+
+	ASSERT(plotType < PLOT_TYPE_ENUM_END);
+}
 void RangeStreamData::checkSelfConsistent() const
 {
 	if(!rangeFile)
@@ -450,6 +552,51 @@ FilterStreamData::FilterStreamData() : parent(0),cached((unsigned int)-1)
 {
 }
 
+FilterStreamData::FilterStreamData(const Filter  *theParent) : parent(theParent),cached((unsigned int)-1)
+{
+}
+
+
+unsigned int IonStreamData::exportStreams(const std::vector<const FilterStreamData * > &selectedStreams,
+		const std::string &outFile, unsigned int format)
+{
+
+	//test file open, and truncate file to zero bytes
+	std::ofstream f(outFile.c_str(),std::ios::trunc);
+	
+	if(!f)
+		return 1;
+
+	f.close();
+
+	for(unsigned int ui=0; ui<selectedStreams.size(); ui++)
+	{
+		switch(selectedStreams[ui]->getStreamType())
+		{
+			case STREAM_TYPE_IONS:
+			{
+				const IonStreamData *ionData;
+				ionData=((const IonStreamData *)(selectedStreams[ui]));
+				switch(format)
+				{
+					case IONFORMAT_POS:
+					{
+						//Append this ion stream to the posfile
+						IonHit::appendPos(ionData->data,outFile.c_str());
+
+						break;
+					}
+					default:
+						ASSERT(false);
+						break;
+				}
+			}
+		}
+	}
+
+	return 0;
+}
+
 IonStreamData::IonStreamData() : representationType(ION_REPRESENT_POINTS), 
 	r(1.0f), g(0.0f), b(0.0f), a(1.0f), 
 	ionSize(2.0f), valueType("Mass-to-Charge (amu/e)")
@@ -457,6 +604,14 @@ IonStreamData::IonStreamData() : representationType(ION_REPRESENT_POINTS),
 	streamType=STREAM_TYPE_IONS;
 }
 
+IonStreamData::IonStreamData(const Filter *f) : FilterStreamData(f), representationType(ION_REPRESENT_POINTS), 
+	r(1.0f), g(0.0f), b(0.0f), a(1.0f), 
+	ionSize(2.0f), valueType("Mass-to-Charge (amu/e)")
+{
+	streamType=STREAM_TYPE_IONS;
+}
+
+
 void IonStreamData::clear()
 {
 	data.clear();
@@ -503,6 +658,30 @@ VoxelStreamData::VoxelStreamData() : representationType(VOXEL_REPRESENT_POINTCLO
 	r(1.0f),g(0.0f),b(0.0f),a(0.3f), splatSize(2.0f),isoLevel(0.5f)
 {
 	streamType=STREAM_TYPE_VOXEL;
+	data = new Voxels<float>;
+}
+
+VoxelStreamData::VoxelStreamData(const Filter *f) : FilterStreamData(f), representationType(VOXEL_REPRESENT_POINTCLOUD),
+	r(1.0f),g(0.0f),b(0.0f),a(0.3f), splatSize(2.0f),isoLevel(0.5f)
+{
+	streamType=STREAM_TYPE_VOXEL;
+	data = new Voxels<float>;
+}
+
+VoxelStreamData::~VoxelStreamData()
+{
+	if(data)
+		delete data;
+}
+
+size_t VoxelStreamData::getNumBasicObjects() const
+{
+	return data->getSize(); 
+}
+
+void VoxelStreamData::clear()
+{
+	data->clear();
 }
 
 RangeStreamData::RangeStreamData() : rangeFile(0)
@@ -510,6 +689,11 @@ RangeStreamData::RangeStreamData() : rangeFile(0)
 	streamType = STREAM_TYPE_RANGE;
 }
 
+RangeStreamData::RangeStreamData(const Filter *f) : FilterStreamData(f), rangeFile(0)
+{
+	streamType = STREAM_TYPE_RANGE;
+}
+
 bool RangeStreamData::save(const char *filename, size_t format) const
 {
 	return !rangeFile->write(filename,format);
@@ -517,6 +701,7 @@ bool RangeStreamData::save(const char *filename, size_t format) const
 
 Filter::Filter() : cache(true), cacheOK(false)
 {
+	COMPILE_ASSERT( THREEDEP_ARRAYSIZE(STREAM_NAMES) == NUM_STREAM_TYPES);
 	for(unsigned int ui=0;ui<NUM_STREAM_TYPES;ui++)
 		numStreamsLastRefresh[ui]=0;
 }
@@ -597,7 +782,7 @@ void Filter::propagateCache(vector<const FilterStreamData *> &getOut) const
 }
 
 void Filter::propagateStreams(const vector<const FilterStreamData *> &dataIn,
-		vector<const FilterStreamData *> &dataOut,size_t mask,bool invertMask) const
+		vector<const FilterStreamData *> &dataOut,size_t mask,bool invertMask)
 {
 	//Propagate any inputs that we don't normally block
 	if(invertMask)
@@ -611,7 +796,7 @@ void Filter::propagateStreams(const vector<const FilterStreamData *> &dataIn,
 
 unsigned int Filter::collateIons(const vector<const FilterStreamData *> &dataIn,
 				vector<IonHit> &outVector, ProgressData &prog,
-				bool (*callback)(bool),size_t totalDataSize)
+				size_t totalDataSize)
 {
 	if(totalDataSize==(size_t)-1)
 		totalDataSize=numElements(dataIn,STREAM_TYPE_IONS);
@@ -639,7 +824,7 @@ unsigned int Filter::collateIons(const vector<const FilterStreamData *> &dataIn,
 				for(size_t ui=0;ui<dataSize; ui++)
 					outVector[offset+ui]=d->data[ui];
 
-				if(!(*callback)(false))
+				if(Filter::wantAbort)
 					return FILTER_ERR_ABORT;
 				offset+=d->data.size();
 
@@ -655,7 +840,7 @@ unsigned int Filter::collateIons(const vector<const FilterStreamData *> &dataIn,
 
 void Filter::updateOutputInfo(const std::vector<const FilterStreamData *> &dataOut)
 {
-	//Reset the number ouf output streams to zero
+	//Reset the number of output streams to zero
 	for(unsigned int ui=0;ui<NUM_STREAM_TYPES;ui++)
 		numStreamsLastRefresh[ui]=0;
 	
@@ -688,6 +873,17 @@ void Filter::initFilter(const std::vector<const FilterStreamData *> &dataIn,
 	std::copy(dataIn.begin(),dataIn.end(),dataOut.begin());
 }
 
+ProgressData::ProgressData()
+{
+	step=0;
+	maxStep=0;
+	curFilter=0;
+	filterProgress=0;
+	totalProgress=0;
+	totalNumFilters=0;
+}
+
+
 bool ProgressData::operator==( const ProgressData &oth) const
 {
 	if(filterProgress!=oth.filterProgress ||
