@@ -1,6 +1,6 @@
 /*
  *	filter.h - Data filter header file. 
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ class ProgressData;
 class RangeFileFilter;
 
 #include "APT/ionhit.h"
+#include "APT/APTFileIO.h"
 
 #include "APT/APTRanges.h"
 #include "common/constants.h"
@@ -32,7 +33,11 @@ class RangeFileFilter;
 #include "gl/select.h"
 #include "gl/drawables.h"
 
-#include "common/voxels.h"
+#include "common/stringFuncs.h"
+#include "common/array2D.h"
+
+template<typename T>
+class Voxels;
 
 //ifdef inclusion as there is some kind of symbol clash...
 #ifdef ATTRIBUTE_PRINTF
@@ -46,6 +51,8 @@ class RangeFileFilter;
 
 
 
+#include <wx/propgrid/propgrid.h>
+
 const unsigned int NUM_CALLBACK=50000;
 
 const unsigned int IONDATA_SIZE=4;
@@ -71,6 +78,7 @@ enum
 	FILTER_TYPE_ENUM_END // not a filter. just end of enum
 };
 
+
 extern const char *FILTER_NAMES[];
 
 
@@ -79,15 +87,16 @@ extern const char *FILTER_NAMES[];
 //the number of stream types that we can have.
 //Current bitmask using functions are
 //	VisController::safeDeleteFilterList
-const unsigned int NUM_STREAM_TYPES=5;
+const unsigned int NUM_STREAM_TYPES=6;
 const unsigned int STREAMTYPE_MASK_ALL= ((1<<(NUM_STREAM_TYPES)) -1 ) & 0x000000FF;
 enum
 {
 	STREAM_TYPE_IONS=1,
 	STREAM_TYPE_PLOT=2,
-	STREAM_TYPE_DRAW=4,
-	STREAM_TYPE_RANGE=8,
-	STREAM_TYPE_VOXEL=16
+	STREAM_TYPE_PLOT2D=4,
+	STREAM_TYPE_DRAW=8,
+	STREAM_TYPE_RANGE=16,
+	STREAM_TYPE_VOXEL=32
 };
 
 
@@ -138,23 +147,19 @@ enum
 };
 
 
+//!Generic filter error codes
 enum
 {
-	FILTER_ERR_ABORT=1, //User abort
-	FILTER_ERR_ENUM_END
+	FILTER_ERR_ABORT = 1000000,
 };
 
 //---
 //
 
-//Forward dec.
-class wxCustomPropGrid;
-
 //!Return the number of elements in a vector of filter data - i.e. the sum of the number of objects within each stream. Only masked streams (STREAM_TYPE_*) will be counted
-size_t numElements(const vector<const FilterStreamData *> &vm, unsigned int mask=STREAMTYPE_MASK_ALL);
+size_t numElements(const std::vector<const FilterStreamData *> &vm, unsigned int mask=STREAMTYPE_MASK_ALL);
 
 
-void updateFilterPropertyGrid(wxCustomPropGrid *g, const Filter *f);
 
 //!Abstract base class for data types that can propagate through filter system
 class FilterStreamData
@@ -171,6 +176,7 @@ class FilterStreamData
 		unsigned int cached;
 
 		FilterStreamData();
+		FilterStreamData(const Filter *);
 		virtual ~FilterStreamData() {}; 
 		virtual size_t getNumBasicObjects() const =0;
 		//!Returns an integer unique to the class to identify type (yes rttid...)
@@ -197,6 +203,9 @@ class FilterProperty
 	size_t key;
 	//!Property data
 	std::string data;
+	//!Secondary property data
+	//	- eg for file, contains wildcard mask for filename
+	std::string dataSecondary;
 	//!name of property
 	std::string name;
 
@@ -242,7 +251,10 @@ class FilterPropGroup
 				groupCount=0;}
 
 		//!Grab all properties from the specified group
-		void getGroup(size_t group, vector<FilterProperty> &groupVec) const;
+		void getGroup(size_t group, std::vector<FilterProperty> &groupVec) const;
+		
+		//Confirm a particular group exists
+		bool hasGroup(size_t group) const;
 		//!Get the nth key
 		const FilterProperty &getNthProp(size_t nthProp) const { return properties[nthProp];};
 
@@ -257,6 +269,7 @@ class IonStreamData : public FilterStreamData
 {
 public:
 	IonStreamData();
+	IonStreamData(const Filter *f);
 	void clear();
 
 	//Sample the data vector to the specified fraction
@@ -278,6 +291,10 @@ public:
 	
 	//!Apply filter to input data stream	
 	std::vector<IonHit> data;
+
+	//!export given filterstream data pointers as ion data
+	static unsigned int exportStreams(const std::vector<const FilterStreamData *> &selected, 
+							const std::string &outFile, unsigned int format=IONFORMAT_POS);
 };
 
 //!Point with m-t-c value data
@@ -285,7 +302,9 @@ class VoxelStreamData : public FilterStreamData
 {
 public:
 	VoxelStreamData();
-	size_t getNumBasicObjects() const { return data.getSize();};
+	~VoxelStreamData();
+	VoxelStreamData( const Filter *f);
+	size_t getNumBasicObjects() const ;
 	void clear();
 	
 	unsigned int representationType;
@@ -293,7 +312,7 @@ public:
 	float splatSize;
 	float isoLevel;
 	//!Apply filter to input data stream	
-	Voxels<float> data;
+	Voxels<float> *data;
 		
 };
 
@@ -302,6 +321,7 @@ class PlotStreamData : public FilterStreamData
 {
 	public:
 		PlotStreamData();
+		PlotStreamData(const Filter *f);
 
 		bool save(const char *filename) const; 
 
@@ -333,13 +353,13 @@ class PlotStreamData : public FilterStreamData
 		//!XY data pairs for plotting curve
 		std::vector<std::pair<float,float> > xyData;
 		//!Rectangular marked regions
-		vector<std::pair<float,float> > regions;
-		vector<string> regionTitle;
+		std::vector<std::pair<float,float> > regions;
+		std::vector<std::string> regionTitle;
 		//!Region colours
-		vector<float> regionR,regionB,regionG;
+		std::vector<float> regionR,regionB,regionG;
 
 		//!Region indices from parent region
-		vector<unsigned int> regionID;
+		std::vector<unsigned int> regionID;
 
 		//!Region parent filter pointer, used for matching interaction 
 		// with region to parent property
@@ -360,14 +380,57 @@ class PlotStreamData : public FilterStreamData
 
 };
 
+//!2D Plotting data
+class Plot2DStreamData : public FilterStreamData
+{
+	public:
+		Plot2DStreamData();
+		Plot2DStreamData(const Filter *f);
+
+		//erase plot contents	
+		void clear() {xyData.clear();};
+			
+		size_t getNumBasicObjects() const; 
+		//title for data
+		std::string dataLabel;
+		//Label for X, Y axes
+		std::string xLabel,yLabel;
+
+		unsigned int plotType;
+
+		//!Structured XY data pairs for plotting curve
+		Array2D<float> xyData;
+		//Only rqeuired for xy plots
+		float xMin,xMax,yMin,yMax;
+	
+
+		float r,g,b,a;
+	
+		//!Unstructured XY points
+		std::vector<std::pair<float,float> > scatterData;
+		//optional intensity data for scatter plots
+		std::vector<float> scatterIntensity;
+	
+		//Do we want to plot the scatter intensity in lgo mode?
+		bool scatterIntensityLog;	
+
+		//!Parent filter index
+		unsigned int index;
+
+#ifdef DEBUG
+		void checkSelfConsistent() const;
+#endif
+};
+
 //!Drawable objects, for 3D decoration. 
 class DrawStreamData: public FilterStreamData
 {
 	public:
 		//!Vector of 3D objects to draw.
-		vector<DrawableObj *> drawables;
+		std::vector<DrawableObj *> drawables;
 		//!constructor
 		DrawStreamData(){ streamType=STREAM_TYPE_DRAW;};
+		DrawStreamData(const Filter *f){ streamType=STREAM_TYPE_DRAW;};
 		//!Destructor
 		~DrawStreamData();
 		//!Returns 0, as this does not store basic object types -- i.e. is not for data storage per se.
@@ -390,13 +453,14 @@ class RangeStreamData :  public FilterStreamData
 		//it merely provides access to existing data.
 		RangeFile *rangeFile;
 		//Enabled ranges from source filter
-		vector<char> enabledRanges;
+		std::vector<char> enabledRanges;
 		//Enabled ions from source filter 
-		vector<char> enabledIons;
+		std::vector<char> enabledIons;
 
 		
 		//!constructor
 		RangeStreamData();
+		RangeStreamData(const Filter *f);
 		//!Destructor
 		~RangeStreamData() {};
 		//!save the range data to a file
@@ -421,13 +485,15 @@ class Filter
 		static bool strongRandom;
 
 
+
 		//!Array of the number of streams propagated on last refresh
 		//This is initialised to -1, which is considered invalid
 		unsigned int numStreamsLastRefresh[NUM_STREAM_TYPES];
 	
 
-		//!temporary console output. Should be only nonzero size immediately after refresh
-		vector<string> consoleOutput;
+		//!Temporary console output. Should be only nonzero size if messages are present
+		//after refresh, until cache is cleared
+		std::vector<std::string> consoleOutput;
 		//!User settable labelling string (human readable ID, etc etc)
 		std::string userString;
 		//Filter output cache
@@ -438,21 +504,41 @@ class Filter
 
 
 		//Collate ions from filterstream data into an ionhit vector
-		static unsigned int collateIons(const vector<const FilterStreamData *> &dataIn,
-				vector<IonHit> &outVector, ProgressData &prog,
-				bool (*callback)(bool),size_t totalDataSize=(size_t)-1);
+		static unsigned int collateIons(const std::vector<const FilterStreamData *> &dataIn,
+				std::vector<IonHit> &outVector, ProgressData &prog, size_t totalDataSize=(size_t)-1);
 
 		//!Propagate the given input data to an output vector
-		void propagateStreams(const vector<const FilterStreamData *> &dataIn,
-				vector<const FilterStreamData *> &dataOut,size_t mask=STREAMTYPE_MASK_ALL,bool invertMask=false) const;
+		static void propagateStreams(const std::vector<const FilterStreamData *> &dataIn,
+				std::vector<const FilterStreamData *> &dataOut,size_t mask=STREAMTYPE_MASK_ALL,bool invertMask=false) ;
 
 		//!Propagate the cache into output
-		void propagateCache(vector<const FilterStreamData *> &dataOut) const;
+		void propagateCache(std::vector<const FilterStreamData *> &dataOut) const;
 
+		//Set a property, without any checking of the new value 
+		// -clears cache on change
+		// - and skipping if no actual change between old and new prop
+		// returns true if change applied OK.
+		template<class T>	
+		bool applyPropertyNow(T &oldProp,const std::string &newVal, bool &needUp);
+	
+
+		//place a stream object into the filter cache, if required
+		// does not place object into filter output - you need to do that yourself
+		void cacheAsNeeded(FilterStreamData *s); 
+
+		//!Get the generic (applies to any filter) error codes
+		static std::string getBaseErrString(unsigned int errCode);
+
+		//!Get the per-filter error codes
+		virtual std::string getSpecificErrString(unsigned int errCode) const=0;
+
+	
 	public:	
 		Filter() ;
 		virtual ~Filter();
-		
+		//Abort pointer . This must be  nonzero during filter refreshes
+		static ATOMIC_BOOL *wantAbort;
+
 		//Pure virtual functions
 		//====
 		//!Duplicate filter contents, excluding cache.
@@ -461,7 +547,7 @@ class Filter
 		//!Apply filter to new data, updating cache as needed. Vector of returned pointers must be deleted manually, first checking ->cached.
 		virtual unsigned int refresh(const std::vector<const FilterStreamData *> &dataIn,
 				std::vector<const FilterStreamData *> &dataOut,
-				ProgressData &progress, bool (*callback)(bool)) =0;
+				ProgressData &progress ) =0;
 		//!Erase cache
 		virtual void clearCache();
 
@@ -488,8 +574,8 @@ class Filter
 		virtual bool setProperty(unsigned int key,
 			const std::string &value, bool &needUpdate) = 0;
 
-		//!Get the human readable error string associated with a particular error code during refresh(...)
-		virtual std::string getErrString(unsigned int code) const =0;
+		//!Get the human readable error string associated with a particular error code during refresh(...). Do *not* override this for specific filter errors. Override getSpecificErrString
+		std::string getErrString(unsigned int code) const;
 
 		//!Dump state to output stream, using specified format
 		/* Current supported formats are STATE_FORMAT_XML
@@ -520,7 +606,7 @@ class Filter
 		//====
 	
 		//!Return the unique name for a given filter -- DO NOT TRANSLATE	
-		string trueName() const { return FILTER_NAMES[getType()];};
+		std::string trueName() const { return FILTER_NAMES[getType()];};
 
 
 
@@ -532,7 +618,7 @@ class Filter
 
 		//!Return the XML elements that refer to external entities (i.e. files) which do not move with the XML file
 		//Each element is to be referred to using "/" as entity separator, for the first pair element, and the attribute name for the second.
-		virtual void getStateOverrides(std::vector<string> &overrides) const {}; 
+		virtual void getStateOverrides(std::vector<std::string> &overrides) const {}; 
 
 		//!Enable/disable caching for this filter
 		void setCaching(bool enableCache) {cache=enableCache;};
@@ -559,7 +645,7 @@ class Filter
 		 * another at this level (for example setting two devices on one primitve,
 		 * with the same mouse/key bindings). So dont do that.
 		 */
-		void getSelectionDevices(vector<SelectionDevice *> &devices) const;
+		void getSelectionDevices(std::vector<SelectionDevice *> &devices) const;
 
 
 		//!Update the output statistics for this filter (num items of streams of each type output)
@@ -592,9 +678,12 @@ class Filter
 
 		//Can we be a useful filter, even if given no input specified by the Use mask?
 		virtual bool isUsefulAsAppend() const { return false;}
+	
+		template<typename T>	
+		static void getStreamsOfType(const std::vector<const FilterStreamData *> &vec, std::vector<const T *> &dataOut);
 #ifdef DEBUG
 		//!Run all the registered unit tests for this filter
-		virtual bool runUnitTests() { cerr << "No test for " << typeString() << endl; return true;} ;
+		virtual bool runUnitTests() { std::cerr << "No test for " << typeString() << std::endl; return true;} ;
 		//!Is the filter caching?
 		bool cacheEnabled() const {return cache;};
 
@@ -605,6 +694,52 @@ class Filter
 
 };
 
+template<typename T>
+void Filter::getStreamsOfType(const std::vector<const FilterStreamData *> &vec, std::vector<const T *> &dataOut)
+{
+	T dummyInstance;
+	for(size_t ui=0;ui<vec.size() ; ui++)
+	{
+		if(vec[ui]->getStreamType() == dummyInstance.getStreamType())
+			dataOut.push_back((const T*)vec[ui]);
+	}
+}
+
+//Template specialisations & def for  applyPropertyNow
+//--
+template<>
+bool Filter::applyPropertyNow(Point3D &prop, const std::string &val, bool &needUp);
+
+template<>
+bool Filter::applyPropertyNow(bool &prop, const std::string &val, bool &needUp);
+
+template<>
+bool Filter::applyPropertyNow(std::string &prop, const std::string &val, bool &needUp);
+
+template<class T>
+bool Filter::applyPropertyNow(T &prop, const std::string &val, bool &needUp)
+{
+	// no update initially needed
+	needUp=false;
+
+	//convert to type T
+	std::string s;
+	s=stripWhite(val);
+	T tmp;
+	if(stream_cast(tmp,s))
+		return false;
+	
+	//return true, as it is technically ok that we assign to self.
+	// needUp however stays false, as the property is the same.
+	if(tmp == prop)
+		return true;
+	
+	prop=tmp;
+	clearCache();
+	needUp=true;
+	return true;	
+}
+//--
 
 //!Class that tracks the progress of scene updates
 class ProgressData
@@ -623,12 +758,14 @@ class ProgressData
 		//!Maximum steps
 		unsigned int maxStep;
 		
-		//!Pointer to the current filter that is being updated. Only valid during an update callback
+		//!Pointer to the current filter that is being updated. 
 		const Filter *curFilter;
 
 		//!Name of current operation, if specified
 		std::string stepName;
 
+		ProgressData(); 
+
 		bool operator==(const ProgressData &o) const;
 		const ProgressData &operator=(const ProgressData &o);
 
