@@ -1,6 +1,6 @@
 /*
  *	boundingBox.cpp - Place a bounding box around point datasets
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,6 +19,11 @@
 
 #include "filterCommon.h"
 
+using std::vector;
+using std::string;
+using std::pair;
+using std::make_pair;
+
 enum
 {
 	KEY_VISIBLE=1,
@@ -62,8 +67,7 @@ const char *BOUND_STYLE[] =
 
 
 BoundingBoxFilter::BoundingBoxFilter() : isVisible(true), boundStyle(BOUND_STYLE_TICKS),
-	fixedNumTicks(true), fontSize(5), rLine(0.0f), gLine(0.0f), bLine(1.0f), aLine(1.0f),
-	lineWidth(2.0f), threeDText(true)
+	fixedNumTicks(true), fontSize(5), lineColour(0,0,1.0f), lineWidth(2.0f), threeDText(true)
 {
 	for(unsigned int ui=0;ui<3;ui++)
 	{
@@ -71,7 +75,6 @@ BoundingBoxFilter::BoundingBoxFilter() : isVisible(true), boundStyle(BOUND_STYLE
 		tickSpacing[ui]=5.0f;
 	}
 
-
 	cacheOK=false;
 	cache=false; 
 }
@@ -90,16 +93,12 @@ Filter *BoundingBoxFilter::cloneUncached() const
 	p->boundStyle=boundStyle;
 
 
-	p->rLine=rLine;
-	p->gLine=gLine;
-	p->bLine=bLine;
-	p->aLine=aLine;
 	p->threeDText=threeDText;	
 
 	p->lineWidth=lineWidth;
 	p->fontSize=fontSize;
 
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -119,7 +118,7 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 	//Add the rectangle drawable
 	DrawRectPrism *dP = new DrawRectPrism;
 	dP->setAxisAligned(bTotal);
-	dP->setColour(rLine,gLine,bLine,aLine);
+	dP->setColour(lineColour.r(),lineColour.g(),lineColour.b(),lineColour.a());
 	dP->setLineWidth(lineWidth);
 	d->drawables.push_back(dP);
 
@@ -185,7 +184,8 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 			dV->setDrawArrow(false);
 			dV->setOrigin(tickPosition);
 			dV->setVector(tickVector);
-			dV->setColour(rLine,gLine,bLine,aLine);
+			dV->setColour(lineColour.r(),lineColour.g(),
+					lineColour.b(), lineColour.a());
 
 			d->drawables.push_back(dV);
 	
@@ -206,7 +206,8 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 				dT->setString(buffer);
 				dT->setSize(fontSize);
 				
-				dT->setColour(rLine,gLine,bLine,aLine);
+				dT->setColour(lineColour.r(),lineColour.g(),
+						lineColour.b(),lineColour.a());
 				dT->setOrigin(tickPosition + tickVector*2);	
 				dT->setUp(Point3D(0,0,1));	
 				dT->setTextDir(textVector);
@@ -226,7 +227,8 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 	//Handle "0" text value
 	dT->setString("0");
 	
-	dT->setColour(rLine,gLine,bLine,aLine);
+	dT->setColour(lineColour.r(),lineColour.g(),
+		lineColour.b(),lineColour.a());
 	dT->setSize(fontSize);
 	dT->setOrigin(tickOrigin+ Point3D(-1,-1,-1));
 	dT->setAlignment(DRAWTEXT_ALIGN_RIGHT);
@@ -241,13 +243,14 @@ void BoundingBoxFilter::drawDimension(const BoundCube &bTotal, DrawStreamData *d
 	//Add the rectangle drawable
 	DrawRectPrism *dP = new DrawRectPrism;
 	dP->setAxisAligned(bTotal);
-	dP->setColour(rLine,gLine,bLine,aLine);
+	dP->setColour(lineColour.r(),lineColour.g(),
+				lineColour.b(),lineColour.a());
 	dP->setLineWidth(lineWidth);
 	d->drawables.push_back(dP);
 
 
 
-	//Add the arrows from the start ot the end
+	//Add the arrows from the start to the end
 	//Create the position from which to draw the tick origins
 	Point3D tickOrigin,tickEnd;
 	bTotal.getBounds(tickOrigin,tickEnd);
@@ -288,7 +291,8 @@ void BoundingBoxFilter::drawDimension(const BoundCube &bTotal, DrawStreamData *d
 		DrawVector *dV;
 		dV= new DrawVector;
 
-		dV->setColour(rLine,gLine,bLine,aLine);	
+		dV->setColour(lineColour.r(),lineColour.g(),
+				lineColour.b(),lineColour.a());	
 		dV->wantsLight=true;
 		
 		dV->setArrowSize(maxLen*ARROW_SCALE_FACTOR);
@@ -325,7 +329,8 @@ void BoundingBoxFilter::drawDimension(const BoundCube &bTotal, DrawStreamData *d
 		dT->setString(buffer);
 		dT->setSize(fontSize);
 
-		dT->setColour(rLine,gLine,bLine,aLine);
+		dT->setColour(lineColour.r(),lineColour.g(),
+				lineColour.b(),lineColour.a());
 		dT->setOrigin(centrePt[ui]);	
 		switch(ui)
 		{
@@ -355,7 +360,7 @@ void BoundingBoxFilter::drawDimension(const BoundCube &bTotal, DrawStreamData *d
 }
 
 unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+	std::vector<const FilterStreamData *> &getOut, ProgressData &progress)
 {
 
 	if(!isVisible)
@@ -429,7 +434,7 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 
 						if(thisT == 0)
 						{
-							if(!(*callback)(false))
+							if(*Filter::wantAbort)
 								spin=true;
 						}
 					}
@@ -456,7 +461,7 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 					{
 						n+=NUM_CALLBACK;
 						progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-						if(!(*callback)(false))
+						if(*Filter::wantAbort)
 						{
 							delete d;
 							return BOUNDINGBOX_ABORT_ERR;
@@ -465,6 +470,7 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 				}
 #endif
 				bTotal.expand(bThis);
+				progress.filterProgress=100;
 				break;
 			}
 			default:
@@ -488,7 +494,8 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 				//Add the rectangle drawable
 				DrawRectPrism *dP = new DrawRectPrism;
 				dP->setAxisAligned(bTotal);
-				dP->setColour(rLine,gLine,bLine,aLine);
+				dP->setColour(lineColour.r(),lineColour.g(),
+						lineColour.b(),lineColour.a());
 				dP->setLineWidth(lineWidth);
 				d->drawables.push_back(dP);
 				break;
@@ -540,11 +547,12 @@ void BoundingBoxFilter::getProperties(FilterPropGroup &propertyList) const
 		p.helpText=TRANS("Box display mode");
 		p.key=KEY_STYLE;
 		propertyList.addProperty(p,curGroup);
+		propertyList.setGroupTitle(curGroup,TRANS("Display mode"));
+		curGroup++;
 
 		
 		if(boundStyle == BOUND_STYLE_TICKS)
 		{
-			curGroup++;
 
 			//Properties are X Y and Z counts on ticks
 			stream_cast(tmpStr,fixedNumTicks);
@@ -612,14 +620,11 @@ void BoundingBoxFilter::getProperties(FilterPropGroup &propertyList) const
 
 		}
 
-		//Box Line properties 
-		curGroup++;
 
 		//Colour
-		genColString((unsigned char)(rLine*255.0),(unsigned char)(gLine*255.0),
-			(unsigned char)(bLine*255),(unsigned char)(aLine*255),tmpStr);
+
 		p.name=TRANS("Box Colour");
-		p.data= tmpStr;
+		p.data= lineColour.toColourRGBA().rgbString();
 		p.key=KEY_LINECOLOUR;
 		p.type=PROPERTY_TYPE_COLOUR;
 		p.helpText=TRANS("Colour of the bounding box");
@@ -645,8 +650,8 @@ void BoundingBoxFilter::getProperties(FilterPropGroup &propertyList) const
 			p.helpText=TRANS("Relative size for text");
 			propertyList.addProperty(p,curGroup);
 		}
-		propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
 	}
+	propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
 }
 
 bool BoundingBoxFilter::setProperty(  unsigned int key,
@@ -658,18 +663,8 @@ bool BoundingBoxFilter::setProperty(  unsigned int key,
 	{
 		case KEY_VISIBLE:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(isVisible,value,needUpdate))
 				return false;
-
-			bool lastVal=isVisible;
-			isVisible=(stripped == "1");
-			
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=isVisible)
-				needUpdate=true;
 			break;
 		}	
 		case KEY_STYLE:
@@ -700,18 +695,8 @@ bool BoundingBoxFilter::setProperty(  unsigned int key,
 		}	
 		case KEY_FIXEDOUT:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(fixedNumTicks,value,needUpdate))
 				return false;
-
-			bool lastVal=fixedNumTicks;
-			fixedNumTicks=(stripped=="1");
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=fixedNumTicks)
-				needUpdate=true;
 			break;
 		}	
 		case KEY_COUNT_X:
@@ -733,18 +718,15 @@ bool BoundingBoxFilter::setProperty(  unsigned int key,
 		}
 		case KEY_LINECOLOUR:
 		{
-			unsigned char newR,newG,newB,newA;
+			ColourRGBA newLineColour;
+			if(!newLineColour.parse(value))
+				return false;
 
-			parseColString(value,newR,newG,newB,newA);
 
-			if(newB != bLine || newR != rLine ||
-				newG !=gLine || newA != aLine)
+			if(lineColour.toColourRGBA() != newLineColour) 
 				needUpdate=true;
+			lineColour=newLineColour.toRGBAf();
 
-			rLine=newR/255.0;
-			gLine=newG/255.0;
-			bLine=newB/255.0;
-			aLine=newA/255.0;
 			needUpdate=true;
 			break;
 		}
@@ -779,12 +761,8 @@ bool BoundingBoxFilter::setProperty(  unsigned int key,
 		}
 		case KEY_FONTSIZE:
 		{
-			unsigned int newCount;
-			if(stream_cast(newCount,value))
+			if(!applyPropertyNow(fontSize,value,needUpdate))
 				return false;
-
-			fontSize=newCount;
-			needUpdate=true;
 			break;
 		}
 		default:
@@ -794,7 +772,7 @@ bool BoundingBoxFilter::setProperty(  unsigned int key,
 }
 
 
-std::string  BoundingBoxFilter::getErrString(unsigned int code) const
+std::string  BoundingBoxFilter::getSpecificErrString(unsigned int code) const
 {
 
 	//Currently the only error is aborting
@@ -824,8 +802,8 @@ bool BoundingBoxFilter::writeState(std::ostream &f,unsigned int format, unsigned
 				<< tickSpacing[1] << "\" z=\""<< tickSpacing[2] <<"\"/>"  << endl;
 			f << tabs(depth+1) << "<linewidth value=\"" << lineWidth << "\"/>"<<endl;
 			f << tabs(depth+1) << "<fontsize value=\"" << fontSize << "\"/>"<<endl;
-			f << tabs(depth+1) << "<colour r=\"" <<  rLine<< "\" g=\"" << gLine << "\" b=\"" <<bLine  
-								<< "\" a=\"" << aLine << "\"/>" <<endl;
+			f << tabs(depth+1) << "<colour r=\"" <<  lineColour.r()<< "\" g=\"" << lineColour.g() << "\" b=\"" <<lineColour.b()  
+								<< "\" a=\"" << lineColour.a() << "\"/>" <<endl;
 			f << tabs(depth) << "</" <<trueName()<< ">" << endl;
 			break;
 		}
@@ -862,11 +840,7 @@ bool BoundingBoxFilter::readState(xmlNodePtr &nodePtr, const std::string &stateF
 		return false;
 	tmpStr=(char *)xmlString;
 
-	if(tmpStr == "0")
-		isVisible=false;
-	else if(tmpStr == "1")
-		isVisible=true;
-	else
+	if(!boolStrDec(tmpStr,isVisible))
 		return false;
 
 	xmlFree(xmlString);
@@ -899,11 +873,7 @@ bool BoundingBoxFilter::readState(xmlNodePtr &nodePtr, const std::string &stateF
 		return false;
 	tmpStr=(char *)xmlString;
 
-	if(tmpStr == "0")
-		fixedNumTicks=false;
-	else if(tmpStr == "1")
-		fixedNumTicks=true;
-	else
+	if(!boolStrDec(tmpStr,fixedNumTicks))
 		return false;
 
 	xmlFree(xmlString);
@@ -1029,8 +999,10 @@ bool BoundingBoxFilter::readState(xmlNodePtr &nodePtr, const std::string &stateF
 	if(XMLHelpFwdToElem(nodePtr,"colour"))
 		return false;
 
-	if(!parseXMLColour(nodePtr,rLine,gLine,bLine,aLine))
+	ColourRGBAf tmpCol;
+	if(!parseXMLColour(nodePtr,tmpCol))
 		return false;
+	lineColour=tmpCol;
 	//====
 
 	return true;	
@@ -1104,7 +1076,7 @@ bool boxVolumeTest()
 
 
 	ProgressData p;
-	TEST(!b->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!b->refresh(streamIn,streamOut,p),"Refresh error code");
 	//---
 
 	//Run tests 
