@@ -1,6 +1,6 @@
 /*
  *	compositionProfile.cpp - Compute composition profiles from valued point clouds
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -21,25 +21,35 @@
 #include "filterCommon.h"
 #include "geometryHelpers.h"
 
+using std::vector;
+using std::string;
+using std::pair;
+using std::make_pair;
+using std::map;
+
 
 //!Possible primitive types for composition profiles
 enum
 {
-	PRIMITIVE_CYLINDER,
+	PRIMITIVE_CYLINDER_AXIAL,
+	PRIMITIVE_CYLINDER_RADIAL,
 	PRIMITIVE_SPHERE,
 	PRIMITIVE_END, //Not actually a primitive, just end of enum
 };
 
+
 //!Error codes
 enum
 {
 	ERR_NUMBINS=1,
 	ERR_MEMALLOC,
-	ERR_ABORT
+	ERR_ABORT,
+	ERR_COMP_ENUM_END
 };
 
 const char *PRIMITIVE_NAME[]={
-	NTRANS("Cylinder"),
+	NTRANS("Cylinder (axial)"),
+	NTRANS("Cylinder (radial)"),
 	NTRANS("Sphere")
 };
 
@@ -48,9 +58,9 @@ const float DEFAULT_RADIUS = 10.0f;
 const unsigned int MINEVENTS_DEFAULT =10;
 
 
-CompositionProfileFilter::CompositionProfileFilter() : primitiveType(PRIMITIVE_CYLINDER),
+CompositionProfileFilter::CompositionProfileFilter() : primitiveType(PRIMITIVE_CYLINDER_AXIAL),
 	showPrimitive(true), lockAxisMag(false),normalise(true), fixedBins(0),
-	nBins(1000), binWidth(0.5f), minEvents(MINEVENTS_DEFAULT), r(0.0f),g(0.0f),b(1.0f),a(1.0f), plotStyle(0)
+	nBins(1000), binWidth(0.5f), minEvents(MINEVENTS_DEFAULT), rgba(0,0,1), plotStyle(0)
 {
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(PRIMITIVE_NAME) == PRIMITIVE_END);
 	
@@ -64,7 +74,8 @@ CompositionProfileFilter::CompositionProfileFilter() : primitiveType(PRIMITIVE_C
 	haveRangeParent=false;
 }
 
-
+//Puts an ion in its appropriate range position, given ionID mapping,
+//range data (if any), mass to charge and the output table
 void CompositionProfileFilter::binIon(unsigned int targetBin, const RangeStreamData* rng, 
 	const map<unsigned int,unsigned int> &ionIDMapping,
 	vector<vector<size_t> > &frequencyTable, float massToCharge) 
@@ -78,7 +89,12 @@ void CompositionProfileFilter::binIon(unsigned int targetBin, const RangeStreamD
 		//that makes the target bin equate to the table size. 
 		//disallow this.
 		if(targetBin < frequencyTable[0].size())
-			frequencyTable[0][targetBin]++;
+		{
+			vector<size_t>::iterator it;
+			it=frequencyTable[0].begin()+targetBin;
+			#pragma omp critical
+			(*it)++;
+		}
 		return;
 	}
 
@@ -92,7 +108,10 @@ void CompositionProfileFilter::binIon(unsigned int targetBin, const RangeStreamD
 		unsigned int ionID=rng->rangeFile->getIonID(rangeID); 
 		unsigned int pos;
 		pos = ionIDMapping.find(ionID)->second;
-		frequencyTable[pos][targetBin]++;
+		vector<size_t>::iterator it;
+		it=frequencyTable[pos].begin()+targetBin;
+		#pragma omp critical
+		(*it)++;
 	}
 }
 
@@ -112,16 +131,13 @@ Filter *CompositionProfileFilter::cloneUncached() const
 	p->normalise=normalise;	
 	p->fixedBins=fixedBins;
 	p->lockAxisMag=lockAxisMag;
-
+	
+	p->rgba=rgba;
 	p->binWidth=binWidth;
 	p->nBins = nBins;
-	p->r=r;	
-	p->g=g;	
-	p->b=b;	
-	p->a=a;	
 	p->plotStyle=plotStyle;
 	p->errMode=errMode;
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -145,8 +161,7 @@ void CompositionProfileFilter::initFilter(const std::vector<const FilterStreamDa
 }
 
 unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-			std::vector<const FilterStreamData *> &getOut, ProgressData &progress, 
-								bool (*callback)(bool))
+			std::vector<const FilterStreamData *> &getOut, ProgressData &progress) 
 {
 	//Clear selection devices
 	// FIXME: Leaking drawables.
@@ -160,7 +175,8 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		drawData->parent=this;
 		switch(primitiveType)
 		{
-			case PRIMITIVE_CYLINDER:
+			case PRIMITIVE_CYLINDER_AXIAL:
+			case PRIMITIVE_CYLINDER_RADIAL:
 			{
 				//Origin + normal
 				ASSERT(vectorParams.size() == 2);
@@ -170,7 +186,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 				dC->setRadius(scalarParams[0]);
 				dC->setColour(0.5,0.5,0.5,0.3);
 				dC->setSlices(40);
-				dC->setLength(sqrt(vectorParams[1].sqrMag())*2.0f);
+				dC->setLength(sqrtf(vectorParams[1].sqrMag())*2.0f);
 				dC->setDirection(vectorParams[1]);
 				dC->wantsLight=true;
 				drawData->drawables.push_back(dC);
@@ -301,7 +317,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 
 		ASSERT(filterOutputs.back()->getStreamType() == STREAM_TYPE_PLOT);
 
-			
+		progress.filterProgress=100;
 		return 0;
 	}
 			
@@ -319,12 +335,14 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		}
 	}
 
-	float length;
 	unsigned int numBins, errCode;
+	{
+	float length;
 	errCode=getBinData(numBins,length);
 
 	if(!numBins)
 		return 0;
+	}
 
 	if(errCode)
 		return errCode;
@@ -391,11 +409,12 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 	size_t totalSize=numElements(dataIn);
 
 	map<size_t,size_t> primitiveMap;
-	primitiveMap[PRIMITIVE_CYLINDER] = CROP_CYLINDER_INSIDE;
+	primitiveMap[PRIMITIVE_CYLINDER_AXIAL] = CROP_CYLINDER_INSIDE_AXIAL;
+	primitiveMap[PRIMITIVE_CYLINDER_RADIAL] = CROP_CYLINDER_INSIDE_RADIAL;
 	primitiveMap[PRIMITIVE_SPHERE] = CROP_SPHERE_INSIDE;
 
-	CropHelper dataMapping(callback,&progress.filterProgress,
-			totalSize,primitiveMap[primitiveType], vectorParams,scalarParams  );
+	CropHelper dataMapping(totalSize,primitiveMap[primitiveType], 
+					vectorParams,scalarParams  );
 	dataMapping.setMapMaxima(numBins);
 
 	unsigned int curProg=NUM_CALLBACK;
@@ -406,12 +425,22 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		{
 			case STREAM_TYPE_IONS:
 			{
+				const IonStreamData *dIon = (const IonStreamData*)dataIn[ui];
+#ifdef _OPENMP
+				bool spin=false;
+#endif
 				//Process ion streams
-				for(vector<IonHit>::const_iterator it=((const IonStreamData *)dataIn[ui])->data.begin();
-					       it!=((const IonStreamData *)dataIn[ui])->data.end(); ++it)
+			
+				size_t nIons=dIon->data.size();	
+				#pragma omp parallel for
+				for(size_t uj=0;uj<nIons;uj++)
 				{
+#ifdef _OPENMP
+					//if parallelised, abort computaiton
+					if(spin) continue;
+#endif
 					unsigned int targetBin;
-					targetBin=dataMapping.mapIon1D(*it);
+					targetBin=dataMapping.mapIon1D(dIon->data[uj]);
 
 					//Keep ion if inside cylinder 
 					if(targetBin!=(unsigned int)-1)
@@ -419,19 +448,37 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 						//Push data into the correct bin.
 						// based upon eg ranging information and target 1D bin
 						binIon(targetBin,rngData,ionIDMapping,ionFrequencies,
-								it->getMassToCharge());
+								dIon->data[uj].getMassToCharge());
 					}
 
+#ifdef _OPENMP
+					if(omp_get_thread_num() == 0)	
+					{
+#endif
 					//update progress every CALLBACK ions
 					if(!curProg--)
 					{
 						n+=NUM_CALLBACK;
 						progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
 						curProg=NUM_CALLBACK;
-						if(!(*callback)(false))
+						if(*Filter::wantAbort)
+						{
+							#ifdef _OPENMP
+								spin=true;
+							#else
 							return ERR_ABORT;
+							#endif 
+						}
 					}
+#ifdef _OPENMP
+					}
+#endif
 				}
+
+#ifdef _OPENMP
+				if(spin)
+					return ERR_ABORT;
+#endif
 					
 				break;
 			}
@@ -457,34 +504,43 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 	normalisationFactor.resize(ionFrequencies[0].size());
 	normalisationCount.resize(ionFrequencies[0].size());
 	bool needNormalise=false;
-	bool needNormaliseCount=false;
 
 	//Perform the appropriate normalisation
 	if(!rngData && normalise)
 	{
-		float dx;
-		if(fixedBins)
-			dx=(length/(float)numBins);
-		else
-			dx=binWidth;
 		// For density plots, normalise by
 		//  the volume of the primitive's shell
 		switch(primitiveType)
 		{
-			case PRIMITIVE_CYLINDER:
+			case PRIMITIVE_CYLINDER_AXIAL:
+			case PRIMITIVE_CYLINDER_RADIAL:
 			{
+
+				float dx;
+				if(fixedBins)
+					dx=(sqrtf(vectorParams[1].sqrMag())/(float)numBins);
+
+				else
+					dx=binWidth;
 				needNormalise=true;
-				for(unsigned int uj=0;uj<ionFrequencies[0].size(); uj++)
-				{
-					//Normalise by cylinder volume, pi*r^2*h
-					normalisationFactor[uj] = 1.0/(M_PI*
-						scalarParams[0]*scalarParams[0]*dx);
-				}
+				float nFact;
+				//Normalise by cylinder slice volume, pi*r^2*h.
+				// This is the same in both radial and axial mode as the radial slices are equi-volume, 
+				// same as axial mode
+				nFact=1.0/(M_PI*scalarParams[0]*scalarParams[0]*dx);
+				for(unsigned int uj=0;uj<normalisationFactor.size(); uj++)
+					normalisationFactor[uj] = nFact;
 				break;
 			}
 			case PRIMITIVE_SPHERE:
 			{
-				for(unsigned int uj=0;uj<ionFrequencies[0].size(); uj++)
+				float dx;
+				if(fixedBins)
+					dx=(scalarParams[0]/(float)numBins);
+
+				else
+					dx=binWidth;
+				for(unsigned int uj=0;uj<normalisationFactor.size(); uj++)
 				{
 					//Normalise by sphere shell volume, 
 					// 4/3 *PI*dx^3*((n+1)^3-n^3)
@@ -503,7 +559,6 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		// the loops' nesting is reversed as we need to sum over distinct plots
 		//Density profiles (non-ranged plots) have a fixed normalisation factor
 		needNormalise=true;
-		needNormaliseCount = true; // we also need to have at least one count to normalise
 
 		for(unsigned int uj=0;uj<ionFrequencies[0].size(); uj++)
 		{
@@ -571,10 +626,10 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 			//If it only has one component, then 
 			//it's not really a composition profile is it?
 			plotData[ui]->dataLabel= TRANS("Freq. Profile");
-			plotData[ui]->r = r;
-			plotData[ui]->g = g;
-			plotData[ui]->b = b;
-			plotData[ui]->a = a;
+			plotData[ui]->r = rgba.r();
+			plotData[ui]->g = rgba.g();
+			plotData[ui]->b = rgba.b();
+			plotData[ui]->a = rgba.a();
 		}
 
 		plotData[ui]->xyData.reserve(ionFrequencies[ui].size());
@@ -584,7 +639,11 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		for(unsigned int uj=0;uj<ionFrequencies[ui].size(); uj++)
 		{
 			float xPos;
-			xPos = (((float)uj+0.5f)/(float)ionFrequencies[ui].size())*length;
+			xPos = getBinPosition(uj);
+
+			if(ionFrequencies[ui][uj] < minEvents)
+				continue;
+
 			//Recompute normalisation value for this bin, if needed
 			if(needNormalise)
 			{
@@ -592,12 +651,9 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 
 				//keep the data if we are not using minimum threshold for normalisation, or we met the 
 				// threhsold
-				if(!needNormaliseCount || normalisationCount[uj] > minEvents)
-				{
-					plotData[ui]->xyData.push_back(
-						std::make_pair(xPos,
-						normFactor*(float)ionFrequencies[ui][uj]));
-				}
+				plotData[ui]->xyData.push_back(
+					std::make_pair(xPos,
+					normFactor*(float)ionFrequencies[ui][uj]));
 			}
 			else
 			{	
@@ -608,13 +664,6 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 
 
 
-		if(cache)
-		{
-			plotData[ui]->cached=1;
-			filterOutputs.push_back(plotData[ui]);	
-		}
-		else
-			plotData[ui]->cached=0;
 
 		plotData[ui]->plotStyle = plotStyle;
 		plotData[ui]->plotMode=PLOT_MODE_1D;
@@ -623,28 +672,30 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		// otherwise, trash the plot info
 		if(plotData[ui]->xyData.size())
 		{
-			cacheOK=cache;
+			cacheAsNeeded(plotData[ui]);
 			getOut.push_back(plotData[ui]);
 		}
 		else
+		{
+			consoleOutput.push_back(TRANS("No data remained in profile - cannot display result"));
 			delete plotData[ui];
+		}
 	}
 
 	return 0;
 }
 
-std::string  CompositionProfileFilter::getErrString(unsigned int code) const
+std::string  CompositionProfileFilter::getSpecificErrString(unsigned int code) const
 {
-	switch(code)
-	{
-		case ERR_NUMBINS:
-			return std::string(TRANS("Too many bins in comp. profile."));
-		case ERR_MEMALLOC:
-			return std::string(TRANS("Not enough memory for comp. profile."));
-		case ERR_ABORT:
-			return std::string(TRANS("Aborted composition prof."));
-	}
-	return std::string("BUG: (CompositionProfileFilter::getErrString) Shouldn't see this!");
+	const char *errCodes[] =   { "",
+		"Too many bins in comp. profile.",
+		"Not enough memory for comp. profile.",
+		"Aborted composition prof." }; 
+
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(errCodes) == ERR_COMP_ENUM_END);
+	ASSERT(code < ERR_COMP_ENUM_END);
+
+	return errCodes[code];
 }
 
 bool CompositionProfileFilter::setProperty( unsigned int key, 
@@ -660,7 +711,7 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 			if(stream_cast(newBinWidth,value))
 				return false;
 
-			if(newBinWidth < sqrt(std::numeric_limits<float>::epsilon()))
+			if(newBinWidth < sqrtf(std::numeric_limits<float>::epsilon()))
 				return false;
 
 			binWidth=newBinWidth;
@@ -670,24 +721,8 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 		}
 		case COMPOSITION_KEY_FIXEDBINS:
 		{
-			unsigned int valueInt;
-			if(stream_cast(valueInt,value))
+			if(!applyPropertyNow(fixedBins,value,needUpdate))
 				return false;
-
-			if(valueInt ==0 || valueInt == 1)
-			{
-				if(fixedBins!= (bool)valueInt)
-				{
-					needUpdate=true;
-					fixedBins=valueInt;
-				}
-				else
-					needUpdate=false;
-			}
-			else
-				return false;
-			clearCache();
-			needUpdate=true;	
 			break;	
 		}
 		case COMPOSITION_KEY_NORMAL:
@@ -696,16 +731,16 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 			if(!newPt.parse(value))
 				return false;
 
-			if(primitiveType == PRIMITIVE_CYLINDER)
+			if(primitiveType == PRIMITIVE_CYLINDER_AXIAL)
 			{
 				if(lockAxisMag && 
-					newPt.sqrMag() > sqrt(std::numeric_limits<float>::epsilon()))
+					newPt.sqrMag() > sqrtf(std::numeric_limits<float>::epsilon()))
 				{
 					newPt.normalise();
-					newPt*=sqrt(vectorParams[1].sqrMag());
+					newPt*=sqrtf(vectorParams[1].sqrMag());
 				}
 			}
-			if(newPt.sqrMag() < sqrt(std::numeric_limits<float>::epsilon()))
+			if(newPt.sqrMag() < sqrtf(std::numeric_limits<float>::epsilon()))
 				return false;
 
 			if(!(vectorParams[1] == newPt ))
@@ -718,24 +753,10 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 		}
 		case COMPOSITION_KEY_MINEVENTS:
 		{
-			unsigned int valueInt;
-			if(stream_cast(valueInt,value))
+			if(!applyPropertyNow(minEvents,value,needUpdate))
 				return false;
-
-			if(minEvents!= valueInt)
-			{
-				needUpdate=true;
-				minEvents=valueInt;
-			}
-			else
-				needUpdate=false;
-
-			clearCache();
-			needUpdate=true;	
 			break;	
 		}
-	
-		
 		case COMPOSITION_KEY_NUMBINS:
 		{
 			unsigned int newNumBins;
@@ -754,17 +775,8 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 		}
 		case COMPOSITION_KEY_ORIGIN:
 		{
-			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!applyPropertyNow(vectorParams[0],value,needUpdate))
 				return false;
-
-			if(!(vectorParams[0] == newPt ))
-			{
-				vectorParams[0] = newPt;
-				needUpdate=true;
-				clearCache();
-			}
-
 			return true;
 		}
 		case COMPOSITION_KEY_PRIMITIVETYPE:
@@ -781,7 +793,8 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 			// preserving data where possible
 			switch(primitiveType)
 			{
-				case PRIMITIVE_CYLINDER:
+				case PRIMITIVE_CYLINDER_AXIAL:
+				case PRIMITIVE_CYLINDER_RADIAL:
 				{
 					if(vectorParams.size() != 2)
 					{
@@ -805,6 +818,10 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 						else
 							scalarParams.resize(1);
 					}
+
+					if(primitiveType == PRIMITIVE_CYLINDER_RADIAL)
+						fixedBins=true;
+
 					break;
 				}
 				case PRIMITIVE_SPHERE:
@@ -841,7 +858,7 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 			if(stream_cast(newRad,value))
 				return false;
 
-			if(newRad < sqrt(std::numeric_limits<float>::epsilon()))
+			if(newRad < sqrtf(std::numeric_limits<float>::epsilon()))
 				return false;
 
 			if(scalarParams[0] != newRad )
@@ -854,67 +871,29 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 		}
 		case COMPOSITION_KEY_SHOWPRIMITIVE:
 		{
-			unsigned int valueInt;
-			if(stream_cast(valueInt,value))
+			if(!applyPropertyNow(showPrimitive,value,needUpdate))
 				return false;
-
-			if(valueInt ==0 || valueInt == 1)
-			{
-				if(showPrimitive!= (bool)valueInt)
-				{
-					needUpdate=true;
-					showPrimitive=valueInt;
-				}
-				else
-					needUpdate=false;
-			}
-			else
-				return false;		
 			break;	
 		}
-
 		case COMPOSITION_KEY_NORMALISE:
 		{
-			unsigned int valueInt;
-			if(stream_cast(valueInt,value))
+			if(!applyPropertyNow(normalise,value,needUpdate))
 				return false;
-
-			if(!(valueInt ==0 || valueInt == 1))
-				return false;
-			
-			if(normalise!= (bool)valueInt)
-			{
-				needUpdate=true;
-				normalise=valueInt;
-			}
-			else
-				needUpdate=false;
-		
-			clearCache();
-			needUpdate=true;	
 			break;	
 		}
 		case COMPOSITION_KEY_LOCKAXISMAG:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(lockAxisMag,value,needUpdate))
 				return false;
-
-			lockAxisMag=(stripped=="1");
-
-			needUpdate=true;
-
 			break;
 		}
-
 		case COMPOSITION_KEY_PLOTTYPE:
 		{
 			unsigned int tmpPlotType;
 
 			tmpPlotType=plotID(value);
 
-			if(tmpPlotType >= PLOT_TRACE_ENDOFENUM)
+			if(tmpPlotType >= PLOT_LINE_NONE)
 				return false;
 
 			plotStyle = tmpPlotType;
@@ -923,14 +902,11 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 		}
 		case COMPOSITION_KEY_COLOUR:
 		{
-			unsigned char newR,newG,newB,newA;
-			parseColString(value,newR,newG,newB,newA);
-
-			r=((float)newR)/255.0f;
-			g=((float)newG)/255.0f;
-			b=((float)newB)/255.0f;
-			a=1.0;
-
+			ColourRGBA tmpRgba;
+			if(!tmpRgba.parse(value))
+				return false;
+			
+			rgba=tmpRgba.toRGBAf();
 			needUpdate=true;
 			break;	
 		}
@@ -992,6 +968,7 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 		p.type=PROPERTY_TYPE_CHOICE;
 		p.helpText=TRANS("Basic shape to use for profile");
 		propertyList.addProperty(p,curGroup);
+		propertyList.setGroupTitle(curGroup,TRANS("Primitive"));	
 		curGroup++;
 	}
 
@@ -1006,7 +983,8 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 
 	switch(primitiveType)
 	{
-		case PRIMITIVE_CYLINDER:
+		case PRIMITIVE_CYLINDER_AXIAL:
+		case PRIMITIVE_CYLINDER_RADIAL:
 		{
 			ASSERT(vectorParams.size() == 2);
 			ASSERT(scalarParams.size() == 1);
@@ -1026,10 +1004,7 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 			p.helpText=TRANS("Vector between ends of cylinder");
 			propertyList.addProperty(p,curGroup);
 
-			if(lockAxisMag)
-				str="1";
-			else
-				str="0";
+			str=boolStrEnc(lockAxisMag);
 			p.key=COMPOSITION_KEY_LOCKAXISMAG;
 			p.name=TRANS("Lock Axis Mag.");
 			p.data= str;
@@ -1072,13 +1047,17 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 			ASSERT(false);
 	}
 
-	p.key=COMPOSITION_KEY_FIXEDBINS;
-	stream_cast(str,fixedBins);
-	p.name=TRANS("Fixed Bin Num");
-	p.data=str;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("If true, use a fixed number of bins for profile, otherwise use fixed step size");
-	propertyList.addProperty(p,curGroup);
+	//Must be fixed bin num in radial mode. Disallow turning this off
+	if(primitiveType!= PRIMITIVE_CYLINDER_RADIAL)
+	{
+		p.key=COMPOSITION_KEY_FIXEDBINS;
+		stream_cast(str,fixedBins);
+		p.name=TRANS("Fixed Bin Num");
+		p.data=str;
+		p.type=PROPERTY_TYPE_BOOL;
+		p.helpText=TRANS("If true, use a fixed number of bins for profile, otherwise use fixed step size");
+		propertyList.addProperty(p,curGroup);
+	}
 
 	if(fixedBins)
 	{
@@ -1093,6 +1072,7 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 	}
 	else
 	{
+		ASSERT(primitiveType!=PRIMITIVE_CYLINDER_RADIAL);
 		str = TRANS("Bin width");
 		stream_cast(tmpStr,binWidth);
 		p.name=str;
@@ -1119,6 +1099,7 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 	p.helpText=TRANS("Drop data that does not have this many events");
 	propertyList.addProperty(p,curGroup);
 
+	propertyList.setGroupTitle(curGroup,TRANS("Settings"));	
 
 
 
@@ -1130,14 +1111,14 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 	vector<pair<unsigned int,string> > choices;
 
 
-	tmpStr=plotString(PLOT_TRACE_LINES);
-	choices.push_back(make_pair((unsigned int) PLOT_TRACE_LINES,tmpStr));
-	tmpStr=plotString(PLOT_TRACE_BARS);
-	choices.push_back(make_pair((unsigned int)PLOT_TRACE_BARS,tmpStr));
-	tmpStr=plotString(PLOT_TRACE_STEPS);
-	choices.push_back(make_pair((unsigned int)PLOT_TRACE_STEPS,tmpStr));
-	tmpStr=plotString(PLOT_TRACE_STEM);
-	choices.push_back(make_pair((unsigned int)PLOT_TRACE_STEM,tmpStr));
+	tmpStr=plotString(PLOT_LINE_LINES);
+	choices.push_back(make_pair((unsigned int) PLOT_LINE_LINES,tmpStr));
+	tmpStr=plotString(PLOT_LINE_BARS);
+	choices.push_back(make_pair((unsigned int)PLOT_LINE_BARS,tmpStr));
+	tmpStr=plotString(PLOT_LINE_STEPS);
+	choices.push_back(make_pair((unsigned int)PLOT_LINE_STEPS,tmpStr));
+	tmpStr=plotString(PLOT_LINE_STEM);
+	choices.push_back(make_pair((unsigned int)PLOT_LINE_STEM,tmpStr));
 
 	tmpStr= choiceString(choices,plotStyle);
 	p.name=TRANS("Plot Type");
@@ -1149,12 +1130,9 @@ void CompositionProfileFilter::getProperties(FilterPropGroup &propertyList) cons
 	//Convert the colour to a hex string
 	if(!haveRangeParent)
 	{
-		string thisCol;
-		genColString((unsigned char)(r*255.0),(unsigned char)(g*255.0),
-		(unsigned char)(b*255.0),(unsigned char)(a*255.0),thisCol);
 
 		p.name=TRANS("Colour");
-		p.data=thisCol; 
+		p.data=rgba.toColourRGBA().rgbString();
 		p.type=PROPERTY_TYPE_COLOUR;
 		p.helpText=TRANS("Colour of plot");
 		p.key=COMPOSITION_KEY_COLOUR;
@@ -1202,9 +1180,13 @@ unsigned int CompositionProfileFilter::getBinData(unsigned int &numBins, float &
 			//radius of sphere
 			length=scalarParams[0];
 			break;
-		case PRIMITIVE_CYLINDER:
+		case PRIMITIVE_CYLINDER_AXIAL:
 			//length of cylinder, full axis length
-			length=sqrt(vectorParams[1].sqrMag());
+			length=sqrtf(vectorParams[1].sqrMag());
+			break;
+		case PRIMITIVE_CYLINDER_RADIAL:
+			//radius of cylinder
+			length =scalarParams[0];
 			break;
 		default:
 			ASSERT(false);
@@ -1216,7 +1198,8 @@ unsigned int CompositionProfileFilter::getBinData(unsigned int &numBins, float &
 	{
 		switch(primitiveType)
 		{
-			case PRIMITIVE_CYLINDER:
+			case PRIMITIVE_CYLINDER_AXIAL:
+			case PRIMITIVE_CYLINDER_RADIAL:
 			case PRIMITIVE_SPHERE:
 			{
 
@@ -1238,6 +1221,26 @@ unsigned int CompositionProfileFilter::getBinData(unsigned int &numBins, float &
 	return 0;
 }
 
+float CompositionProfileFilter::getBinPosition(unsigned int nBin) const
+{
+	unsigned int nBinsMax; float fullLen, xPos;
+	getBinData(nBinsMax,fullLen);	
+	ASSERT(nBin < nBinsMax)
+	xPos = ((float) nBin + 0.5)/(float)nBinsMax;
+	if( primitiveType == PRIMITIVE_CYLINDER_RADIAL)
+	{
+		float maxPosSqr = fullLen*fullLen;
+		//compute fraction
+		xPos = sqrt ( xPos*maxPosSqr);
+	}
+	else
+	{
+		xPos = xPos*fullLen;
+	}			
+
+	return xPos;
+}
+
 //!Get approx number of bytes for caching output
 size_t CompositionProfileFilter::numBytesForCache(size_t nObjects) const
 {
@@ -1281,8 +1284,8 @@ bool CompositionProfileFilter::writeState(std::ostream &f,unsigned int format, u
 			f << tabs(depth+1) << "<fixedbins value=\"" << (int)fixedBins << "\"/>" << endl;
 			f << tabs(depth+1) << "<nbins value=\"" << nBins << "\"/>" << endl;
 			f << tabs(depth+1) << "<binwidth value=\"" << binWidth << "\"/>" << endl;
-			f << tabs(depth+1) << "<colour r=\"" <<  r<< "\" g=\"" << g << "\" b=\"" <<b
-				<< "\" a=\"" << a << "\"/>" <<endl;
+			f << tabs(depth+1) << "<colour r=\"" <<  rgba.r() << "\" g=\"" << rgba.g() << "\" b=\"" << rgba.b()
+				<< "\" a=\"" << rgba.a() << "\"/>" <<endl;
 
 			f << tabs(depth+1) << "<plottype value=\"" << plotStyle << "\"/>" << endl;
 			f << tabs(depth) << "</" << trueName()  << ">" << endl;
@@ -1326,6 +1329,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	if(XMLHelpFwdToElem(nodePtr,"primitivetype"))
 		return false;
 
+
 	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
 	if(!xmlString)
 		return false;
@@ -1335,6 +1339,16 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	if(stream_cast(primitiveType,tmpStr))
 		return false;
 
+	//FIXME: DEPRECATE 3Depict versions <=0.0.17 had only two primitives,
+	// cylinder and sphere. 
+/*	if(versionCheckGreater(Filter::stateWriterVersion,("0.0.17")))
+	{
+		//remap the primitive type as needed
+		if(primitiveType == PRIMITIVE_CYLINDER_RADIAL)
+			primitiveType=PRIMITIVE_SPHERE;
+
+	}
+*/
 	if(primitiveType >= PRIMITIVE_END)
 	       return false;	
 	xmlFree(xmlString);
@@ -1350,11 +1364,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 		return false;
 	tmpStr=(char *)xmlString;
 
-	if(tmpStr == "0")
-		showPrimitive=false;
-	else if(tmpStr == "1")
-		showPrimitive=true;
-	else
+	if(!boolStrDec(tmpStr,showPrimitive))
 		return false;
 
 	xmlFree(xmlString);
@@ -1370,11 +1380,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 		return false;
 	tmpStr=(char *)xmlString;
 
-	if(tmpStr == "0")
-		lockAxisMag=false;
-	else if(tmpStr == "1")
-		lockAxisMag=true;
-	else
+	if(!boolStrDec(tmpStr,lockAxisMag))
 		return false;
 
 	xmlFree(xmlString);
@@ -1459,7 +1465,8 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	//Check the scalar params match the selected primitive	
 	switch(primitiveType)
 	{
-		case PRIMITIVE_CYLINDER:
+		case PRIMITIVE_CYLINDER_AXIAL:
+		case PRIMITIVE_CYLINDER_RADIAL:
 			if(vectorParams.size() != 2 || scalarParams.size() !=1)
 				return false;
 			break;
@@ -1485,11 +1492,8 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 		return false;
 	tmpStr=(char *)xmlString;
 
-	if(tmpStr == "0")
-		normalise=false;
-	else if(tmpStr == "1")
-		normalise=true;
-	else
+
+	if(!boolStrDec(tmpStr,normalise))
 		return false;
 
 	xmlFree(xmlString);
@@ -1505,11 +1509,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 		return false;
 	tmpStr=(char *)xmlString;
 
-	if(tmpStr == "0")
-		fixedBins=false;
-	else if(tmpStr == "1")
-		fixedBins=true;
-	else
+	if(!boolStrDec(tmpStr,fixedBins))
 		return false;
 
 
@@ -1553,7 +1553,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	//====
 	if(XMLHelpFwdToElem(nodePtr,"colour"))
 		return false;
-	if(!parseXMLColour(nodePtr,r,g,b,a))
+	if(!parseXMLColour(nodePtr,rgba))
 		return false;
 	//====
 	
@@ -1571,7 +1571,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	if(stream_cast(plotStyle,tmpStr))
 		return false;
 
-	if(plotStyle >= PLOT_TRACE_ENDOFENUM)
+	if(plotStyle >= PLOT_LINE_NONE)
 	       return false;	
 	xmlFree(xmlString);
 	//====
@@ -1710,6 +1710,7 @@ bool testCompositionCylinder()
 	bool needUp; std::string s;
 	stream_cast(s,Point3D((startPt+endPt)*0.5f));
 	TEST(f->setProperty(COMPOSITION_KEY_ORIGIN,s,needUp),"set origin");
+	TEST(f->setProperty(COMPOSITION_KEY_MINEVENTS,"0",needUp),"set origin");
 	
 	stream_cast(s,Point3D((endPt-startPt)*0.5f));
 	TEST(f->setProperty(COMPOSITION_KEY_NORMAL,s,needUp),"set direction");
@@ -1726,7 +1727,7 @@ bool testCompositionCylinder()
 
 
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 
 	//2* plot, 1*rng, 1*draw
 	TEST(streamOut.size() == 4, "output stream count");
@@ -1757,7 +1758,7 @@ bool testCompositionCylinder()
 			break;
 		}
 	}
-
+	TEST(plotData,"Should have plot data");
 	TEST(plotData->xyData.size(),"Plot data size");
 
 	for(size_t ui=0;ui<plotData->xyData.size(); ui++)
@@ -1814,7 +1815,7 @@ bool testDensityCylinder()
 	TEST(f->setProperty(COMPOSITION_KEY_RADIUS,"5",needUp),"Set radius");
 
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 	delete f;
 	delete d;
 
