@@ -1,6 +1,6 @@
 /*
  *	dataLoad.cpp - filter to  load datasets from various source
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -27,6 +27,11 @@
 
 #include "backend/APT/APTFileIO.h"
 
+using std::string;
+using std::pair;
+using std::make_pair;
+using std::endl;
+
 //Default number of ions to load
 const size_t MAX_IONS_LOAD_DEFAULT=5*1024*1024/(4*sizeof(float)); //5 MB worth.
 
@@ -68,7 +73,7 @@ const char *DEFAULT_LABEL="Mass-to-Charge (amu/e)";
 
 // == Pos load filter ==
 DataLoadFilter::DataLoadFilter() : fileType(FILEDATA_TYPE_POS), doSample(true), maxIons(MAX_IONS_LOAD_DEFAULT),
-	r(1.0f),g(0.0f),b(0.0f),a(1.0f),ionSize(2.0f), numColumns(4), enabled(true),
+	rgbaf(1.0f,0.0f,0.0f,1.0f),ionSize(2.0f), numColumns(4), enabled(true),
 	volumeRestrict(false), monitorTimestamp(-1),monitorSize((size_t)-1),wantMonitor(false),
 	valueLabel(TRANS(DEFAULT_LABEL)), endianMode(0)
 {
@@ -91,12 +96,8 @@ Filter *DataLoadFilter::cloneUncached() const
 	p->maxIons=maxIons;
 	p->ionSize=ionSize;
 	p->fileType=fileType;
-	p->guessType=guessType;
 	//Colours
-	p->r=r;	
-	p->g=g;	
-	p->b=b;	
-	p->a=a;	
+	p->rgbaf=rgbaf;	
 	//Bounding volume
 	p->bound.setBounds(bound);
 	p->volumeRestrict=volumeRestrict;
@@ -106,7 +107,7 @@ Filter *DataLoadFilter::cloneUncached() const
 	for(size_t ui=0;ui<INDEX_LENGTH;ui++)
 		p->index[ui]=index[ui];
 
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -159,6 +160,10 @@ void DataLoadFilter::guessNumColumns()
 	if(ionFilename.size() > 4)
 		extension = ionFilename.substr ( ionFilename.size() - 4, 4 );
 
+	//By default, return 4. If you want to have other file types,
+	// uncomment the below
+	numColumns=4;
+	/*
 	//Set extension to lowercase version
 	for(size_t ui=0;ui<extension.size();ui++)
 		extension[ui] = tolower(extension[ui]);
@@ -167,7 +172,7 @@ void DataLoadFilter::guessNumColumns()
 		numColumns = 4;
 		return;
 	}
-	numColumns = 4;
+	numColumns = 4;*/
 }
 
 //!Get (approx) number of bytes required for cache
@@ -185,7 +190,7 @@ size_t DataLoadFilter::numBytesForCache(size_t nObjects) const
 }
 
 unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+	std::vector<const FilterStreamData *> &getOut, ProgressData &progress)
 {
 
 	errStr="";
@@ -198,7 +203,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 		//the same timestamp as on the file.
 		if(wantMonitor)
 		{
-			if(!wxFile::Exists(wxStr(ionFilename)))
+			if(!wxFile::Exists((ionFilename)))
 			{
 				monitorTimestamp=-1;
 				monitorSize=-1;
@@ -214,7 +219,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				size_t fileSizeVal;
 				getFilesize(ionFilename.c_str(),fileSizeVal);
 
-				if(wxFileModificationTime(wxStr(ionFilename)) ==monitorTimestamp
+				if(wxFileModificationTime((ionFilename)) ==monitorTimestamp
 					||  fileSizeVal!= monitorSize)
 				{
 					doUseCache=false;
@@ -230,14 +235,15 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 			propagateCache(getOut);
 
 			propagateStreams(dataIn,getOut);
+			progress.filterProgress=100;
 			return 0;
 		}
 	}
 
 	//If theres no file, then there is not a lot we can do..
-	if(!wxFile::Exists(wxStr(ionFilename)))
+	if(!wxFile::Exists((ionFilename)))
 	{
-		wxFileName f(wxStr(ionFilename));
+		wxFileName f((ionFilename));
 		
 		errStr= stlStr(f.GetFullName())  + TRANS(" does not exist");
 		return ERR_FILE_OPEN;
@@ -245,7 +251,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 	
 	//If we have disable the filter, or we are are monitoring and 
 	//there is no file
-	if(!enabled ||(wantMonitor && !wxFile::Exists(wxStr(ionFilename))) )
+	if(!enabled ||(wantMonitor && !wxFile::Exists((ionFilename))) )
 	{
 		monitorTimestamp=-1;
 		monitorSize=-1;
@@ -257,7 +263,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 
 	//Update the monitoring timestamp such that we know
 	//when the file was last loaded
-	monitorTimestamp = wxFileModificationTime(wxStr(ionFilename));
+	monitorTimestamp = wxFileModificationTime((ionFilename));
 	size_t tmp;
 	if(getFilesize(ionFilename.c_str(),tmp))
 		monitorSize=tmp;
@@ -265,6 +271,10 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 	IonStreamData *ionData = new IonStreamData;
 	ionData->parent=this;	
 
+	progress.step=1;
+	progress.stepName=TRANS("Reading File");
+	progress.maxStep=1;
+
 	unsigned int uiErr;	
 	switch(fileType)
 	{
@@ -275,7 +285,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				
 				//Load the pos file, limiting how much you pull from it
 				if((uiErr = LimitLoadPosFile(numColumns, INDEX_LENGTH, index, ionData->data, ionFilename.c_str(),
-									maxIons,progress.filterProgress,callback,strongRandom)))
+									maxIons,progress.filterProgress,(*Filter::wantAbort),strongRandom)))
 				{
 					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
@@ -288,7 +298,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 			{
 				//Load the entirety of the file
 				if((uiErr = GenericLoadFloatFile(numColumns, INDEX_LENGTH, index, ionData->data, ionFilename.c_str(),
-									progress.filterProgress,callback)))
+									progress.filterProgress,(*Filter::wantAbort))))
 				{
 					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
@@ -331,7 +341,8 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 			{
 				//Load the output data using a random sampling technique. Load up to 4 data columns
 				if((uiErr=limitLoadTextFile(4,outDat,ionFilename.c_str(),
-						TEXT_DELIMINATORS,maxIons,progress.filterProgress,callback,strongRandom)))
+						TEXT_DELIMINATORS,maxIons,progress.filterProgress,(*Filter::wantAbort),strongRandom)))
+
 				{
 					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
@@ -399,11 +410,10 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 		}
 		case FILEDATA_TYPE_ATO:
 		{
-			//TODO: Load Ato file with sampling?
-
-			//Load the pos file, limiting how much you pull from it
+			//TODO: Load Ato file with sampling
+			//Load the file
 			if((uiErr = LoadATOFile(ionFilename.c_str(), ionData->data,
-						progress.filterProgress,callback)))
+						progress.filterProgress,(*Filter::wantAbort))))
 			{
 				consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
 				delete ionData;
@@ -423,10 +433,10 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 			ASSERT(false);
 	}
 
-	ionData->r = r;
-	ionData->g = g;
-	ionData->b = b;
-	ionData->a = a;
+	ionData->r = rgbaf.r();
+	ionData->g = rgbaf.g();
+	ionData->b = rgbaf.b();
+	ionData->a = rgbaf.a();
 	ionData->ionSize=ionSize;
 	ionData->valueType=valueLabel;
 
@@ -438,6 +448,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 		delete ionData;
 		return	0;
 	}
+	progress.filterProgress=100;
 
 
 	BoundCube dataCube;
@@ -451,15 +462,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 			       "(magnitude too large). Consider rescaling data before loading"));
 	}
 
-	if(cache)
-	{
-		ionData->cached=1;
-		filterOutputs.push_back(ionData);
-		cacheOK=true;
-	}
-	else
-		ionData->cached=0;
-
+	cacheAsNeeded(ionData);
 
 	//Append the ion data 
 	getOut.push_back(ionData);
@@ -476,11 +479,13 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 
 	size_t curGroup=0;
 
-	p.type=PROPERTY_TYPE_STRING;
+	p.type=PROPERTY_TYPE_FILE;
 	p.key=DATALOAD_KEY_FILE;
 	p.name=TRANS("File");
 	p.helpText=TRANS("File from which to load data");
 	p.data=ionFilename;
+	//Wx- acceptable string format
+	p.dataSecondary = TRANS("Readable files (*.xml, *.pos, *.txt,*.csv, *.ato)|*.xml;*.pos;*.txt;*.csv;*.ato|All Files|*") ;
 
 	propertyList.addProperty(p,curGroup);
 
@@ -497,6 +502,7 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 	
 	propertyList.addProperty(p,curGroup);
 
+	propertyList.setGroupTitle(curGroup,TRANS("File"));
 	//---------
 	curGroup++;	
 	
@@ -603,7 +609,7 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 	{
 		std::string tmpStr;
 
-		//FIXME: ATO Files need an imeplmentation of sampling read
+		//FIXME: ATO Files need an implementation of sampling read
 		if(fileType!=FILEDATA_TYPE_ATO)
 		{
 			stream_cast(tmpStr,doSample);
@@ -633,18 +639,17 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 		p.type=PROPERTY_TYPE_BOOL;
 		p.helpText=TRANS("Watch file timestamp to track changes to file contents from other programs");
 		propertyList.addProperty(p,curGroup);
-		propertyList.setGroupTitle(curGroup,TRANS("Load params."));
+	}
+	
+	propertyList.setGroupTitle(curGroup,TRANS("Load params."));
 
+	if(enabled)
+	{
 		
 		curGroup++;
 
-		string thisCol;
-		//Convert the ion colour to a hex string	
-		genColString((unsigned char)(r*255),(unsigned char)(g*255),
-				(unsigned char)(b*255),(unsigned char)(a*255),thisCol);
-
 		p.name=TRANS("Default colour ");
-		p.data=thisCol; 
+		p.data=rgbaf.toColourRGBA().rgbaString(); 
 		p.type=PROPERTY_TYPE_COLOUR;
 		p.helpText=TRANS("Default colour for points, if not overridden by other filters");
 		p.key=DATALOAD_KEY_COLOUR;
@@ -656,6 +661,7 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 		p.type=PROPERTY_TYPE_REAL;
 		p.helpText=TRANS("Default size for points, if not overridden by other filters");
 		p.key=DATALOAD_KEY_IONSIZE;
+		
 		propertyList.addProperty(p,curGroup);
 		propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
 	}
@@ -715,57 +721,21 @@ bool DataLoadFilter::setProperty(  unsigned int key,
 		}
 		case DATALOAD_KEY_ENABLED:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(enabled,value,needUpdate))
 				return false;
-
-			bool lastVal=enabled;
-			enabled=(stripped == "1");
-			
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=enabled)
-				needUpdate=true;
-			
-			clearCache();
 			break;
 		}
 		case DATALOAD_KEY_MONITOR:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(wantMonitor,value,needUpdate))
 				return false;
-
-			bool lastVal=wantMonitor;
-			wantMonitor=(stripped=="1");
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=wantMonitor)
-				needUpdate=true;
-			
-			clearCache();
 			break;
 		}
 		
 		case DATALOAD_KEY_SAMPLE:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(doSample,value,needUpdate))
 				return false;
-
-			bool lastVal=doSample;
-			doSample=(stripped == "1");
-			
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=doSample)
-				needUpdate=true;
-			
-			clearCache();
 			break;
 		}
 		case DATALOAD_KEY_SIZE:
@@ -789,17 +759,12 @@ bool DataLoadFilter::setProperty(  unsigned int key,
 		}
 		case DATALOAD_KEY_COLOUR:
 		{
-			unsigned char newR,newG,newB,newA;
+			ColourRGBA tmpRgba;
+			tmpRgba.parse(value);
 
-			parseColString(value,newR,newG,newB,newA);
-
-			if(newB != b || newR != r ||
-				newG !=g || newA != a)
+			if(tmpRgba != rgbaf.toColourRGBA())
 			{
-				r=newR/255.0;
-				g=newG/255.0;
-				b=newB/255.0;
-				a=newA/255.0;
+				rgbaf=tmpRgba.toRGBAf();
 
 
 				//Check the cache, updating it if needed
@@ -811,10 +776,10 @@ bool DataLoadFilter::setProperty(  unsigned int key,
 						{
 							IonStreamData *i;
 							i=(IonStreamData *)filterOutputs[ui];
-							i->r=r;
-							i->g=g;
-							i->b=b;
-							i->a=a;
+							i->r=rgbaf.r();
+							i->g=rgbaf.g();
+							i->b=rgbaf.b();
+							i->a=rgbaf.a();
 						}
 					}
 
@@ -1145,7 +1110,7 @@ bool DataLoadFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	if(XMLHelpFwdToElem(nodePtr,"colour"))
 		return false;
 
-	if(!parseXMLColour(nodePtr,r,g,b,a))
+	if(!parseXMLColour(nodePtr,rgbaf))
 		return false;	
 	//====
 
@@ -1177,7 +1142,7 @@ unsigned int DataLoadFilter::getRefreshUseMask() const
 	return 0;
 }
 
-std::string  DataLoadFilter::getErrString(unsigned int code) const
+std::string  DataLoadFilter::getSpecificErrString(unsigned int code) const
 {
 	ASSERT(errStr.size());
 	return errStr;
@@ -1190,7 +1155,6 @@ void DataLoadFilter::setPropFromBinding(const SelectionBinding &b)
 
 bool DataLoadFilter::writeState(std::ostream &f,unsigned int format, unsigned int depth) const
 {
-	using std::endl;
 	switch(format)
 	{
 		case STATE_FORMAT_XML:
@@ -1207,8 +1171,8 @@ bool DataLoadFilter::writeState(std::ostream &f,unsigned int format, unsigned in
 			f << tabs(depth+1) << "<dosample value=\"" << doSample << "\"/>" << endl;
 			f << tabs(depth+1) << "<maxions value=\"" << maxIons << "\"/>" << endl;
 
-			f << tabs(depth+1) << "<colour r=\"" <<  r<< "\" g=\"" << g << "\" b=\"" <<b
-				<< "\" a=\"" << a << "\"/>" <<endl;
+			f << tabs(depth+1) << "<colour r=\"" <<  rgbaf.r() << "\" g=\"" << rgbaf.g() 
+				<< "\" b=\"" << rgbaf.b() << "\" a=\"" << rgbaf.a() << "\"/>" <<endl;
 			f << tabs(depth+1) << "<ionsize value=\"" << ionSize << "\"/>" << endl;
 			f << tabs(depth) << "</" << trueName() << ">" << endl;
 			break;
@@ -1257,7 +1221,7 @@ bool DataLoadFilter::monitorNeedsRefresh() const
 	{
 		//Check to see that the file exists, if
 		// not fall back to the cache.
-		if(!wxFile::Exists(wxStr(ionFilename)))
+		if(!wxFile::Exists((ionFilename)))
 			return cacheOK;
 
 
@@ -1266,7 +1230,7 @@ bool DataLoadFilter::monitorNeedsRefresh() const
 		if(sizeVal != monitorSize)
 			return true;
 
-		return( wxFileModificationTime(wxStr(ionFilename))
+		return( wxFileModificationTime((ionFilename))
 						!=monitorTimestamp);
 		
 		
@@ -1344,7 +1308,7 @@ bool posFileTest()
 
 	vector<const FilterStreamData*> streamIn,streamOut;
 	ProgressData prog;
-	TEST(!d->refresh(streamIn,streamOut,prog,dummyCallback),"Refresh error code");
+	TEST(!d->refresh(streamIn,streamOut,prog),"Refresh error code");
 	delete d;
 
 
@@ -1376,7 +1340,7 @@ bool textFileTest()
 	//TODO: do better than this
 	const char *FILENAME="test-3mdfuneaascn.txt";
 	//see if we can open the file for input. If so, it must exist,
-	//and thus we don't want to overwite it, as it may contain useful data.
+	//and thus we don't want to overwrite it, as it may contain useful data.
 	std::ifstream inFile(FILENAME);
 	if(inFile)
 	{
@@ -1429,7 +1393,7 @@ bool textFileTest()
 
 	vector<const FilterStreamData*> streamIn,streamOut;
 	ProgressData prog;
-	TEST(!d->refresh(streamIn,streamOut,prog,dummyCallback),"Refresh error code");
+	TEST(!d->refresh(streamIn,streamOut,prog),"Refresh error code");
 	delete d;
 
 
@@ -1439,7 +1403,7 @@ bool textFileTest()
 	TEST(streamOut[0]->getNumBasicObjects() == NUM_PTS,"Stream count");
 
 #if defined(__LINUX__) || defined(__APPLE__)
-	//Hackish mathod to delete file
+	//Hackish method to delete file
 	std::string s;
 	s=string("rm -f ") + string(FILENAME);
 	system(s.c_str());
