@@ -1,6 +1,6 @@
 /*
  *	filterCommon.cpp - Helper routines for filter classes
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include "filterCommon.h"
 
 #include "common/colourmap.h"
-
+#include "wx/wxcommon.h"
 
 
 //TODO: Work out where the payoff for this is
@@ -35,7 +35,8 @@ bool qhullInited=false;
 
 //Wrapper for qhull single-pass run
 unsigned int doHull(unsigned int bufferSize, double *buffer, 
-			vector<Point3D> &resHull, Point3D &midPoint,bool freeHullOnExit);
+			vector<Point3D> &resHull, Point3D &midPoint,	
+			bool wantVolume, bool freeHullOnExit);
 
 void writeVectorsXML(ostream &f,const char *containerName,
 		const vector<Point3D> &vectorParams, unsigned int depth)
@@ -111,9 +112,11 @@ bool readVectorsXML(xmlNodePtr nodePtr,	std::vector<Point3D> &vectorParams)
 	return true;
 }
 
-bool parseXMLColour(xmlNodePtr &nodePtr, float &r,float&g,float&b,float&a)
+bool parseXMLColour(xmlNodePtr &nodePtr, ColourRGBAf &rgba)
 {
 	xmlChar *xmlString;
+
+	float r,g,b,a;
 	std::string tmpStr;
 	//--red--
 	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"r");
@@ -185,6 +188,10 @@ bool parseXMLColour(xmlNodePtr &nodePtr, float &r,float&g,float&b,float&a)
 	//disallow negative or values gt 1.
 	if(a < 0.0f || a > 1.0f)
 		return false;
+	rgba.r(r);
+	rgba.g(g);
+	rgba.b(b);
+	rgba.a(a);
 
 	return true;
 }
@@ -241,7 +248,7 @@ unsigned int getIonstreamIonID(const IonStreamData *d, const RangeFile *r)
 
 //!Extend a point data vector using some ion data
 unsigned int extendPointVector(std::vector<Point3D> &dest, const std::vector<IonHit> &vIonData,
-				bool (*callback)(bool),unsigned int &progress, size_t offset)
+				unsigned int &progress, size_t offset)
 {
 	unsigned int curProg=NUM_CALLBACK;
 	unsigned int n =offset;
@@ -264,7 +271,7 @@ unsigned int extendPointVector(std::vector<Point3D> &dest, const std::vector<Ion
 			progress= (unsigned int)(((float)n/(float)dest.size())*100.0f);
 			if(!omp_get_thread_num())
 			{
-				if(!(*callback)(false))
+				if(*Filter::wantAbort)
 					spin=true;
 			}
 			}
@@ -285,7 +292,7 @@ unsigned int extendPointVector(std::vector<Point3D> &dest, const std::vector<Ion
 		{
 			n+=NUM_CALLBACK;
 			progress= (unsigned int)(((float)n/(float)dest.size())*100.0f);
-			if(!(*callback)(false))
+			if(*(Filter::wantAbort))
 				return 1;
 		}
 
@@ -297,8 +304,9 @@ unsigned int extendPointVector(std::vector<Point3D> &dest, const std::vector<Ion
 }
 
 
+//FIXME: Abort pointer?
 unsigned int computeConvexHull(const vector<const FilterStreamData*> &data, unsigned int *progress,
-					bool (*callback)(bool),std::vector<Point3D> &curHull, bool freeHull)
+					std::vector<Point3D> &curHull, bool wantVolume,bool freeHull)
 {
 
 	size_t numPts;
@@ -319,11 +327,10 @@ unsigned int computeConvexHull(const vector<const FilterStreamData*> &data, unsi
 
 	//Do the convex hull in steps for two reasons
 	// 1) qhull chokes on large data
-	// 2) we need to run the callback every now and again, so we have to
+	// 2) we need to check for abort every now and again, so we have to
 	//   work in batches.
 	Point3D midPoint;
 	float maxSqrDist=-1;
-	size_t progressReduce=PROGRESS_REDUCE;
 	size_t n=0;
 	for(size_t ui=0; ui<data.size(); ui++)
 	{
@@ -367,7 +374,7 @@ unsigned int computeConvexHull(const vector<const FilterStreamData*> &data, unsi
 
 					unsigned int errCode=0;
 					
-					errCode=doHull(bufferOffset,buffer,curHull,midPoint,freeHull);
+					errCode=doHull(bufferOffset,buffer,curHull,midPoint,wantVolume,freeHull);
 					if(errCode)
 					{
 						free(buffer);
@@ -387,19 +394,15 @@ unsigned int computeConvexHull(const vector<const FilterStreamData*> &data, unsi
 			}
 			n++;
 
-			//Update the progress information, and run callback periodically
-			if(!progressReduce--)
+			//Update the progress information, and run abort check
+			if(*Filter::wantAbort)
 			{
-				if(!(*callback)(false))
-				{
-					free(buffer);
-					return HULL_ERR_USER_ABORT;
-				}
-	
-				*progress= (unsigned int)((float)(n)/((float)numPts)*100.0f);
-
-				progressReduce=PROGRESS_REDUCE;
+				free(buffer);
+				return HULL_ERR_USER_ABORT;
 			}
+
+			*progress= (unsigned int)((float)(n)/((float)numPts)*100.0f);
+
 		}
 	}
 
@@ -424,12 +427,13 @@ unsigned int computeConvexHull(const vector<const FilterStreamData*> &data, unsi
 			buffer[3*(bufferOffset+ui)+2]=curHull[ui][2];
 		}
 
-		unsigned int errCode=doHull(bufferOffset+curHull.size(),buffer,curHull,midPoint,freeHull);
+		unsigned int errCode=doHull(bufferOffset+curHull.size(),buffer,
+						curHull,midPoint,wantVolume,freeHull);
 
 		if(errCode)
 		{
 			free(buffer);
-			//Free the last convex hull mem
+			//FIXME: Free the last convex hull mem??
 			return errCode;
 		}
 	}
@@ -440,7 +444,7 @@ unsigned int computeConvexHull(const vector<const FilterStreamData*> &data, unsi
 }
 
 unsigned int computeConvexHull(const vector<Point3D> &data, unsigned int *progress,
-					bool (*callback)(bool),std::vector<Point3D> &curHull, bool freeHull)
+				const bool &abortPtr,std::vector<Point3D> &curHull, bool wantVolume, bool freeHull)
 {
 
 	//Easy case of no data
@@ -465,7 +469,6 @@ unsigned int computeConvexHull(const vector<Point3D> &data, unsigned int *progre
 	float maxSqrDist=-1;
 
 
-	size_t progressReduce=PROGRESS_REDUCE;
 
 	for(size_t uj=0; uj<data.size(); uj++)
 	{
@@ -504,7 +507,7 @@ unsigned int computeConvexHull(const vector<Point3D> &data, unsigned int *progre
 
 				unsigned int errCode=0;
 				
-				errCode=doHull(bufferOffset,buffer,curHull,midPoint,freeHull);
+				errCode=doHull(bufferOffset,buffer,curHull,midPoint,wantVolume,freeHull);
 				if(errCode)
 					return errCode;
 
@@ -519,18 +522,14 @@ unsigned int computeConvexHull(const vector<Point3D> &data, unsigned int *progre
 			}
 		}
 
-		if(!progressReduce--)
+		if(*Filter::wantAbort)
 		{
-			if(!(*callback)(false))
-			{
-				free(buffer);
-				return HULL_ERR_USER_ABORT;
-			}
-	
-			*progress= (unsigned int)((float)(uj)/((float)data.size())*100.0f);
-
-			progressReduce=PROGRESS_REDUCE;
+			free(buffer);
+			return HULL_ERR_USER_ABORT;
 		}
+
+		*progress= (unsigned int)((float)(uj)/((float)data.size())*100.0f);
+
 	}
 
 
@@ -557,7 +556,7 @@ unsigned int computeConvexHull(const vector<Point3D> &data, unsigned int *progre
 			buffer[3*(bufferOffset+ui)+2]=curHull[ui][2];
 		}
 
-		unsigned int errCode=doHull(bufferOffset+curHull.size(),buffer,curHull,midPoint,freeHull);
+		unsigned int errCode=doHull(bufferOffset+curHull.size(),buffer,curHull,midPoint,wantVolume,freeHull);
 
 		if(errCode)
 		{
@@ -573,14 +572,20 @@ unsigned int computeConvexHull(const vector<Point3D> &data, unsigned int *progre
 }
 
 unsigned int doHull(unsigned int bufferSize, double *buffer, 
-			vector<Point3D> &resHull, Point3D &midPoint, bool freeHullOnExit)
+			vector<Point3D> &resHull, Point3D &midPoint, bool wantVolume ,
+			bool freeHullOnExit)
 {
 	if(qhullInited)
 	{
-		qh_freeqhull(true);
+		qh_freeqhull(qh_ALL);
+		int curlong,totlong;
+		//This seems to be required? Cannot find any documentation on the difference
+		// between qh_freeqhull and qh_memfreeshort. qhull appears to leak when just using qh_freeqhull
+		qh_memfreeshort (&curlong, &totlong);    
 		qhullInited=false;
 	}
 
+
 	const int dim=3;
 	//Now compute the new hull
 	//Generate the convex hull
@@ -603,11 +608,23 @@ unsigned int doHull(unsigned int bufferSize, double *buffer,
 		outSquelch=stderr;
 	}
 
+	 //Joggle the output, such that only simplical facets are generated, Also compute area/volume
+	const char *argsOptions[]= { "qhull QJ FA",
+				   "qhull QJ"
+					};
+
+	const char *args;
+	if(wantVolume)
+		args = argsOptions[0];
+	else
+		args=argsOptions[1];
+	
+
 	qh_new_qhull(	dim,
 			bufferSize,
 			buffer,
 			false,
-			(char *)"qhull QJ", //Joggle the output, such that only simplical facets are generated
+			(char *)args ,
 			outSquelch, //QHULL's interface is bizarre, no way to set null pointer in qhull 2012 - result is inf. loop in qhull_fprintf and error reporting func. 
 			outSquelch);
 	qhullInited=true;
@@ -660,7 +677,11 @@ unsigned int doHull(unsigned int bufferSize, double *buffer,
 	//--
 	if(freeHullOnExit)
 	{
-		qh_freeqhull(true);
+		qh_freeqhull(qh_ALL);
+		int curlong,totlong;
+		//This seems to be required? Cannot find any documentation on the difference
+		// between qh_freeqhull and qh_memfreeshort. qhull appears to leak when just using qh_freeqhull
+		qh_memfreeshort (&curlong, &totlong);    
 		qhullInited=false;
 	
 	}
@@ -671,11 +692,15 @@ unsigned int doHull(unsigned int bufferSize, double *buffer,
 
 void freeConvexHull()
 {
-	qh_freeqhull(true);
+	qh_freeqhull(qh_ALL);
+	int curlong,totlong;
+	//This seems to be required? Cannot find any documentation on the difference
+	// between qh_freeqhull and qh_memfreeshort. qhull appears to leak when just using qh_freeqhull
+	qh_memfreeshort (&curlong, &totlong);    
 	qhullInited=false;
 }
 
-DrawColourBarOverlay *makeColourBar(float minV, float maxV,size_t nColours,size_t colourMap, bool reverseMap) 
+DrawColourBarOverlay *makeColourBar(float minV, float maxV,size_t nColours,size_t colourMap, bool reverseMap, float alpha) 
 {
 	//Set up the colour bar. Place it in a draw stream type
 	DrawColourBarOverlay *dc = new DrawColourBarOverlay;
@@ -702,7 +727,39 @@ DrawColourBarOverlay *makeColourBar(float minV, float maxV,size_t nColours,size_
 	dc->setSize(0.08,0.6);
 	dc->setPosition(0.1,0.1);
 	dc->setMinMax(minV,maxV);
-
+	dc->setAlpha(alpha);
 
 	return dc;
 }
+
+//creates a temporary filename for use
+std::string createTmpFilename(const char *dir,const char *extension)
+{
+	wxString tmpFilename,tmpDir;
+	
+	if(!dir)
+	{
+		tmpDir=wxFileName::GetTempDir();
+
+
+	#if defined(__WIN32__) || defined(__WIN64__)
+		tmpDir=tmpDir + wxT("\\3Depict\\");
+
+	#else
+		tmpDir=tmpDir + wxT("/3Depict/");
+	#endif
+
+	}
+	else
+		tmpDir=dir;
+	
+	if(!wxDirExists(tmpDir))
+		wxMkdir(tmpDir);
+	tmpFilename=wxFileName::CreateTempFileName(tmpDir+ wxT("unittest-"));
+	wxRemoveFile(tmpFilename);
+	if(extension)
+		tmpFilename+=wxT(".pos");
+
+	return stlStr(tmpFilename);
+}
+
