@@ -1,6 +1,6 @@
 /*
  *	ionClip.cpp -3Depict filter to perform clipping of 3D point clouds
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -23,12 +23,18 @@
 
 #include <map>
 
+using std::vector;
+using std::string;
+using std::pair;
+using std::make_pair;
+using std::map;
 
 //!Error codes
 enum 
 {
 	CALLBACK_FAIL=1,
 	BAD_ALLOC,
+	IONCLIP_ERR_ENUM_END
 };
 
 //!Possible primitive types for ion clipping
@@ -106,7 +112,7 @@ Filter *IonClipFilter::cloneUncached() const
 
 	p->lockAxisMag = lockAxisMag;
 
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -124,8 +130,7 @@ size_t IonClipFilter::numBytesForCache(size_t nObjects) const
 
 //!update filter
 unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-			std::vector<const FilterStreamData *> &getOut, ProgressData &progress, 
-								bool (*callback)(bool))
+			std::vector<const FilterStreamData *> &getOut, ProgressData &progress) 
 {
 	ASSERT(vectorParams.size() || scalarParams.size());	
 	//Clear selection devices, first deleting any we have
@@ -251,7 +256,7 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 				dC->setRadius(scalarParams[0]);
 				dC->setColour(0.5,0.5,0.5,1.0);
 				dC->setSlices(40);
-				dC->setLength(sqrt(vectorParams[1].sqrMag()));
+				dC->setLength(sqrtf(vectorParams[1].sqrMag()));
 				dC->setDirection(vectorParams[1]);
 				dC->wantsLight=true;
 				drawData->drawables.push_back(dC);
@@ -378,6 +383,7 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 				getOut.push_back(dataIn[ui]);
 		}
 			
+		progress.filterProgress=100;
 		return 0;
 	}
 
@@ -395,7 +401,7 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 		primitiveTypeMap[make_pair((size_t)PRIMITIVE_SPHERE,true)]=CROP_SPHERE_OUTSIDE;
 		primitiveTypeMap[make_pair((size_t)PRIMITIVE_PLANE,false)]=CROP_PLANE_FRONT;
 		primitiveTypeMap[make_pair((size_t)PRIMITIVE_PLANE,true)]=CROP_PLANE_BACK;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,false)]=CROP_CYLINDER_INSIDE;
+		primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,false)]=CROP_CYLINDER_INSIDE_AXIAL;
 		primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,true)]=CROP_CYLINDER_OUTSIDE;
 		primitiveTypeMap[make_pair((size_t)PRIMITIVE_AAB,false)]=CROP_AAB_INSIDE;
 		primitiveTypeMap[make_pair((size_t)PRIMITIVE_AAB,true)]=CROP_AAB_OUTSIDE;
@@ -403,10 +409,10 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 		size_t mode;
 		mode = primitiveTypeMap[make_pair(primitiveType,invertedClip)];
 		size_t totalSize=numElements(dataIn);
-		CropHelper cropper(callback,&progress.filterProgress,
-					totalSize,mode,
-					vectorParams,scalarParams  );
+		CropHelper cropper(totalSize,mode,vectorParams,scalarParams  );
 
+		float minProg,maxProg;
+		size_t cumulativeSize=0;
 		for(unsigned int ui=0;ui<dataIn.size() ;ui++)
 		{
 			//Loop through each data set
@@ -416,9 +422,13 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 				{
 					d=new IonStreamData;
 					d->parent=this;
+					minProg=cumulativeSize/(float)totalSize;
+					cumulativeSize+=d->data.size();
+					maxProg=cumulativeSize/(float)totalSize;
 
 					//Filter input data to output data.
-					if(cropper.runFilter(((const IonStreamData *)dataIn[ui])->data,d->data))
+					if(cropper.runFilter(((const IonStreamData *)dataIn[ui])->data,
+						d->data,minProg,maxProg,progress.filterProgress))
 					{
 						delete d;
 						return CALLBACK_FAIL; 
@@ -435,15 +445,7 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 						d->representationType=((IonStreamData *)dataIn[ui])->representationType;
 
 						//getOut is const, so shouldn't be modified
-						if(cache)
-						{
-							d->cached=1;
-							filterOutputs.push_back(d);
-							cacheOK=true;
-						}
-						else
-							d->cached=0;
-						
+						cacheAsNeeded(d);
 
 						getOut.push_back(d);
 						d=0;
@@ -581,10 +583,7 @@ void IonClipFilter::getProperties(FilterPropGroup &propertyList) const
 			p.helpText=TRANS("Positive vector for cylinder");
 			propertyList.addProperty(p,curGroup);
 			
-			if(lockAxisMag)
-				tmpStr="1";
-			else
-				tmpStr="0";
+			tmpStr=boolStrEnc(lockAxisMag);
 			p.key=KEY_AXIS_LOCKMAG;
 			p.name=TRANS("Lock Axis Mag.");
 			p.data=tmpStr;
@@ -625,7 +624,8 @@ void IonClipFilter::getProperties(FilterPropGroup &propertyList) const
 		default:
 			ASSERT(false);
 	}
-	
+
+	propertyList.setGroupTitle(curGroup,TRANS("Clipping"));
 }
 
 //!Set the properties for the nth filter. Returns true if prop set OK
@@ -737,50 +737,21 @@ bool IonClipFilter::setProperty(unsigned int key,
 		}
 		case KEY_ORIGIN:
 		{
-			ASSERT(vectorParams.size() >= 1);
-			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!applyPropertyNow(vectorParams[0],value,needUpdate))
 				return false;
-
-			if(!(vectorParams[0] == newPt ))
-			{
-				vectorParams[0] = newPt;
-				needUpdate=true;
-				clearCache();
-			}
-
-			return true;
+			break;
 		}
 		case KEY_CORNER:
 		{
-			ASSERT(vectorParams.size() >= 2);
-			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!applyPropertyNow(vectorParams[1],value,needUpdate))
 				return false;
-
-			if(!(vectorParams[1] == newPt ))
-			{
-				vectorParams[1] = newPt;
-				needUpdate=true;
-				clearCache();
-			}
-
-			return true;
+			break;
 		}
 		case KEY_RADIUS:
 		{
-			ASSERT(scalarParams.size() >=1);
-			float newRad;
-			if(stream_cast(newRad,value))
+			if(!applyPropertyNow(scalarParams[0],value,needUpdate))
 				return false;
-
-			if(scalarParams[0] != newRad )
-			{
-				scalarParams[0] = newRad;
-				needUpdate=true;
-				clearCache();
-			}
-			return true;
+			break;
 		}
 		case KEY_NORMAL:
 		{
@@ -792,10 +763,10 @@ bool IonClipFilter::setProperty(unsigned int key,
 			if(primitiveType == PRIMITIVE_CYLINDER)
 			{
 				if(lockAxisMag && 
-					newPt.sqrMag() > sqrt(std::numeric_limits<float>::epsilon()))
+					newPt.sqrMag() > sqrtf(std::numeric_limits<float>::epsilon()))
 				{
 					newPt.normalise();
-					newPt*=sqrt(vectorParams[1].sqrMag());
+					newPt*=sqrtf(vectorParams[1].sqrMag());
 				}
 			}
 			if(!(vectorParams[1] == newPt ))
@@ -808,58 +779,20 @@ bool IonClipFilter::setProperty(unsigned int key,
 		}
 		case KEY_PRIMITIVE_SHOW:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(showPrimitive,value,needUpdate))
 				return false;
-
-			if(stripped=="1")
-				showPrimitive=true;
-			else
-				showPrimitive=false;
-
-			needUpdate=true;
-
 			break;
 		}
 		case KEY_PRIMITIVE_INVERTCLIP:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(invertedClip,value,needUpdate))
 				return false;
-
-			bool lastVal=invertedClip;
-			if(stripped=="1")
-				invertedClip=true;
-			else
-				invertedClip=false;
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=invertedClip)
-			{
-				needUpdate=true;
-				clearCache();
-			}
-
 			break;
 		}
-
 		case KEY_AXIS_LOCKMAG:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(lockAxisMag,value,needUpdate))
 				return false;
-
-			if(stripped=="1")
-				lockAxisMag=true;
-			else
-				lockAxisMag=false;
-
-			needUpdate=true;
-
 			break;
 		}
 		default:
@@ -873,16 +806,15 @@ bool IonClipFilter::setProperty(unsigned int key,
 }
 
 //!Get the human readable error string associated with a particular error code during refresh(...)
-std::string IonClipFilter::getErrString(unsigned int code) const
+std::string IonClipFilter::getSpecificErrString(unsigned int code) const
 {
-	switch(code)
-	{
-		case BAD_ALLOC:
-			return std::string("Insufficient mem. for Ionclip");
-		case CALLBACK_FAIL:
-			return std::string("Ionclip Aborted");
-	}
-	ASSERT(false);
+	const char *errCode[] = { "",
+				"Insufficient mem. for Ionclip",
+				"Ionclip Aborted"
+	};
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(errCode) == IONCLIP_ERR_ENUM_END);
+	ASSERT(code < IONCLIP_ERR_ENUM_END);
+	return errCode[code];
 }
 
 bool IonClipFilter::writeState(std::ostream &f,unsigned int format, unsigned int depth) const
@@ -942,23 +874,16 @@ bool IonClipFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFileD
 	//
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"invertedclip","value"))
 		return false;
-	if(tmpStr == "0")
-		invertedClip=false;
-	else if(tmpStr == "1")
-		invertedClip=true;
-	else
+
+	if(!boolStrDec(tmpStr,invertedClip))
 		return false;
 	//====
 	
-	//Retrieve primitive visiblity 
+	//Retrieve primitive visibility 
 	//====
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"showprimitive","value"))
 		return false;
-	if(tmpStr == "0")
-		showPrimitive=false;
-	else if(tmpStr == "1")
-		showPrimitive=true;
-	else
+	if(!boolStrDec(tmpStr,showPrimitive))
 		return false;
 	//====
 	
@@ -966,11 +891,8 @@ bool IonClipFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFileD
 	//====
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"lockaxismag","value"))
 		return false;
-	if(tmpStr == "0")
-		lockAxisMag=false;
-	else if(tmpStr == "1")
-		lockAxisMag=true;
-	else
+
+	if(!boolStrDec(tmpStr,lockAxisMag))
 		return false;
 	//====
 	
@@ -1164,7 +1086,7 @@ bool sphereTest()
 
 	//Do the refresh
 	ProgressData p;
-	f->refresh(streamIn,streamOut,p,dummyCallback);
+	f->refresh(streamIn,streamOut,p);
 
 	delete f;
 	delete d;
@@ -1182,7 +1104,7 @@ bool sphereTest()
 		Point3D p;
 		p=dOut->data[ui].getPos();
 
-		TEST(sqrt(p.sqrDist(pOrigin)) 
+		TEST(sqrtf(p.sqrDist(pOrigin)) 
 			<= TEST_RADIUS,"Sphere containment");
 	}
 	
@@ -1221,7 +1143,7 @@ bool planeTest()
 
 	//Do the refresh
 	ProgressData p;
-	f->refresh(streamIn,streamOut,p,dummyCallback);
+	f->refresh(streamIn,streamOut,p);
 
 	delete f;
 	delete d;
@@ -1273,7 +1195,7 @@ bool cylinderTest(const Point3D &pAxis, const unsigned int *span, float testRadi
 	TEST(f->setProperty(KEY_PRIMITIVE_SHOW,"0",needUp),"Set prop");
 	//Do the refresh
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 	delete f;
 	delete d;
 	
@@ -1285,7 +1207,7 @@ bool cylinderTest(const Point3D &pAxis, const unsigned int *span, float testRadi
 	DrawCylinder *dC = new DrawCylinder;
 	dC->setRadius(testRadius);
 	dC->setOrigin(pOrigin);
-	float len = sqrt(pAxis.sqrMag());
+	float len = sqrtf(pAxis.sqrMag());
 
 	Point3D axisNormal(pAxis);
 	axisNormal.normalise();
@@ -1298,7 +1220,7 @@ bool cylinderTest(const Point3D &pAxis, const unsigned int *span, float testRadi
 
 	delete dC;
 
-	b.expand(sqrt(std::numeric_limits<float>::epsilon()));
+	b.expand(sqrtf(std::numeric_limits<float>::epsilon()));
 	for(unsigned int ui=0;ui<dOut->data.size();ui++)
 	{
 		Point3D p;
@@ -1345,7 +1267,7 @@ bool rectTest()
 	TEST(f->setProperty(KEY_CORNER,s,needUp),"Set prop");
 
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 	delete f;
 	delete d;
 
