@@ -1,6 +1,6 @@
 /*
  *	ionDownsample.cpp - Filter to perform sampling without replacement on input ion data
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -22,12 +22,14 @@
 
 #include "ionDownsample.h"
 
+using std::vector;
+using std::string;
 
 //!Downsampling filter
 enum
 {
-	IONDOWNSAMPLE_ABORT_ERR=1,
-	IONDOWNSAMPLE_BAD_ALLOC,
+	IONDOWNSAMPLE_BAD_ALLOC=1,
+	IONDOWNSAMPLE_ERR_ENUM_END
 };
 
 
@@ -148,7 +150,7 @@ Filter *IonDownsampleFilter::cloneUncached() const
 	std::copy(ionLimits.begin(),ionLimits.end(),p->ionLimits.begin());
 
 
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -173,7 +175,7 @@ size_t IonDownsampleFilter::numBytesForCache(size_t nObjects) const
 }
 
 unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+	std::vector<const FilterStreamData *> &getOut, ProgressData &progress)
 {
 	//use the cached copy if we have it.
 	if(cacheOK)
@@ -183,7 +185,10 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 
 		return 0;
 	}
-
+	
+	progress.step=1;
+	progress.maxStep=1;
+	progress.stepName=TRANS("Sampling");
 
 	size_t totalSize = numElements(dataIn,STREAM_TYPE_IONS);
 	if(!perSpecies)	
@@ -208,14 +213,19 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 							frac = (float)(((const IonStreamData*)dataIn[ui])->data.size())/(float)totalSize;
 
 							randomSelect(d->data,((const IonStreamData *)dataIn[ui])->data,
-										rng,maxAfterFilter*frac,progress.filterProgress,callback,strongRandom);
+										rng,maxAfterFilter*frac,progress.filterProgress,
+												*wantAbort,strongRandom);
 
+							if(*Filter::wantAbort)
+							{
+								delete d;
+								return FILTER_ERR_ABORT;
+							}
 
 						}
 						else
 						{
 
-							unsigned int curProg=NUM_CALLBACK;
 							size_t n=0;
 							//Reserve 90% of storage needed.
 							//highly likely with even modest numbers of ions
@@ -224,23 +234,17 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 
 							ASSERT(dataIn[ui]->getStreamType() == STREAM_TYPE_IONS);
 
-							for(vector<IonHit>::const_iterator it=((const IonStreamData *)dataIn[ui])->data.begin();
+							for(std::vector<IonHit>::const_iterator it=((const IonStreamData *)dataIn[ui])->data.begin();
 								       it!=((const IonStreamData *)dataIn[ui])->data.end(); ++it)
 							{
 								if(rng.genUniformDev() <  fraction)
 									d->data.push_back(*it);
 							
-								//update progress every CALLBACK ions
-								if(!curProg--)
+								progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+								if(*Filter::wantAbort)
 								{
-									curProg=NUM_CALLBACK;
-									n+=NUM_CALLBACK;
-									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-									if(!(*callback)(false))
-									{
-										delete d;
-										return IONDOWNSAMPLE_ABORT_ERR;
-									}
+									delete d;
+									return FILTER_ERR_ABORT;
 								}
 							}
 						}
@@ -268,15 +272,7 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 					d->valueType=((IonStreamData *)dataIn[ui])->valueType;
 
 					//getOut is const, so shouldn't be modified
-					if(cache)
-					{
-						d->cached=1;
-						filterOutputs.push_back(d);
-						cacheOK=true;
-					}
-					else
-						d->cached=0;
-			
+					cacheAsNeeded(d);
 
 					getOut.push_back(d);
 					break;
@@ -354,14 +350,16 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 							//The total number of ions is the specified value for this ionID, multiplied by
 							//this stream's fraction of the total incoming data
 							randomSelect(d->data,input->data, rng,(size_t)(frac*ionLimits[ionIDVec[idPos]]),
-									progress.filterProgress,callback,strongRandom);
+									progress.filterProgress,*wantAbort,strongRandom);
+							if(*Filter::wantAbort)
+							{
+								delete d;
+								return FILTER_ERR_ABORT;
+							}
 						}
 						else
 						{
 							//Use the direct fractions as entered in by user. 
-
-							unsigned int curProg=NUM_CALLBACK;
-
 							float thisFraction = ionFractions[ionIDVec[idPos]];
 							
 							//Reserve 90% of storage needed.
@@ -377,17 +375,13 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 									if(rng.genUniformDev() <  thisFraction)
 										d->data.push_back(*it);
 								
-									//update progress every CALLBACK ions
-									if(!curProg--)
+									//update progress
+									progress.filterProgress= 
+										(unsigned int)((float)(n)/((float)totalSize)*100.0f);
+									if(*Filter::wantAbort)
 									{
-										n+=NUM_CALLBACK;
-										progress.filterProgress= 
-											(unsigned int)((float)(n)/((float)totalSize)*100.0f);
-										if(!(*callback)(false))
-										{
-											delete d;
-											return IONDOWNSAMPLE_ABORT_ERR;
-										}
+										delete d;
+										return FILTER_ERR_ABORT;
 									}
 								}
 						
@@ -414,15 +408,7 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 
 
 						//getOut is const, so shouldn't be modified
-						if(cache)
-						{
-							d->cached=1;
-							filterOutputs.push_back(d);
-							cacheOK=true;
-						}
-						else
-							d->cached=0;
-			
+						cacheAsNeeded(d);
 
 						getOut.push_back(d);
 					}
@@ -444,6 +430,7 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 
 	}	
 
+	progress.filterProgress=100;
 	return 0;
 }
 
@@ -475,7 +462,7 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 	}	
 
 
-	propertyList.setGroupTitle(curGroup,TRANS("Sampling rates"));
+	propertyList.setGroupTitle(curGroup,TRANS("Mode"));
 	curGroup++;
 	if(rsdIncoming && perSpecies)
 	{
@@ -485,6 +472,7 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 		else
 			typeVal=PROPERTY_TYPE_REAL;
 
+		bool haveProp=false;
 		//create a  single line for each
 		for(unsigned  int ui=0; ui<rsdIncoming->enabledIons.size(); ui++)
 		{
@@ -501,8 +489,12 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 				p.helpText=TRANS("Sampling value for species");
 				p.key=KEY_IONDOWNSAMPLE_DYNAMIC+ui;
 				propertyList.addProperty(p,curGroup);
+				
+				haveProp=true;
 			}
 		}
+		if(haveProp)
+			propertyList.setGroupTitle(curGroup,TRANS("Sampling rates"));
 	}
 	else
 	{
@@ -526,6 +518,7 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 
 		}
 		propertyList.addProperty(p,curGroup);
+		propertyList.setGroupTitle(curGroup,TRANS("Sampling rates"));
 	}
 }
 
@@ -537,21 +530,8 @@ bool IonDownsampleFilter::setProperty(  unsigned int key,
 	{
 		case KEY_IONDOWNSAMPLE_FIXEDOUT: 
 		{
-			string stripped=stripWhite(value);
-
-			bool lastVal=fixedNumOut;
-
-			if(!boolStrDec(stripped,fixedNumOut))
+			if(!applyPropertyNow(fixedNumOut,value,needUpdate))
 				return false;
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=fixedNumOut)
-			{
-				needUpdate=true;
-				clearCache();
-			}
-
 			break;
 		}	
 		case KEY_IONDOWNSAMPLE_FRACTION:
@@ -578,38 +558,14 @@ bool IonDownsampleFilter::setProperty(  unsigned int key,
 		}
 		case KEY_IONDOWNSAMPLE_COUNT:
 		{
-			size_t count;
-
-			if(stream_cast(count,value))
+			if(!applyPropertyNow(maxAfterFilter,value,needUpdate))
 				return false;
-
-			maxAfterFilter=count;
-			//In the case of fixed number output, 
-			//our cache is invalidated
-			if(fixedNumOut)
-			{
-				needUpdate=true;
-				clearCache();
-			}
-			
 			break;
 		}	
 		case KEY_IONDOWNSAMPLE_PERSPECIES: 
 		{
-			string stripped=stripWhite(value);
-
-			bool lastVal=perSpecies;
-			if(!boolStrDec(stripped,perSpecies))
+			if(!applyPropertyNow(perSpecies,value,needUpdate))
 				return false;
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(lastVal!=perSpecies)
-			{
-				needUpdate=true;
-				clearCache();
-			}
-
 			break;
 		}	
 		default:
@@ -661,17 +617,14 @@ bool IonDownsampleFilter::setProperty(  unsigned int key,
 }
 
 
-std::string  IonDownsampleFilter::getErrString(unsigned int code) const
+std::string  IonDownsampleFilter::getSpecificErrString(unsigned int code) const
 {
-	switch(code)
-	{
-		case IONDOWNSAMPLE_ABORT_ERR:
-			return std::string(TRANS("Downsample Aborted"));
-		case IONDOWNSAMPLE_BAD_ALLOC:
-			return std::string(TRANS("Insuffient memory for downsample"));
-	}	
-
-	return std::string("BUG! Should not see this (IonDownsample)");
+	const char *errStrs[] = { "",	
+		"Insuffient memory for downsample",
+	};	
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(errStrs) == IONDOWNSAMPLE_ERR_ENUM_END);
+	ASSERT(code < IONDOWNSAMPLE_ERR_ENUM_END);
+	return errStrs[code];
 }
 
 void IonDownsampleFilter::setPropFromBinding(const SelectionBinding &b)
@@ -804,7 +757,7 @@ unsigned int IonDownsampleFilter::getRefreshUseMask() const
 
 //Create a synthetic dataset of points
 // returned pointer *must* be deleted. Span must have 3 elements, 
-// and for best results could be co-prime with one another; eg all prime numbers
+// and for best results could be co-prime with one another; e.g. all prime numbers
 IonStreamData *synthDataPts(unsigned int span[],unsigned int numPts);
 
 //Test for fixed number of output ions
@@ -856,7 +809,7 @@ bool fixedSampleTest()
 
 	//Do the refresh
 	ProgressData p;
-	f->refresh(streamIn,streamOut,p,dummyCallback);
+	f->refresh(streamIn,streamOut,p);
 
 	delete f;
 	delete d;
@@ -893,7 +846,7 @@ bool variableSampleTest()
 
 	//Do the refresh
 	ProgressData p;
-	TEST(!(f->refresh(streamIn,streamOut,p,dummyCallback)),"refresh error code");
+	TEST(!(f->refresh(streamIn,streamOut,p)),"refresh error code");
 
 	delete f;
 	delete d;
