@@ -1,6 +1,6 @@
 /*
  *	rangeFile.cpp - bins ions into different value ranges given an input range file
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 #include <map>
 
 using std::map;
+using std::vector;
+using std::string;
 
 const unsigned int NUM_ROWS_ION=3;
 const unsigned int NUM_ROWS_RANGE=4;
@@ -30,13 +32,15 @@ const unsigned int NUM_ROWS_RANGE=4;
 enum
 {
 	RANGEFILE_ABORT_FAIL=1,
-	RANGEFILE_BAD_ALLOC
+	RANGEFILE_BAD_ALLOC,
+	RANGEFILE_ERR_ENUM_END
 };
 //== Range File Filter == 
 
 RangeFileFilter::RangeFileFilter()
 {
 	dropUnranged=true;
+	showLegend=false;
 	assumedFileFormat=RANGE_FORMAT_ORNL;
 }
 
@@ -55,7 +59,7 @@ Filter *RangeFileFilter::cloneUncached() const
 	p->assumedFileFormat=assumedFileFormat;
 	p->dropUnranged=dropUnranged;
 
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -96,7 +100,7 @@ void RangeFileFilter::initFilter(const std::vector<const FilterStreamData *> &da
 }
 
 unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-		std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+		std::vector<const FilterStreamData *> &getOut, ProgressData &progress)
 {
 
 	//use the cached copy of the data if we have it.
@@ -163,6 +167,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 			d[ui]->parent=this;
 		}
 		
+		const unsigned int RANGE_ALLOC_STEP=157; //is prime - less likely to form sequence?
 		if(!haveEnabled)
 		{
 			//There are no enabled ranges at all.
@@ -173,6 +178,8 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 			//Step 1: Do a first sweep to obtain range sizes needed
 			// then reserve the same amount of mem as we need on the output
 			//========================
+			//Don't examine every ion to determine the allocation size.
+			// perform step-wise examination of dataset
 			for(unsigned int ui=0;ui<dataIn.size() ;ui++)
 			{
 				switch(dataIn[ui]->getStreamType())
@@ -195,7 +202,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 						unsigned int curProg=NUM_CALLBACK;
 						bool spin=false;
 						#pragma omp parallel for firstprivate(curProg)
-						for(size_t uj=0; uj<src->data.size();uj++)
+						for(size_t uj=0; uj<src->data.size();uj+=RANGE_ALLOC_STEP)
 						{
 #ifdef _OPENMP
 							unsigned int thisT=omp_get_thread_num();
@@ -235,7 +242,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 								curProg=NUM_CALLBACK;
 
 
-								if(!(*callback)(false))
+								if(*Filter::wantAbort)
 									spin=true;
 								}
 							}
@@ -261,8 +268,9 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 		//reserve the vector to the exact size we need
 		try
 		{
+			//slightly over-allocate to allow for any variance
 			for(size_t ui=0;ui<d.size();ui++)
-				d[ui]->data.reserve(dSizes[ui]);
+				d[ui]->data.reserve(dSizes[ui]*1.f*RANGE_ALLOC_STEP+10);
 		}
 		catch(std::bad_alloc)
 		{
@@ -286,7 +294,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 		//=========================================
 		if(haveEnabled)
 		{
-			//	I tried parallelising this a few different ways, but the linear performance wa simply better.
+			//	I tried parallelising this a few different ways, but the linear performance was simply better.
 			//		- Tried an array of openmp locks
 			//		- Tried keeping a unique offset number and fixing the size of the output vectors
 			//		- Tried straight criticalling the push_back
@@ -357,7 +365,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 								curProg=NUM_CALLBACK;
 
 								
-								if(!(*callback)(false))
+								if(*Filter::wantAbort)
 								{
 									//Free space allocated for output ion streams...
 									for(unsigned int ui=0;ui<d.size();ui++)
@@ -408,7 +416,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 				}
 			}
 
-			if(!(*callback)(false))
+			if(*Filter::wantAbort)
 			{
 				//Free space allocated for output ion streams...
 				for(unsigned int ui=0;ui<d.size();ui++)
@@ -467,26 +475,23 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 		//======================================
 
 		//Having ranged all streams, merge them back into one ranged stream.
-		if(cache)
-		{
-			for(unsigned int ui=0;ui<d.size(); ui++)
-			{
-				d[ui]->cached=1; //IMPORTANT: ->cached must be set PRIOR to push back
-				filterOutputs.push_back(d[ui]);
-			}
-		}
-		else
-		{
-			for(unsigned int ui=0;ui<d.size(); ui++)
-				d[ui]->cached=0; //IMPORTANT: ->cached must be set PRIOR to push back
-			cacheOK=false;
-		}
+		for(unsigned int ui=0;ui<d.size(); ui++)
+			cacheAsNeeded(d[ui]);
 		
 		for(unsigned int ui=0;ui<d.size(); ui++)
 			getOut.push_back(d[ui]);
 	}
 
+	if(haveEnabled && showLegend)
+	{
+		DrawStreamData *dS = createLegend();
+		cacheAsNeeded(dS);
 
+		getOut.push_back(dS);
+
+	}
+
+	
 	//Put out rangeData
 	RangeStreamData *rngData=new RangeStreamData;
 	rngData->parent=this;
@@ -497,33 +502,25 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 	rngData->enabledIons.resize(enabledIons.size());	
 	std::copy(enabledIons.begin(),enabledIons.end(),rngData->enabledIons.begin());
 	
-	
-	rngData->cached=cache;
-	
-	if(cache)
-		filterOutputs.push_back(rngData);
 
+	cacheAsNeeded(rngData);	
 	getOut.push_back(rngData);
 		
 	cacheOK=cache;
 	return 0;
 }
 
+
+
 bool RangeFileFilter::updateRng()
 {
 	if(!rng.openGuessFormat(rngName.c_str()))
 		return false;
 
 	unsigned int nRng = rng.getNumRanges();
-	enabledRanges.resize(nRng);
+	enabledRanges.resize(nRng,(char)1);
 	unsigned int nIon = rng.getNumIons();
-	enabledIons.resize(nIon);
-	//Turn all ranges to "on" 
-	for(unsigned int ui=0;ui<enabledRanges.size(); ui++)
-		enabledRanges[ui]=(char)1;
-	//Turn all ions to "on" 
-	for(unsigned int ui=0;ui<enabledIons.size(); ui++)
-		enabledIons[ui]=(char)1;
+	enabledIons.resize(nIon,(char)1);
 
 	return true;
 }
@@ -533,15 +530,9 @@ void RangeFileFilter::setRangeData(const RangeFile &rngNew)
 	rng=rngNew;		
 	
 	unsigned int nRng = rng.getNumRanges();
-	enabledRanges.resize(nRng);
+	enabledRanges.resize(nRng,1);
 	unsigned int nIon = rng.getNumIons();
-	enabledIons.resize(nIon);
-	//Turn all ranges to "on" 
-	for(unsigned int ui=0;ui<enabledRanges.size(); ui++)
-		enabledRanges[ui]=(char)1;
-	//Turn all ions to "on" 
-	for(unsigned int ui=0;ui<enabledIons.size(); ui++)
-		enabledIons[ui]=(char)1;
+	enabledIons.resize(nIon,1);
 
 	clearCache();
 }
@@ -565,19 +556,18 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 	size_t curGroup=0;
 
 	prop.name=TRANS("File");
-	prop.type=PROPERTY_TYPE_STRING;
+	//Wx- acceptable string format
+	prop.type=PROPERTY_TYPE_FILE;
 	prop.helpText=TRANS("File to use for range data");
 	prop.key=RANGE_KEY_RANGE_FILENAME;
 	prop.data=rngName;
+	prop.dataSecondary=TRANS(RANGEFILE_WX_CONSTANT);
+
 
 	p.addProperty(prop,curGroup);	
 
 	std::string tmpStr;
-	if(dropUnranged)
-		tmpStr="1";
-	else
-		tmpStr="0";
-
+	tmpStr=boolStrEnc(dropUnranged);
 	prop.name=TRANS("Drop unranged");
 	prop.type=PROPERTY_TYPE_BOOL;
 	prop.helpText=TRANS("Remove unranged points when generating output");
@@ -585,8 +575,20 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 	prop.data=tmpStr;
 
 	p.addProperty(prop,curGroup);
+	p.setGroupTitle(curGroup,TRANS("File"));
 
 	curGroup++;
+
+	
+	prop.name = TRANS("Legend");
+	prop.type = PROPERTY_TYPE_BOOL;
+	prop.helpText = TRANS("Display colour legend for enabled ions");
+	prop.key= RANGE_KEY_ENABLE_LEGEND;
+	prop.data=boolStrEnc(showLegend);
+	p.addProperty(prop,curGroup);
+	p.setGroupTitle(curGroup,TRANS("View"));
+	curGroup++;
+
 	//---
 	//Option to disable/enable all ions
 	if(rng.getNumIons())
@@ -624,11 +626,8 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 
 			p.addProperty(prop,curGroup);
 
-			string str;	
-			if(enabledIons[ui])
-				str="1";
-			else
-				str="0";
+			string str;
+			str=boolStrEnc(enabledIons[ui]);
 			
 			prop.name=TRANS("Active Ion ") + suffix;
 			prop.type=PROPERTY_TYPE_BOOL;
@@ -638,16 +637,13 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 		
 			p.addProperty(prop,curGroup);
 
-			RGBf col;
-			string thisCol;
-		
+			ColourRGBAf col;
 			//Convert the ion colour to a hex string	
 			col=rng.getColour(ui);
-			genColString((unsigned char)(col.red*255),(unsigned char)(col.green*255),
-					(unsigned char)(col.blue*255),255,thisCol);
-
+			
 			prop.name=TRANS("Colour ") + suffix;
-			prop.data=thisCol;
+			prop.data=col.toColourRGBA().rgbaString();
+
 			prop.type=PROPERTY_TYPE_COLOUR;
 			prop.helpText=TRANS("Colour used to represent ion");
 			prop.key=NUM_ROWS_ION*ui+3+RANGE_KEY_ENABLE_ALL_IONS;
@@ -655,12 +651,11 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			p.addProperty(prop,curGroup);
 		}
 
+		p.setGroupTitle(curGroup,TRANS("Ions"));
+		curGroup++;
 	}
 	//----
 
-
-	curGroup++;
-
 	//----
 	if(rng.getNumRanges())
 	{
@@ -686,11 +681,8 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			std::string suffix;
 			stream_cast(suffix,ui);
 
-			string str;	
-			if(enabledRanges[ui])
-				str="1";
-			else
-				str="0";
+			string str;
+			str=boolStrEnc(enabledRanges[ui]);
 
 			prop.name=TRANS("Active Rng ")+suffix;
 			prop.data=str;
@@ -773,37 +765,61 @@ bool RangeFileFilter::setProperty(unsigned int key,
 		}
 		case RANGE_KEY_DROP_UNRANGED: //Enable/disable unranged dropping
 		{
-			unsigned int valueInt;
-			if(stream_cast(valueInt,value))
+			if(!applyPropertyNow(dropUnranged,value,needUpdate))
+				return false;
+			break;
+		}	
+		case RANGE_KEY_ENABLE_LEGEND:
+		{
+			if(!cacheOK || !cache)
+			{
+				if(!applyPropertyNow(showLegend,value,needUpdate))
+					return false;
+				break;
+			}
+
+			//Manually decode the value, then fiddle with the cache
+			// so we don't invalidate it and cause a full recomputation
+			bool newShow;
+			if(!boolStrDec(value,newShow))
 				return false;
 
-			if(valueInt ==0 || valueInt == 1)
+			if(showLegend == newShow)
+				return false;
+
+			if(showLegend)
 			{
-				if(dropUnranged!= (bool)valueInt)
+				//disabling legend, find it and destroy it in cache, rather
+				// than recomputing all ranging
+				for(size_t ui=0; ui<filterOutputs.size(); ui++)
 				{
-					needUpdate=true;
-					dropUnranged=valueInt;
+					if(filterOutputs[ui]->getStreamType() == STREAM_TYPE_DRAW)
+					{
+						delete filterOutputs[ui];
+						std::swap(filterOutputs[ui],filterOutputs.back());
+						filterOutputs.pop_back();
+						break;
+					}	
 				}
-				else
-					needUpdate=false;
 			}
 			else
-				return false;		
-			
-			if(needUpdate)
-				clearCache();
+			{
+				//enabling legend, create the new legened and add it to the cache
+				DrawStreamData *ds = createLegend();
+				ds->cached=1;
+				filterOutputs.push_back(ds);
+					
+			}
 
+			showLegend=newShow;
+			needUpdate=true;
 			break;
 		}	
 		case RANGE_KEY_ENABLE_ALL_RANGES:
 		{
 
 			bool allEnable;
-			if(value == "1")
-				allEnable=true;
-			else if ( value == "0")
-				allEnable=false;
-			else
+			if(!boolStrDec(value,allEnable))
 				return false;
 
 			//set them to the opposite of whatever we have now
@@ -826,11 +842,7 @@ bool RangeFileFilter::setProperty(unsigned int key,
 		{
 
 			bool allEnable;
-			if(value == "1")
-				allEnable=true;
-			else if ( value == "0")
-				allEnable=false;
-			else
+			if(!boolStrDec(value,allEnable))
 				return false;
 
 			//set them to the opposite of whatever we have now
@@ -902,15 +914,9 @@ bool RangeFileFilter::setProperty(unsigned int key,
 					}	
 					case 2: //Colour of the ion
 					{
-						unsigned char r,g,b,a;
-						parseColString(value,r,g,b,a);
-
-						RGBf newCol;
-						newCol.red=(float)r/255.0f;
-						newCol.green=(float)g/255.0f;
-						newCol.blue=(float)b/255.0f;
-
-						rng.setColour(ionID,newCol);
+						ColourRGBA rgba;
+						rgba.parse(value);
+						rng.setColour(ionID,rgba.toRGBAf().toRGBf());
 						needUpdate=true;
 						break;
 					}
@@ -1021,17 +1027,17 @@ bool RangeFileFilter::setProperty(unsigned int key,
 }
 
 
-std::string  RangeFileFilter::getErrString(unsigned int code) const
+std::string  RangeFileFilter::getSpecificErrString(unsigned int code) const
 {
-	switch(code)
-	{
-		case RANGEFILE_ABORT_FAIL:
-			return std::string(TRANS("Ranging aborted by user"));
-		case RANGEFILE_BAD_ALLOC:
-			return std::string(TRANS("Insufficient memory for range"));
-	}
+	const char *errStrs[] ={ "",
+		"Ranging aborted by user",
+		"Insufficient memory for range",
+	};
 
-	return std::string("BUG(range file filter): Shouldn't see this!");
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(errStrs) == RANGEFILE_ERR_ENUM_END);
+	ASSERT(code < RANGEFILE_ERR_ENUM_END);
+
+	return errStrs[code];
 }
 
 void RangeFileFilter::setPropFromBinding(const SelectionBinding &b)
@@ -1057,18 +1063,17 @@ bool RangeFileFilter::writeState(std::ostream &f,unsigned int format, unsigned i
 			f << tabs(depth+1) << "<userstring value=\""<< escapeXML(userString) << "\"/>"  << endl;
 
 			f << tabs(depth+1) << "<file name=\""<< escapeXML(convertFileStringToCanonical(rngName)) << "\"/>"  << endl;
+			f << tabs(depth+1) << "<legend enabled=\""<< boolStrEnc(showLegend) << "\"/>"  << endl;
 			f << tabs(depth+1) << "<dropunranged value=\""<<(int)dropUnranged<< "\"/>"  << endl;
 			f << tabs(depth+1) << "<enabledions>"<< endl;
 			for(unsigned int ui=0;ui<enabledIons.size();ui++)
 			{
-				RGBf col;
-				string colourString;
+				ColourRGBAf col;
 				col = rng.getColour(ui);
 
-				genColString((unsigned char)(col.red*255),(unsigned char)(col.green*255),
-						(unsigned char)(col.blue*255),255,colourString);
+				
 				f<< tabs(depth+2) << "<ion id=\"" << ui << "\" enabled=\"" 
-					<< (int)enabledIons[ui] << "\" colour=\"" << colourString << "\"/>" << endl;
+					<< (int)enabledIons[ui] << "\" colour=\"" << col.toColourRGBA().rgbString()<< "\"/>" << endl;
 			}
 			f << tabs(depth+1) << "</enabledions>"<< endl;
 
@@ -1118,7 +1123,7 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	rngName=(char *)xmlString;
 	xmlFree(xmlString);
 
-	//Override the string, as needed
+	//Override the string to strip leading ./ notation, as needed
 	if( (stateFileDir.size()) &&
 		(rngName.size() > 2 && rngName.substr(0,2) == "./") )
 	{
@@ -1132,18 +1137,28 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 		return false;
 		
 	//==
-	
+
+
+	//TODO: Deprecate me. Did not exist prior to 0.0.17/
+	// internal 3e88134daeea
+	xmlNodePtr tmpNode=nodePtr;
+	if(!XMLHelpFwdToElem(tmpNode,"legend"))
+	{
+		if(XMLHelpGetProp(showLegend,tmpNode,"enabled"))
+		{
+			return false;
+		}
+	}
+	else
+		showLegend=false;
+
 	std::string tmpStr;
-	//Retrieve user string
+	//Retrieve range status 
 	//==
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"dropunranged","value"))
 		return false;
-	
-	if(tmpStr=="1")
-		dropUnranged=true;
-	else if(tmpStr=="0")
-		dropUnranged=false;
-	else
+
+	if(!boolStrDec(tmpStr,dropUnranged))
 		return false;
 
 	//==
@@ -1153,7 +1168,7 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	//===
 	if(XMLHelpFwdToElem(nodePtr,"enabledions"))
 		return false;
-	xmlNodePtr tmpNode=nodePtr;
+	tmpNode=nodePtr;
 
 	nodePtr=nodePtr->xmlChildrenNode;
 
@@ -1161,7 +1176,7 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	bool enabled;
 	//By default, turn ions off, but use state file to turn them on
 	map<unsigned int ,char> tmpEnabledIons;
-	map<unsigned int, RGBf> tmpCol;
+	map<unsigned int ,RGBf> tmpCol;
 	while(!XMLHelpFwdToElem(nodePtr,"ion"))
 	{
 		//Get ID value
@@ -1183,13 +1198,8 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 			return false;
 		tmpStr=(char *)xmlString;
 
-		if(tmpStr == "0")
-			enabled=false;
-		else if(tmpStr == "1")
-			enabled=true;
-		else
+		if(!boolStrDec(tmpStr,enabled))
 			return false;
-
 		tmpEnabledIons[ionID]=enabled;
 		xmlFree(xmlString);
 		
@@ -1201,15 +1211,11 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 
 		tmpStr=(char *)xmlString;
 
-		unsigned char r,g,b,a;
-		if(!parseColString(tmpStr,r,g,b,a))
+		ColourRGBA rgbaTmp;
+		if(!rgbaTmp.parse(tmpStr))
 			return false;
 		
-		RGBf col;
-		col.red=(float)r/255.0f;
-		col.green=(float)g/255.0f;
-		col.blue=(float)b/255.0f;
-		tmpCol[ionID]=col;	
+		tmpCol[ionID]=rgbaTmp.toRGBAf().toRGBf();	
 		xmlFree(xmlString);
 	}
 
@@ -1250,11 +1256,7 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 			return false;
 		tmpStr=(char *)xmlString;
 
-		if(tmpStr == "0")
-			enabled=false;
-		else if(tmpStr == "1")
-			enabled=true;
-		else
+		if(!boolStrDec(tmpStr,enabled))
 			return false;
 
 		xmlFree(xmlString);
@@ -1271,7 +1273,11 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	if(tmpEnabledIons.size() == rng.getNumIons())
 	{
 		for(size_t ui=0;ui<tmpEnabledIons.size(); ui++)
+		{
 			enabledIons[ui] = tmpEnabledIons[ui];
+			rng.setColour(ui,tmpCol[ui]);
+			
+		}
 		
 		for(size_t ui=0;ui<tmpEnabledRanges.size(); ui++)
 			enabledRanges[ui] = tmpEnabledRanges[ui];
@@ -1291,7 +1297,10 @@ unsigned int RangeFileFilter::getRefreshBlockMask() const
 
 unsigned int RangeFileFilter::getRefreshEmitMask() const
 {
-	return STREAM_TYPE_RANGE | STREAM_TYPE_IONS ;
+	unsigned int retmask=  STREAM_TYPE_RANGE | STREAM_TYPE_IONS;
+	if(showLegend)
+		retmask |= STREAM_TYPE_DRAW;
+	return retmask;
 }
 
 unsigned int RangeFileFilter::getRefreshUseMask() const
@@ -1347,6 +1356,31 @@ bool RangeFileFilter::writePackageState(std::ostream &f, unsigned int format,
 	return result;
 }
 
+DrawStreamData *RangeFileFilter::createLegend() const
+{
+	//Create a legend bar, which shows the ions that are present
+	DrawStreamData *dS = new DrawStreamData;
+
+	dS->parent=this;
+
+	DrawPointLegendOverlay *dl=new DrawPointLegendOverlay;
+	dl->setPosition(0.1,0.1);
+
+	for(unsigned int ui=0;ui<enabledIons.size();ui++)
+	{	
+		if(!enabledIons[ui])
+			continue;
+
+		RGBf curRGBf;
+		curRGBf =rng.getColour(ui);
+		dl->addItem(rng.getName(ui),
+			curRGBf.red, curRGBf.green,curRGBf.blue);
+	}
+
+	dS->drawables.push_back(dl);
+
+	return dS;
+}
 #ifdef DEBUG
 
 bool testRanged();
@@ -1413,7 +1447,7 @@ bool testRanged()
 
 	//Run the initialisation stage
 	ProgressData prog;
-	TEST(!r->refresh(streamIn,streamOut,prog,dummyCallback),"Refresh error code");
+	TEST(!r->refresh(streamIn,streamOut,prog),"Refresh error code");
 	//--
 	
 	//Run the tests
