@@ -1,6 +1,6 @@
 /*
  *	transform.cpp - Perform geometrical transform operations on point clouds
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,7 +19,10 @@
 
 #include "filterCommon.h"
 
-
+using std::vector;
+using std::string;
+using std::pair;
+using std::make_pair;
 
 
 enum
@@ -70,8 +73,9 @@ enum
 //!Error codes
 enum
 {
-	ERR_CALLBACK_FAIL=1,
-	ERR_NOMEM
+	
+	ERR_NOMEM=1,
+	TRANSFORM_ERR_ENUM_END
 };
 
 const char *TRANSFORM_MODE_STRING[] = { NTRANS("Translate"),
@@ -128,7 +132,7 @@ Filter *TransformFilter::cloneUncached() const
 	p->transformMode=transformMode;
 	p->showOrigin=showOrigin;
 	p->noiseType=noiseType;
-	//We are copying wether to cache or not,
+	//We are copying whether to cache or not,
 	//not the cache itself
 	p->cache=cache;
 	p->cacheOK=false;
@@ -184,7 +188,7 @@ DrawStreamData* TransformFilter::makeMarkerSphere(SelectionDevice* &s) const
 }
 
 unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+	std::vector<const FilterStreamData *> &getOut, ProgressData &progress)
 {
 	//use the cached copy if we have it.
 	if(cacheOK)
@@ -273,13 +277,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		DrawStreamData *d=makeMarkerSphere(s);
 		if(s)
 			devices.push_back(s);
-		if(cache)
-		{
-			d->cached=1;
-			filterOutputs.push_back(d);
-		}
-		else
-			d->cached=0;
+
+		cacheAsNeeded(d);
 		
 		getOut.push_back(d);
 	}
@@ -305,8 +304,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 
 	if( transformMode != MODE_VALUE_SHUFFLE)
 	{
-		//Dont cross the streams. Why? It would be bad.
-		//  - Im fuzzy on the whole good-bad thing, what do you mean bad?"
+		//Don't cross the streams. Why? It would be bad.
+		//  - I'm fuzzy on the whole good-bad thing, what do you mean bad?"
 		//  - Every ion in the data body can be operated on independently.
 		//
 		//  OK, important safety tip.
@@ -350,19 +349,19 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							d->valueType=src->valueType;
 
 							ASSERT(src->data.size() <= totalSize);
-							unsigned int curProg=NUM_CALLBACK;
 #ifdef _OPENMP
+							unsigned int curProg=PROGRESS_REDUCE;
 							//Parallel version
 							bool spin=false;
-							#pragma omp parallel for shared(spin)
+							#pragma omp parallel for shared(spin,n)
 							for(unsigned int ui=0;ui<src->data.size();ui++)
 							{
-								unsigned int thisT=omp_get_thread_num();
 								if(spin)
 									continue;
 
 								if(!curProg--)
 								{
+									unsigned int thisT=omp_get_thread_num();
 									#pragma omp critical
 									{
 									n+=NUM_CALLBACK;
@@ -372,7 +371,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 
 									if(thisT == 0)
 									{
-										if(!(*callback)(false))
+										if(*Filter::wantAbort)
 											spin=true;
 									}
 								}
@@ -385,7 +384,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							if(spin)
 							{			
 								delete d;
-								return ERR_CALLBACK_FAIL;
+								return FILTER_ERR_ABORT;
 							}
 
 #else
@@ -398,17 +397,12 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								//set the position for the given ion
 								d->data[pos].setPos((it->getPosRef() - origin)*scaleFactor+origin);
 								d->data[pos].setMassToCharge(it->getMassToCharge());
-								//update progress every CALLBACK ions
-								if(!curProg--)
+								//update progress 
+								progress.filterProgress= (unsigned int)((float)(n++)/((float)totalSize)*100.0f);
+								if(*Filter::wantAbort)
 								{
-									n+=NUM_CALLBACK;
-									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-									if(!(*callback)(false))
-									{
-										delete d;
-										return ERR_CALLBACK_FAIL;
-									}
-									curProg=NUM_CALLBACK;
+									delete d;
+									return FILTER_ERR_ABORT;
 								}
 								pos++;
 							}
@@ -417,14 +411,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 #endif
 							ASSERT(d->data.size() == src->data.size());
 
-							if(cache)
-							{
-								d->cached=1;
-								filterOutputs.push_back(d);
-								cacheOK=true;
-							}
-							else
-								d->cached=0;
+							cacheAsNeeded(d);
 
 							getOut.push_back(d);
 							break;
@@ -471,8 +458,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							d->valueType=src->valueType;
 
 							ASSERT(src->data.size() <= totalSize);
-							unsigned int curProg=NUM_CALLBACK;
 #ifdef _OPENMP
+							unsigned int curProg=PROGRESS_REDUCE;
 							//Parallel version
 							bool spin=false;
 							#pragma omp parallel for shared(spin)
@@ -493,7 +480,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 
 									if(thisT == 0)
 									{
-										if(!(*callback)(false))
+										if(*Filter::wantAbort)
 											spin=true;
 									}
 								}
@@ -506,7 +493,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							if(spin)
 							{			
 								delete d;
-								return ERR_CALLBACK_FAIL;
+								return FILTER_ERR_ABORT;
 							}
 
 #else
@@ -519,17 +506,11 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								//set the position for the given ion
 								d->data[pos].setPos((it->getPosRef() - origin)*transformVec+origin);
 								d->data[pos].setMassToCharge(it->getMassToCharge());
-								//update progress every CALLBACK ions
-								if(!curProg--)
+								progress.filterProgress= (unsigned int)((float)(n++)/((float)totalSize)*100.0f);
+								if(*Filter::wantAbort)
 								{
-									n+=NUM_CALLBACK;
-									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-									if(!(*callback)(false))
-									{
-										delete d;
-										return ERR_CALLBACK_FAIL;
-									}
-									curProg=NUM_CALLBACK;
+									delete d;
+									return FILTER_ERR_ABORT;
 								}
 								pos++;
 							}
@@ -538,14 +519,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 #endif
 							ASSERT(d->data.size() == src->data.size());
 
-							if(cache)
-							{
-								d->cached=1;
-								filterOutputs.push_back(d);
-								cacheOK=true;
-							}
-							else
-								d->cached=0;
+							cacheAsNeeded(d);
 
 							getOut.push_back(d);
 							break;
@@ -594,14 +568,13 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							d->valueType=src->valueType;
 							
 							ASSERT(src->data.size() <= totalSize);
-							unsigned int curProg=NUM_CALLBACK;
 #ifdef _OPENMP
 							//Parallel version
+							unsigned int curProg=PROGRESS_REDUCE;
 							bool spin=false;
 #pragma omp parallel for shared(spin)
 							for(unsigned int ui=0;ui<src->data.size();ui++)
 							{
-								unsigned int thisT=omp_get_thread_num();
 								if(spin)
 									continue;
 								
@@ -609,14 +582,14 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								{
 #pragma omp critical
 									{
-										n+=NUM_CALLBACK;
+										n+=PROGRESS_REDUCE;
 										progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
 									}
 									
 									
-									if(thisT == 0)
+									if(omp_get_thread_num() == 0)
 									{
-										if(!(*callback)(false))
+										if(*Filter::wantAbort)
 											spin=true;
 									}
 								}
@@ -629,7 +602,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							if(spin)
 							{			
 								delete d;
-								return ERR_CALLBACK_FAIL;
+								return FILTER_ERR_ABORT;
 							}
 							
 #else
@@ -643,30 +616,19 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								d->data[pos].setPos((it->getPosRef() - origin));
 								d->data[pos].setMassToCharge(it->getMassToCharge());
 								//update progress every CALLBACK ions
-								if(!curProg--)
+								progress.filterProgress= (unsigned int)((float)(n++)/((float)totalSize)*100.0f);
+								if(*Filter::wantAbort)
 								{
-									n+=NUM_CALLBACK;
-									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-									if(!(*callback)(false))
-									{
-										delete d;
-										return ERR_CALLBACK_FAIL;
-									}
-									curProg=NUM_CALLBACK;
+									delete d;
+									return FILTER_ERR_ABORT;
 								}
 								pos++;
 							}
 							ASSERT(pos == d->data.size());
 #endif
 							ASSERT(d->data.size() == src->data.size());
-							if(cache)
-							{
-								d->cached=1;
-								filterOutputs.push_back(d);
-								cacheOK=true;
-							}
-							else
-								d->cached=0;
+
+							cacheAsNeeded(d);
 							
 							getOut.push_back(d);
 							break;
@@ -733,7 +695,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 									
 									if(thisT == 0)
 									{
-										if(!(*callback)(false))
+										if(*Filter::wantAbort)
 											spin=true;
 									}
 								}
@@ -746,7 +708,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							if(spin)
 							{			
 								delete d;
-								return ERR_CALLBACK_FAIL;
+								return FILTER_ERR_ABORT;
 							}
 							
 #else
@@ -764,10 +726,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								{
 									n+=NUM_CALLBACK;
 									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-									if(!(*callback)(false))
+									if(*Filter::wantAbort)
 									{
 										delete d;
-										return ERR_CALLBACK_FAIL;
+										return FILTER_ERR_ABORT;
 									}
 									curProg=NUM_CALLBACK;
 								}
@@ -776,14 +738,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							ASSERT(pos == d->data.size());
 #endif
 							ASSERT(d->data.size() == src->data.size());
-							if(cache)
-							{
-								d->cached=1;
-								filterOutputs.push_back(d);
-								cacheOK=true;
-							}
-							else
-								d->cached=0;
+
+							cacheAsNeeded(d);
 							
 							getOut.push_back(d);
 							break;
@@ -865,10 +821,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								{
 									n+=NUM_CALLBACK;
 									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-									if(!(*callback)(false))
+									if(*Filter::wantAbort)
 									{
 										delete d;
-										return ERR_CALLBACK_FAIL;
+										return FILTER_ERR_ABORT;
 									}
 									curProg=NUM_CALLBACK;
 								}
@@ -876,14 +832,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							}
 
 							ASSERT(d->data.size() == src->data.size());
-							if(cache)
-							{
-								d->cached=1;
-								filterOutputs.push_back(d);
-								cacheOK=true;
-							}
-							else
-								d->cached=0;
+
+							cacheAsNeeded(d);
 							
 							getOut.push_back(d);
 							break;
@@ -962,10 +912,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 										{
 											curProg=NUM_CALLBACK;
 											progress.filterProgress= (unsigned int)((float)(ui)/((float)totalSize)*100.0f);
-											if(!(*callback)(false))
+											if(*Filter::wantAbort)
 											{
 												delete d;
-												return ERR_CALLBACK_FAIL;
+												return FILTER_ERR_ABORT;
 											}
 										}
 									}
@@ -992,10 +942,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 										{
 											curProg=NUM_CALLBACK;
 											progress.filterProgress= (unsigned int)((float)(ui)/((float)totalSize)*100.0f);
-											if(!(*callback)(false))
+											if(*Filter::wantAbort)
 											{
 												delete d;
-												return ERR_CALLBACK_FAIL;
+												return FILTER_ERR_ABORT;
 											}
 										}
 									}
@@ -1005,14 +955,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							}
 							
 							ASSERT(d->data.size() == src->data.size());
-							if(cache)
-							{
-								d->cached=1;
-								filterOutputs.push_back(d);
-								cacheOK=true;
-							}
-							else
-								d->cached=0;
+
+							cacheAsNeeded(d);
 							
 							getOut.push_back(d);
 							break;
@@ -1032,8 +976,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		progress.filterProgress=0;
 		progress.stepName=TRANS("Collate");
 		progress.maxStep=3;
-		if(!(*callback)(true))
-			return ERR_CALLBACK_FAIL;
+		if(*Filter::wantAbort)
+			return FILTER_ERR_ABORT;
 		//we have to cross the streams (I thought that was bad?) 
 		//  - Each dataset is no longer independent, and needs to
 		//  be mixed with the other datasets. Bugger; sounds mem. expensive.
@@ -1085,10 +1029,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 						d->data[uj+curPos].setPos(src->data[uj].getPos());
 					}
 				
-					if(!(*callback)(true))
+					if(*Filter::wantAbort)
 					{
 						delete d;
-						return ERR_CALLBACK_FAIL;
+						return FILTER_ERR_ABORT;
 					}
 
 					curPos+=src->data.size();
@@ -1104,10 +1048,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		progress.step=2;
 		progress.filterProgress=0;
 		progress.stepName=TRANS("Shuffle");
-		if(!(*callback)(true))
+		if(*Filter::wantAbort)
 		{
 			delete d;
-			return ERR_CALLBACK_FAIL;
+			return FILTER_ERR_ABORT;
 		}
 		//Shuffle the value data.TODO: callback functor	
 
@@ -1119,19 +1063,19 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		r.seed(time(0));
 		std::shuffle(massData.begin(),massData.end(),r);
 #endif
-		if(!(*callback)(true))
+		if(*Filter::wantAbort)
 		{
 			delete d;
-			return ERR_CALLBACK_FAIL;
+			return FILTER_ERR_ABORT;
 		}
 
 		progress.step=3;
 		progress.filterProgress=0;
 		progress.stepName=TRANS("Splice");
-		if(!(*callback)(true))
+		if(*Filter::wantAbort)
 		{
 			delete d;
-			return ERR_CALLBACK_FAIL;
+			return FILTER_ERR_ABORT;
 		}
 		
 		
@@ -1142,23 +1086,16 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		for(size_t uj=0;uj<totalSize;uj++)
 			d->data[uj].setMassToCharge(massData[uj]);
 		
-		if(!(*callback)(true))
+		if(*Filter::wantAbort)
 		{
 			delete d;
-			return ERR_CALLBACK_FAIL;
+			return FILTER_ERR_ABORT;
 		}
 
 		massData.clear();
 
-		if(cache)
-		{
-			d->cached=1;
-			filterOutputs.push_back(d);
-			cacheOK=true;
-		}
-		else
-			d->cached=0;
-
+		cacheAsNeeded(d);
+		
 		getOut.push_back(d);
 		
 	}
@@ -1186,6 +1123,7 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 	p.key=KEY_MODE;
 	propertyList.addProperty(p,curGroup);
 	
+	propertyList.setGroupTitle(curGroup,TRANS("Algorithm"));
 	curGroup++;	
 	
 	//non-translation transforms require a user to select an origin	
@@ -1444,55 +1382,27 @@ bool TransformFilter::setProperty(  unsigned int key,
 			break;
 		}
 		//The rotation angle, and the scale factor are both stored
-		//in scalaraparams[0]. All we need ot do is set that,
+		//in scalaraparams[0]. All we need to do is set that,
 		//as either can take any valid floating pt value
 		case KEY_ROTATE_ANGLE:
 		case KEY_SCALEFACTOR:
 		case KEY_NOISELEVEL:
 		case KEY_ORIGIN_VALUE:
 		{
-			ASSERT(scalarParams.size());
-
-			float newScale;
-			if(stream_cast(newScale,value))
+			if(!applyPropertyNow(scalarParams[0],value,needUpdate))
 				return false;
-
-			if(scalarParams[0] != newScale )
-			{
-				scalarParams[0] = newScale;
-				needUpdate=true;
-				clearCache();
-			}
 			return true;
 		}
 		case KEY_SCALEFACTOR_ANISOTROPIC:
 		{
-			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!applyPropertyNow(vectorParams[1],value,needUpdate))
 				return false;
-
-			if(!(vectorParams[1] == newPt ))
-			{
-				vectorParams[1] = newPt;
-				needUpdate=true;
-				clearCache();
-			}
-
 			return true;
 		}
 		case KEY_ORIGIN:
 		{
-			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!applyPropertyNow(vectorParams[0],value,needUpdate))
 				return false;
-
-			if(!(vectorParams[0] == newPt ))
-			{
-				vectorParams[0] = newPt;
-				needUpdate=true;
-				clearCache();
-			}
-
 			return true;
 		}
 		case KEY_ROTATE_AXIS:
@@ -1534,15 +1444,8 @@ bool TransformFilter::setProperty(  unsigned int key,
 		}
 		case KEY_TRANSFORM_SHOWORIGIN:
 		{
-			string stripped=stripWhite(value);
-
-			if(!(stripped == "1"|| stripped == "0"))
+			if(!applyPropertyNow(showOrigin,value,needUpdate))
 				return false;
-
-			showOrigin=(stripped=="1");
-
-			needUpdate=true;
-
 			break;
 		}
 		case KEY_NOISETYPE:
@@ -1569,19 +1472,15 @@ bool TransformFilter::setProperty(  unsigned int key,
 }
 
 
-std::string  TransformFilter::getErrString(unsigned int code) const
+std::string  TransformFilter::getSpecificErrString(unsigned int code) const
 {
+	const char *errStrs[] = { "",
+		"Unable to allocate memory"//Caught a memory issue,
+	};
 
-	switch(code)
-	{
-		//User aborted in a callback
-		case ERR_CALLBACK_FAIL:
-			return std::string(TRANS("Aborted"));
-		//Caught a memory issue
-		case ERR_NOMEM:
-			return std::string(TRANS("Unable to allocate memory"));
-	}
-	ASSERT(false);
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(errStrs) == TRANSFORM_ERR_ENUM_END);
+	ASSERT(code < TRANSFORM_ERR_ENUM_END);
+	return errStrs[code]; 
 }
 
 bool TransformFilter::writeState(std::ostream &f,unsigned int format, unsigned int depth) const
@@ -1887,7 +1786,7 @@ bool rotateTest()
 	//OK, so now do the rotation
 	//Do the refresh
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"refresh error code");
 	delete f;
 
 	TEST(streamOut.size() == 1,"stream count");
@@ -1907,7 +1806,7 @@ bool rotateTest()
 
 
 	TEST((massCentre[0]-massCentre[1]).sqrMag() < 
-			2.0*sqrt(std::numeric_limits<float>::epsilon()),"mass centre invariance");
+			2.0*sqrtf(std::numeric_limits<float>::epsilon()),"mass centre invariance");
 
 	//Rotating a sphere around its centre of mass
 	// should not massively change the bounding box
@@ -1969,7 +1868,7 @@ bool translateTest()
 	
 	//Do the refresh
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 	delete f;
 	
 	TEST(streamOut.size() == 1,"stream count");
@@ -1989,7 +1888,7 @@ bool translateTest()
 		{
 			float f;
 			f=bc[0].getBound(ui,uj) -bc[1].getBound(ui,uj);
-			TEST(fabs(f-offsetPt[ui]) < sqrt(std::numeric_limits<float>::epsilon()), "bound translation");
+			TEST(fabs(f-offsetPt[ui]) < sqrtf(std::numeric_limits<float>::epsilon()), "bound translation");
 		}
 	}
 
@@ -2049,7 +1948,7 @@ bool scaleTest()
 
 	//Do the refresh
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"refresh error code");
 	delete f;
 
 	TEST(streamOut.size() == 1,"stream count");
@@ -2069,7 +1968,7 @@ bool scaleTest()
 	float volumeDelta;
 	volumeDelta=fabs(bc[1].volume()/cubeOfScale - bc[0].volume() );
 
-	TEST(volumeDelta < 100.0f*sqrt(std::numeric_limits<float>::epsilon()), "scaled volume test");
+	TEST(volumeDelta < 100.0f*sqrtf(std::numeric_limits<float>::epsilon()), "scaled volume test");
 
 	delete streamOut[0];
 	delete d;
@@ -2122,7 +2021,7 @@ bool scaleAnisoTest()
 
 	//Do the refresh
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"refresh error code");
 	delete f;
 
 	TEST(streamOut.size() == 1,"stream count");
@@ -2164,7 +2063,7 @@ bool shuffleTest()
 	//OK, so now run the shuffle 
 	//Do the refresh
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"refresh error code");
 	delete f;
 
 	TEST(streamOut.size() == 1,"stream count");
