@@ -1,6 +1,6 @@
 /*
  *	voxelise.cpp - Compute 3D binning (voxelisation) of point clouds
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -50,8 +50,8 @@ enum
 	KEY_VOXEL_SLICE_INTERP,
 	
 	KEY_FILTER_MODE,
-	KEY_FILTER_BOUNDARY_MODE,
-	KEY_FILTER_BINS,
+	KEY_FILTER_RATIO,
+	KEY_FILTER_STDEV,
 	KEY_ENABLE_NUMERATOR,
 	KEY_ENABLE_DENOMINATOR
 };
@@ -71,6 +71,7 @@ enum
 {
 	VOXELISE_FILTERTYPE_NONE,
 	VOXELISE_FILTERTYPE_GAUSS,
+	VOXELISE_FILTERTYPE_LAPLACE,
 	VOXELISE_FILTERTYPE_MAX // keep this at the end so it's a bookend for the last value
 };
 
@@ -88,10 +89,11 @@ enum
 //--
 enum
 {
-	VOXELISE_ABORT_ERR,
+	VOXELISE_ABORT_ERR=1,
 	VOXELISE_MEMORY_ERR,
 	VOXELISE_CONVOLVE_ERR,
-	VOXELISE_BOUNDS_INVALID_ERR
+	VOXELISE_BOUNDS_INVALID_ERR,
+	VOXELISE_ERR_ENUM_END
 };
 //--
 
@@ -120,12 +122,8 @@ const char *REPRESENTATION_TYPE_STRING[] = {
 
 const char *VOXELISE_FILTER_TYPE_STRING[]={
 	NTRANS("None"),
-	NTRANS("Gaussian (2𝜎)"),
-	};
-
-const char *VOXELISE_FILTER_BOUND_STRING[] ={
-	NTRANS("Zero"),
-	NTRANS("Bounce")
+	NTRANS("Gaussian (blur)"),
+	NTRANS("Lapl. of Gauss. (edges)"),
 	};
 
 const char *VOXELISE_SLICE_INTERP_STRING[]={
@@ -135,7 +133,7 @@ const char *VOXELISE_SLICE_INTERP_STRING[]={
 
 //This is not a member of voxels.h, as the voxels do not have any concept of the IonHit
 int countPoints(Voxels<float> &v, const std::vector<IonHit> &points, 
-				bool noWrap,bool (*callback)(bool))
+				bool noWrap)
 {
 
 	size_t x,y,z;
@@ -147,7 +145,7 @@ int countPoints(Voxels<float> &v, const std::vector<IonHit> &points,
 	{
 		if(!downSample--)
 		{
-			if(!(*callback)(false))
+			if(*Filter::wantAbort)
 				return 1;
 			downSample=MAX_CALLBACK;
 		}
@@ -159,6 +157,7 @@ int countPoints(Voxels<float> &v, const std::vector<IonHit> &points,
 				float value;
 				value=v.getData(x,y,z)+1.0f;
 
+				ASSERT(value >= 0.0f);
 				//Prevent wrap-around errors
 				if (noWrap) {
 					if (value > v.getData(x,y,z))
@@ -178,19 +177,16 @@ VoxeliseFilter::VoxeliseFilter()
 {
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(NORMALISE_TYPE_STRING) ==  VOXELISE_NORMALISETYPE_MAX);
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(VOXELISE_FILTER_TYPE_STRING) == VOXELISE_FILTERTYPE_MAX );
-	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(VOXELISE_FILTER_BOUND_STRING) ==  VOXELISE_FILTERBOUNDMODE_MAX);
 
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(REPRESENTATION_TYPE_STRING) == VOXEL_REPRESENT_END);
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(VOXEL_REPRESENT_KEEPCACHE) == VOXEL_REPRESENT_END);
 
 	splatSize=1.0f;
-	a=0.9f;
-	r=g=b=0.5;
+	rgba=ColourRGBAf(0.5,0.5,0.5,0.9f);
 	isoLevel=0.5;
 	
-	filterBins=3;
+	filterRatio=3.0;
 	filterMode=VOXELISE_FILTERTYPE_NONE;
-	filterBoundaryMode=VOXELISE_FILTERBOUNDMODE_BOUNCE;
 	gaussDev=0.5;	
 	
 	representation=VOXEL_REPRESENT_POINTCLOUD;
@@ -213,7 +209,6 @@ VoxeliseFilter::VoxeliseFilter()
 	denominatorAll = true;
 
 	sliceInterpolate=SLICE_INTERP_NONE;
-	sliceBoundMode=BOUND_MIRROR;
 	sliceAxis=0;
 	sliceOffset=0.5;
 	showColourBar=false;
@@ -231,16 +226,12 @@ Filter *VoxeliseFilter::cloneUncached() const
 {
 	VoxeliseFilter *p=new VoxeliseFilter();
 	p->splatSize=splatSize;
-	p->a=a;
-	p->r=r;
-	p->g=g;
-	p->b=b;
+	p->rgba=rgba;
 
 	p->isoLevel=isoLevel;
 	
 	p->filterMode=filterMode;
-	p->filterBoundaryMode=filterBoundaryMode;
-	p->filterBins=filterBins;
+	p->filterRatio=filterRatio;
 	p->gaussDev=gaussDev;
 
 	p->representation=representation;
@@ -282,7 +273,6 @@ Filter *VoxeliseFilter::cloneUncached() const
 	p->colourMapBounds[1] = colourMapBounds[1];
 
 	p->sliceInterpolate = sliceInterpolate;
-	p->sliceBoundMode= sliceBoundMode;
 	p->sliceAxis = sliceAxis;
 	p->sliceOffset = sliceOffset;
 
@@ -381,7 +371,7 @@ void VoxeliseFilter::initFilter(const std::vector<const FilterStreamData *> &dat
 }
 
 unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-		  std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+		  std::vector<const FilterStreamData *> &getOut, ProgressData &progress)
 {
 	//Disallow copying of anything in the blockmask. Copy everything else
 	propagateStreams(dataIn,getOut,getRefreshBlockMask(),true);
@@ -431,9 +421,8 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 		if(minP == maxP)
 			return 0;
 	
-		//Rebuild the voxels from the point dta
+		//Rebuild the voxels from the point data
 		Voxels<float> vsDenom;
-		voxelData.setCallbackMethod(callback);
 		voxelData.init(nBins[0], nBins[1], nBins[2], bc);
 		voxelData.fill(0);
 		
@@ -441,7 +430,6 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 			normaliseType == VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL) {
 			//Check we actually have incoming data
 			ASSERT(rsdIncoming);
-			vsDenom.setCallbackMethod(callback);
 			vsDenom.init(nBins[0], nBins[1], nBins[2], bc);
 			vsDenom.fill(0);
 		}
@@ -475,7 +463,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 
 					if(thisIonEnabled)
 					{
-						countPoints(voxelData,is->data,true,callback);
+						countPoints(voxelData,is->data,true);
 					}
 				}
 			
@@ -495,14 +483,14 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 							thisIonEnabled=false;
 
 						if(thisIonEnabled)
-							countPoints(vsDenom,is->data,true,callback);
+							countPoints(vsDenom,is->data,true);
 					}
 				} else if (normaliseType == VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL)
 				{
-					countPoints(vsDenom,is->data,true,callback);
+					countPoints(vsDenom,is->data,true);
 				}
 
-				if(!(*callback)(false))
+				if(*Filter::wantAbort)
 					return VOXELISE_ABORT_ERR;
 			}
 		
@@ -523,9 +511,9 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 				{
 					is= (const IonStreamData *)dataIn[i];
 
-					countPoints(voxelData,is->data,true,callback);
+					countPoints(voxelData,is->data,true);
 					
-					if(!(*callback)(false))
+					if(*Filter::wantAbort)
 						return VOXELISE_ABORT_ERR;
 
 				}
@@ -545,41 +533,13 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 			case VOXELISE_FILTERTYPE_NONE:
 				break;
 			case VOXELISE_FILTERTYPE_GAUSS:
+			{	
+				voxelData.isotropicGaussianSmooth(gaussDev,filterRatio);
+				break;
+			}
+			case VOXELISE_FILTERTYPE_LAPLACE:
 			{
-				Voxels<float> kernel,res;
-
-				map<unsigned int, unsigned int> modeMap;
-
-
-				modeMap[VOXELISE_FILTERBOUNDMODE_ZERO]=BOUND_ZERO;
-				modeMap[VOXELISE_FILTERBOUNDMODE_BOUNCE]=BOUND_MIRROR;
-
-				//FIXME: This will be SLOW. need to use IIR or some other
-				//fast technique
-				
-				//Construct the gaussian convolution
-				kernel.setGaussianKernelCube(gaussDev,(float)filterBins,filterBins);
-				//Normalise the kernel
-				float sum;
-				sum=kernel.getSum();
-				kernel/=sum;
-
-				cerr << "Kernel (min/max):" << kernel.min() << "," << kernel.max() << endl;
-			
-				if(res.resize(voxelData))
-					return VOXELISE_MEMORY_ERR; 
-				
-
-				cerr << "Data:" << voxelData.min() << "," << voxelData.max() << endl;
-				//Gaussian kernel is separable (rank 1)
-				if(voxelData.convolve(kernel,res,BOUND_MIRROR))
-					return VOXELISE_CONVOLVE_ERR;
-				
-				cerr << "Result (min/max):" << res.min() << "," << res.max() << endl;
-
-				voxelData.swap(res);
-
-				res.clear();
+				voxelData.laplaceOfGaussian(gaussDev,filterRatio);
 				break;
 			}
 			default:
@@ -620,14 +580,14 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 		{
 			VoxelStreamData *vs = new VoxelStreamData();
 			vs->parent=this;
-			std::swap(vs->data,voxelData);
+			std::swap(*(vs->data),voxelData);
 			vs->representationType= representation;
 			vs->splatSize = splatSize;
 			vs->isoLevel=isoLevel;
-			vs->r=r;
-			vs->g=g;
-			vs->b=b;
-			vs->a=a;
+			vs->r=rgba.r();
+			vs->g=rgba.g();
+			vs->b=rgba.b();
+			vs->a=rgba.a();
 
 			if(cache)
 			{
@@ -654,6 +614,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 			getTexturedSlice(voxelData,sliceAxis,sliceOffset,
 						sliceInterpolate,minV,maxV,*dq);
 
+			dq->setColour(1,1,1,rgba.a());
 			dq->canSelect=true;
 
 
@@ -738,12 +699,6 @@ std::string VoxeliseFilter::getFilterTypeString(int type)
 }
 
 
-std::string VoxeliseFilter::getFilterBoundTypeString(int type) 
-{
-	ASSERT(type < VOXELISE_FILTERBOUNDMODE_MAX);
-	return std::string(TRANS(VOXELISE_FILTER_BOUND_STRING[type]));
-}
-
 void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 {
 	FilterProperty p;
@@ -814,6 +769,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 
 	//Let the user know what the valid values for voxel value types are
 	vector<pair<unsigned int,string> > choices;
+	unsigned int defaultChoice=normaliseType;
 	tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_NONE);
 	choices.push_back(make_pair((unsigned int)VOXELISE_NORMALISETYPE_NONE,tmpStr));
 	tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_VOLUME);
@@ -827,8 +783,16 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 		tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_COUNT2INVOXEL);
 		choices.push_back(make_pair((unsigned int)VOXELISE_NORMALISETYPE_COUNT2INVOXEL,tmpStr));
 	}
+	else
+	{
+		//prevent the case where we used to have an incoming range stream, but now we don't.
+		// selected item within choice string must still be valid
+		if(normaliseType > VOXELISE_NORMALISETYPE_VOLUME)
+			defaultChoice= VOXELISE_NORMALISETYPE_NONE;
+		
+	}
 
-	tmpStr= choiceString(choices,normaliseType);
+	tmpStr= choiceString(choices,defaultChoice);
 	p.name=TRANS("Normalise by");
 	p.data=tmpStr;
 	p.type=PROPERTY_TYPE_CHOICE;
@@ -843,7 +807,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 	if (rsdIncoming) 
 	{
 		p.name=TRANS("Numerator");
-		p.data=numeratorAll ? "1" : "0";
+		p.data=boolStrEnc(numeratorAll);
 		p.type=PROPERTY_TYPE_BOOL;
 		p.helpText=TRANS("Parmeter \"a\" used in fraction (a/b) to get voxel value");
 		p.key=KEY_ENABLE_NUMERATOR;
@@ -856,10 +820,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 		for(unsigned  int ui=0; ui<rsdIncoming->enabledIons.size(); ui++)
 		{
 			string str;
-			if(enabledIons[0][ui])
-				str="1";
-			else
-				str="0";
+			str=boolStrEnc(enabledIons[0][ui]);
 
 			//Append the ion name with a checkbox
 			p.name=rsdIncoming->rangeFile->getName(ui);
@@ -870,6 +831,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			propertyList.addProperty(p,curGroup);
 		}
 	
+		propertyList.setGroupTitle(curGroup,TRANS("Ranges"));
 		curGroup++;
 	}
 	
@@ -877,7 +839,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 	if (normaliseType == VOXELISE_NORMALISETYPE_COUNT2INVOXEL && rsdIncoming) 
 	{
 		p.name=TRANS("Denominator");
-		p.data=denominatorAll ? "1" : "0";
+		p.data=boolStrEnc(denominatorAll );
 		p.type=PROPERTY_TYPE_BOOL;
 		p.helpText=TRANS("Parameter \"b\" used in fraction (a/b) to get voxel value");
 		p.key=KEY_ENABLE_DENOMINATOR;
@@ -885,10 +847,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 		for(unsigned  int ui=0; ui<rsdIncoming->enabledIons.size(); ui++)
 		{			
 			string str;
-			if(enabledIons[1][ui])
-				str="1";
-			else
-				str="0";
+			str=boolStrEnc(enabledIons[1][ui]);
 
 			//Append the ion name with a checkbox
 			p.key=KEY_ENABLE_DENOMINATOR*1000 + ui;
@@ -899,6 +858,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 
 			propertyList.addProperty(p,curGroup);
 		}
+		propertyList.setGroupTitle(curGroup,TRANS("Mode"));
 		curGroup++;
 	}
 
@@ -906,7 +866,6 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 	//----
 	//TODO: Other filtering? threshold/median? laplacian? etc
 	
-/*
 	choices.clear();
 	//Post-filtering method
 	for(unsigned int ui=0;ui<VOXELISE_FILTERTYPE_MAX; ui++)
@@ -915,6 +874,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 		choices.push_back(make_pair(ui,tmpStr));
 	}
 	tmpStr= choiceString(choices,filterMode);
+	choices.clear();
 
 	p.name=TRANS("Filtering");
 	p.data=tmpStr;
@@ -928,33 +888,29 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 	{
 
 		//Filter size
-		stream_cast(tmpStr,filterBins);
-		p.name=TRANS("Kernel Bins");
+		stream_cast(tmpStr,gaussDev);
+		p.name=TRANS("Standard Dev");
 		p.data=tmpStr;
-		p.key=KEY_FILTER_BINS;
-		p.type=PROPERTY_TYPE_INTEGER;
-		p.helpText=TRANS("Number of bins in convolution kernel");
+		p.key=KEY_FILTER_STDEV;
+		p.type=PROPERTY_TYPE_REAL;
+		p.helpText=TRANS("Filtering Scale");
 		propertyList.addProperty(p,curGroup);
-		//Boundary wrapping mode selection
-		choices.clear();
-		for(unsigned int ui=0;ui<VOXELISE_FILTERBOUNDMODE_MAX; ui++)
-		{
-			tmpStr=getFilterBoundTypeString(ui);
-			choices.push_back(make_pair(ui,tmpStr));
-		}
+
 		
-		tmpStr= choiceString(choices,filterBoundaryMode);
-		p.name=TRANS("Exterior values");
+		//Filter size
+		stream_cast(tmpStr,filterRatio);
+		p.name=TRANS("Kernel Size");
 		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_CHOICE;
-		p.helpText=TRANS("Method to use to treat boundaries of voxel data for convolution");
-		p.key=KEY_FILTER_BOUNDARY_MODE;
+		p.key=KEY_FILTER_RATIO;
+		p.type=PROPERTY_TYPE_REAL;
+		p.helpText=TRANS("Filter radius, in multiples of std. dev. Larger -> slower, more accurate");
 		propertyList.addProperty(p,curGroup);
+
 	}
 	propertyList.setGroupTitle(curGroup,TRANS("Filtering"));
 	curGroup++;
 	//----
-*/
+
 	//start a new group for the visual representation
 	//----------------------------
 	choices.clear();
@@ -988,7 +944,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			p.key=KEY_SPOTSIZE;
 			propertyList.addProperty(p,curGroup);
 
-			stream_cast(tmpStr,1.0-a);
+			stream_cast(tmpStr,1.0-rgba.a());
 			p.name=TRANS("Transparency");
 			p.data=tmpStr;
 			p.type=PROPERTY_TYPE_REAL;
@@ -1012,21 +968,18 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			propertyList.addProperty(p,curGroup);
 		
 			//-- 
-				
+			propertyList.setGroupTitle(curGroup,TRANS("Surface"));	
 			curGroup++;
 
 			//-- Isosurface appearance --
-			//Convert the ion colour to a hex string	
-			genColString((unsigned char)(r*255),(unsigned char)(g*255),
-					(unsigned char)(b*255),(unsigned char)(a*255),tmpStr);
 			p.name=TRANS("Colour");
-			p.data=tmpStr;
+			p.data=rgba.toColourRGBA().rgbString();
 			p.type=PROPERTY_TYPE_COLOUR;
 			p.helpText=TRANS("Colour of isosurface");
 			p.key=KEY_COLOUR;
 			propertyList.addProperty(p,curGroup);
 
-			stream_cast(tmpStr,1.0-a);
+			stream_cast(tmpStr,1.0-rgba.a());
 			p.name=TRANS("Transparency");
 			p.data=tmpStr;
 			p.type=PROPERTY_TYPE_REAL;
@@ -1034,7 +987,6 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			p.key=KEY_TRANSPARENCY;
 			propertyList.addProperty(p,curGroup);
 			
-			propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
 			//----
 			
 			break;
@@ -1081,6 +1033,7 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			propertyList.addProperty(p,curGroup);
 			choices.clear();
 			// ---	
+			propertyList.setGroupTitle(curGroup,TRANS("Surface"));	
 			curGroup++;
 
 			
@@ -1097,22 +1050,23 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			p.helpText=TRANS("Colour scheme used to assign points colours by value");
 			p.key=KEY_VOXEL_COLOURMODE;
 			propertyList.addProperty(p,curGroup);
-
-			if(showColourBar)
-				tmpStr="1";
-			else
-				tmpStr="0";
 			
+			stream_cast(tmpStr,1.0-rgba.a());
+			p.name=TRANS("Transparency");
+			p.data=tmpStr;
+			p.type=PROPERTY_TYPE_REAL;
+			p.helpText=TRANS("How \"see through\" each facet is (0 - opaque, 1 - invisible)");
+			p.key=KEY_TRANSPARENCY;
+			propertyList.addProperty(p,curGroup);
+
+			tmpStr=boolStrEnc(showColourBar);
 			p.name=TRANS("Show Bar");
 			p.key=KEY_SHOW_COLOURBAR;
 			p.data=tmpStr;
 			p.type=PROPERTY_TYPE_BOOL;
 			propertyList.addProperty(p,curGroup);
 
-			if(autoColourMap)
-				tmpStr="1";
-			else
-				tmpStr="0";	
+			tmpStr=boolStrEnc(autoColourMap);
 			p.name=TRANS("Auto Bounds");
 			p.helpText=TRANS("Auto-compute min/max values in map"); 
 			p.data= tmpStr;
@@ -1147,6 +1101,8 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 			ASSERT(false);
 			;
 	}
+	
+	propertyList.setGroupTitle(curGroup,TRANS("Appearance"));	
 	curGroup++;
 
 	//----------------------------
@@ -1161,125 +1117,26 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 	{
 		case KEY_FIXEDWIDTH: 
 		{
-			bool b;
-			if(stream_cast(b,value))
+			if(!applyPropertyNow(fixedWidth,value,needUpdate))
 				return false;
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(b!=fixedWidth)
-			{
-				needUpdate=true;
-				fixedWidth=b;
-				clearCache();
-			}
 			break;
 		}	
 		case KEY_NBINSX:
-		{
-			 unsigned int i;
-			if(stream_cast(i,value))
-				return false;
-			if(!i)
-				return false;
-
-			//if the result is different, the
-			//cache should be invalidated
-			if(i!=nBins[0])
-			{
-				needUpdate=true;
-				nBins[0]=i;
-				calculateWidthsFromNumBins(binWidth, nBins);
-				clearCache();
-			}
-			break;
-		}
 		case KEY_NBINSY:
-		{
-			 unsigned int i;
-			if(stream_cast(i,value))
-				return false;
-			if(!i)
-				return false;
-			needUpdate=true;
-			//if the result is different, the
-			//cache should be invalidated
-			if(i!=nBins[1])
-			{
-				needUpdate=true;
-				nBins[1]=i;
-				calculateWidthsFromNumBins(binWidth, nBins);
-				clearCache();
-			}
-			break;
-		}
 		case KEY_NBINSZ:
 		{
-			 unsigned int i;
-			if(stream_cast(i,value))
+			if(!applyPropertyNow(nBins[key-KEY_NBINSX],value,needUpdate))
 				return false;
-			if(!i)
-				return false;
-			
-			//if the result is different, the
-			//cache should be invalidated
-			if(i!=nBins[2])
-			{
-				needUpdate=true;
-				nBins[2]=i;
-				calculateWidthsFromNumBins(binWidth, nBins);
-				clearCache();
-			}
+			calculateWidthsFromNumBins(binWidth, nBins);
 			break;
 		}
 		case KEY_WIDTHBINSX:
-		{
-			float f;
-			if(stream_cast(f,value))
-				return false;
-			if(f <= 0.0f)
-				return false;
-		
-			if(f!=binWidth[0])
-			{
-				needUpdate=true;
-				binWidth[0]=f;
-				calculateNumBinsFromWidths(binWidth, nBins);
-				clearCache();
-			}
-			break;
-		}
 		case KEY_WIDTHBINSY:
-		{
-			float f;
-			if(stream_cast(f,value))
-				return false;
-			if(f <= 0.0f)
-				return false;
-			
-			if(f!=binWidth[1])
-			{
-				needUpdate=true;
-				binWidth[1]=f;
-				calculateNumBinsFromWidths(binWidth, nBins);
-				clearCache();
-			}
-			break;
-		}
 		case KEY_WIDTHBINSZ:
 		{
-			float f;
-			if(stream_cast(f,value))
+			if(!applyPropertyNow(binWidth[key-KEY_WIDTHBINSX],value,needUpdate))
 				return false;
-			if(f <= 0.0f)
-				return false;
-			if(f!=binWidth[2])
-			{
-				needUpdate=true;
-				binWidth[2]=f;
-				calculateNumBinsFromWidths(binWidth, nBins);
-				clearCache();
-			}
+			calculateNumBinsFromWidths(binWidth, nBins);
 			break;
 		}
 		case KEY_NORMALISE_TYPE:
@@ -1334,7 +1191,7 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 				return false;
 			needUpdate=true;
 			//Alpha is opacity, which is 1-transparancy
-			a=1.0f-f;
+			rgba.a(1.0f-f);
 			//Go in and manually adjust the cached
 			//entries to have the new value, rather
 			//than doing a full recomputation
@@ -1344,7 +1201,7 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 				{
 					VoxelStreamData *d;
 					d=(VoxelStreamData*)filterOutputs[ui];
-					d->a=a;
+					d->a=rgba.a();
 				}
 			}
 			break;
@@ -1374,16 +1231,17 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 		}
 		case KEY_COLOUR:
 		{
-			unsigned char newR,newG,newB,newA;
+			ColourRGBA tmpRGBA;
 
-			parseColString(value,newR,newG,newB,newA);
+			if(!tmpRGBA.parse(value))
+				return false;
 
-			if(newB != b || newR != r ||
-				newG !=g || newA != a)
+			if(tmpRGBA.toRGBAf() != rgba)
+			{
+				rgba=tmpRGBA.toRGBAf();
 				needUpdate=true;
-			r=newR/255.0;
-			g=newG/255.0;
-			b=newB/255.0;
+			}
+
 			//Go in and manually adjust the cached
 			//entries to have the new value, rather
 			//than doing a full recomputation
@@ -1393,9 +1251,9 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 				{
 					VoxelStreamData *d;
 					d=(VoxelStreamData*)filterOutputs[ui];
-					d->r=r;
-					d->g=g;
-					d->b=b;
+					d->r=rgba.r();
+					d->g=rgba.g();
+					d->b=rgba.b();
 				}
 			}
 			break;
@@ -1477,35 +1335,34 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 			}
 			break;
 		}
-		case KEY_FILTER_BOUNDARY_MODE:
+		case KEY_FILTER_RATIO:
 		{
-			 unsigned int i;
-			for (i = 0; i < VOXELISE_FILTERBOUNDMODE_MAX; i++)
-				if (value == getFilterBoundTypeString(i)) break;
-			if (i == VOXELISE_FILTERTYPE_MAX)
+			float i;
+			if(stream_cast(i,value))
 				return false;
-			
-			if(i != filterBoundaryMode)
+			//forbid negative sizes
+			if(i <= 0)
+				return false;
+			if(i != filterRatio)
 			{
-				filterBoundaryMode=i;
 				needUpdate=true;
+				filterRatio=i;
 				clearCache();
 			}
 			break;
 		}
-		case KEY_FILTER_BINS:
+		case KEY_FILTER_STDEV:
 		{
-			 unsigned int i;
+			float i;
 			if(stream_cast(i,value))
 				return false;
-
-			//FIXME: Min restriction is artificial and imposed due to incomplete separable convolution filter implementation
-			if(i == 0 || i > std::min(nBins[0],std::min(nBins[1],nBins[2])))
+			//forbid negative sizes
+			if(i <= 0)
 				return false;
-			if(i != filterBins)
+			if(i != gaussDev)
 			{
 				needUpdate=true;
-				filterBins=i;
+				gaussDev=i;
 				clearCache();
 			}
 			break;
@@ -1513,7 +1370,7 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 		case KEY_VOXEL_SLICE_COLOURAUTO:
 		{
 			bool b;
-			if(stream_cast(b,value))
+			if(!boolStrDec(value,b))
 				return false;
 
 			//if the result is different, the
@@ -1615,7 +1472,7 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 		case KEY_SHOW_COLOURBAR:
 		{
 			bool b;
-			if(stream_cast(b,value))
+			if(!boolStrDec(value,b))
 				return false;
 
 			//if the result is different, the
@@ -1672,7 +1529,7 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 			// TODO: This is a bit of a hack.
 			if (key >= KEY_ENABLE_DENOMINATOR*1000) {
 				bool b;
-				if(stream_cast(b,value))
+				if(!boolStrDec(value,b))
 					return false;
 
 				enabledIons[1][key - KEY_ENABLE_DENOMINATOR*1000]=b;
@@ -1683,7 +1540,7 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 				clearCache();
 			} else if (key >= KEY_ENABLE_NUMERATOR*1000) {
 				bool b;
-				if(stream_cast(b,value))
+				if(!boolStrDec(value,b))
 					return false;
 				
 				enabledIons[0][key - KEY_ENABLE_NUMERATOR*1000]=b;
@@ -1703,21 +1560,19 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 	return true;
 }
 
-std::string  VoxeliseFilter::getErrString(unsigned int code) const
+std::string  VoxeliseFilter::getSpecificErrString(unsigned int code) const
 {
-	switch(code)
-	{
-		case VOXELISE_ABORT_ERR:
-			return std::string(TRANS("Voxelisation aborted"));
-		case VOXELISE_MEMORY_ERR:
-			return std::string(TRANS("Out of memory"));
-		case VOXELISE_CONVOLVE_ERR:
-			return std::string(TRANS("Unable to perform filter convolution"));
-		case VOXELISE_BOUNDS_INVALID_ERR:
-			return std::string(TRANS("Voxelisation bounds are invalid"));
-	}	
+	const char *errStrs[]={
+	 	"",
+		"Voxelisation aborted",
+		"Out of memory",
+		"Unable to perform filter convolution",
+		"Voxelisation bounds are invalid",
+	};
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(errStrs) == VOXELISE_ERR_ENUM_END);	
 	
-	return std::string("BUG! Should not see this (VoxeliseFilter)");
+	ASSERT(code < VOXELISE_ERR_ENUM_END);
+	return errStrs[code];
 }
 
 bool VoxeliseFilter::writeState(std::ostream &f,unsigned int format, unsigned int depth) const
@@ -1737,29 +1592,28 @@ bool VoxeliseFilter::writeState(std::ostream &f,unsigned int format, unsigned in
 
 			f << tabs(depth+2) << "<numerator>" << endl;
 			for(unsigned int ui=0;ui<enabledIons[0].size(); ui++)
-				f << tabs(depth+3) << "<enabled value=\"" << (enabledIons[0][ui]?1:0) << "\"/>" << endl;
+				f << tabs(depth+3) << "<enabled value=\"" << boolStrEnc(enabledIons[0][ui]) << "\"/>" << endl;
 			f << tabs(depth+2) << "</numerator>" << endl;
 
 			f << tabs(depth+2) << "<denominator>" << endl;
 			for(unsigned int ui=0;ui<enabledIons[1].size(); ui++)
-				f << tabs(depth+3) << "<enabled value=\"" << (enabledIons[1][ui]?1:0) << "\"/>" << endl;
+				f << tabs(depth+3) << "<enabled value=\"" << boolStrEnc(enabledIons[1][ui]) << "\"/>" << endl;
 			f << tabs(depth+2) << "</denominator>" << endl;
 
 			f << tabs(depth+1) << "</enabledions>" << endl;
 
 			f << tabs(depth+1) << "<representation value=\""<<representation << "\"/>" << endl;
 			f << tabs(depth+1) << "<isovalue value=\""<<isoLevel << "\"/>" << endl;
-			f << tabs(depth+1) << "<colour r=\"" <<  r<< "\" g=\"" << g << "\" b=\"" <<b
-				<< "\" a=\"" << a << "\"/>" <<endl;
+			f << tabs(depth+1) << "<colour r=\"" <<  rgba.r()<< "\" g=\"" << rgba.g() << "\" b=\"" <<rgba.b()
+				<< "\" a=\"" << rgba.a() << "\"/>" <<endl;
 
 			f << tabs(depth+1) << "<axialslice>" << endl;
 			f << tabs(depth+2) << "<offset value=\""<<sliceOffset<< "\"/>" << endl;
 			f << tabs(depth+2) << "<interpolate value=\""<<sliceInterpolate<< "\"/>" << endl;
 			f << tabs(depth+2) << "<axis value=\""<<sliceAxis<< "\"/>" << endl;
-			f << tabs(depth+2) << "<colourbar show=\""<<(showColourBar?1:0)<< 
-					"\" auto=\"" << (autoColourMap?1:0)<< "\" min=\"" <<
-					colourMapBounds[0] << "\" max=\"" << 
-					colourMapBounds[1] << "\"/>" << endl;
+			f << tabs(depth+2) << "<colourbar show=\""<<boolStrEnc(showColourBar)<< 
+						"\" auto=\"" << boolStrEnc(autoColourMap)<< "\" min=\"" <<
+					colourMapBounds[0] << "\" max=\"" <<  colourMapBounds[1] << "\"/>" << endl;
 			f << tabs(depth+1) << "</axialslice>" << endl;
 
 
@@ -1797,11 +1651,7 @@ bool VoxeliseFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	//Retrieve fixedWidth mode
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"fixedwidth","value"))
 		return false;
-	if(tmpStr == "1") 
-		fixedWidth=true;
-	else if(tmpStr== "0")
-		fixedWidth=false;
-	else
+	if(!boolStrDec(tmpStr,fixedWidth))
 		return false;
 	
 	//Retrieve nBins	
@@ -1942,8 +1792,10 @@ bool VoxeliseFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	//====
 	if(XMLHelpFwdToElem(nodePtr,"colour"))
 		return false;
-	if(!parseXMLColour(nodePtr,r,g,b,a))
+	ColourRGBAf tmpRgba;
+	if(!parseXMLColour(nodePtr,tmpRgba))
 		return false;
+	rgba=tmpRgba;
 
 	//====
 
@@ -2054,7 +1906,7 @@ void VoxeliseFilter::getTexturedSlice(const Voxels<float> &v,
 
 	ASSERT(offset >=0 && offset <=1.0f);
 
-	v.getSlice(axis,offset,data,interpolateMode,sliceBoundMode);
+	v.getInterpSlice(axis,offset,data,interpolateMode);
 
 	if(autoColourMap)
 	{
@@ -2114,8 +1966,6 @@ void VoxeliseFilter::getTexturedSlice(const Voxels<float> &v,
 	texQ.setVertices(verts);
 	//--
 
-	//Upload the texture to the video card
-	texQ.rebindTexture();
 
 }
 
@@ -2155,7 +2005,7 @@ bool voxelSingleCountTest()
 	streamIn.push_back(ionData);
 
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 	delete f;
 
 	TEST(streamOut.size() == 1,"stream count");
@@ -2164,16 +2014,16 @@ bool voxelSingleCountTest()
 
 	const VoxelStreamData *v= (const VoxelStreamData*)streamOut[0];
 
-	TEST(v->data.max() <=numIons,
+	TEST(v->data->max() <=numIons,
 			"voxel max less than input stream")
 
-	TEST(v->data.min() >= 0.0f,"voxel counting minimum sanity");
+	TEST(v->data->min() >= 0.0f,"voxel counting minimum sanity");
 
 	
 	float dataSum;
-	sumVoxels(v->data,dataSum);
+	sumVoxels(*(v->data),dataSum);
 	TEST(fabs(dataSum - (float)numIons ) < 
-		sqrt(std::numeric_limits<float>::epsilon()),"voxel counting all input ions ");
+		sqrtf(std::numeric_limits<float>::epsilon()),"voxel counting all input ions ");
 
 	delete ionData;
 	delete streamOut[0];
@@ -2202,6 +2052,10 @@ bool voxelMultiCountTest()
 
 	RGBf col; col.red=col.green=col.blue=1.0f;
 
+	//create several input ion streams, each
+	//containing the above data, but with differeing
+	//mass to charge values.
+	// - we range this data though!
 	const unsigned int MAX_NUM_RANGES=2;
 	for(unsigned int ui=0;ui<MAX_NUM_RANGES;ui++)
 	{
@@ -2249,7 +2103,7 @@ bool voxelMultiCountTest()
 				"Set normalise mode");
 
 	ProgressData p;
-	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"Refresh error code");
+	TEST(!f->refresh(streamIn,streamOut,p),"Refresh error code");
 	delete f;
 	for(unsigned int ui=0;ui<MAX_NUM_RANGES;ui++)
 		delete streamIn[ui];
@@ -2258,16 +2112,17 @@ bool voxelMultiCountTest()
 	
 	const VoxelStreamData *v= (const VoxelStreamData*)streamOut[1];
 
-	TEST(v->data.max() <=1.0f,
+	TEST(v->data->max() <=1.0f,
 			"voxel max less than input stream")
-	TEST(v->data.min() >= 0.0f,"voxel counting minimum sanity");
+	TEST(v->data->min() >= 0.0f,"voxel counting minimum sanity");
 
 
-	for(unsigned int ui=0;ui<v->data.getSize();ui++)
+	//all data should lie between 0 and 1
+	for(unsigned int ui=0;ui<v->data->getSize();ui++)
 	{
-		float delta;
-		delta=(v->data.getData(ui) - v->data.getData(0) );
-		ASSERT( v->data.getData(ui) == 0 || delta < std::numeric_limits<float>::epsilon());
+		float val;
+		val=v->data->getData(ui);
+		ASSERT(  val >= 0 && val <= 1.0f); 
 	}
 
 	delete v;
