@@ -1,6 +1,6 @@
 /*
  * 	filtertree.cpp - Filter tree topology and data propagation handling
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -32,6 +32,15 @@ enum
 	CACHE_NEVER,
 };
 
+//Unlock helper class for toggling a  boolean value at exit
+class AutoUnlocker
+{
+	private:
+		bool *lockBool;
+	public:
+		AutoUnlocker(bool *b) { lockBool= b; *lockBool=true;}
+		~AutoUnlocker() { *lockBool=false;}
+};
 
 
 
@@ -43,6 +52,9 @@ class FilterRefreshCollector
 		//Pile of lists of pointers that we are tracking
 		vector<list<const FilterStreamData *> > nodes;
 
+		//List of pointers we should *not* erase
+		set<const FilterStreamData *> forgottenNodes;
+
 		//Find out if a filter tracks itself or not
 		static bool tracksSelf(const FilterStreamData *p) { return p->cached;}
 
@@ -65,7 +77,10 @@ class FilterRefreshCollector
 		void collectAll();
 		unsigned int getLevel() const { return nodes.size();}
 		void collectToLevel(unsigned int level);
-	
+
+#ifdef DEBUG
+		bool  isTracked(const FilterStreamData *ptr) const;
+#endif	
 };
 
 #ifdef DEBUG
@@ -78,23 +93,32 @@ void FilterRefreshCollector::checkSanity()
 		for(list<const FilterStreamData *>::iterator it=nodes[ui].begin();
 				it!=nodes[ui].end(); ++it)
 		{
-			//Check that we hve not already inserted this
+			//Should never have something that tracks itself
+			ASSERT(!tracksSelf(*it) )
+			//Check that we have not already inserted this
 			ASSERT(s.find(*it) == s.end())
 			s.insert(*it);
+
+			ASSERT(forgottenNodes.find(*it) == forgottenNodes.end());
 		}
 	}
 	s.clear();
 
-	//Should never have something that tracks itself
-	for(size_t ui=0;ui<nodes.size();ui++)
+}
+
+bool FilterRefreshCollector::isTracked(const FilterStreamData *ptr) const
+{
+	for(unsigned int ui=0;ui<nodes.size();ui++)
 	{
-		for(list<const FilterStreamData *>::iterator it=nodes[ui].begin();
-				it!=nodes[ui].end(); ++it)
+		for(list<const FilterStreamData *>::const_iterator it=nodes[ui].begin();
+			it!=nodes[ui].end(); ++it)
 		{
-			ASSERT(!tracksSelf(*it) )
+			if(*it == ptr)
+				return true;
 		}
 	}
 
+	return false;
 }
 #endif
 
@@ -126,6 +150,8 @@ void FilterRefreshCollector::trackPointers(const vector<const FilterStreamData *
 
 			}
 
+			found|=(forgottenNodes.find(v[ui])  != forgottenNodes.end());
+
 			if(!found)
 				lKeep.push_back(v[ui]);
 		}
@@ -150,9 +176,10 @@ void FilterRefreshCollector::forgetPointers(const vector<const FilterStreamData
 			it=std::find(nodes[ui].begin(),nodes[ui].end(),v[uj]) ;
 			if(it != nodes[ui].end())
 			{
-				nodes[ui].erase(it);
+				forgottenNodes.insert(*it);
 				//We deleted the source of this, no need to continue
 				// checking for this particular pointer.
+				nodes[ui].erase(it);
 				break;
 			}
 		}
@@ -208,6 +235,7 @@ FilterTree::FilterTree()
 {
 	maxCachePercent=DEFAULT_MAX_CACHE_PERCENT;
 	cacheStrategy=CACHE_DEPTH_FIRST;
+	amRefreshing=false;
 }
 
 FilterTree::~FilterTree()
@@ -224,6 +252,8 @@ FilterTree::FilterTree(const FilterTree &orig) :
 	for(tree<Filter *>::pre_order_iterator it=filters.begin();
 		it!=filters.end();++it)
 		(*it)=(*it)->cloneUncached();
+	
+	amRefreshing=false;
 }
 
 size_t FilterTree::depth(const tree<Filter*>::pre_order_iterator &it) const
@@ -241,6 +271,7 @@ void FilterTree::swap(FilterTree &other)
 
 const FilterTree &FilterTree::operator=(const FilterTree &orig)
 {
+	ASSERT(!amRefreshing)
 	clear();
 
 	cacheStrategy=orig.cacheStrategy;
@@ -546,13 +577,64 @@ void FilterTree::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &propS
 
 
 
+}
+
+void FilterTree::getConsoleMessagesToNodes(std::vector<tree<Filter *>::iterator > &nodes,
+			std::vector<pair<const Filter*,string> > &messages) const
+{
+
+
+	//obtain a unique list of all filters who are parents of the nodes
+	if(!nodes.size())
+		return;
+
+	std::set<Filter*> filterSet;
+	for(unsigned int ui=0;ui<nodes.size();ui++)
+	{
+		tree<Filter*>::iterator it;
+		it=nodes[ui];
+
+		if(it==filters.begin())
+			filterSet.insert(*it);
+		
+		while(filters.is_valid(it) && 
+			filters.parent(it)!= filters.begin())
+		{
+			filterSet.insert(*it);	
+			it=filters.parent(it);
+		}
+	}
+
+	filterSet.insert(*(filters.begin()));
+
+
+	//now loop through the filters and obtain the console messages
+	for(set<Filter *>::iterator it=filterSet.begin();it!=filterSet.end();it++)
+	{
+		vector<string> tmpMsgs;
+		(*it)->getConsoleStrings(tmpMsgs);
+
+		for(size_t ui=0;ui<tmpMsgs.size();ui++)
+		{
+			messages.push_back(make_pair(*it,tmpMsgs[ui]));
+		}
+	}
 }
 
 unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData, 
 		std::vector<SelectionDevice *> &devices,
 		vector<pair<const Filter* , string> > &consoleMessages,
-	       	ProgressData &curProg, bool (*callback)(bool)) const
+	       	ProgressData &curProg, ATOMIC_BOOL &abortRefresh) const
 {
+
+	//initially, we should not want to abort refreshing 
+	ASSERT(!abortRefresh);
+	//Tell the filter system about our abort flag
+	Filter::wantAbort=&abortRefresh;
+
+
+	//Lock the refresh state.
+	AutoUnlocker unlocker(&amRefreshing);
 	
 	unsigned int errCode=0;
 
@@ -608,7 +690,6 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 	curProg.totalNumFilters=countChildFilters(filters,baseTreeNodes)+baseTreeNodes.size();
 
 
-	(*callback)(false);
 
 	for(unsigned int itPos=0;itPos<baseTreeNodes.size(); itPos++)
 	{
@@ -633,7 +714,7 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			//Step 0 : Pop the cache until we reach our current level, 
 			//	delete any pointers that would otherwise be lost.
 			//	Recall that the zero size in the stack may not correspond to the
-			//	tree root, but rather corresponds to our insertion level
+			//	tree root, but rather corresponds to the filter we started refreshing from
 			//---
 			size_t popLevel;
 			popLevel=filters.depth(filtIt) - filters.depth(baseTreeNodes[itPos])+1;
@@ -652,37 +733,39 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			//Step 2: Check if we should cache this filter or not.
 			//Get the number of bytes that the filter expects to use
 			//---
-			unsigned long long cacheBytes;
-			if(inDataStack.empty())
-				cacheBytes=currentFilter->numBytesForCache(0);
-			else
-				cacheBytes=currentFilter->numBytesForCache(numElements(inDataStack.top()));
-
-			if(cacheBytes != (unsigned long long)(-1))
+			if(!currentFilter->haveCache())
 			{
-				//As long as we have caching enabled, let us cache according to the
-				//selected strategy
-				switch(cacheStrategy)
+				unsigned long long cacheBytes;
+				if(inDataStack.empty())
+					cacheBytes=currentFilter->numBytesForCache(0);
+				else
+					cacheBytes=currentFilter->numBytesForCache(numElements(inDataStack.top()));
+
+				if(cacheBytes != (unsigned long long)(-1))
 				{
-					case CACHE_NEVER:
-						currentFilter->setCaching(false);
-						break;
-					case CACHE_DEPTH_FIRST:
+					//As long as we have caching enabled, let us cache according to the
+					//selected strategy
+					switch(cacheStrategy)
 					{
-						float ramFreeForUse;
-						ramFreeForUse= maxCachePercent/(float)100.0f*getAvailRAM();
+						case CACHE_NEVER:
+							currentFilter->setCaching(false);
+							break;
+						case CACHE_DEPTH_FIRST:
+						{
+							float ramFreeForUse;
+							ramFreeForUse= maxCachePercent/(float)100.0f*getAvailRAM();
 
-						bool cache;
-						cache=(cacheBytes/(1024*1024) ) < ramFreeForUse;
+							bool cache;
+							cache=((float)cacheBytes/(1024*1024) ) < ramFreeForUse;
 
-						currentFilter->setCaching( cache);
-						break;
+							currentFilter->setCaching( cache);
+							break;
+						}
 					}
 				}
+				else
+					currentFilter->setCaching(false);
 			}
-			else
-				currentFilter->setCaching(false);
-
 			//---
 
 			//Step 3: Take the stack top, and turn it into "curdata" and refresh using the filter.
@@ -691,7 +774,6 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			//	This is the guts of the system.
 			//---
 			//
-			(*callback)(false);
 
 			if(!currentFilter->haveCache())
 				currentFilter->clearConsole();
@@ -700,11 +782,7 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			try
 			{
 
-				errCode=currentFilter->refresh(inDataStack.top(),
-							curData,curProg,callback);
-
-				//error codes above this value are reserved
-				ASSERT(errCode <=FILTERTREE_REFRESH_ERR_BEGIN);
+				errCode=currentFilter->refresh(inDataStack.top(),curData,curProg);
 			}
 			catch(std::bad_alloc)
 			{
@@ -717,11 +795,15 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			//Perform sanity checks on filter output
 			checkRefreshValidity(curData,currentFilter);
 			ASSERT(curProg.step == curProg.maxStep || errCode);
+			//when completing, we should have full progress 
+			std::string progWarn = std::string("Progress did not reach 100\% for filter: ");
+			progWarn+=currentFilter->getUserString();
+			
+			WARN( (curProg.filterProgress == 100 || errCode),progWarn.c_str());
 #endif
 			//Ensure that (1) yield is called, regardless of what filter does
 			//(2) yield is called after 100% update	
 			curProg.filterProgress=100;	
-			(*callback)(false);
 
 
 			vector<SelectionDevice *> curDevices;
@@ -743,30 +825,40 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 				consoleMessages.push_back(make_pair(currentFilter,tmpMessages[ui]));
 
 			//check for any error in filter update (including user abort)
-			if(errCode)
+			if(errCode || abortRefresh)
 			{
 				//clear any intermediary pointers
 				popPointerStack(inDataStack,0);
 				ASSERT(inDataStack.empty());
-				//Clean up the output that we didn't use
+				
+				//remove duplicates, as more than one output data may
+				// output the same pointer
+				std::set<const FilterStreamData *> uniqSet;
 				for(list<FILTER_OUTPUT_DATA>::iterator it=outData.begin();
 						it!=outData.end();++it)
 				{
 					for(size_t ui=0;ui<it->second.size();ui++)
-					{
-						const FilterStreamData *data;
-						data = it->second[ui];
-
-						//Output data is uncached - delete it
-						if(!data->cached)
-							delete data;
-					}
+						uniqSet.insert(it->second[ui]);
 				}
+
+	
+				//Clean up the output that we didn't use
+				for(std::set<const FilterStreamData *>::iterator it=uniqSet.begin();
+					it!=uniqSet.end(); ++it)
+				{
+					const FilterStreamData *data;
+					data = *it;
+					//Output data is uncached - it is our job to delete it
+					if(!data->cached)
+						delete data;
+				}
+				if(abortRefresh)
+					return FILTER_ERR_ABORT;
 				return errCode;
 			}
 
 
-			//Update the filter output statistics, eg num objects of each type output 
+			//Update the filter output statistics, e.g. num objects of each type output 
 			currentFilter->updateOutputInfo(curData);
 			
 			
@@ -779,7 +871,7 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 				refreshCollector.trackPointers(curData);
 				
 				//Put this in the intermediary stack, 
-				//so it is available for any other children at this leve.
+				//so it is available for any other children at this level.
 				inDataStack.push(curData);
 			}
 			else if(curData.size())
@@ -788,7 +880,7 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 				outData.push_back(make_pair(currentFilter,curData));
 				refreshCollector.forgetPointers(curData);
 			}	
-			//Cur data is recorded either in outDta or on the data stack
+			//Cur data is recorded either in outData or on the data stack
 			curData.clear();
 			//---
 			
@@ -807,19 +899,19 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 	// minimises unnecessary output)
 	//Construct a single list of all pointers in output,
 	//checking for uniqueness. Delete duplicates
-	list<const FilterStreamData *> flatList;
 	
 	for(list<FILTER_OUTPUT_DATA>::iterator it=outData.begin();it!=outData.end(); )
 	{
-		vector<const FilterStreamData *>::iterator itJ;
+		std::set<const FilterStreamData *> uniqueSet;
 
+		vector<const FilterStreamData *>::iterator itJ;
 		itJ=it->second.begin();
 		while(itJ!=it->second.end()) 
 		{
 			//Each stream data pointer should only occur once in the entire lot.
-			if(find(flatList.begin(),flatList.end(),*itJ) == flatList.end())
+			if(uniqueSet.find(*itJ) == uniqueSet.end())
 			{
-				flatList.push_back(*itJ);
+				uniqueSet.insert(*itJ);
 				++itJ;
 			}
 			else
@@ -833,10 +925,6 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 		else
 			++it;
 	}
-
-	//outData List is complete, and contains only unique entries. clear the checking list.
-	flatList.clear();
-
 	//======
 
 
@@ -846,7 +934,7 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 
 string FilterTree::getRefreshErrString(unsigned int code)
 {
-	ASSERT(code >FILTERTREE_REFRESH_ERR_BEGIN && code < FILTERTREE_REFRESH_ERR_ENUM_END);
+	
 	const char *REFRESH_ERR_STRINGS[] = {"",
 		"Insufficient memory for refresh",
 		};
@@ -1014,7 +1102,7 @@ unsigned int FilterTree::loadXML(const xmlNodePtr &treeParent, std::ostream &err
 		newFilt=0;
 		nodeUnderstood=true; //assume by default we understand, and set false if not
 
-		//If we encounter a "children" node. then we need to look at the children of this filter
+		//If we encounter a "children" node. Then we need to look at the children of this filter
 		if (!xmlStrcmp(nodePtr->name,(const xmlChar*)"children"))
 		{
 			//Can't have children without parent
@@ -1224,7 +1312,7 @@ void FilterTree::stripHazardousContents()
 
 bool FilterTree::isChild(const tree<Filter *> &treeInst,
 				const tree<Filter *>::iterator &testParent,
-				tree<Filter *>::iterator testChild) const 
+				tree<Filter *>::iterator testChild) 
 {
 	// NOTE: A comparison against tree root (treeInst.begin())is INVALID
 	// for trees that have multiple base nodes.
@@ -1320,7 +1408,6 @@ void FilterTree::checkRefreshValidity(const vector< const FilterStreamData *> &c
 	//Filter outputs should
 	//	- Always have isCached set to 0 or 1.
 	//	- Filter should report that it has a cache, if it is emitting cached objects
-	//	- If caching is disabled, filter should not be caching objects
 	bool hasSomeCached=false;
 	for(size_t ui=0; ui<curData.size(); ui++)
 	{
@@ -1328,13 +1415,7 @@ void FilterTree::checkRefreshValidity(const vector< const FilterStreamData *> &c
 				curData[ui]->cached == 0);
 
 		if(curData[ui]->parent == refreshFilter)
-		{
-			if(!(refreshFilter->cacheEnabled()) )
-			{
-				ASSERT(curData[ui]->cached==0);
-			}
 			hasSomeCached|=curData[ui]->cached;
-		}
 	}
 
 	ASSERT(!(hasSomeCached == false && refreshFilter->haveCache()));
@@ -1420,7 +1501,7 @@ void FilterTree::checkRefreshValidity(const vector< const FilterStreamData *> &c
 #endif
 
 void FilterTree::safeDeleteFilterList( std::list<FILTER_OUTPUT_DATA> &outData, 
-						size_t typeMask, bool maskPrevents) const
+						size_t typeMask, bool maskPrevents) 
 {
 	//Loop through the list of vectors of filterstreamdata, then drop any elements that are deleted
 	for(list<FILTER_OUTPUT_DATA> ::iterator it=outData.begin(); 
@@ -1663,7 +1744,7 @@ void FilterTree::removeSubtree(Filter *removeFilt)
 	initFilterTree();
 }
 
-void FilterTree::cloneSubtree(FilterTree &f,Filter *targetFilt) const
+void FilterTree::cloneSubtree(FilterTree &f,const Filter *targetFilt) const
 {
 	ASSERT(!f.filters.size()); //Should only be passing empty trees
 	
