@@ -1,6 +1,6 @@
 /*
  *	filtertreeAnalyse.cpp - Performs correctness checking of filter trees
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -132,6 +132,24 @@ bool needsRangeParent(const Filter *f)
 
 }
 
+unsigned int needsUnrangedData(const Filter *f)
+{
+	if(f->getType() == FILTER_TYPE_IONINFO)
+	{
+		const IonInfoFilter *fInfo = (const IonInfoFilter *)f;
+
+		return fInfo->needsUnrangedData();
+	}
+
+	if(f->getType() == FILTER_TYPE_SPECTRUMPLOT)
+	{
+		const SpectrumPlotFilter *fSp = (const SpectrumPlotFilter *)f;
+
+		return fSp->needsUnrangedData(); 
+	}
+	return false;
+}
+
 void FilterTreeAnalyse::getAnalysisResults(std::vector<FILTERTREE_ERR> &errs) const
 {
 	errs.resize(analysisResults.size());
@@ -140,6 +158,8 @@ void FilterTreeAnalyse::getAnalysisResults(std::vector<FILTERTREE_ERR> &errs) co
 
 void FilterTreeAnalyse::analyse(const FilterTree &f)
 {
+	clear();
+
 	f.getAccumulatedPropagationMaps(emitTypes,blockTypes);
 
 	//Check for a data pair where the output is entirely blocked,
@@ -155,6 +175,10 @@ void FilterTreeAnalyse::analyse(const FilterTree &f)
 	//Check for filters that do not have a parent, which is required
 	checkRequiredParent(f);
 
+	//check for unranged data required by child
+	checkUnrangedData(f);
+
+
 	emitTypes.clear();
 	blockTypes.clear();
 
@@ -213,7 +237,7 @@ void FilterTreeAnalyse::blockingPairError(const FilterTree &f)
 			
 				analysisResults.push_back(treeErr);
 			}
-			//If the parent does not emit a useable objects 
+			//If the parent does not emit a usable objects 
 			//for the child filter, this is bad too.
 			// - else if, so we don't double up on warnings
 			else if( !(parentEmit & curUse) && !childFilter->isUsefulAsAppend())
@@ -356,7 +380,7 @@ void FilterTreeAnalyse::checkRequiredParent(const FilterTree &f)
 			}
 		}
 
-		//If we couldnt find a parent, then this is an error.
+		//If we couldn't find a parent, then this is an error.
 		// let the user know
 		if(!foundParent)
 		{
@@ -380,6 +404,40 @@ void FilterTreeAnalyse::checkRequiredParent(const FilterTree &f)
 	}
 }
 
+void FilterTreeAnalyse::checkUnrangedData(const FilterTree &f)
+{
+	const tree<Filter *> &treeFilt=f.getTree();
+	for(tree<Filter*>::pre_order_iterator it(treeFilt.begin()); it!=treeFilt.end(); ++it)
+	{
+		//Check to see if we have a filter that can be affected by unranged data, missing or present
+
+		if((*it)->getType() == FILTER_TYPE_RANGEFILE)
+		{
+			const RangeFileFilter *rngF = (const RangeFileFilter *)(*it);
+
+			//we only need to investigate filters which drop data
+			if( !rngF->getDropUnranged() )
+				continue;
+
+			for(tree<Filter*>::pre_order_iterator itJ(it); itJ!=treeFilt.end();++itJ)
+			{
+				//we need ranged data, but don't have it. Warn 
+				if(needsUnrangedData(*itJ))
+				{
+					FILTERTREE_ERR treeErr;
+					treeErr.reportedFilters.push_back(*it);
+					treeErr.reportedFilters.push_back(*itJ);
+					treeErr.shortReportMessage=TRANS("Bad range filter settings");
+					treeErr.verboseReportMessage=TRANS("Rangefile set to drop unranged data, however a child filter requires it.");						
+					treeErr.severity=ANALYSE_SEVERITY_WARNING;
+
+					analysisResults.push_back(treeErr);
+				} 
+			}
+
+		}
+	} 
+}
 
 bool filterAltersComposition(const Filter *f)
 {
@@ -401,13 +459,14 @@ bool filterAltersComposition(const Filter *f)
 
 			p=props.getPropValue(KEY_IONDOWNSAMPLE_PERSPECIES);
 
+			const int GROUP_SAMPLING=1;
 			
-			if(p.data== "1")
+			if(p.data== "1" && props.hasGroup(GROUP_SAMPLING))
 			{
 				vector<FilterProperty> propVec;
-				const int GROUP_SAMPLING=1;
 				props.getGroup(GROUP_SAMPLING,propVec);
 
+
 				//If using per-species mode, then
 				// we may affect the output ion composition
 				// if we have differing values
