@@ -1,6 +1,6 @@
 /*
  *	plot.h - plotting wraper for mathgl
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -21,9 +21,10 @@
 
 
 #include "backend/filter.h"
-
+#include "common/array2D.h"
 #include <map>
-
+#include <vector>
+#include <utility>
 
 #if  defined(WIN32) || defined(WIN64)
 	//Help mathgl out a bit: we don't need the GSL on this platform
@@ -32,28 +33,40 @@
 
 //Use config header to determine if we need to enable mgl2 support
 #include "../config.h"
-#ifdef USE_MGL2
 #include <mgl2/mgl.h>
-#else
-#include <mgl/mgl.h>
-#endif
 
 //mathgl shadows std::isnan
 #undef isnan
 
 
-enum
+//Plot style/mode enum
+enum 
 {
-	PLOT_TYPE_ONED,
-	PLOT_TYPE_MIXED,
-	PLOT_TYPE_ENUM_END //not a plot, just end of enum
+	PLOT_LINE_LINES=0,
+	PLOT_LINE_BARS,
+	PLOT_LINE_STEPS,
+	PLOT_LINE_STEM,
+	PLOT_LINE_POINTS,
+	PLOT_LINE_NONE, // not a plot, just marker for enum break
+	PLOT_2D_DENS,
+	PLOT_2D_SCATTER,
+	PLOT_TYPE_ENUM_END
 };
 
-
-
+//This is the plot mode,
+// it can be determined as a function of the
+// plot-style/mode enum
+enum
+{
+	PLOT_MODE_1D,
+	PLOT_MODE_2D,
+	PLOT_MODE_COLUMN,
+	PLOT_MODE_MIXED, //special marker - different types of plots, when looking at multiple plots
+	PLOT_MODE_ENUM_END //not a plot, just end of enum
+};
 
 //!Return a human readable string for a given plot type
-std::string plotString(unsigned int traceType);
+std::string plotString(unsigned int plotMode);
 
 //!Return a human readable string for the plot error mode
 std::string plotErrmodeString(unsigned int errMode);
@@ -64,26 +77,6 @@ unsigned int plotID(const std::string &plotString);
 //!Return the error mode type, given the human readable string
 unsigned int plotErrmodeID(const std::string &s);
 		
-//!Nasty hack class to change mathgl API from named char palette to rgb specification
-class MGLColourFixer
-{
-	private:
-		vector<float> rs,gs,bs;
-		static int maxCols;
-	public:
-		//Restore the MGL colour strings
-		void reset();
-		//Return the exact colour, if there is one
-		char haveExactColour(float r, float g, float b) const;
-		//Get the best colour that is available
-		// returns the char to give to mathgl; may be exact,
-		// maybe nearest match, depending upon number of colours used
-		// and mgl palette size
-		char getNextBestColour(float r, float g, float b);
-
-		static unsigned int getMaxColours();
-};
-
 
 //!Data class  for holding info about non-overlapping 
 // interactive rectilinear "zones" overlaid on plots 
@@ -105,7 +98,7 @@ class PlotRegion
 		};
 		
 		//Bounding limits for axial bind
-		vector<std::pair<float,float> > bounds;
+		std::vector<std::pair<float,float> > bounds;
 
 		//Bounding region colour
 		float r,g,b;
@@ -125,7 +118,7 @@ class PlotRegion
 		void setUpdateMethod(size_t updateMethod, void *parentObject);
 
 		//Update the parent object using the curretn update method
-		void updateParent(size_t method, const vector<float> &newBounds, bool updateSelf=true);
+		void updateParent(size_t method, const std::vector<float> &newBounds, bool updateSelf=true);
 
 		//Retrieve the parent as a filter object - must be in ACCESS_MODE_FILTER
 		Filter *getParentAsFilter() const { ASSERT(accessMode==ACCESS_MODE_FILTER); return (Filter*)parentObject;};
@@ -137,19 +130,18 @@ class PlotRegion
 };
 
 //Handles an array of regions, for drawing and editing of the array
-
 class RegionGroup
 {
 	private:
 		//cache for region overlaps, to reduce need to search
-		mutable vector<pair<size_t,size_t> > overlapIdCache;
-		mutable vector<pair<float,float> > overlapCoordsCache;
+		mutable std::vector<std::pair<size_t,size_t> > overlapIdCache;
+		mutable std::vector<std::pair<float,float> > overlapCoordsCache;
 
 		mutable bool haveOverlapCache;
 	public:
 		RegionGroup() { haveOverlapCache=false;};
 		//!Interactive, or otherwise marked plot regions
-		vector<PlotRegion> regions;
+		std::vector<PlotRegion> regions;
 
 
 		void clear() {regions.clear(); };
@@ -174,32 +166,34 @@ class RegionGroup
 				unsigned int movementType, float &maxX, float &maxY) const;
 
 
-		void getOverlaps(vector<pair<size_t,size_t> > &ids,
-				vector< pair<float,float> > &coords) const;
+		void getOverlaps(std::vector<std::pair<size_t,size_t> > &ids,
+				std::vector< std::pair<float,float> > &coords) const;
 };
 
 struct  OVERLAY_DATA
 {
 	//Coordinate and amplitude
-	vector<pair<float, float> > coordData;
+	std::vector<std::pair<float, float> > coordData;
 	//title for all of te specified overlay data
-	string title;
+	std::string title;
 	//If the overlay is enabled or not
 	bool enabled;
 };
 
+//!Thse are 1D-stem style overlays that can be used
+// to draw onto the plot
 class PlotOverlays
 {
 	private:
 		bool isEnabled;
 		// List of the overlays that can be shown on the given plot
-		vector<OVERLAY_DATA>  overlayData;
+		std::vector<OVERLAY_DATA>  overlayData;
 	public:
 		PlotOverlays() : isEnabled(true) {}
 		//Add a new overlay to the plot
 		void add(const OVERLAY_DATA &overlay) {overlayData.push_back(overlay);}
 		//Draw the overlay on the current plot
-		void draw(mglGraph *g,MGLColourFixer &fixer, 
+		void draw(mglGraph *g,
 			const mglPoint &boundMin, const mglPoint &boundMax,bool logMode) const;
 		//Enable the specified overlay
 		void setEnabled(size_t offset,bool isEnabled) 
@@ -211,38 +205,60 @@ class PlotOverlays
 		void clear() {overlayData.clear();}
 		void erase(size_t item) {ASSERT(item < overlayData.size()); overlayData.erase(overlayData.begin() + item);;}
 
-		const vector<OVERLAY_DATA> &getOverlays() const { return overlayData;};
+		const std::vector<OVERLAY_DATA> &getOverlays() const { return overlayData;};
 
 };
 
 //!Base class for data plotting
 class PlotBase
 {
+	protected:
+		//!Sub type of plot (eg lines, bars for 1D)
+		unsigned int plotMode;
+		//!xaxis label
+		std::string xLabel;
+		//!y axis label
+		std::string yLabel;
+		//!Plot title
+		std::string title;
+		
+		//plot colour (for single coloured plots)
+		float r,g,b;
+		
+		//The type of plot (ie what class is it?)	
+		unsigned int plotType;
+		
+		void copyBase(PlotBase *target) const;
+
+		//Find the upper and lower limit of a given dataset
+		static void computeDataBounds(const std::vector<float> &d, float &minV,float &maxV) ;
+		
+		//Find the upper and lower limit of a given dataset
+		static void computeDataBounds(const std::vector<float> &d, const std::vector<float> &errorBar,
+								float &minV,float &maxV);
+
+		//Find the upper and lower limit of a given dataset
+		static void computeDataBounds(const std::vector<std::pair<float,float> > &d, 
+					float &minVx,float &maxVx,float &minVy, float &maxVy);
 	public:
-		PlotBase(){};
+		PlotBase();
 		virtual ~PlotBase(){};
 
 		virtual PlotBase *clone() const = 0;
 
-		//The type of plot (ie what class is it?)	
-		unsigned int plotType;
-		
-		//!Bounding box for plot -  may exceed plot data area
+	
+		//return the plot type
+		unsigned int getType() const;
+
+		//return the plot mode, which is derived
+		// uniquely from the plot type
+		unsigned int getMode() const;
+
+		//!Bounding box for plot 
 		float minX,maxX,minY,maxY;
 
-		//!Colour of trace
-		float r,g,b;
 		//!Is trace visible?
 		bool visible;
-		//!Type of plot (lines, bars, sticks, etc)
-		unsigned int traceType;
-		//!xaxis label
-		std::wstring xLabel;
-		//!y axis label
-		std::wstring yLabel;
-		//!Plot title
-		std::wstring title;
-
 		//!Use the plot title for Y data label when exporting raw data
 		// (true), or use the yLabel
 		bool titleAsRawDataLabel;
@@ -258,25 +274,45 @@ class PlotBase
 		RegionGroup regionGroup;
 
 	
-		//True if the plot has data
-		virtual bool empty() const=0;
+		//True if the plot has no data
+		virtual bool isEmpty() const=0;
 
 		//Draw the plot onto a given MGL graph
-		virtual void drawPlot(mglGraph *graph, MGLColourFixer &fixer) const=0;
+		virtual void drawPlot(mglGraph *graph) const=0;
 
-		//!Scan for the data bounds.
+		//!Return the true data bounds for this plot
 		virtual void getBounds(float &xMin,float &xMax,
-					float &yMin,float &yMax) const = 0;
+					float &yMin,float &yMax) const;
+		
 
 		//Retrieve the raw data associated with this plot.
-		virtual void getRawData(vector<vector<float> > &f,
-				std::vector<std::wstring> &labels) const=0;
+		virtual void getRawData(std::vector<std::vector<float> > &f,
+				std::vector<std::string> &labels) const=0;
+
+		//set the plot axis strings (x,y and title)
+		void setStrings(const std::string &x, 
+			const std::string &y,const std::string &t);
+
+		void setColour(float rNew, float gNew, float bNew);
+
+		std::string getXLabel() const { return xLabel;}
+		std::string getTitle() const { return title;}
+		std::string getYLabel() const { return yLabel;}
+
+		//Plot mode is, eg 2D or 1D plot
+		unsigned int getPlotMode() const { return plotMode;}
+		//FIXME: Deprecate me. Plot mode should be intrinsic
+		void setPlotMode(unsigned int newMode) { plotMode= newMode;}
 
 
+		void getColour(float &r, float &g, float &b) const ;
+
+#ifdef DEBUG
+		void checkConsistent() const;
+#endif
 };
 
-//!1D Function f(x) Plot with ranges
-// data must be a pure Function.
+//!1D Function f(x) 
 class Plot1D : public PlotBase
 {
 	private: 	
@@ -285,41 +321,42 @@ class Plot1D : public PlotBase
 		bool logarithmic;
 		//!Data
 		std::vector<float> xValues,yValues,errBars;
-		
+
+		//Do we want to draw error bars?
+		PLOT_ERROR errMode;	
+
+		//Set the error bars for this plot
+		void genErrBars();	
 	public:
 		Plot1D();
-		virtual bool empty() const;
+		virtual bool isEmpty() const;
 		virtual PlotBase *clone() const;
 			
-		void getBounds(float &xMin,float &xMax,float &yMin,float &yMax) const;
-
 		//!Set the plot data from a pair and symmetric Y error
-		void setData(const vector<std::pair<float,float> > &v);
-		void setData(const vector<std::pair<float,float> > &v,const vector<float> &symYErr);
+		void setData(const std::vector<std::pair<float,float> > &v);
+		void setData(const std::vector<std::pair<float,float> > &v,const std::vector<float> &symYErr);
 		//!Set the plot data from two vectors and symmetric Y error
-		void setData(const vector<float> &vX, const vector<float> &vY);
-		void setData(const vector<float> &vX, const vector<float> &vY,
-							const vector<float> &symYErr);
+		void setData(const std::vector<float> &vX, const std::vector<float> &vY);
+		void setData(const std::vector<float> &vX, const std::vector<float> &vY,
+							const std::vector<float> &symYErr);
 
 		
 		//!Move a region to a new location. 
 		void moveRegion(unsigned int region, unsigned int method, float newPos);
 
-		void clear(bool preserveVisibility);
-		
 		
 		//Draw the plot onto a given MGL graph
-		virtual void drawPlot(mglGraph *graph,MGLColourFixer &fixer) const;
+		virtual void drawPlot(mglGraph *graph) const;
 
 		//Draw the associated regions		
-		void drawRegions(mglGraph *graph, MGLColourFixer &fixer,
-				const mglPoint &min, const mglPoint &max) const;
+		void drawRegions(mglGraph *graph,
+			const mglPoint &min, const mglPoint &max) const;
 
 
 		//!Retrieve the raw data associated with the currently visible plots. 
 		//note that this is the FULL data not the zoomed data for the current user bounds
 		void getRawData(std::vector<std::vector<float> >  &rawData,
-				std::vector<std::wstring> &labels) const;
+				std::vector<std::string> &labels) const;
 		
 		//!Retrieve the ID of the non-overlapping region in X-Y space
 		bool getRegionIdAtPosition(float x, float y, unsigned int &id) const;
@@ -339,6 +376,64 @@ class Plot1D : public PlotBase
 
 		bool wantLogPlot() const { return logarithmic;};
 		void setLogarithmic(bool p){logarithmic=p;};
+
+		//obtain the smallest nonzero value, if possible (otherwise, returns 0)
+		// - used to get limits for logarithmic plots, for example
+		float getSmallestNonzero() const;
+
+		//Set the current error mode
+		void setErrMode(PLOT_ERROR newErrMode) ;  
+
+};
+
+//!2D function, f(x,y). 
+class Plot2DFunc : public PlotBase
+{
+	private: 	
+		//2D array, for f(x,y) plots 
+		Array2D<float> xyValues;
+	public:
+		Plot2DFunc();
+		virtual bool isEmpty() const;
+		virtual PlotBase *clone() const;
+			
+		//Draw the plot onto a given MGL graph
+		virtual void drawPlot(mglGraph *graph) const;
+
+		//!Retrieve the raw data associated with the currently visible plots. 
+		//note that this is the FULL data not the zoomed data for the current user bounds
+		void getRawData(std::vector<std::vector<float> >  &rawData,
+				std::vector<std::string> &labels) const;
+	
+		void setData(const Array2D<float> &a, float xLow, float xHigh, float yLow, float yHigh) ;
+
+};
+
+//!2D scatter plot, {x,y}_i
+class Plot2DScatter : public PlotBase
+{
+	private:
+		std::vector<std::pair<float,float> > points;
+		std::vector<float > intensity;
+	public:
+		//Do we want to display the points in logarithmic terms?
+		bool scatterIntensityLog;
+
+		Plot2DScatter();
+		virtual bool isEmpty() const;
+		virtual PlotBase *clone() const;
+			
+		//Draw the plot onto a given MGL graph
+		virtual void drawPlot(mglGraph *graph) const;
+
+		//!Retrieve the raw data associated with the currently visible plots. 
+		//note that this is the FULL data not the zoomed data for the current user bounds
+		void getRawData(std::vector<std::vector<float> >  &rawData,
+				std::vector<std::string> &labels) const;
+
+		//reset the data stored in the plot
+		void setData(const std::vector<std::pair<float,float> > &pts);
+		void setData(const std::vector<std::pair<float,float> > &pts ,const std::vector<float> &intens);
 };
 
 //Wrapper class for containing multiple plots 
@@ -378,6 +473,8 @@ class PlotWrapper
 		//!Do we want to highlight positions where regions overlap?
 		bool highlightRegionOverlaps;
 
+
+		void getAppliedBounds(mglPoint &min,mglPoint &max) const;
 	public:
 		//"stick" type overlays for marking amplitudes on top of the plot
 		PlotOverlays overlays;
@@ -394,10 +491,10 @@ class PlotWrapper
 		size_t numPlots() const { return plottingData.size();}
 
 		//Retrieve the IDs for the stored plots
-		void getPlotIDs(vector<unsigned int> &ids) const ;
+		void getPlotIDs(std::vector<unsigned int> &ids) const ;
 
 		//Retrieve the title of the plot
-		std::wstring getTitle(size_t plotId) const;
+		std::string getTitle(size_t plotId) const;
 
 
 		void setEnableHighlightOverlap(bool enable=true) { highlightRegionOverlaps=enable;}
@@ -435,9 +532,6 @@ class PlotWrapper
 				const char *x, const char *y, const char *t);
 		void setStrings(unsigned int plotID,const std::string &x, 
 				const std::string &y,const std::string &t);
-		//!Set the parent information for a given plot
-		void setParentData(unsigned int plotID,
-				const void *parentObj, unsigned int plotIndex);
 
 		//TODO: Type hack - should return const Filter *
 		//!Get the parent object fo rthis plot
@@ -469,7 +563,7 @@ class PlotWrapper
 		bool isPlotVisible(unsigned int plotID) const;
 		
 		
-		void getVisibleIDs(vector<unsigned int> &plotID) const;
+		void getVisibleIDs(std::vector<unsigned int> &plotID) const;
 
 		//!Disable user bounds
 		void disableUserBounds(){plotChanged=true;applyUserBounds=false;};
@@ -503,22 +597,22 @@ class PlotWrapper
 		void getRegion(unsigned int plotId, unsigned int regionId, PlotRegion &r) const;
 
 		//Get all of the (id, regions) for plots. Bool allows for only the plots that are visible to be obtained
-		void getRegions(vector<pair<size_t,vector<PlotRegion> > > &regions, bool visibleOnly=true) const;
+		void getRegions(std::vector<std::pair<size_t,std::vector<PlotRegion> > > &regions, bool visibleOnly=true) const;
 		
 		//Return the ID and coordinates of any overlapping regions
 		// - this only returns overlaps for individual plots - not between plots
-		void getRegionOverlaps(std::vector<pair<size_t,size_t> > &ids,
-							std::vector< pair<float,float> > &coords) const;
+		void getRegionOverlaps(std::vector<std::pair<size_t,size_t> > &ids,
+							std::vector< std::pair<float,float> > &coords) const;
 
 		//!Retrieve the raw data associated with the selected plots.
-		void getRawData(vector<vector<vector<float> > >  &data, std::vector<std::vector<std::wstring> >  &labels) const;
+		void getRawData(std::vector<std::vector<std::vector<float> > >  &data, std::vector<std::vector<std::string> >  &labels) const;
 	
 
 		//!obtain the type of a plot, given the plot's uniqueID
 		unsigned int plotType(unsigned int plotId) const;
 
 		//Retrieve the types of visible plots
-		unsigned int getVisibleType() const;
+		unsigned int getVisibleMode() const;
 
 		//!Obtain limit of motion for a given region movement type
 		void findRegionLimit(unsigned int plotId, unsigned int regionId,
@@ -537,7 +631,7 @@ class PlotWrapper
 
 		//TODO: convert to serialised parent path
 		//Override the last-visible selection. This allows for overriding which plots were selected, which is normally handled internally
-		void overrideLastVisible(vector< pair<const void *,unsigned int>  > &overridden); 
+		void overrideLastVisible(std::vector< std::pair<const void *,unsigned int>  > &overridden); 
 };
 
 #endif
