@@ -1,6 +1,6 @@
 /*
  *	basics.cpp  - basic functions header
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -49,21 +49,7 @@ using std::vector;
 using std::list;
 
 
-//Name of the  DTD file for state loading
-const char *DTD_NAME="threeDepict-state.dtd";
-//Program name
-const char *PROGRAM_NAME = "3Depict";
-//Program version
-const char *PROGRAM_VERSION = "0.0.16";
-//Path to font for Default FTGL  font
-const char *FONT_FILE= "FreeSans.ttf";
 
-const char *TEXT_LOAD_ERR_STRINGS[] = { "",
-       					NTRANS("Error opening file"),
-       					NTRANS("Error whilst reading file contents"),
-					NTRANS("Error interpreting field in file"),
-					NTRANS("Inconsistent number of columns found")
-					};
 
 //default font to use.
 std::string defaultFontFile;
@@ -151,7 +137,7 @@ std::string getDefaultFontFile()
 void tickSpacingsFromInterspace(float start, float end, 
 		float interSpacing, std::vector<float> &spacings)
 {
-	ASSERT(interSpacing > sqrt(std::numeric_limits<float>::epsilon()));
+	ASSERT(interSpacing > sqrtf(std::numeric_limits<float>::epsilon()));
 	unsigned int nTicks;
 
 	if(end < start)
@@ -310,7 +296,334 @@ string veryFuzzyTimeSince( time_t origTime, time_t nowTime)
 	return TRANS("moments ago");
 }
 
+ColourRGBA::ColourRGBA()
+{
+}
 
+ColourRGBA::ColourRGBA(unsigned char r, unsigned char g, unsigned char b, unsigned char a) 
+{
+	data[0]=r;
+	data[1]=g;
+	data[2]=b;
+	data[3]=a;
+}
+
+
+ColourRGBA::ColourRGBA(unsigned char r, unsigned char g, unsigned char b) 
+{
+	data[0]=r;
+	data[1]=g;
+	data[2]=b;
+}
+
+unsigned char ColourRGBA::at(unsigned int idx) const
+{
+	ASSERT(idx< 4);
+	return data[idx];
+}
+
+unsigned char ColourRGBA::r() const
+{
+    return data[0];
+}
+
+unsigned char ColourRGBA::g() const
+{
+    return data[1];
+}
+
+unsigned char ColourRGBA::b() const
+{
+    return data[2];
+}
+
+unsigned char ColourRGBA::a() const
+{
+    return data[3];
+}
+
+bool ColourRGBA::parse(const std::string &str)	
+{
+	//Input string is in 2 char hex form, 3 or 4 colour, with # leading. RGB order
+	//lowercase string.
+	if(str.size() != 9 && str.size() != 7)
+		return false;
+
+	if(str[0] != '#')
+		return false;
+
+	string rS,gS,bS,aS;
+	rS=str.substr(1,2);
+	gS=str.substr(3,2);
+	bS=str.substr(5,2);
+
+	if(!isxdigit(rS[0]) || !isxdigit(rS[1]))
+		return false;
+	if(!isxdigit(gS[0]) || !isxdigit(gS[1]))
+		return false;
+	if(!isxdigit(bS[0]) || !isxdigit(bS[1]))
+		return false;
+
+	unsigned char r,g,b,a;
+	hexStrToUChar(str.substr(1,2),r);	
+	hexStrToUChar(str.substr(3,2),g);	
+	hexStrToUChar(str.substr(5,2),b);	
+	//3 colour must have a=255.
+	if(str.size() == 7)
+		a = 255;
+	else
+	{
+		aS=str.substr(7,2);
+		if(!isxdigit(aS[0]) || !isxdigit(aS[1]))
+			return false;
+		hexStrToUChar(str.substr(7,2),a);	
+	}
+	
+	data[0]=r;
+	data[1]=g;
+	data[2]=b;
+	data[3]=a;
+	return true;
+}
+
+std::string ColourRGBA::rgbaString() const
+{
+	std::string s="#", tmp;
+	ucharToHexStr(data[0],tmp);
+	s+=tmp;
+	ucharToHexStr(data[1],tmp);
+	s+=tmp;
+	ucharToHexStr(data[2],tmp);
+	s+=tmp;
+	ucharToHexStr(data[3],tmp);
+	s+=tmp;
+
+	return s;
+}
+
+std::string ColourRGBA::rgbString() const
+{
+	string tmp,s;
+	s="#";
+	ucharToHexStr(data[0],tmp);
+	s+=tmp;
+	ucharToHexStr(data[1],tmp);
+	s+=tmp;
+	ucharToHexStr(data[2],tmp);
+	s+=tmp;
+
+	return s;
+}
+
+RGBf ColourRGBA::toFloat() const
+{
+	RGBf ret;
+	ret.red=(float)data[0]/255.0f;
+	ret.green=(float)data[1]/255.0f;
+	ret.blue=(float)data[2]/255.0f;
+
+	return ret;
+}
+
+ColourRGBAf ColourRGBA::toRGBAf() const
+{
+	ColourRGBAf tmp;
+		
+	for(unsigned int ui=0;ui<4;ui++)
+	{
+		tmp[ui] = (float)data[ui]/255.0f;
+	}
+
+	return tmp;
+}
+
+void ColourRGBA::fromRGBf(const RGBf &oth) 
+{
+	data[0]=oth.red*255.0f;
+	data[1]=oth.green*255.0f;
+	data[2]=oth.blue*255.0f;
+	data[3]=255.0f;
+}
+
+bool ColourRGBA::operator==(const ColourRGBA &oth) const
+{
+	for(unsigned int ui=0;ui<4;ui++)
+	{
+		if(data[ui] != oth.data[ui])
+			return false;
+	}
+	return true;
+}
+
+bool ColourRGBA::operator==(const ColourRGBAf &oth) const
+{
+	for(unsigned int ui=0;ui<4;ui++)
+	{
+		if(data[ui] != oth.at(ui))
+			return false;
+	}
+	return true;
+}
+
+bool ColourRGBA::operator==(const RGBf &oth) const
+{
+	return (data[0]/255.0f == oth.red && data[1]/255.0f == oth.green && data[2]/255.0f == oth.blue);
+		
+}
+
+bool ColourRGBA::operator!=(const ColourRGBA &oth) const
+{
+	return !(*this == oth);
+}
+
+bool ColourRGBA::operator!=(const ColourRGBAf &oth) const
+{
+	return !(*this == oth);
+}
+
+
+ColourRGBAf::ColourRGBAf()
+{
+}
+
+ColourRGBAf::ColourRGBAf(float r, float g, float b, float a) 
+{
+	ASSERT(r >=0 && r <=1.0f); 
+	ASSERT(g >=0 && g <=1.0f); 
+	ASSERT(b >=0 && b <=1.0f); 
+	ASSERT(a >=0 && a <=1.0f); 
+	data[0]=r;
+	data[1]=g;
+	data[2]=b;
+	data[3]=a;
+}
+
+
+ColourRGBAf::ColourRGBAf(float r, float g, float b) 
+{
+	ASSERT(r >=0 && r <=1.0f); 
+	ASSERT(g >=0 && g <=1.0f); 
+	ASSERT(b >=0 && b <=1.0f); 
+	data[0]=r;
+	data[1]=g;
+	data[2]=b;
+	data[3]=1.0f;
+}
+float ColourRGBAf::r() const
+{
+    return data[0];
+}
+
+float ColourRGBAf::g() const
+{
+    return data[1];
+}
+
+float ColourRGBAf::b() const
+{
+    return data[2];
+}
+
+float ColourRGBAf::a() const
+{
+    return data[3];
+}
+
+void ColourRGBAf::r(float v)
+{
+	ASSERT(v >=0.0f && v <=1.0f);
+	data[0]=v;
+}
+
+void ColourRGBAf::g(float v)
+{
+	ASSERT(v >=0.0f && v <=1.0f);
+	data[1]=v;
+}
+
+void ColourRGBAf::b(float v)
+{
+	ASSERT(v >=0.0f && v <=1.0f);
+	data[2]=v;
+}
+
+void ColourRGBAf::a(float v)
+{
+	ASSERT(v >=0.0f && v <=1.0f);
+	data[3]=v;
+}
+float &ColourRGBAf::operator[](unsigned int idx) 
+{
+	ASSERT(idx < 4);
+	return data[idx];
+}
+
+float ColourRGBAf::at(unsigned int idx) const
+{
+	return data[idx];
+}
+
+		
+ColourRGBAf ColourRGBAf::interpolate(float delta, const ColourRGBAf &other)
+{
+	ColourRGBAf result;
+
+	for(unsigned int ui=0;ui<3;ui++)
+		result[ui] = data[ui] + (other.data[ui] - data[ui])*delta;
+	return result;
+}
+
+ColourRGBA ColourRGBAf::toColourRGBA() const
+{
+	ColourRGBA tmp(data[0]*255.0f,data[1]*255.0f,
+			data[2]*255.0f,data[3]*255.0f);
+	return tmp;
+}
+
+RGBf ColourRGBAf::toRGBf() const
+{
+	RGBf tmp;
+	tmp.red=data[0];
+	tmp.green=data[1];
+	tmp.blue=data[2];
+	return tmp;
+}
+
+
+void ColourRGBAf::operator=(const RGBf &oth)
+{
+	data[0]= oth.red;
+	data[1]= oth.green;
+	data[2]= oth.blue;
+	data[3]= 1.0f;
+}
+
+
+bool ColourRGBAf::operator==(const ColourRGBA &oth) const
+{
+	for(unsigned int ui=0;ui<3;ui++)
+	{
+		if(data[ui] != (float)oth.at(ui)/255.0f)
+			return false;
+	}
+
+	return true;
+}
+
+bool ColourRGBAf::operator==(const ColourRGBAf &oth) const
+{
+	for(unsigned int ui=0;ui<3;ui++)
+	{
+		if(data[ui] != (float)oth.data[ui])
+			return false;
+	}
+
+	return true;
+}
+bool ColourRGBAf::operator!=(const ColourRGBAf  &oth) const
+{
+	return !(*this == oth);
+}
 void BoundCube::getBound(Point3D &retBound, unsigned int minMax) const
 {
 	retBound=Point3D(bounds[0][minMax],
@@ -367,6 +680,17 @@ void BoundCube::setBounds(const std::vector<Point3D> &points)
 #endif
 }
 
+void BoundCube::setBounds(const Point3D &p, float r)
+{
+	for(unsigned int dim=0;dim<3;dim++)
+	{
+		bounds[dim][0] = p[dim] - r;
+		bounds[dim][1] = p[dim] + r;
+		valid[dim][0]=true;
+		valid[dim][1]=true;
+	}
+}
+
 void BoundCube::getVertices(std::vector<Point3D> &points, bool centre) const
 {
 	points.resize(8);
@@ -430,7 +754,7 @@ bool BoundCube::isFlat() const
 
 bool BoundCube::isNumericallyBig() const
 {
-	const float TOO_BIG=sqrt(std::numeric_limits<float>::max());
+	const float TOO_BIG=sqrtf(std::numeric_limits<float>::max());
 	for(unsigned int ui=0;ui<2; ui++)
 	{
 		for(unsigned int uj=0;uj<3; uj++)
@@ -610,6 +934,38 @@ bool BoundCube::intersects(const Point3D &pt, float sqrRad)
 	return (nearPt.sqrDist(pt) <=sqrRad);
 }
 
+BoundCube BoundCube::makeUnion(const BoundCube &bC) const
+{
+	BoundCube res;
+	for(unsigned int dim=0;dim<3;dim++)
+	{
+		float a,b;
+		a=bounds[dim][0]; b=bC.bounds[dim][0];
+		res.setBound(dim,0,std::max(a,b));
+		a=bounds[dim][1]; b=bC.bounds[dim][1];
+		res.setBound(dim,1,std::min(a,b));
+	}
+
+	return res;
+}
+
+unsigned int BoundCube::segmentTriple(unsigned int dim, float slice) const
+{
+	ASSERT(dim < 3);
+
+	//check lower
+	if( slice < bounds[dim][0])
+		return 0;
+	
+	//check upper
+	if( slice >=bounds[dim][1])
+		return 2;
+
+	return 1;
+
+}
+
+
 Point3D BoundCube::getCentroid() const
 {
 #ifdef DEBUG
@@ -1115,6 +1471,30 @@ unsigned int loadTextStringData(const char *cpFilename, vector<vector<string> >
 	return 0;
 }
 
+
+#if !defined(__WIN32__) && !defined(__WIN64)
+	
+bool isNotDirectory(const char *filename)
+{
+	struct stat statbuf;
+
+	if(stat(filename,&statbuf) == -1)
+		return false;
+
+	return (statbuf.st_mode !=S_IFDIR);
+}
+
+bool rmFile(const std::string &filename)
+{
+	return remove(filename.c_str()) == 0;
+}
+#elif defined(__WIN32) || defined(__WIN64)
+bool rmFile(const std::string &filename)
+{ 
+	return DeleteFile((const wchar_t*)filename.c_str()) == 0;
+}
+#endif
+
 #ifdef DEBUG
 bool isValidXML(const char *filename)
 {
@@ -1146,37 +1526,4 @@ bool isValidXML(const char *filename)
 	WARN(!result,"xmllint not installed in system PATH, cannot perform debug check")
 	return true;
 }
-
-	
-//FIXME: Why negative?
-bool isNotDirectory(const char *filename)
-{
-#if !defined(__WIN32__) && !defined(__WIN64__)
-	struct stat statbuf;
-
-	if(stat(filename,&statbuf) == -1)
-		return false;
-
-	return (statbuf.st_mode !=S_IFDIR);
-#else
-
-	WARN(false, "Untested function. calling win api");
-	DWORD fileAttribs;
-	fileAttribs=GetFileAttributes((LPCWSTR)filename);
-	if(fileAttribs == INVALID_FILE_ATTRIBUTES)
-		return false;
-
-	return !(fileAttribs & FILE_ATTRIBUTE_DIRECTORY);
-#endif
-}
-
-bool rmFile(const std::string &filename)
-{
-#if !defined(__WIN32__) && !defined(__WIN64__)
-	return remove(filename.c_str()) == 0;
-#else
-	WARN(false, "Untested function. calling win api");
-	return DeleteFile((LPCWSTR)filename.c_str());
-#endif
-}
 #endif
