@@ -1,6 +1,6 @@
 /*
  *	common/basics.h - Basic functionality header 
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -21,6 +21,39 @@
 #define BASICS_H
 //!Basic objects header file
 
+
+#if (_MSC_VER >= 1600) ||  (__cplusplus > 199711L) 
+	#define HAVE_CPP_1X
+#endif
+
+//C-style Array size macro
+#ifdef HAVE_CPP_1X
+	template<typename T, unsigned int s> constexpr unsigned int THREEDEP_ARRAYSIZE(T (&)[s]) {     return s;  } 
+#else
+	#define THREEDEP_ARRAYSIZE(f) (sizeof (f) / sizeof(*f))
+#endif
+
+//macro to switch between normal bool and atomic, as available.
+// do *NOT* declare const ATOMIC_BOOLs. This has wierd CPU caching
+// assumptions, which cause the type to not work properly if not using
+// true atomics	
+#ifndef ATOMIC_BOOL
+
+
+	#ifdef HAVE_CPP_1X
+		// Killed? By an ATOMIC bool? No sir, I guess I don't take 
+		// much solace that the implosion trigger
+		// functioned perfectly. 
+
+		//This bool is reliable for attempting to perform inter-thread flagging.
+		#include <atomic>
+		#define ATOMIC_BOOL std::atomic<bool>
+	#else
+		//C++ <1X does not provide a truly safe bool type. Most implementations it seems to work though (provided you don't use const).
+		#define ATOMIC_BOOL bool
+	#endif
+#endif
+
 #include "mathfuncs.h"
 #include "common/assertion.h"
 
@@ -34,14 +67,6 @@
 class K3DTree;
 
 
-bool dummyCallback(bool);
-
-extern const char *DTD_NAME;
-extern const char *PROGRAM_NAME;
-extern const char *PROGRAM_VERSION;
-extern const char *FONT_FILE;
-
-
 //Set new locale code. Must be followed by a popLocale call before completion
 // Only one locale type can be pushed at a time this way
 void pushLocale(const char *newLocale, int type);
@@ -95,7 +120,12 @@ enum
 	ERR_FILE_ENUM_END // not an error, just end of enum
 };
 
-extern const char *TEXT_LOAD_ERR_STRINGS[];
+//Exclusive or operator
+template<class T>
+bool xorFunc(const T a, const T b)
+{
+  return (a || b) && !(a && b);
+}
 
 //Perform a a<-b<-c<-a rotation of data
 template<class T>
@@ -359,8 +389,16 @@ public:
     void setBounds( const Point3D &p, const Point3D &q);
     //!Obtain bounds from an array of Point3Ds
     void setBounds(const std::vector<Point3D> &ptArray);
+
+    //!Set bounds via cube that contains given sphere
+    void setBounds(const Point3D &p, float radius);
+
+    //Set & set-like operations
     //!Checks if a point intersects a sphere of centre Pt, radius^2 sqrRad
     bool intersects(const Point3D &pt, float sqrRad);
+   
+    //Create a union of two bounding cubes, which is itself a cube 
+    BoundCube makeUnion(const BoundCube &b) const;
     //Check to see if the point is contained in, or part of the walls
     //of the cube
     bool containsPt(const Point3D &pt) const;
@@ -370,6 +408,8 @@ public:
     //!Is this bounding cube completely contained within a sphere centred on pt of sqr size sqrRad?
     bool containedInSphere(const Point3D &pt, float sqrRad) const;
 
+
+    unsigned int segmentTriple(unsigned int dim, float slice) const;
     //!Returns maximum distnace to box corners (which is an upper bound on max box distance). 
     //Bounding box must be valid.
     float getMaxDistanceToBox(const Point3D &pt) const;
@@ -397,78 +437,103 @@ public:
     friend class K3DTreeMk2;
 };
 
+//!Data holder for colour as float
+typedef struct RGBf
+{
+	float red;
+	float green;
+	float blue;
+} RGBf;
 
+class ColourRGBAf;
 
-
-//OK, this is a bit tricky. We override the operators to call
-//a callback, so the UI updates keep happening, even inside the STL function
-//----
-template<class T>
-class GreaterWithCallback 
+//Colour storage class. Uses uchar internally
+class ColourRGBA
 {
-	private:
-		bool (*callback)(bool);
-		//!Reduction frequency (use callback every k its)
-		unsigned int redMax;
-		//!Current reduction counter
-		unsigned int reduction;
-		//!pointer to progress value
-		unsigned int *prgPtr;
 	public:
-		//!Second argument is a "reduction" value to set the number of calls
-		//to the random functor before initiating a callback
-		GreaterWithCallback( bool (*ptr)(bool),unsigned int red) : callback(ptr), redMax(red), reduction(red)
-	{};
+		unsigned char data[4];
+	public:
+		ColourRGBA();
+		ColourRGBA(unsigned char , unsigned char, unsigned char);
+		ColourRGBA(unsigned char , unsigned char, unsigned char, unsigned char);
+		unsigned char r() const;
+		unsigned char g() const;
+		unsigned char b() const;
+		unsigned char a() const;
+		
+		//Parse a colour string, such as #aabbccdd into its RGBA 8-bit components. alpha value (last) can be omitted. Will assume 255.
+		bool parse(const std::string &);
 
-		bool operator()(const T &a, const T &b) 
-		{
-			if(!reduction--)
-			{
-				reduction=redMax;
-				//Execute callback
-				(*callback)(false);
-			}
+		//Convert an RGB its 
+		// hexadecimal colour string
+		// format is "#rrggbb" such as "#11ee00"
+		std::string rgbString() const;
+		//Convert RGB to hex colour string, with alpha channel
+		std::string rgbaString() const;
+
+		//convert data from RGB/[0->255] integers to [0->1] float.
+		// alpha channel is not used
+		RGBf toFloat() const;
+
+		void fromRGBAf(const ColourRGBAf &);
+		ColourRGBAf toRGBAf() const;
+		
+		void fromRGBf(const RGBf &);
+
+		bool operator==(const ColourRGBA &oth) const;
+		bool operator==(const ColourRGBAf &oth) const;
+		bool operator==(const RGBf &oth) const;
+		
+		bool operator!=(const ColourRGBA &oth) const;
+		bool operator!=(const ColourRGBAf &oth) const;
+		
+		unsigned char at(unsigned int idx) const;
 
-			return a < b;
-		}
 };
 
-
-template<class T>
-class EqualWithCallback 
+//Colour storage class. Uses float internally
+class ColourRGBAf
 {
 	private:
-		bool (*callback)(bool);
-		//!Reduction frequency (use callback every k its)
-		unsigned int redMax;
-		//!Current reduction counter
-		unsigned int reduction;
+		float data[4];
 	public:
-		//!Second argument is a "reduction" value to set the number of calls
-		//to the random functor before initiating a callback
-		EqualWithCallback( bool (*ptr)(bool),unsigned int red) : callback(ptr), redMax(red), reduction(red)
-			{ };
+		ColourRGBAf();
+		ColourRGBAf(float, float, float);
+		ColourRGBAf(float, float, float,float);
+		float r() const;
+		float g() const;
+		float b() const;
+		float a() const;
+		
+		void r(float);
+		void g(float);
+		void b(float);
+		void a(float);
 
-		bool operator()(const T &a, const T &b) 
-		{
-			if(!reduction--)
-			{
-				reduction=redMax;
-				//Execute callback
-				(*callback)(false);
-			}
 
-			return a ==b;
-		}
+		ColourRGBAf interpolate(float delta, const ColourRGBAf &other);
+		//convert to a ColourRGBA (uchar representation)
+		//TODO : Rename me!
+		ColourRGBA toColourRGBA() const; 
+		RGBf toRGBf() const; 
+		bool operator==(const ColourRGBA &oth) const;
+		bool operator!=(const ColourRGBA &oth) const;
+		bool operator==(const ColourRGBAf &oth) const;
+		bool operator!=(const ColourRGBAf &oth) const;
+		
+		//TODO: Deprecate me!
+		bool operator==(const RGBf &oth) const;
+		
+		void operator=(const RGBf &oth);
+		float &operator[](unsigned int idx) ;
+		float at(unsigned int idx) const;
 };
-//----
 
-
-//Randomly select subset. Subset will be (somewhat) sorted on output
+//Randomly select subset. Subset will be (somewhat) sorted on output.
+// Returns -1 on abort, otherwise returns number of randomly selected items
 template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<T> &source, 
-							RandNumGen &rng, size_t num,unsigned int &progress,bool (*callback)(bool), bool strongRandom=false)
+							RandNumGen &rng, size_t num,unsigned int &progress, ATOMIC_BOOL &wantAbort, bool strongRandom=false)
 {
-	const unsigned int NUM_CALLBACK=50000;
 	//If there are not enough points, just copy it across in whole
 	if(source.size() <= num)
 	{
@@ -502,11 +567,7 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 		for(size_t ui=0; ui<numTicksNeeded; ui++)
 			ticks[ui]=(size_t)(rng.genUniformDev()*(source.size()-1));
 
-		//Remove duplicates. Intersperse some callbacks to be nice
-		GreaterWithCallback<size_t> gFunctor(callback,50000);
-		std::sort(ticks.begin(),ticks.end(),gFunctor);
-		EqualWithCallback<size_t> eqFunctor(callback,50000);
-
+		std::sort(ticks.begin(),ticks.end());
 		std::vector<size_t>::iterator newLast;
 		newLast=std::unique(ticks.begin(),ticks.end());	
 		ticks.erase(newLast,ticks.end());
@@ -514,7 +575,7 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 		//Top up with unique entries
 		//TODO: Overcommit & Discard implementation?
 		// - Should be possible to predict how many we need after collisions, and then overcommit and discard randomly. Removes need to loop-sort like this
-		while(ticks.size() < numTicksNeeded)
+		while(ticks.size() < numTicksNeeded && !wantAbort)
 		{
 			size_t moreTicks=numTicksNeeded-ticks.size();
 			for(size_t uk=0;uk<moreTicks;uk++)
@@ -526,17 +587,18 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 	
 			}
 
-			std::sort(ticks.begin(),ticks.end(),gFunctor);
-			newLast=std::unique(ticks.begin(),ticks.end(),eqFunctor);	
+			std::sort(ticks.begin(),ticks.end());
+			newLast=std::unique(ticks.begin(),ticks.end());	
 			ticks.erase(newLast,ticks.end());
 		}
 
+		if(wantAbort)
+			return -1;
+
 		ASSERT(ticks.size() == numTicksNeeded);
 		//---------
 		
 		//Transfer the output
-		unsigned int curProg=NUM_CALLBACK;
-
 		if(num < source.size()/2)
 		{
 			size_t pos=0;
@@ -545,18 +607,13 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 
 				result[pos]=source[*it];
 				pos++;
-				if(!curProg--)
-				{
-					progress= (unsigned int)((float)(pos)/((float)num)*100.0f);
-					(*callback)(false);
-					curProg=NUM_CALLBACK;
-				}
+				progress= (unsigned int)((float)(pos)/((float)num)*100.0f);
 			}
 		}
 		else
 		{
 			//Sort the ticks properly (mostly sorted anyway..)
-			std::sort(ticks.begin(),ticks.end(),gFunctor);
+			std::sort(ticks.begin(),ticks.end());
 
 			unsigned int curTick=0;
 			for(size_t ui=0;ui<source.size(); ui++)
@@ -570,12 +627,7 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 					result[ui-curTick]=source[ui];
 				}
 				
-				if(!curProg--)
-				{
-					progress= (unsigned int)(((float)(ui)/(float)source.size())*100.0f);
-					(*callback)(false);
-					curProg=NUM_CALLBACK;
-				}
+				progress= (unsigned int)(((float)(ui)/(float)source.size())*100.0f);
 			}
 		}
 
@@ -608,7 +660,6 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 		l.setState(start);
 
 		size_t ui=0;	
-		unsigned int curProg=NUM_CALLBACK;
 		//generate unique weak random numbers.
 		while(ui<num)
 		{
@@ -622,12 +673,7 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 				result[ui] =source[res];
 				ui++;
 			}
-			if(!curProg--)
-			{
-				progress= (unsigned int)((float)(ui)/((float)source.size())*100.0f);
-				(*callback)(false);
-				curProg=NUM_CALLBACK;
-			}
+			progress= (unsigned int)((float)(ui)/((float)source.size())*100.0f);
 		}
 
 	}
@@ -637,7 +683,7 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 
 //Randomly select subset [0,max). Subset will be (somewhat) sorted on output
 template<class T> size_t randomDigitSelection(std::vector<T> &result, const size_t max,
-			RandNumGen &rng, size_t num,unsigned int &progress,bool (*callback)(bool),
+			RandNumGen &rng, size_t num,unsigned int &progress,
 			bool strongRandom=false)
 {
 	//If there are not enough points, just copy it across in whole
@@ -676,13 +722,10 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 		for(size_t ui=0; ui<numTicksNeeded; ui++)
 			ticks[ui]=(size_t)(rng.genUniformDev()*(max-1));
 
-		//Remove duplicates. Intersperse some callbacks to be nice
-		GreaterWithCallback<size_t> gFunctor(callback,50000);
-		std::sort(ticks.begin(),ticks.end(),gFunctor);
-		EqualWithCallback<size_t> eqFunctor(callback,50000);
-		
+		//Remove duplicates
+		std::sort(ticks.begin(),ticks.end());
 		std::vector<size_t>::iterator itLast;
-		itLast=std::unique(ticks.begin(),ticks.end(),eqFunctor);	
+		itLast=std::unique(ticks.begin(),ticks.end());	
 		ticks.erase(itLast,ticks.end());
 		
 		//Top up with unique entries
@@ -698,8 +741,8 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 	
 			}
 
-			std::sort(ticks.begin(),ticks.end(),gFunctor);
-			itLast=std::unique(ticks.begin(),ticks.end(),eqFunctor);	
+			std::sort(ticks.begin(),ticks.end());
+			itLast=std::unique(ticks.begin(),ticks.end());	
 			ticks.erase(itLast,ticks.end());
 		}
 
@@ -719,18 +762,13 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 
 				result[pos]=*it;
 				pos++;
-				if(!curProg--)
-				{
-					progress= (unsigned int)((float)(curProg)/((float)num)*100.0f);
-					(*callback)(false);
-					curProg=CURPROG;
-				}
+				progress= (unsigned int)((float)(curProg)/((float)num)*100.0f);
 			}
 		}
 		else
 		{
 			//Sort the ticks properly (mostly sorted anyway..)
-			std::sort(ticks.begin(),ticks.end(),gFunctor);
+			std::sort(ticks.begin(),ticks.end());
 			
 			unsigned int curTick=0;
 			for(size_t ui=0;ui<numTicksNeeded; ui++)
@@ -741,12 +779,7 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 				else
 					result[ui-curTick]=ui;
 				
-				if(!curProg--)
-				{
-					progress= (unsigned int)((float)(curProg)/((float)num)*100.0f);
-					(*callback)(false);
-					curProg=CURPROG;
-				}
+				progress= (unsigned int)((float)(curProg)/((float)num)*100.0f);
 			}
 		}
 
