@@ -1,6 +1,6 @@
 /*
  * common/constants.h  - Common constants used across program
- * Copyright (C) 2013  D Haley
+ * Copyright (C) 2015  D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,37 +18,13 @@
 #ifndef COMMONCONSTANTS_H
 #define COMMONCONSTANTS_H
 
-//C-style Array size macro
-#define THREEDEP_ARRAYSIZE(f) (sizeof (f) / sizeof(*f))
-//C++1x availabilty (as far as we are concerned)
-#if __cplusplus > 199711L
-	#define HAVE_CPP1X 1
-#endif
+#include "translation.h"
 
 //Hack-ish variable for minimal size of data to execute
 // openMP conditionals as parallel regions. This is highly empirical!
 const unsigned int OPENMP_MIN_DATASIZE=1000;
 
 
-enum
-{
-  PLOT_TRACE_LINES=0,
-  PLOT_TRACE_BARS,
-  PLOT_TRACE_STEPS,
-  PLOT_TRACE_STEM,
-  PLOT_TRACE_POINTS,
-  PLOT_TRACE_ENDOFENUM
-};
-
-//Plot types
-enum
-{
-	PLOT_MODE_1D,
-	PLOT_MODE_2D,
-	PLOT_MODE_ENUM_END
-};
-
-
 //Plot error types
 enum
 {
@@ -73,6 +49,8 @@ enum
 	PROPERTY_TYPE_STRING,
 	PROPERTY_TYPE_POINT3D,
 	PROPERTY_TYPE_CHOICE,
+	PROPERTY_TYPE_FILE,
+	PROPERTY_TYPE_DIR,
 	PROPERTY_TYPE_ENUM_END //Not a prop, just end of enum
 };
 
@@ -97,4 +75,11 @@ struct PLOT_ERROR
 	unsigned int edgeMode;
 };
 
+extern const char *RANGEFILE_WX_CONSTANT;
+
+extern const char *DTD_NAME;
+extern const char *PROGRAM_NAME;
+extern const char *PROGRAM_VERSION;
+extern const char *FONT_FILE;
+
 #endif
