@@ -1,6 +1,6 @@
 /*
  *	stringFuncs.cpp - string manipulation routines
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -334,73 +334,6 @@ std::string choiceString(std::vector<std::pair<unsigned int, std::string> > comb
 	return s;
 }
 
-bool parseColString(const std::string &str,
-	unsigned char &r, unsigned char &g, unsigned char &b, unsigned char &a)
-{
-	//Input string is in 2 char hex form, 3 or 4 colour, with # leading. RGB order
-	//lowercase string.
-	if(str.size() != 9 && str.size() != 7)
-		return false;
-
-	if(str[0] != '#')
-		return false;
-
-	string rS,gS,bS,aS;
-	rS=str.substr(1,2);
-	gS=str.substr(3,2);
-	bS=str.substr(5,2);
-
-	if(!isxdigit(rS[0]) || !isxdigit(rS[1]))
-		return false;
-	if(!isxdigit(gS[0]) || !isxdigit(gS[1]))
-		return false;
-	if(!isxdigit(bS[0]) || !isxdigit(bS[1]))
-		return false;
-
-	hexStrToUChar(str.substr(1,2),r);	
-	hexStrToUChar(str.substr(3,2),g);	
-	hexStrToUChar(str.substr(5,2),b);	
-	//3 colour must have a=255.
-	if(str.size() == 7)
-		a = 255;
-	else
-	{
-		aS=str.substr(7,2);
-		if(!isxdigit(aS[0]) || !isxdigit(aS[1]))
-			return false;
-		hexStrToUChar(str.substr(7,2),a);	
-	}
-	return true;
-}
-
-void genColString(unsigned char r, unsigned char g, 
-			unsigned char b, unsigned char a, std::string &s)
-{
-	s="#";
-	string tmp;
-	ucharToHexStr(r,tmp);
-	s+=tmp;
-	ucharToHexStr(g,tmp);
-	s+=tmp;
-	ucharToHexStr(b,tmp);
-	s+=tmp;
-	ucharToHexStr(a,tmp);
-	s+=tmp;
-
-}
-
-void genColString(unsigned char r, unsigned char g, 
-			unsigned char b, std::string &s)
-{
-	string tmp;
-	s="#";
-	ucharToHexStr(r,tmp);
-	s+=tmp;
-	ucharToHexStr(g,tmp);
-	s+=tmp;
-	ucharToHexStr(b,tmp);
-	s+=tmp;
-}
 
 //Strip "whitespace"
 std::string stripWhite(const std::string &str)
@@ -457,6 +390,16 @@ std::string lowercase(std::string s)
 	return s;
 }
 
+std::string uppercase(std::string s)
+{
+	for(unsigned int ui=0;ui<s.size();ui++)
+	{
+		if(isascii(s[ui]) && islower(s[ui]))
+			s[ui] = toupper(s[ui]);
+	}
+	return s;
+}
+
 //Split strings around a delimiter
 void splitStrsRef(const char *cpStr, const char delim,std::vector<string> &v )
 {
