@@ -1,6 +1,6 @@
 /*
  *	drawables.cpp - opengl drawable objects cpp file
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -24,7 +24,9 @@
 
 #include "common/colourmap.h"
 
+#include "common/voxels.h"
 
+#include "glDebug.h"
 
 
 const float DEPTH_SORT_REORDER_EPSILON = 1e-2;
@@ -38,11 +40,13 @@ float DrawableObj::backgroundG;
 float DrawableObj::backgroundB;
 
 bool DrawableObj::useAlphaBlend;
-TexturePool *DrawableObj::texPool;
+TexturePool *DrawableObj::texPool=0;
 
 unsigned int DrawableObj::winX;
 unsigned int DrawableObj::winY;
 
+DrawTexturedQuad DrawPointLegendOverlay::dQuad;
+bool DrawPointLegendOverlay::quadSet=false;
 //==
 
 
@@ -54,7 +58,7 @@ void drawCone(const Point3D &axisVec, const Point3D &origin,
 {
 	Point3D axis;
 	axis =axisVec;
-	if(axis.sqrMag() < sqrt(std::numeric_limits<float>::epsilon()))
+	if(axis.sqrMag() < sqrtf(std::numeric_limits<float>::epsilon()))
 		axis=Point3D(0,0,1);
 	else
 		axis.normalise();
@@ -70,7 +74,7 @@ void drawCone(const Point3D &axisVec, const Point3D &origin,
 	
 	Point3D *ptArray = new Point3D[numSegments];
 
-	const float ROT_TOL=sqrt(std::numeric_limits<float>::epsilon()) ;
+	const float ROT_TOL=sqrtf(std::numeric_limits<float>::epsilon()) ;
 
 	//Only rotate if the angle is nonzero (note 2PI wraparound is possible from acos)
 	if((tiltAngle > ROT_TOL	|| fabs(tiltAngle - 2*M_PI) > ROT_TOL) && 
@@ -229,6 +233,36 @@ DrawableObj::DrawableObj() : active(true), haveChanged(true), canSelect(false),
 DrawableObj::~DrawableObj()
 {
 }
+	
+	
+float DrawableObj::getHighContrastValue() 
+{
+	//Perform luminence check on background to try to create most appropriate
+	// colour
+	//-------
+	// TODO: I have this in a few places now, need to refactor into a single colour class
+
+	//weights
+ 	const float CHANNEL_LUM_WEIGHTS[3] = { 0.299f,0.587f,0.114f};
+	float totalBright=backgroundR*CHANNEL_LUM_WEIGHTS[0] +
+			backgroundG*CHANNEL_LUM_WEIGHTS[1] +
+			backgroundB*CHANNEL_LUM_WEIGHTS[2];
+
+	float contrastCol;
+	if(totalBright > 0.5f)
+	{
+		//"bright" scene, use black text
+		contrastCol=0.0f;
+	}
+	else
+	{
+		//"Dark" background, use white text
+		contrastCol=1.0f;
+	}
+
+	return contrastCol;
+
+}
 
 void DrawableObj::explode(std::vector<DrawableObj *> &simpleObjects)
 {
@@ -248,7 +282,7 @@ void DrawableObj::clearTexPool()
 {
 	ASSERT(texPool);
 	delete texPool;
-
+	texPool=0;
 }
 
 Point3D DrawableObj::getCentroid() const
@@ -369,7 +403,7 @@ void DrawVector::draw() const
 	{
 		//Back off the distance a little, because otherwise the line can poke out
 		// the sides of the cone.
-		float backoffFactor = std::max(radius/sqrt(vector.sqrMag()),0.0f);
+		float backoffFactor = std::max(radius/sqrtf(vector.sqrMag()),0.0f);
 		Point3D tmpVec=vector*(1.0f-backoffFactor) + origin;
 		
 		if(doubleEnded)
@@ -397,7 +431,7 @@ void DrawVector::draw() const
 	glPopAttrib();
 
 	//If we only wanted the line, then we are done here.
-	if(arrowSize < sqrt(std::numeric_limits<float>::epsilon()) || !drawArrow)
+	if(arrowSize < sqrtf(std::numeric_limits<float>::epsilon()) || !drawArrow)
 		return ;
 
 	//Now compute & draw the cone tip
@@ -508,6 +542,27 @@ void DrawQuad::setVertices(const Point3D *v)
 		vertices[ui]=v[ui];
 }
 
+void DrawQuad::setVertex(unsigned int v, const Point3D &p)
+{
+	ASSERT(v <4);
+	vertices[v] = p;
+}
+
+void DrawQuad::setColour(float rNew, float gNew, float bNew, float aNew)
+{
+	ASSERT(rNew >=0 && rNew <=1.0f);
+	ASSERT(gNew >=0 && gNew <=1.0f);
+	ASSERT(bNew >=0 && bNew <=1.0f);
+	ASSERT(aNew >=0 && aNew <=1.0f);
+	for(unsigned int ui=0;ui<4; ui++)
+	{
+		r[ui]=rNew;		
+		g[ui]=gNew;		
+		b[ui]=bNew;		
+		a[ui]=aNew;		
+	}
+}
+
 Point3D DrawQuad::getOrigin() const
 {
 	return Point3D::centroid(vertices,4);
@@ -537,41 +592,66 @@ void DrawQuad::recomputeParams(const vector<Point3D> &vecs,
 	}
 }
 
-DrawTexturedQuad::DrawTexturedQuad() :textureData(0), textureId((unsigned int)-1)
+DrawTexturedQuad::DrawTexturedQuad() :textureData(0), textureId((unsigned int)-1), noColour(false) , needsBinding(true)
 {
 }
 
+DrawTexturedQuad::DrawTexturedQuad(const DrawTexturedQuad &oth)
+{
+	ASSERT(false);
+}
+
 DrawTexturedQuad::~DrawTexturedQuad()
 {
-	if(textureData)
-		delete[] textureData;
-
-	texPool->closeTexture(textureId);
+	//hack to work around static construct/destruct.
+	// normally we use the texture pool do to everything
+	if(texPool && textureId != -1)
+	{
+		texPool->closeTexture(textureId);
+		textureId=-1;
+	}
 }
 
 void DrawTexturedQuad::draw() const
 {
+	if(needsBinding)
+	{
+		rebindTexture();
+	}
+
 	ASSERT(glIsTexture(textureId));
 	
 	glEnable(GL_TEXTURE_2D);
-	glBindTexture(GL_TEXTURE_2D,textureId);
 	glPushAttrib(GL_CULL_FACE);
 	glDisable(GL_CULL_FACE);
 	
 	
 	glBindTexture(GL_TEXTURE_2D,textureId);
-
-	glColor4f(1.0f,1.0f,1.0f,1.0f);
 	const float COORD_SEQ_X[]={ 0,0,1,1};
 	const float COORD_SEQ_Y[]={ 0,1,1,0};
-	glBegin(GL_QUADS);
+
+	if(!noColour)
+	{
+		glBegin(GL_QUADS);
 		for(size_t ui=0;ui<4;ui++)
 		{
+			glColor4f(r[ui],g[ui],b[ui],a[ui]);
 			glTexCoord2f(COORD_SEQ_X[ui],COORD_SEQ_Y[ui]);
 			glVertex3fv(vertices[ui].getValueArr());
 		}
-	glEnd();
-
+		glEnd();
+	}	
+	else
+	{
+		glBegin(GL_QUADS);
+			for(size_t ui=0;ui<4;ui++)
+			{
+				glColor4f(1.0f,1.0f,1.0f,a[ui]);
+				glTexCoord2f(COORD_SEQ_X[ui],COORD_SEQ_Y[ui]);
+				glVertex3fv(vertices[ui].getValueArr());
+			}
+		glEnd();
+	}
 	glPopAttrib();
 	glDisable(GL_TEXTURE_2D);	
 }
@@ -601,32 +681,33 @@ void DrawTexturedQuad::resize(size_t numX, size_t numY,
 
 }
 
-void DrawTexturedQuad::rebindTexture()
+void DrawTexturedQuad::rebindTexture(unsigned int mode) const
 {
 	ASSERT(texPool);
 	ASSERT(textureData);
 	if(textureId == (unsigned int)-1)
 		texPool->genTexID(textureId);
-	
-	//Construc the texture
+
+	ASSERT(!(mode == GL_RGB && channels !=3 ));
+	ASSERT(!(mode == GL_RGBA && channels !=4 ));
+
+	//Construct the texture
 	glBindTexture(GL_TEXTURE_2D,textureId);
 	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE); 
-	ASSERT(channels == 3);
-
-	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,nX,nY,
-		0,GL_RGB,GL_UNSIGNED_BYTE,textureData);
-
 
+	glTexImage2D(GL_TEXTURE_2D,0,mode,nX,nY,
+		0,mode,GL_UNSIGNED_BYTE,textureData);
 
+	needsBinding=false;	
 }
 
 void DrawTexturedQuad::setData(size_t x, size_t y, unsigned char *entry)
 {
 	ASSERT(textureData);
-	ASSERT(channels == 3);
+	ASSERT(x < nX && y < nY);
 
 	for(size_t ui=0;ui<channels;ui++)
 		textureData[(y*nX + x)*channels + ui] = entry[ui]; 	
@@ -760,7 +841,7 @@ void DrawCylinder::draw() const
 	if(!q || !qCap[0] || !qCap[1])
 		return;
 
-	//Cross product desired drection with default
+	//Cross product desired direction with default
 	//direction to produce rotation vector
 	Point3D dir(0.0f,0.0f,1.0f);
 
@@ -772,8 +853,8 @@ void DrawCylinder::draw() const
 
 	float length=sqrtf(direction.sqrMag());
 	float angle = dir.angle(dirNormal);
-	if(angle < M_PI - sqrt(std::numeric_limits<float>::epsilon()) &&
-		angle > sqrt(std::numeric_limits<float>::epsilon()))
+	if(angle < M_PI - sqrtf(std::numeric_limits<float>::epsilon()) &&
+		angle > sqrtf(std::numeric_limits<float>::epsilon()))
 	{
 		//we need to rotate
 		dir = dir.crossProd(dirNormal);
@@ -1232,7 +1313,6 @@ void DrawGLText::draw() const
 			axis.fz=rotateAxis[2];
 
 
-//			cerr << "Gl rotate (1):" << rotateAxis << " , " << angle << endl;
 			glRotatef(angle*180.0f/M_PI,rotateAxis[0],rotateAxis[1],rotateAxis[2]);
 			quat_rot(&tmp,&axis,angle); //angle is in radiians
 
@@ -1250,7 +1330,6 @@ void DrawGLText::draw() const
 			rotateAxis = newUp.crossProd(Point3D(0,-1,0));
 			rotateAxis.normalise();
 			glRotatef(angle*180.0f/M_PI,rotateAxis[0],rotateAxis[1],rotateAxis[2]);
-			//cerr << "Gl rotate (2):" << rotateAxis << " , " << angle << endl;
 		}
 
 		//Ensure that the text is not back-culled (i.e. if the
@@ -1293,7 +1372,7 @@ void DrawGLText::draw() const
 					//the text direction is now the x-axis
 					glTranslatef(advance/2.0f,halfHeight,0);
 					//spin text around its front direction 180 degrees
-					//no need to trnaslate as text sits at its baseline
+					//no need to translate as text sits at its baseline
 					glRotatef(180,0,0,1);
 					//move halfway along text, noting that 
 					//the text direction is now the x-axis
@@ -1640,7 +1719,12 @@ void DrawRectPrism::recomputeParams(const vector<Point3D> &vecs,
 	}
 }
 
+DrawableOverlay::~DrawableOverlay()
+{
+}
+
 DrawTexturedQuadOverlay::DrawTexturedQuadOverlay()  
+:  textureId(-1),textureOK(false)
 {
 }
 
@@ -1649,19 +1733,17 @@ DrawTexturedQuadOverlay::~DrawTexturedQuadOverlay()
 	texPool->closeTexture(textureId);
 }
 
-void DrawTexturedQuadOverlay::setSize(float s)
-{
-	length=s;
-}
-
-
 void DrawTexturedQuadOverlay::draw() const
 {
 	if(!textureOK)
 		return;
 
+	ASSERT(height == width);
+
 	ASSERT(glIsTexture(textureId));
 	
+	//TODO: Is this redundant? might be already handled
+	// by scene?
 	glMatrixMode(GL_PROJECTION);	
 	glPushMatrix();
 	glLoadIdentity();
@@ -1679,13 +1761,13 @@ void DrawTexturedQuadOverlay::draw() const
 	glColor3f(1.0f,1.0f,1.0f);
 	glBegin(GL_QUADS);
 		glTexCoord2f(0.0f,0.0f);
-		glVertex3f(position[0]-length/2.0,position[1]-length/2.0,0.0);
+		glVertex3f(position[0]-height/2.0,position[1]-height/2.0,0.0);
 		glTexCoord2f(0.0f,1.0f);
-		glVertex3f(position[0]-length/2.0,position[1]+length/2.0,0.0);
+		glVertex3f(position[0]-height/2.0,position[1]+height/2.0,0.0);
 		glTexCoord2f(1.0f,1.0f);
-		glVertex3f(position[0]+length/2.0,position[1]+length/2.0,0.0);
+		glVertex3f(position[0]+height/2.0,position[1]+height/2.0,0.0);
 		glTexCoord2f(1.0f,0.0f);
-		glVertex3f(position[0]+length/2.0,position[1]-length/2.0,0.0);
+		glVertex3f(position[0]+height/2.0,position[1]-height/2.0,0.0);
 	glEnd();
 
 	glDisable(GL_TEXTURE_2D);	
@@ -1707,11 +1789,207 @@ bool DrawTexturedQuadOverlay::setTexture(const char *textureFile)
 	return textureOK;
 }
 
-void DrawTexturedQuadOverlay::getBoundingBox(BoundCube &b) const
+
+DrawProgressCircleOverlay::DrawProgressCircleOverlay()
 {
-	b.setInvalid();
+	stepProgress=0;
+	step=0;
+	maxStep=0;
+
 }
 
+DrawProgressCircleOverlay::~DrawProgressCircleOverlay()
+{
+}
+
+void DrawProgressCircleOverlay::reset()
+{
+	stepProgress=0;
+	maxStep=0;
+	totalFilters=0;
+	curFilter=0;	
+}
+
+void DrawProgressCircleOverlay::draw( )const
+{
+	if(!maxStep)
+		return;
+
+	ASSERT(curFilter <=totalFilters);
+	//TODO: Is this redundant? might be already handled
+	// by scene?
+	glMatrixMode(GL_PROJECTION);	
+	glPushMatrix();
+	glLoadIdentity();
+	gluOrtho2D(0, winX, winY, 0);
+	
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	glLoadIdentity();
+
+	glDisable(GL_CULL_FACE);
+	glEnable(GL_DEPTH_TEST);
+	//size of gap to fit, in degrees
+	const float FILTER_SPACING_ANGLE= 20.0f/(float)totalFilters;
+	//Angular step when drawing wheel
+	const float DEG_STEP= 2;
+	
+	//fraction of radius for inner section
+	const float IN_RADIUS_FRACTION=0.85;
+	//compute circle radius
+	float radiusOut,radiusIn;
+	radiusOut = std::min(height,width)/2.0f;
+	radiusIn= radiusOut*IN_RADIUS_FRACTION; 
+
+
+
+	//Allows for gap between each step, and one at start and end
+	float thetaPerFilter = (360.0f - FILTER_SPACING_ANGLE*(totalFilters))/totalFilters;
+		
+	//Draw the complete filters 
+	float curTheta=FILTER_SPACING_ANGLE/2.0f;	
+	for(size_t ui=1;ui<curFilter;ui++)
+	{
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerFilter,true);
+		curTheta+=thetaPerFilter+FILTER_SPACING_ANGLE;
+	}
+
+
+	//Draw the completed Steps
+	float thetaPerStep =  thetaPerFilter/maxStep;
+	for(size_t ui=1;ui<step;ui++)
+	{
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerStep,true);
+		curTheta+=thetaPerStep;
+
+		if(ui < step-1)
+		{
+			//Draw a line to mark the step
+			glColor4f(1.0f,0.0f,0.0f,1.0f);
+			glBegin(GL_LINES);
+				glVertex3f(radiusIn*sin(curTheta),radiusIn*cos(curTheta),0);
+				glVertex3f(radiusOut*sin(curTheta),radiusOut*cos(curTheta),0);
+			glEnd();
+		}
+	}
+	//Draw the current step
+	if(stepProgress == 100)
+	{
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerStep,true);
+	}
+	else if (!stepProgress)
+	{
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerStep,false);
+	}
+	else
+	{
+		//draw two segments, one with the current progress value in the complete style
+		float interpFrac = (float)stepProgress/100.0f;
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerStep*interpFrac,true);
+		// and one with the incomplete style
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta+thetaPerStep*interpFrac, 
+			curTheta+thetaPerStep,false);
+
+	}
+	curTheta+=thetaPerStep;
+	//Draw the remaining incomplete steps
+	for(size_t ui=step+1; ui<=maxStep;ui++)
+	{
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerStep,false);
+		curTheta+=thetaPerStep;
+	}
+
+	curTheta+=FILTER_SPACING_ANGLE;
+
+	//Draw the incomplete filters
+	for(size_t ui=curFilter+1; ui<=totalFilters;ui++)
+	{
+		drawSection(DEG_STEP,
+			IN_RADIUS_FRACTION*radiusIn, 
+			radiusOut,curTheta, 
+			curTheta+thetaPerFilter,false);
+		curTheta+=thetaPerFilter+FILTER_SPACING_ANGLE;
+	}
+	
+	glPopMatrix(); //Pop modelview matrix
+
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+
+	glMatrixMode(GL_MODELVIEW);
+
+	glDisable(GL_DEPTH_TEST);
+	glEnable(GL_CULL_FACE);
+}
+
+void DrawProgressCircleOverlay::drawSection(unsigned int degreeStep, 
+	float rIn, float rOut,float startTheta, float stopTheta, bool complete) const
+{
+	//TODO: all calulations that call this should use radiians	
+	float startThetaRad = startTheta*M_PI/180.0f;
+	float endThetaRad = stopTheta*M_PI/180.0f;
+	float degStepRad = degreeStep*M_PI/180.0f;
+
+	unsigned int nSegments = (endThetaRad-startThetaRad)/degStepRad;
+
+	if(!nSegments)
+		return;
+
+	float alphaBase,dt;
+	getAnimationStat(alphaBase,dt);	
+	if(alphaBase == 0.0f)
+		return;
+
+
+
+
+	const float ALPHA_COMPLETE=0.5*alphaBase;
+	const float ALPHA_INCOMPLETE=0.15*alphaBase;
+	if(complete)	
+		glColor4f(1.0f,1.0f,1.0f,ALPHA_COMPLETE);
+	else
+		glColor4f(1.0f,1.0f,1.0f,ALPHA_INCOMPLETE);
+
+	//Draw arc
+	glBegin(GL_TRIANGLE_STRIP);
+	float thetaOne=startThetaRad;
+	float thetaTwo=startThetaRad+degStepRad;
+	glVertex2f(position[0] + rIn*cos(thetaOne),position[1] + rIn*sin(thetaOne));
+	for(size_t ui=0;ui<nSegments;ui++)
+	{
+		thetaOne=startThetaRad + ui*degStepRad;;
+		thetaTwo=startThetaRad + (ui+1)*degStepRad;
+		
+		glVertex2f(position[0] + rOut*cos(thetaOne),position[1] + rOut*sin(thetaOne));
+		glVertex2f(position[0] + rIn*cos(thetaTwo),position[1] + rIn*sin(thetaTwo));
+
+
+	}
+	glVertex2f(position[0] + rOut*cos(thetaTwo),position[1] + rOut*sin(thetaTwo));
+	glEnd();
+}
+
+
 DrawAnimatedOverlay::DrawAnimatedOverlay()
 {
 	fadeIn=0.0f;
@@ -1738,36 +2016,47 @@ bool DrawAnimatedOverlay::setTexture(const vector<string> &texFiles,
 	return textureOK;
 }
 
-void DrawAnimatedOverlay::setSize(float newLen)
+void DrawAnimatedOverlay::getAnimationStat(float &alpha , float &animDeltaTime) const
 {
-	length=newLen;
-}
+		
+	timeval t;
+	gettimeofday(&t,NULL);
+	animDeltaTime=(float)(t.tv_sec - animStartTime.tv_sec) +
+		(t.tv_usec-animStartTime.tv_usec)/1.0e6;
 
 
+	//Skip if we wish to show later
+	if(animDeltaTime < delayBeforeShow)
+	{
+		alpha= 0;
+		return;
+	}
+
+	animDeltaTime-=delayBeforeShow;
+
+	if(fadeIn > 0.0f && (fadeIn > animDeltaTime) ) 
+	{
+		alpha= (animDeltaTime )/(fadeIn) ;
+	}
+	else
+		alpha= 1.0f;
+	
+}
+
 void DrawAnimatedOverlay::draw() const
 {
 	if(!textureOK)
 		return;
 
-	float texCoordZ;
-	float alphaVal=1.0f;
-	{
-	timeval t;
-	gettimeofday(&t,NULL);
-	float animDeltaTime=(float)(t.tv_sec - animStartTime.tv_sec) +
-		(t.tv_usec-animStartTime.tv_usec)/1.0e6;
+	float alphaVal, animDeltaTime;
+	getAnimationStat(alphaVal,animDeltaTime);
 
-	//Skip if we wish to show later
-	if(animDeltaTime < delayBeforeShow)
+	if(alphaVal== 0.0f)
 		return;
-
-
-	if(fadeIn > 0.0f && (delayBeforeShow + fadeIn > animDeltaTime) ) 
-		alphaVal= (animDeltaTime - delayBeforeShow )/(fadeIn) ;
-
+	float texCoordZ;
 	texCoordZ=fmod(animDeltaTime,repeatInterval);
 	texCoordZ/=repeatInterval;
-	}
+
 	ASSERT(glIsTexture(textureId));
 	
 	glMatrixMode(GL_PROJECTION);	
@@ -1781,19 +2070,24 @@ void DrawAnimatedOverlay::draw() const
 
 	glEnable(GL_TEXTURE_3D);
 	glBindTexture(GL_TEXTURE_3D,textureId);
-	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MULT); 
-	
+
+	//TODO: Find correct blending mode. Default is good, but may change...	
+//	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND); 
+//	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA);
+
 	// Draw overlay quad 
+	ASSERT(width == height); // width/height should be the same
 	glColor4f(1.0f,1.0f,1.0f,alphaVal);
+
 	glBegin(GL_QUADS);
 		glTexCoord3f(0.0f,0.0f,texCoordZ);
-		glVertex3f(position[0]-length/2.0,position[1]-length/2.0,0.0);
+		glVertex3f(position[0]-width/2.0,position[1]-width/2.0,0.0);
 		glTexCoord3f(0.0f,1.0f,texCoordZ);
-		glVertex3f(position[0]-length/2.0,position[1]+length/2.0,0.0);
+		glVertex3f(position[0]-width/2.0,position[1]+width/2.0,0.0);
 		glTexCoord3f(1.0f,1.0f,texCoordZ);
-		glVertex3f(position[0]+length/2.0,position[1]+length/2.0,0.0);
+		glVertex3f(position[0]+width/2.0,position[1]+width/2.0,0.0);
 		glTexCoord3f(1.0f,0.0f,texCoordZ);
-		glVertex3f(position[0]+length/2.0,position[1]-length/2.0,0.0);
+		glVertex3f(position[0]+width/2.0,position[1]-width/2.0,0.0);
 	glEnd();
 
 	glDisable(GL_TEXTURE_3D);	
@@ -1806,10 +2100,7 @@ void DrawAnimatedOverlay::draw() const
 	glMatrixMode(GL_MODELVIEW);
 }
 
-void DrawAnimatedOverlay::getBoundingBox(BoundCube &b) const
-{
-	b.setInvalid();
-}
+
 
 DrawColourBarOverlay::DrawColourBarOverlay() 
 {
@@ -1834,8 +2125,8 @@ DrawColourBarOverlay::DrawColourBarOverlay(const DrawColourBarOverlay &oth)
 	height=oth.height;
 	width=oth.width;
 	
-	tlX=oth.tlX;
-	tlY=oth.tlY;
+	position[0]=oth.position[0];
+	position[1]=oth.position[1];
 };
 
 void DrawColourBarOverlay::draw() const
@@ -1851,52 +2142,30 @@ void DrawColourBarOverlay::draw() const
 		//Set the quad colour for bar element
 		glColor4f(rgb[rgb.size()-(ui+1)].v[0],
 				rgb[rgb.size()-(ui+1)].v[1],
-				rgb[rgb.size()-(ui+1)].v[2],1.0);
+				rgb[rgb.size()-(ui+1)].v[2],a);
 
 		//draw this quad (bar element)
-		glVertex3f(tlX,tlY+(float)ui*elemHeight,0);
-		glVertex3f(tlX,tlY+(float)(ui+1)*elemHeight,0);
-		glVertex3f(tlX+barWidth,tlY+(float)(ui+1)*elemHeight,0);
-		glVertex3f(tlX+barWidth,tlY+(float)(ui)*elemHeight,0);
+		glVertex2f(position[0],position[1]+(float)ui*elemHeight);
+		glVertex2f(position[0],position[1]+(float)(ui+1)*elemHeight);
+		glVertex2f(position[0]+barWidth,position[1]+(float)(ui+1)*elemHeight);
+		glVertex2f(position[0]+barWidth,position[1]+(float)(ui)*elemHeight);
 	}
 
 	glEnd();
 
-	//Perform luminence check on background to try to create most appropriate
-	// colour
-	//-------
-	// TODO: I have this in a few places now, need to refactor into a single colour class
-
-	//weights
- 	const float CHANNEL_LUM_WEIGHTS[3] = { 0.299f,0.587f,0.114f};
-	float totalBright=backgroundR*CHANNEL_LUM_WEIGHTS[0] +
-			backgroundG*CHANNEL_LUM_WEIGHTS[1] +
-			backgroundB*CHANNEL_LUM_WEIGHTS[2];
-
-	float textGrey;
-	if(totalBright > 0.5f)
-	{
-		//"bright" scene, use black text
-		textGrey=0.0f;
-	}
-	else
-	{
-		//"Dark" background, use white text
-		textGrey=1.0f;
-	}
-	
 
 	//-------
 
+	float textGrey=getHighContrastValue();
 	//Draw ticks on colour bar
 	glBegin(GL_LINES);
-		glColor4f(textGrey,textGrey,textGrey,1.0f);
+		glColor4f(textGrey,textGrey,textGrey,a);
 		//Top tick
-		glVertex3f(tlX,tlY,0);
-		glVertex3f(tlX+width,tlY,0);
+		glVertex2f(position[0],position[1]);
+		glVertex2f(position[0]+width,position[1]);
 		//Bottom tick
-		glVertex3f(tlX,tlY+height,0);
-		glVertex3f(tlX+width,tlY+height,0);
+		glVertex2f(position[0],position[1]+height);
+		glVertex2f(position[0]+width,position[1]+height);
 	glEnd();
 
 
@@ -1922,7 +2191,7 @@ void DrawColourBarOverlay::draw() const
 	font->FaceSize(3);
 	glDisable(GL_CULL_FACE);
 	glPushMatrix();
-	glTranslatef(tlX+width,tlY,0);
+	glTranslatef(position[0]+width,position[1],0);
 	string s;
 	stream_cast(s,max);
 	//Note negative sign to flip from y-down screen (opengl) to text dir
@@ -1933,7 +2202,7 @@ void DrawColourBarOverlay::draw() const
 	glPopMatrix();
 
 	glPushMatrix();
-	glTranslatef(tlX+width,tlY+height,0);
+	glTranslatef(position[0]+width,position[1]+height,0);
 	stream_cast(s,min);
 	//Note negative sign to flip from y-down screen (opengl) to text dir
 	//(y up)
@@ -1962,9 +2231,154 @@ void DrawColourBarOverlay::setColourVec(const vector<float> &r,
 
 }
 
-void DrawColourBarOverlay::getBoundingBox(BoundCube &b) const
+DrawPointLegendOverlay::DrawPointLegendOverlay() : enabled(true)
 {
-	b.setInvalid();
+	a=1.0f;
+
+	std::string tmpStr =getDefaultFontFile();
+	font = new FTGLPolygonFont(tmpStr.c_str());
+
+	//check to see if we need to init the texture quad
+	if(!quadSet &&  texPool)
+	{
+
+		dQuad.setUseColouring(false);
+
+		//Create a circular texture
+		const unsigned int N_CHANNELS=4;
+		unsigned int LEG_TEX_SIZE = 256; 
+		unsigned char colourWhite[N_CHANNELS]= { 255,255,255,255 };
+		unsigned char colourBlack[N_CHANNELS]= { 0,0,0,0 };
+
+		//TODO: Convert to single channel texture, to save space?
+		// DrawQuad does not support single channel at this time
+		dQuad.resize(LEG_TEX_SIZE,LEG_TEX_SIZE,N_CHANNELS);
+		const float HALF_CIRCLE_R2 = 0.25; 
+		#pragma omp parallel for
+		for(unsigned int nX=0;nX<LEG_TEX_SIZE;nX++)
+		{
+			float fx;
+			fx= (float) nX/(float)LEG_TEX_SIZE - 0.5;
+			for(unsigned int nY=0;nY<LEG_TEX_SIZE;nY++)
+			{
+				float fy;
+				fy = (float) nY/(float)LEG_TEX_SIZE -0.5;
+				if( fx*fx + fy*fy < HALF_CIRCLE_R2) 
+					dQuad.setData(nX,nY,colourWhite);
+				else
+					dQuad.setData(nX,nY,colourBlack);
+			}
+
+		}
+
+		dQuad.rebindTexture(GL_RGBA);
+
+		quadSet=true;
+
+	}
+}
+
+DrawPointLegendOverlay::~DrawPointLegendOverlay()
+{
+}
+
+DrawableObj *DrawPointLegendOverlay::clone() const
+{
+	DrawPointLegendOverlay *dp = new DrawPointLegendOverlay(*this);
+
+	return dp;
+}
+
+DrawPointLegendOverlay::DrawPointLegendOverlay(const DrawPointLegendOverlay &oth)
+{
+	string f;
+	f=getDefaultFontFile();
+	
+	font = new FTGLPolygonFont(f.c_str());
+	a=oth.a;
+	legendItems = oth.legendItems;
+	enabled = oth.enabled;
+
+	height=oth.height;
+	width=oth.width;
+	
+	position[0]=oth.position[0];
+	position[1]=oth.position[1];
+}
+
+void DrawPointLegendOverlay::draw() const
+{
+
+	if(!enabled || legendItems.empty())
+		return;
+
+	ASSERT(winX >0 && winY > 0);
+	float curX = position[0];
+	float curY = position[1];
+
+	float delta = std::max(std::min(1.0f/legendItems.size(),0.02f),0.05f);
+	float size = delta*0.9f; 
+	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+
+	float maxTextWidth=0;
+
+
+	if(font)	
+		font->FaceSize(1);
+	for(unsigned int ui=0; ui<legendItems.size();ui++)
+	{
+		for(;ui<legendItems.size();ui++)
+		{
+
+			//Draw textured quad (circle)
+			//--
+			dQuad.setVertex(0,Point3D(curX,curY,0));
+			dQuad.setVertex(1,Point3D(curX+size,curY,0));
+			dQuad.setVertex(2,Point3D(curX+size,curY+size,0));
+			dQuad.setVertex(3,Point3D(curX,curY+size,0));
+
+			const RGBFloat *f;
+			f = &legendItems[ui].second;
+			glColor3f(f->v[0],f->v[1],f->v[2]);
+			dQuad.draw();
+
+
+			//--
+
+			//Draw text, if possible
+			if( font && !font->Error())
+			{
+				float textGrey=getHighContrastValue();
+				glColor3f(textGrey,textGrey,textGrey);
+				float fminX,fminY,fminZ;
+				float fmaxX,fmaxY,fmaxZ;
+				font->BBox(legendItems[ui].first.c_str(),fminX,
+						fminY,fminZ,fmaxX,fmaxY,fmaxZ);
+				glPushMatrix();
+				glTranslatef(curX+1.5*size,curY+0.85*size,0.0f);
+				glScalef(size,-size,0);
+				font->Render(legendItems[ui].first.c_str());
+				glPopMatrix();
+				maxTextWidth=std::max(fmaxX-fminX,maxTextWidth);
+			}
+			
+			
+			curY+=delta;
+		}
+
+		curX+=maxTextWidth + size;
+		curY=position[1] + 0.5*delta;
+	}
+	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+}
+
+void DrawPointLegendOverlay::addItem(const std::string &s, float r, float g, float b)
+{
+	RGBFloat rgb;
+	rgb.v[0]=r;
+	rgb.v[1]= g;
+	rgb.v[2]= b;
+	legendItems.push_back(make_pair(s,rgb));
 }
 
 
@@ -2009,7 +2423,7 @@ void DrawField3D::setColourMinMax()
 			
 void DrawField3D::draw() const
 {
-	if(alphaVal < sqrt(std::numeric_limits<float>::epsilon()))
+	if(alphaVal < sqrtf(std::numeric_limits<float>::epsilon()))
 		return;
 
 	ASSERT(field);
@@ -2132,7 +2546,6 @@ void DrawField3D::draw() const
 		drawBox(field->getMinBounds(),field->getMaxBounds(),
 			boxColourR, boxColourG,boxColourB,alphaUse);
 	}
-	//Draw the projections
 }
 
 void DrawField3D::setAlpha(float newAlpha)
@@ -2213,7 +2626,7 @@ void DrawIsoSurface::getBoundingBox(BoundCube &b) const
 
 void DrawIsoSurface::draw() const
 {
-	if(a< sqrt(std::numeric_limits<float>::epsilon()))
+	if(a< sqrtf(std::numeric_limits<float>::epsilon()))
 		return;
 
 	if(!cacheOK)
