@@ -1,6 +1,6 @@
 /*
  *	drawables.h - Opengl drawable objects header
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,15 +19,6 @@
 #ifndef DRAWABLES_H
 #define DRAWABLES_H
 
-#include "textures.h"
-#include "cameras.h"
-#include "isoSurface.h"
-
-//STL includes
-
-
-
-
 //MacOS is "special" and puts it elsewhere
 #ifdef __APPLE__ 
 	#include <OpenGL/glu.h>
@@ -37,6 +28,13 @@
 
 #include <sys/time.h>
 
+#include "textures.h"
+#include "cameras.h"
+#include "isoSurface.h"
+
+template<class T>
+class Voxels;
+
 //TODO: Work out if there is any way of obtaining the maximum 
 //number of items that can be drawn in an opengl context
 //For now Max it out at 10 million (~120MB of vertex data)
@@ -102,6 +100,8 @@ enum
 	DRAW_TYPE_FIELD3D,
 	DRAW_TYPE_ISOSURFACE,
 	DRAW_TYPE_AXIS,
+	DRAW_TYPE_LEGENDOVERLAY,
+	DRAW_TYPE_PROGRESSCIRCLE_OVERLAY,
 };
 
 //TODO: It seems unnecessary to have multiple types for the bind
@@ -151,7 +151,10 @@ class DrawableObj
 
 		static bool useAlphaBlend;
 	
+		//Size of the opengl window
 		static unsigned int winX,winY;
+
+		static float getHighContrastValue();
 	public: 
 		//!Can be selected from openGL viewport interactively?
 		bool canSelect;
@@ -204,7 +207,7 @@ class DrawableObj
 		virtual ~DrawableObj();
 
 		//!If we offer any kind of external pointer interface; use this to do a recomputation as needed. This is needed for selection binding behaviour
-		virtual void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode) {};
+		virtual void recomputeParams(const std::vector<Point3D> &vecs, const std::vector<float> &scalars, unsigned int mode) {};
 		
 		//!Set the current camera
 		static void setCurCamera(const Camera *c){curCamera=c;};
@@ -413,9 +416,6 @@ class DrawTriangle : public DrawableObj
  */
 class DrawQuad : public DrawableObj
 {
-	private:
-		//!Colours of the vertices (rgba colour model)
-		float r[4],g[4],b[4],a[4];
 	protected:
 		//!Vertices of the quad
 		Point3D vertices[4];
@@ -425,6 +425,8 @@ class DrawQuad : public DrawableObj
 		/*! Lighting for this class is per triangle only no
 		 * per vertex lighting */
 		Point3D normal;
+		//!Colours of the vertices (rgba colour model)
+		float r[4],g[4],b[4],a[4];
 	public:
 		//!Constructor
 		DrawQuad() {};
@@ -444,6 +446,9 @@ class DrawQuad : public DrawableObj
 		void setVertices(const Point3D *);
 		//!Set the colour of a vertex
 		void setColour(unsigned int, float r, float g, float b, float a);
+		//!Set the colour of all vertices
+		void setColour(float r, float g, float b, float a);
+
 		//!Update the normal to the surface from vertices
 		/*!Uses the first 3 vertices to calculate the normal.
 		 */
@@ -456,12 +461,13 @@ class DrawQuad : public DrawableObj
 		
 		//!Recompute the internal parameters using the input vector information
 		// i.e. this is used for (eg) mouse interaction
-		void recomputeParams(const vector<Point3D> &vecs, 
-				const vector<float> &scalars, unsigned int mode);
+		void recomputeParams(const std::vector<Point3D> &vecs, 
+				const std::vector<float> &scalars, unsigned int mode);
 };
 
 class DrawTexturedQuad : public DrawQuad
 {
+	private:
 	//TODO: Move this back
 	// into the texture pool
 		unsigned char *textureData;
@@ -469,13 +475,26 @@ class DrawTexturedQuad : public DrawQuad
 		size_t channels;
 		size_t displayMode;
 
+		//FIXME: This should be non-mutable.  We need
+		// to move texture rebinding to a pre-processing step, not at draw time
 		//ID of the texture to use when drawing, -1 if not bound
 		// to opengl
-		unsigned int textureId;
+		mutable unsigned int textureId;
+		
+		//!FTGL font instance
+		FTFont *font;
+		
+		//disallow resetting base colour to white 
+		bool noColour;
+
+		//we can only bind from the main thread.
+		// this is true by default, until the texture is bound
+		mutable bool needsBinding;
 		
 	public:
 		DrawTexturedQuad();
 		~DrawTexturedQuad();
+		DrawTexturedQuad(const DrawTexturedQuad &d);
 
 		//Resize the texture contents, destroying any existing contents
 		void resize(size_t nx, size_t nY, unsigned int nChannels);
@@ -484,82 +503,13 @@ class DrawTexturedQuad : public DrawQuad
 		//Set the specified pixel in the texture to this value 
 		void setData(size_t x, size_t y, unsigned char *entry);
 		//Send the texture to the video card. 
-		void rebindTexture();	
-};
-
-
-class DrawAnimatedOverlay : public DrawQuad
-{
-	private:
-		size_t nX,nY,nZ;
-
-		//ID of the texture to use when drawing, -1 if not bound
-		// to opengl
-		unsigned int textureId;
-
-		float position[2];
-
-		timeval animStartTime;
-
-		bool textureOK;
-
-		//Time delta before repeating animation
-		float repeatInterval;
+		void rebindTexture(unsigned int mode=GL_RGB) const;	
 		
-		//Length to use for the quad to represent icon
-		float length;
-
-		//Time before showing the image
-		float delayBeforeShow;
-
-		//Time for fadein after show
-		float fadeIn;
-
-	public:
-		DrawAnimatedOverlay();
-		~DrawAnimatedOverlay();
-
-		virtual unsigned int getType() const {return DRAW_TYPE_ANIMATEDOVERLAY;}
-
-		//!This is an overlay
-		bool isOverlay() const {return true;};
-
-
-		//!Set the texture position in XY (z is ignored)
-		void setPos(float xp,float yp)
-		{
-			position[0]=xp;
-			position[1]=yp;
-		};
-
-		void setSize(float size);
-
-		//Set the time between repeats for the animation
-		void setRepeatTime(float timeV) { repeatInterval=timeV;}
-
-		//Set the time before the texture appears
-		void setShowDelayTime(float showDelayTime) 
-			{ ASSERT(showDelayTime >=0.0f);  delayBeforeShow = showDelayTime;}
-
-		//Set the time during which the alpha value will be ramped up.
-		// activated after the delay time (ie time before 100% visible is fadeInTime + 
-		//	delayTime.
-		void setFadeInTime(float fadeInTime)
-			{ ASSERT(fadeInTime >=0.0f); fadeIn=fadeInTime;}
-
-		//!Set the texture by name
-		bool setTexture(const vector<string> &textureFiles, float timeRepeat=1.0f);
-
-		void resetTime() ;
-
-		//!Draw object
-		void draw() const;
-
-		void getBoundingBox(BoundCube &b) const ;
-
-		bool isOK() const { return textureOK; }
+		void setUseColouring(bool useColouring) {noColour= !useColouring;};
 };
 
+
+
 //!A sphere drawing 
 class DrawSphere : public DrawableObj
 {
@@ -603,8 +553,8 @@ class DrawSphere : public DrawableObj
 
 		//!Recompute the internal parameters using the input vector information
 		// i.e. this is used for (eg) mouse interaction
-		void recomputeParams(const vector<Point3D> &vecs, 
-				const vector<float> &scalars, unsigned int mode);
+		void recomputeParams(const std::vector<Point3D> &vecs, 
+				const std::vector<float> &scalars, unsigned int mode);
 
 };
 
@@ -665,7 +615,7 @@ class DrawCylinder : public DrawableObj
 		void getBoundingBox(BoundCube &b) const ;
 
 		//!Recompute the internal parameters using the input vector information
-		void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode);
+		void recomputeParams(const std::vector<Point3D> &vecs, const std::vector<float> &scalars, unsigned int mode);
 
 		virtual bool needsDepthSorting() const;
 
@@ -857,8 +807,8 @@ class DrawGLText : public DrawableObj
 		void setAlignment(unsigned int mode);
 		
 		//Binding parameter recomputation
-		void recomputeParams(const vector<Point3D> &vecs, 
-				const vector<float> &scalars, unsigned int mode);
+		void recomputeParams(const std::vector<Point3D> &vecs, 
+				const std::vector<float> &scalars, unsigned int mode);
 };
 
 
@@ -901,7 +851,7 @@ class DrawRectPrism  : public DrawableObj
 		void getBoundingBox(BoundCube &b) const;
 		
 		//!Recompute the internal parameters using the input vector information
-		void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode);
+		void recomputeParams(const std::vector<Point3D> &vecs, const std::vector<float> &scalars, unsigned int mode);
 };
 
 struct RGBFloat
@@ -909,21 +859,39 @@ struct RGBFloat
 	float v[3];
 };
 
-class DrawColourBarOverlay : public DrawableObj
+//Abstract class as base for overlays
+class DrawableOverlay : public DrawableObj
+{
+	protected:
+		//alpha (transparancy) value
+		float a;
+		//!Height and width of overlay (total)
+		float height,width;
+		//Fractional coordinates for the  top left of the overlay
+		float position[2];
+	public:
+		DrawableOverlay() {} ;
+		//Declared as pure virtual to force ABC
+		virtual ~DrawableOverlay() =0;
+		void setAlpha(float alpha) { a=alpha;};
+		void setSize(float widthN, float heightN) {height=heightN, width=widthN;} 
+		void setSize(float size) {width=height=size;};
+		void setPosition(float newTLX,float newTLY) { position[0]=newTLX; position[1]=newTLY;}
+
+		void getBoundingBox(BoundCube &b) const {b.setInvalid();};
+		//!This is an overlay
+		bool isOverlay() const {return true;};
+};
+
+class DrawColourBarOverlay : public DrawableOverlay
 {
 	private:
 		FTFont *font;
 
 		//!Colours for each element
-		vector<RGBFloat> rgb;
-		//alpha (transparancy) value
-		float a;
+		std::vector<RGBFloat> rgb;
 		//!Minimum and maximum values for the colour bar (for ticks)
 		float min,max;
-		//!Height and width of bar (total)
-		float height,width;
-		//!top left of bar
-		float tlX,tlY;
 
 	public:
 	
@@ -933,63 +901,156 @@ class DrawColourBarOverlay : public DrawableObj
 		
 
 		virtual unsigned int getType() const {return DRAW_TYPE_COLOURBAR;}
-		
-	
-		void getBoundingBox(BoundCube &b) const ;
 
-		//!This is an overlay
-		bool isOverlay() const {return true;};
-		void setColourVec(const vector<float> &r,
-					const vector<float> &g,
-					const vector<float> &b);
+		void setColourVec(const std::vector<float> &r,
+					const std::vector<float> &g,
+					const std::vector<float> &b);
 		//!Draw object
 		void draw() const;
 
-		void setAlpha(float alpha) { a=alpha;};
-		void setSize(float widthN, float heightN) {height=heightN, width=widthN;} 
-		void setPosition(float newTLX,float newTLY) { tlX=newTLX; tlY=newTLY;}
 		void setMinMax(float minNew,float maxNew) { min=minNew;max=maxNew;};
 		
 };
 
 //!A class to hande textures to draw
-class DrawTexturedQuadOverlay : public DrawableObj
+class DrawTexturedQuadOverlay : public DrawableOverlay
 {
 	private:
 		unsigned int textureId;
-		//Fractional coordinates for the 
-		float position[2];
 	
-		//Length of the rectangle to use for the icon
-		float length;
-
 		bool textureOK;
+
 	public:
 		DrawTexturedQuadOverlay();
 		~DrawTexturedQuadOverlay();
-		
-		virtual unsigned int getType() const {return DRAW_TYPE_TEXTUREDOVERLAY;}
 
-		//!This is an overlay
-		bool isOverlay() const {return true;};
+		virtual unsigned int getType() const {return DRAW_TYPE_TEXTUREDOVERLAY;}
 	
 		static void setWindowSize(unsigned int x, unsigned int y){winX=x;winY=y;};	
-
-		//!Set the texture position in XY (z is ignored)
-		void setPos(float xp,float yp){position[0]=xp; position[1]=yp;};
-
-		void setSize(float size);
-
 		//!Set the texture by name
 		bool setTexture(const char *textureFile);
+		//!Draw object
+		void draw() const;
+};
 
+//!Multi-frame texture - Animated overlay
+class DrawAnimatedOverlay : public DrawableOverlay
+{
+	private:
+		//ID of the texture to use when drawing, -1 if not bound
+		// to opengl
+		unsigned int textureId;
+
+		timeval animStartTime;
+
+		bool textureOK;
+
+		//Time delta before repeating animation
+		float repeatInterval;
+		
+		//Time before showing the image
+		float delayBeforeShow;
+
+		//Time for fadein after show
+		float fadeIn;
+
+	protected:		
+		void getAnimationStat(float &alpha, float &deltaTime) const;
+	public:
+		DrawAnimatedOverlay();
+		~DrawAnimatedOverlay();
+
+		virtual unsigned int getType() const {return DRAW_TYPE_ANIMATEDOVERLAY;}
+
+		//Set the time between repeats for the animation
+		void setRepeatTime(float timeV) { repeatInterval=timeV;}
+
+		//Set the time before the texture appears
+		void setShowDelayTime(float showDelayTime) 
+			{ ASSERT(showDelayTime >=0.0f);  delayBeforeShow = showDelayTime;}
+
+		//Set the time during which the alpha value will be ramped up.
+		// activated after the delay time (ie time before 100% visible is fadeInTime + 
+		//	delayTime.
+		void setFadeInTime(float fadeInTime)
+			{ ASSERT(fadeInTime >=0.0f); fadeIn=fadeInTime;}
+
+		//!Set the texture by name
+		bool setTexture(const std::vector<std::string> &textureFiles, float timeRepeat=1.0f);
+
+		void resetTime() ;
 
 		//!Draw object
 		void draw() const;
 
-		void getBoundingBox(BoundCube &b) const ;
+		bool isOK() const { return textureOK; }
 };
 
+//!Draw a progress (segments with completion) overlay
+class DrawProgressCircleOverlay : public DrawAnimatedOverlay
+{
+
+	//Shows the progress of K filters, each with M steps,  and each
+	// step has a (0-100) progress. Result is drawn as filled arcs.
+	// Each filter is one arc, and this is divided into steps.
+	// each step then fills up
+
+	private:
+		//Progress in the current step, range [0,100]
+		unsigned int stepProgress;
+		//Number of steps in process 
+		unsigned int maxStep;
+		//The step that we are currently in
+		unsigned int step;
+		//Number of filters that are to be analysed 
+		unsigned int totalFilters;
+		//Filter that we are analysing (0->n-1)
+		unsigned int curFilter;
+		
+
+		//Draw a 2D wheel shaped section. Complete variable toggles the style of the drawing from a completed, to a n incompleted segment
+		void drawSection(unsigned int degreeStep, 
+	float rIn, float rOut,float startTheta, float stopTheta, bool complete) const;
+	public:
+		DrawProgressCircleOverlay();
+		~DrawProgressCircleOverlay();
+		void setCurFilter(unsigned int v) { curFilter= v;}
+		void setMaxStep(unsigned int v) { maxStep= v;}
+		void setNumFilters(unsigned int v) { totalFilters= v;}
+		void setProgress(unsigned int newProg) { ASSERT(newProg <=100); stepProgress = newProg;}
+		void setStep(unsigned int v) { ASSERT(v<=maxStep); step= v;}
+
+		void reset();
+
+		static void setWindowSize(unsigned int x, unsigned int y){winX=x;winY=y;};
+		virtual unsigned int getType() const {return DRAW_TYPE_PROGRESSCIRCLE_OVERLAY;}
+		void draw() const;
+};
+
+class DrawPointLegendOverlay : public DrawableOverlay
+{
+	private:
+	static DrawTexturedQuad dQuad;
+	static bool quadSet;
+
+	FTFont *font;
+	//Items to draw n overlay, and colour to use to draw
+	std::vector<std::pair<std::string,RGBFloat> > legendItems;
+	bool enabled;
+	public:
+		DrawPointLegendOverlay();
+		~DrawPointLegendOverlay();
+		
+		DrawPointLegendOverlay(const DrawPointLegendOverlay &);
+
+		DrawableObj *clone() const;
+		virtual unsigned int getType() const {return DRAW_TYPE_LEGENDOVERLAY;}
+		void draw() const;
+
+		void clear(); 
+		void addItem(const std::string &s, float r, float g, float b);
+
+};
 
 struct RGBThis
 {
@@ -1090,7 +1151,7 @@ private:
 
 	Voxels<float> *voxels;	
 
-	mutable vector<TriangleWithVertexNorm> mesh;
+	mutable std::vector<TriangleWithVertexNorm> mesh;
 
 	//!Warning. Although I declare this as const, I do some naughty mutating to the cache.
 	void updateMesh() const;	
