@@ -1,6 +1,6 @@
 /*
  *	StashDialog.cpp - filter "Stash" tree editing and viewing dialog
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -20,13 +20,17 @@
 
 #include "wx/wxcommon.h"
 #include "wx/wxcomponents.h"
+#include "wx/propertyGridUpdater.h"
 #include "common/translation.h"
 
 #include "./backend/viscontrol.h"
 
+#include <stack>
+
 using std::pair;
 using std::string;
 using std::stack;
+using std::vector;
 // begin wxGlade: ::extracode
 
 // end wxGlade
@@ -42,19 +46,23 @@ StashDialog::StashDialog(wxWindow* parent, int id, const wxString& title, const
     wxDialog(parent, id, title, pos, size, style)
 {
     // begin wxGlade: StashDialog::StashDialog
-    label_5 = new wxStaticText(this, wxID_ANY, wxTRANS("Stashes"));
+    label_5 = new wxStaticText(this, wxID_ANY, TRANS("Stashes"));
     listStashes = new wxListCtrl(this, ID_LIST_STASH, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxSUNKEN_BORDER);
     btnRemove = new wxButton(this, wxID_REMOVE, wxEmptyString);
-    label_6 = new wxStaticText(this, wxID_ANY, wxTRANS("Stashed Tree"));
+    label_6 = new wxStaticText(this, wxID_ANY, TRANS("Stashed Tree"));
     treeFilters = new wxTreeCtrl(this, ID_TREE_FILTERS, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS|wxTR_NO_LINES|wxTR_DEFAULT_STYLE|wxSUNKEN_BORDER|wxTR_HIDE_ROOT);
-    label_7 = new wxStaticText(this, wxID_ANY, wxTRANS("Properties"));
-    gridProperties = new wxCustomPropGrid(this, ID_GRID_FILTER);
+    label_7 = new wxStaticText(this, wxID_ANY, TRANS("Properties"));
+    gridProperties = new wxPropertyGrid(this, ID_GRID_FILTER);
     btnOK = new wxButton(this, wxID_OK, wxEmptyString);
+    
+    //Due to a bug in wx, empty reports throw an assertion
+    // 'unknown list item format", when there are no columns
+    listStashes->InsertColumn(0,TRANS("Stash Name"));
+    listStashes->InsertColumn(1,TRANS("Filter Count"));
 
     set_properties();
     do_layout();
     // end wxGlade
-    gridProperties->CreateGrid(0, 2);
 
 }
 
@@ -71,7 +79,7 @@ BEGIN_EVENT_TABLE(StashDialog, wxDialog)
     EVT_BUTTON(wxID_REMOVE, StashDialog::OnBtnRemove)
     EVT_LIST_ITEM_SELECTED(ID_LIST_STASH, StashDialog::OnListSelected)
     EVT_TREE_SEL_CHANGED(ID_TREE_FILTERS, StashDialog::OnTreeSelChange)
-    EVT_GRID_CMD_EDITOR_SHOWN(ID_GRID_FILTER,StashDialog::OnGridEditor)
+    EVT_PG_CHANGING(ID_GRID_FILTER,StashDialog::OnGridEditor)
     // end wxGlade
 END_EVENT_TABLE();
 
@@ -83,18 +91,21 @@ void StashDialog::setVisController(VisController *s)
 void StashDialog::set_properties()
 {
     // begin wxGlade: StashDialog::set_properties
-    SetTitle(wxTRANS("Stashed Trees"));
+    SetTitle(TRANS("Stashed Trees"));
     SetSize(wxSize(600, 430));
 
-    btnRemove->SetToolTip(wxTRANS("Erase stashed item"));
-    treeFilters->SetToolTip(wxTRANS("Filter view for current stash"));
-    gridProperties->SetToolTip(wxTRANS("Settings for selected filter in current stash"));
-    listStashes->SetToolTip(wxTRANS("Available stashes"));
+    btnRemove->SetToolTip(TRANS("Erase stashed item"));
+    treeFilters->SetToolTip(TRANS("Filter view for current stash"));
+    gridProperties->SetToolTip(TRANS("Settings for selected filter in current stash"));
+    listStashes->SetToolTip(TRANS("Available stashes"));
     // end wxGlade
 }
 
-void StashDialog::OnGridEditor(wxGridEvent &evt)
+void StashDialog::OnGridEditor(wxPropertyGridEvent &evt)
 {
+	//Silence error mesages
+	evt.SetValidationFailureBehavior(0);
+	
 	evt.Veto();
 }
 
@@ -114,7 +125,7 @@ void StashDialog::OnListKeyDown(wxListEvent &event)
 				if ( item == -1 )
 					break;
 
-				visControl->eraseStash(listStashes->GetItemData(item));
+				visControl->state.eraseStash(listStashes->GetItemData(item));
 			}
 			
 			//Update the filter list
@@ -140,43 +151,38 @@ void StashDialog::OnTreeSelChange(wxTreeEvent &event)
 void StashDialog::updateList()
 {
 	//Generate the stash selection list
-	//
-	vector<pair<string,unsigned int> > stashes;
-
-	visControl->getStashes(stashes);
 
 	//Clear the existing list
-	listStashes->ClearAll();
-	
+	listStashes->Freeze();
+	listStashes->DeleteAllItems();
+
+	unsigned int nStashes=visControl->state.getStashCount();	
 	//Fill it with "stash" entries
 	//Add columns to report listviews
-	listStashes->InsertColumn(0,wxTRANS("Stash Name"),3);
-	listStashes->InsertColumn(1,wxTRANS("Filter Count"),1);
-	for (unsigned int ui=0; ui<stashes.size(); ui++)
+	for (unsigned int ui=0; ui<nStashes; ui++)
 	{
 		string strTmp;
-		FilterTree t;
+		pair<std::string,FilterTree> stash;
 		long itemIdx;
 		
 		//First item is the stash name
-		itemIdx = listStashes->InsertItem(ui,wxStr(stashes[ui].first));
+		itemIdx = listStashes->InsertItem(ui,stash.first);
 
 		//Second column is num filters
-		
-		visControl->getStashTree(stashes[ui].second,t);
-		stream_cast(strTmp,t.size());
-		listStashes->SetItem(ui,1,wxStr(strTmp));
+		visControl->state.copyStashedTree(ui,stash);
+		stream_cast(strTmp,stash.second.size());
+		listStashes->SetItem(itemIdx,1,(strTmp));
 
 		//Set the stash ID as the list data item
 		//this is the key to the stash val
-		listStashes->SetItemData(itemIdx,stashes[ui].second);	
+		listStashes->SetItemData(itemIdx,ui);	
 	}
-
+	listStashes->Thaw();
 }
 
 void StashDialog::updateGrid()
 {
-	gridProperties->clear();
+	gridProperties->Clear();
 	if(!treeFilters->GetCount())
 		return;
 	//Get the selection from the current tree
@@ -198,7 +204,7 @@ void StashDialog::updateGrid()
 	unsigned int filterIdx = ((wxTreeUint *)tData)->value;
 
 	FilterTree t;
-	visControl->getStashTree(stashId,t);
+	visControl->state.copyStashedTree(stashId,t);
 
 	Filter *targetFilter=0;
 	unsigned int pos=0;
@@ -214,31 +220,9 @@ void StashDialog::updateGrid()
 	}
 	
 	ASSERT(targetFilter);	
-	
-	FilterPropGroup p;
-	targetFilter->getProperties(p);
 
-	gridProperties->clearKeys();
-	gridProperties->setNumGroups(p.numGroups());
-	//Create the keys for the property grid to do its thing
-	for(unsigned int ui=0;ui<p.numGroups();ui++)
-	{
-		vector<FilterProperty> propGrouping;
-		p.getGroup(ui,propGrouping);
+	updateFilterPropertyGrid(gridProperties,targetFilter,"");	
 
-		for(size_t uj=0;uj<propGrouping.size();uj++)
-		{
-			gridProperties->addKey(propGrouping[uj].name,ui,
-				propGrouping[uj].key,
-				propGrouping[uj].type,
-				propGrouping[uj].data,
-				propGrouping[uj].helpText);
-		}
-	}
-
-	//Let the property grid layout what it needs to
-	gridProperties->propertyLayout();
-	
 }
 
 bool StashDialog::getStashIdFromList(unsigned int &stashId)
@@ -282,7 +266,7 @@ void StashDialog::updateTree()
 	if(!getStashIdFromList(stashId))
 		return;
 
-	visControl->getStashTree(stashId,curTree);
+	visControl->state.copyStashedTree(stashId,curTree);
 
 
 	uniqueIds.clear();
@@ -326,7 +310,7 @@ void StashDialog::updateTree()
 	
 		//This will use the user label or the type string.	
 		tid=treeFilters->AppendItem(treeIDs.top(),
-			wxStr((*filtIt)->getUserString()));
+			((*filtIt)->getUserString()));
 		
 		treeFilters->SetItemData(tid,new wxTreeUint(pos));
 		pos++;
@@ -354,7 +338,7 @@ void StashDialog::OnBtnRemove(wxCommandEvent &event)
 		if ( item == -1 )
 			break;
 
-		visControl->eraseStash(listStashes->GetItemData(item));
+		visControl->state.eraseStash(listStashes->GetItemData(item));
 		updateList();
 		updateTree();
 		updateGrid();
