@@ -1,6 +1,6 @@
 /*
  *	glPane.cpp - OpenGL panel implementation
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 
 #include "common/stringFuncs.h"
 #include "gl/select.h"
-#include "gl/tr.h"
 #include "glPane.h"
 
 
@@ -46,6 +45,7 @@
     #define GL_BGR 0x80E0
 #endif
 
+using std::string;
 
 enum
 {
@@ -67,7 +67,6 @@ EVT_RIGHT_DOWN(BasicGLPane::mouseDown)
 EVT_LEAVE_WINDOW(BasicGLPane::mouseLeftWindow)
 EVT_SIZE(BasicGLPane::resized)
 EVT_KEY_DOWN(BasicGLPane::keyPressed)
-EVT_CHAR(BasicGLPane::charEvent) 
 EVT_KEY_UP(BasicGLPane::keyReleased)
 EVT_MOUSEWHEEL(BasicGLPane::mouseWheelMoved)
 EVT_PAINT(BasicGLPane::render)
@@ -89,12 +88,7 @@ int attribList[] = {WX_GL_RGBA,
 			1,
 			0,0};
 
-BasicGLPane::BasicGLPane(wxWindow* parent) :
-#if wxCHECK_VERSION(2,9,0)
-wxGLCanvas(parent, wxID_ANY,  attribList)
-#else
-wxGLCanvas(parent, wxID_ANY,  wxDefaultPosition, wxDefaultSize, 0, wxT("GLCanvas"),attribList)
-#endif
+BasicGLPane::BasicGLPane(wxWindow* parent) : wxGLCanvas(parent, wxID_ANY,  attribList)
 {
 	haveCameraUpdates=false;
 	applyingDevice=false;
@@ -102,9 +96,7 @@ wxGLCanvas(parent, wxID_ANY,  wxDefaultPosition, wxDefaultSize, 0, wxT("GLCanvas
 
 	keyDoubleTapTimer=new wxTimer(this,ID_KEYPRESS_TIMER);
 	lastKeyDoubleTap=(unsigned int)-1;
-#if wxCHECK_VERSION(2,9,0)
 	context=0;
-#endif
 
 	mouseMoveFactor=mouseZoomFactor=1.0f;	
 	dragging=false;
@@ -121,26 +113,18 @@ BasicGLPane::~BasicGLPane()
 
 bool BasicGLPane::displaySupported() const
 {
-#if wxCHECK_VERSION(2,9,0)
 	return IsDisplaySupported(attribList);
-#else
-	ASSERT(false);
-	//Lets hope so. If its not, then its just going to fail anyway. 
-	//If it is, then returning false would simply create a roadblock.
-	//Either way, you shouldn't get here.
-	return true; 
-#endif
 }
 
 void BasicGLPane::setSceneInteractionAllowed(bool enabled)
 {
-	currentScene.lockInteraction(!enabled);
+	currentScene->lockInteraction(!enabled);
 }
 
 unsigned int  BasicGLPane::selectionTest(const wxPoint &p,bool &shouldRedraw)
 {
 
-	if(currentScene.isInteractionLocked())
+	if(currentScene->isInteractionLocked())
 	{
 		shouldRedraw=false;
 		return -1; 
@@ -161,8 +145,8 @@ unsigned int  BasicGLPane::selectionTest(const wxPoint &p,bool &shouldRedraw)
 	gluPickMatrix(p.x, oldViewport[3]-p.y,5, 5, oldViewport);
 	glMatrixMode(GL_MODELVIEW);
 
-	int lastSelected = currentScene.getLastSelected();
-	int selectedObject=currentScene.glSelect();
+	int lastSelected = currentScene->getLastSelected();
+	int selectedObject=currentScene->glSelect();
 
 	//If the object selection hasn't changed, we don't need to redraw
 	//if it has changed, we should redraw
@@ -182,7 +166,7 @@ unsigned int  BasicGLPane::selectionTest(const wxPoint &p,bool &shouldRedraw)
 unsigned int  BasicGLPane::hoverTest(const wxPoint &p,bool &shouldRedraw)
 {
 
-	if(currentScene.isInteractionLocked())
+	if(currentScene->isInteractionLocked())
 	{
 		shouldRedraw=false;
 		return -1;
@@ -199,15 +183,15 @@ unsigned int  BasicGLPane::hoverTest(const wxPoint &p,bool &shouldRedraw)
 	gluPickMatrix(p.x, oldViewport[3]-p.y,5, 5, oldViewport);
 	glMatrixMode(GL_MODELVIEW);
 
-	unsigned int lastHover = currentScene.getLastHover();
-	unsigned int hoverObject=currentScene.glSelect(false);
+	unsigned int lastHover = currentScene->getLastHover();
+	unsigned int hoverObject=currentScene->glSelect(false);
 
 	//FIXME: Should be able to make this more efficient	
 	shouldRedraw =  lastHover!=(unsigned int)-1;
 
 	//Set the scene's hover value
-	currentScene.setLastHover(hoverObject);
-	currentScene.setHoverMode(hoverObject != (unsigned int)-1);
+	currentScene->setLastHover(hoverObject);
+	currentScene->setHoverMode(hoverObject != (unsigned int)-1);
 
 	//Restore the previous matirx
 	glPopMatrix();
@@ -226,16 +210,6 @@ void BasicGLPane::forceRedraw()
 	wxPaintEvent ptEvent;
 	wxPostEvent(this,ptEvent);
 
-	//Hack to fix panel redraw commands not working under windows
-	//--
-#if defined(__WIN32) || defined(__WIN64)
-	wxYield();
-
-	Hide();
-	Show();
-
-#endif
-	//--
 }
 
 // some useful events to use
@@ -253,7 +227,7 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 
 	if(selectionMode )
 	{
-		if(currentScene.isInteractionLocked())
+		if(currentScene->isInteractionLocked())
 		{
 			event.Skip();
 			return;
@@ -271,21 +245,13 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 		if(wxm.ShiftDown())
 			keyFlags|=FLAG_SHIFT;
 
-#if wxCHECK_VERSION(2,9,0)
 		if(wxm.LeftIsDown())
 		       	mouseFlags|= SELECT_BUTTON_LEFT;
 		if(wxm.RightIsDown())
 		       	mouseFlags|= SELECT_BUTTON_RIGHT;
 		if(wxm.MiddleIsDown())
 		       	mouseFlags|= SELECT_BUTTON_MIDDLE;
-#else
-		if(wxm.LeftDown())
-		       	mouseFlags|= SELECT_BUTTON_LEFT;
-		if(wxm.RightDown())
-		       	mouseFlags|= SELECT_BUTTON_RIGHT;
-		if(wxm.MiddleDown())
-		       	mouseFlags|= SELECT_BUTTON_MIDDLE;
-#endif
+		
 		//We can get  a mouse move event which reports no buttons before a mouse-up event,
 		//this occurs frequently under windows, but sometimes under GTK
 		if(!mouseFlags)
@@ -298,7 +264,7 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 		GetClientSize(&w, &h);
 
 
-		currentScene.applyDevice((float)draggingStart.x/(float)w,
+		currentScene->applyDevice((float)draggingStart.x/(float)w,
 					(float)draggingStart.y/(float)h,
 					 p.x/(float)w,p.y/(float)h,
 					 keyFlags,mouseFlags,
@@ -335,8 +301,8 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 	{
 		//Commit the current temp cam using the last camera rate
 		//and then restart the motion.
-		if(!lastMoveShiftDown && currentScene.haveTempCam())
-			currentScene.commitTempCam();
+		if(!lastMoveShiftDown && currentScene->haveTempCam())
+			currentScene->commitTempCam();
 
 		camMultRate*=5.0f;
 
@@ -347,8 +313,8 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 	{
 		//Commit the current temp cam using the last camera rate
 		//and then restart the motion.
- 		if(lastMoveShiftDown && currentScene.haveTempCam())
-			currentScene.commitTempCam();
+ 		if(lastMoveShiftDown && currentScene->haveTempCam())
+			currentScene->commitTempCam();
 
 		lastMoveShiftDown=false;
 	}
@@ -383,22 +349,22 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 	switch(camMode)
 	{
 		case CAM_TRANSLATE:
-			currentScene.discardTempCam();
-			currentScene.setTempCam();
-			currentScene.getTempCam()->translate(lrMove,-udMove);
+			currentScene->discardTempCam();
+			currentScene->setTempCam();
+			currentScene->getTempCam()->translate(lrMove,-udMove);
 			break;
 		case CAM_PIVOT:
-			currentScene.discardTempCam();
-			currentScene.setTempCam();
-			currentScene.getTempCam()->pivot(lrMove,udMove);
+			currentScene->discardTempCam();
+			currentScene->setTempCam();
+			currentScene->getTempCam()->pivot(lrMove,udMove);
 			break;
 		case CAM_MOVE:
-			currentScene.setTempCam();
-			currentScene.getTempCam()->move(lrMove,udMove);
+			currentScene->setTempCam();
+			currentScene->getTempCam()->move(lrMove,udMove);
 			break;
 		case CAM_ROLL:
-			currentScene.setTempCam();
-			currentScene.getTempCam()->roll(atan2(udMove,lrMove));
+			currentScene->setTempCam();
+			currentScene->getTempCam()->roll(atan2(udMove,lrMove));
 						
 			break;	
 		default:
@@ -409,7 +375,7 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 	if(!event.m_leftDown)
 	{
 		dragging=false;
-		currentScene.commitTempCam();
+		currentScene->commitTempCam();
 	}
 	
 	haveCameraUpdates=true;
@@ -427,7 +393,7 @@ void BasicGLPane::mouseDown(wxMouseEvent& event)
 	//a temp cam is activated in the scene, or a binding refresh is underway,
 	//which is currently considered bad
 	if(!dragging && !applyingDevice && !selectionMode 
-			&& !currentScene.isInteractionLocked())
+			&& !currentScene->isInteractionLocked())
 	{
 		//Check to see if the user has clicked an object in the scene
 		bool redraw;
@@ -435,11 +401,11 @@ void BasicGLPane::mouseDown(wxMouseEvent& event)
 
 
 		//If the selected object is valid, then
-		//we did select an object. Treat this as a seletion event
-		if(currentScene.getLastSelected() != (unsigned int)-1)
+		//we did select an object. Treat this as a selection event
+		if(currentScene->getLastSelected() != (unsigned int)-1)
 		{
 			selectionMode=true;
-			currentScene.setSelectionMode(true);
+			currentScene->setSelectionMode(true);
 		}
 		else
 		{
@@ -465,7 +431,6 @@ void BasicGLPane::mouseDown(wxMouseEvent& event)
 			Refresh();
 	}
 
-	event.Skip();
 }
 
 void BasicGLPane::mouseWheelMoved(wxMouseEvent& event) 
@@ -481,20 +446,19 @@ void BasicGLPane::mouseWheelMoved(wxMouseEvent& event)
 
 	cameraMoveRate*=CAMERA_SCROLL_RATE;
 	//Move by specified delta
-	currentScene.getActiveCam()->forwardsDolly(cameraMoveRate);
+	currentScene->getActiveCam()->forwardsDolly(cameraMoveRate);
 
 	//if we are using a temporary camera, update that too
-	if(currentScene.haveTempCam())
-		currentScene.getTempCam()->forwardsDolly(cameraMoveRate);
+	if(currentScene->haveTempCam())
+		currentScene->getTempCam()->forwardsDolly(cameraMoveRate);
 
 	haveCameraUpdates=true;
 	Refresh();
-	event.Skip();
 }
 
 void BasicGLPane::mouseReleased(wxMouseEvent& event) 
 {
-	if(currentScene.isInteractionLocked())
+	if(currentScene->isInteractionLocked())
 	{
 		event.Skip();
 		return;
@@ -513,7 +477,7 @@ void BasicGLPane::mouseReleased(wxMouseEvent& event)
 			applyingDevice=true;
 
 
-			currentScene.applyDevice((float)draggingStart.x/(float)w,
+			currentScene->applyDevice((float)draggingStart.x/(float)w,
 						(float)draggingStart.y/(float)h,
 						 p.x/(float)w,p.y/(float)h,
 						 lastKeyFlags,lastMouseFlags,
@@ -523,7 +487,7 @@ void BasicGLPane::mouseReleased(wxMouseEvent& event)
 
 
 			selectionMode=false;
-			currentScene.setSelectionMode(selectionMode);
+			currentScene->setSelectionMode(selectionMode);
 
 			Refresh();
 		}
@@ -532,15 +496,14 @@ void BasicGLPane::mouseReleased(wxMouseEvent& event)
 	}
 	
 
-	if(currentScene.haveTempCam())
-		currentScene.commitTempCam();
-	currentScene.finaliseCam();
+	if(currentScene->haveTempCam())
+		currentScene->commitTempCam();
+	currentScene->finaliseCam();
 
 	haveCameraUpdates=true;
 	dragging=false;
 
 	Refresh();
-	event.Skip();
 	
 }
 
@@ -558,14 +521,14 @@ void BasicGLPane::mouseLeftWindow(wxMouseEvent& event)
 		GetClientSize(&w, &h);
 
 		applyingDevice=true;
-		currentScene.applyDevice((float)draggingStart.x/(float)w,
+		currentScene->applyDevice((float)draggingStart.x/(float)w,
 					(float)draggingStart.y/(float)h,
 					 p.x/(float)w,p.y/(float)h,
 					 lastKeyFlags,lastMouseFlags,
 					 true);
 
 		selectionMode=false;
-		currentScene.setSelectionMode(selectionMode);
+		currentScene->setSelectionMode(selectionMode);
 		Refresh();
 		applyingDevice=false;
 
@@ -576,13 +539,12 @@ void BasicGLPane::mouseLeftWindow(wxMouseEvent& event)
 
 	if(event.m_leftDown)
 	{
-		if(currentScene.haveTempCam())
+		if(currentScene->haveTempCam())
 		{
-			currentScene.commitTempCam();
+			currentScene->commitTempCam();
 			dragging=false;
 		}
 	}
-	event.Skip();
 }
 
 void BasicGLPane::keyPressed(wxKeyEvent& event) 
@@ -600,11 +562,7 @@ void BasicGLPane::keyPressed(wxKeyEvent& event)
             // needs to be control in apple as cmd-space open spotlight
 			unsigned int keyMask;
 #ifdef __APPLE__
-    #if wxCHECK_VERSION(2,9,0)
 			keyMask = (event.RawControlDown() ? 1 : 0);
-    #else
-			keyMask = (event.ControlDown() ? 1 : 0);
-    #endif
 #else
 			keyMask = (event.CmdDown() ? 1 : 0);
 #endif
@@ -666,8 +624,8 @@ void BasicGLPane::keyPressed(wxKeyEvent& event)
 				}
 
 				
-				currentScene.ensureVisible(visibleDir);
-				parentStatusBar->SetStatusText(wxTRANS("Use shift/ctrl-space or double tap to alter reset axis"));
+				currentScene->ensureVisible(visibleDir);
+				parentStatusBar->SetStatusText(TRANS("Use shift/ctrl-space or double tap to alter reset axis"));
 				parentStatusBar->SetBackgroundColour(*wxCYAN)
 					;
 				parentStatusTimer->Start(statusDelay,wxTIMER_ONE_SHOT);
@@ -676,8 +634,9 @@ void BasicGLPane::keyPressed(wxKeyEvent& event)
 			}
 		}
 		break;
+		default:
+			event.Skip();
 	}
-	event.Skip();
 }
 
 void BasicGLPane::setGlClearColour(float r, float g, float b)
@@ -686,7 +645,7 @@ void BasicGLPane::setGlClearColour(float r, float g, float b)
 	ASSERT(g >= 0.0f && g <= 1.0f);
 	ASSERT(b >= 0.0f && b <= 1.0f);
 	
-	currentScene.setBackgroundColour(r,g,b);
+	currentScene->setBackgroundColour(r,g,b);
 	
 	Refresh();
 }
@@ -707,9 +666,9 @@ void BasicGLPane::keyReleased(wxKeyEvent& event)
 		case WXK_SUBTRACT:
 		{
 			//Do a backwards dolly by fixed amount
-			currentScene.getActiveCam()->forwardsDolly(cameraMoveRate);
-			if(currentScene.haveTempCam())
-				currentScene.getTempCam()->forwardsDolly(cameraMoveRate);
+			currentScene->getActiveCam()->forwardsDolly(cameraMoveRate);
+			if(currentScene->haveTempCam())
+				currentScene->getTempCam()->forwardsDolly(cameraMoveRate);
 			break;
 		}
 		case '+':
@@ -722,30 +681,21 @@ void BasicGLPane::keyReleased(wxKeyEvent& event)
 			cameraMoveRate= -cameraMoveRate;
 			
 			//Do a forwards dolly by fixed amount
-			currentScene.getActiveCam()->forwardsDolly(cameraMoveRate);
-			if(currentScene.haveTempCam())
-				currentScene.getTempCam()->forwardsDolly(cameraMoveRate);
+			currentScene->getActiveCam()->forwardsDolly(cameraMoveRate);
+			if(currentScene->haveTempCam())
+				currentScene->getTempCam()->forwardsDolly(cameraMoveRate);
 			break;
 		}
 		default:
-		;
+			event.Skip();
 	}
 
 	Refresh();
-	event.Skip();
 }
 
-void BasicGLPane::charEvent(wxKeyEvent& event) 
-{
-
-}
- 
  
 void BasicGLPane::resized(wxSizeEvent& evt)
 {
-#if !wxCHECK_VERSION(2,9,0)
-	wxGLCanvas::OnSize(evt);
-#endif
 	prepare3DViewport(0,0,getWidth(),getHeight()); 
 	wxClientDC *dc=new wxClientDC(this);
 	Refresh();
@@ -764,8 +714,8 @@ bool BasicGLPane::prepare3DViewport(int tlx, int tly, int brx, int bry)
 	GLint dims[2]; 
 	glGetIntegerv(GL_MAX_VIEWPORT_DIMS, dims); 
 
-	//Ensure that the opengGL function didn't tell us porkies
-	//but double check for the non-debug bulds next line
+	//Ensure that the openGL function didn't tell us porkies
+	//but double check for the non-debug builds next line
 	ASSERT(dims[0] && dims[1]);
 
 	//check for exceeding max viewport and we have some space
@@ -776,8 +726,8 @@ bool BasicGLPane::prepare3DViewport(int tlx, int tly, int brx, int bry)
 	glViewport( tlx, tly, brx-tlx, bry-tly);
 
 	float aspect = (float)(brx-tlx)/(float)(bry-tly);
-	currentScene.setWinSize(brx-tlx,bry-tly);
-	currentScene.setAspect(aspect);
+	currentScene->setWinSize(brx-tlx,bry-tly);
+	currentScene->setAspect(aspect);
 
 	//Set modelview and projection matrices to the identity
 	// matrix
@@ -810,15 +760,12 @@ void BasicGLPane::render( wxPaintEvent& evt )
 	if (!IsShown()) 
 		return;
 	
-#if wxCHECK_VERSION(2,9,0)
 	if(!context)
 	{
 		context = new wxGLContext(this);
 		SetCurrent(*context);
 	}
-#else
-	wxGLCanvas::SetCurrent();
-#endif
+
 	if(!paneInitialised)
 	{
 		paneInitialised=true;
@@ -826,7 +773,7 @@ void BasicGLPane::render( wxPaintEvent& evt )
 	}
 
 	wxPaintDC(this); 
-	currentScene.draw();
+	currentScene->draw();
 	glFlush();
 	SwapBuffers();
 }
@@ -839,7 +786,7 @@ void BasicGLPane::OnEraseBackground(wxEraseEvent &evt)
 void BasicGLPane::updateClearColour()
 {
 	float rClear,gClear,bClear;
-	currentScene.getBackgroundColour(rClear,gClear,bClear);
+	currentScene->getBackgroundColour(rClear,gClear,bClear);
 	//Can't set the opengl window without a proper context
 	ASSERT(paneInitialised);
 	setGlClearColour(rClear,gClear,bClear);
@@ -848,6 +795,28 @@ void BasicGLPane::updateClearColour()
 				bClear,1.0f);
 }
 
+TRcontext *BasicGLPane::generateTileContext(unsigned int width, unsigned int height, unsigned char *imageBuffer, bool alpha) const
+{
+	int panelWidth,panelHeight;
+	GetClientSize(&panelWidth,&panelHeight);
+	
+	//Create TR library tile context
+	TRcontext *tr = trNew();
+	//Tile size
+	trTileSize(tr,panelWidth,panelHeight,0);
+	//Set overall image size
+	trImageSize(tr, width, height);
+	//Set buffer for overall image
+	if(alpha)
+		trImageBuffer(tr, GL_RGBA, GL_UNSIGNED_BYTE, imageBuffer);
+	else
+		trImageBuffer(tr, GL_RGB, GL_UNSIGNED_BYTE, imageBuffer);
+	//Set the row order for the image
+	trRowOrder(tr, TR_BOTTOM_TO_TOP);
+
+	return tr;
+}
+
 bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 		const char *filename, bool showProgress, bool needPostPaint)
 {
@@ -863,40 +832,36 @@ bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 	//create new image
 	wxImage *image = new wxImage(width,height);
 
+
+	unsigned char *imageBuffer= (unsigned char*) malloc(3*(width)*height);
+	if(!imageBuffer)
+		return false;
+
+
+	glLoadIdentity();
+	const Camera *cm = currentScene->getActiveCam();
+
 	//We cannot seem to draw outside the current viewport.
 	//in a cross platform manner.
 	//fall back to stitching the image together by hand
-	int panelWidth,panelHeight;
-	GetClientSize(&panelWidth,&panelHeight);
-
-	unsigned char *imageBuffer= (unsigned char*) malloc(3*(width)*height);
-
-	glLoadIdentity();
-	//Create TR library tile context
-	TRcontext *tr = trNew();
-	const Camera *cm = currentScene.getActiveCam();
 
 	//Initialise tile data
-	{
-	//Tile size
-	trTileSize(tr,panelWidth,panelHeight,0);
-	//Set overall image size
-	trImageSize(tr, width, height);
-	//Set buffer for overall image
-	trImageBuffer(tr, GL_RGB, GL_UNSIGNED_BYTE, imageBuffer);
-	//Set the row order for the image
-	trRowOrder(tr, TR_BOTTOM_TO_TOP);
-
+	TRcontext *tr; 
 	//Inform the tiling system about our camera config
-	BoundCube bc = currentScene.getBound();
-	float farPlane = 1.5*bc.getMaxDistanceToBox(cm->getOrigin());
+	float aspect=currentScene->getAspect();
+	
+	{
+	float farPlane; 
+	tr=generateTileContext(width,height, imageBuffer);
+	BoundCube bc = currentScene->getBound();
+	farPlane = 1.5*bc.getMaxDistanceToBox(cm->getOrigin());
 	
 	if(cm->getProjectionMode() == PROJECTION_MODE_PERSPECTIVE)
 	{
 		if(cm->type() == CAM_LOOKAT)
 		{
-			const CameraLookAt *cl =(const CameraLookAt*) currentScene.getActiveCam();
-			trPerspective(tr,cl->getFOV()/2.0,currentScene.getAspect(),
+			const CameraLookAt *cl =(const CameraLookAt*) currentScene->getActiveCam();
+			trPerspective(tr,cl->getFOV()/2.0,currentScene->getAspect(),
 							cl->getNearPlane(),farPlane);
 		}
 		else
@@ -907,69 +872,86 @@ bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 	}
 	else
 	{
-		float aspect=currentScene.getAspect();
-		float orthoScale=cm->getOrthoScale();
+		float orthoScale = cm->getOrthoScale();
 		trOrtho(tr,-orthoScale*aspect,orthoScale*aspect,
 				-orthoScale,orthoScale,0.0f,farPlane);
 
 	}
 	}
 
-
-	//Obtain tile count
-	unsigned int nRow,nCol;
+	//Obtain tile count from the renderer & init progress
+	//--
+	unsigned int totalTiles;
+	{
+	unsigned int nRow,nCol,nPass;
 	nRow=trGet(tr,TR_ROWS);
 	nCol=trGet(tr,TR_COLUMNS);
+	if(currentScene->hasOverlays())
+		nPass = 2;
+	else
+		nPass=1;
+
+	totalTiles=nRow*nCol*nPass;
+	}
 
 	wxProgressDialog *wxD=0;	
-
-
-	//Only show progress for multiple tiles
-	std::string tmpStr,tmpStrTwo;
-	stream_cast(tmpStrTwo,nRow*nCol);
-	
-	showProgress=showProgress && ( nRow*nCol > 1);
+	showProgress=showProgress && ( totalTiles > 1);
 	if(showProgress)
 	{
-		wxD = new wxProgressDialog(wxTRANS("Image progress"), 
-					wxTRANS("Rendering tiles..."), nRow*nCol);
+		wxD = new wxProgressDialog(TRANS("Image progress"), 
+					TRANS("Rendering tiles..."), totalTiles);
 
-		wxD->Show();
 	}
+	//--
+
+
+	//We have to do two passes. First we have to
+	// do a 3D pass, then we have to separately
+	// draw the overlays.
+
+	// As we have 2 cameras, one for the normal scene
+	// and one for the overlay, we build the images,
+	// then merge the images, rather than trying to composite the entire scene in situ.
+
+	//PASS 1:
+	//--------------	
 
 	//HACK: Flip the all but scene's light z coordinate
-	// for some reason, the frustrum has an inversion
-	// somwhere in the coordinate system, and I can't find it!
-	// inverting the tile frustrum ends up with the depth test 
+	// for some reason, the frustum has an inversion
+	// somewhere in the coordinate system, and I can't find it!
+	// inverting the tile frustum ends up with the depth test 
 	// also inverting.
+	const bool FLIP_LIGHT_HACK=true;
+	//x,y,z and w axis.
+	const bool IMPORTANT_AXIS[4]={true,false,true,false};
+
+	//opengl light has 4 
 	float oldLightPos[4];
-	currentScene.getLightPos(oldLightPos);
-	const int UNTRANS_AXIS=2;
-	for(size_t ui=0;ui<3;ui++)
+	if(FLIP_LIGHT_HACK)
 	{
-		if(ui == UNTRANS_AXIS)
-			continue;
-
-		oldLightPos[ui]=-oldLightPos[ui];
-	
+		currentScene->getLightPos(oldLightPos);
+		float newLightPos[4];
+		for(size_t ui=0;ui<4;ui++)
+		{
+			if(IMPORTANT_AXIS[ui])
+				newLightPos[ui]=oldLightPos[ui];
+			else
+				newLightPos[ui]=-oldLightPos[ui];
+		
+		}
+		currentScene->setLightPos(newLightPos);
 	}
-	currentScene.setLightPos(oldLightPos);
 	
+	if(showProgress)
+		wxD->Show();
 
-	//Loop through the tiles
+	//Loop through the tiles/ 
+	// note that 2D overlays will not be drawn in this pass
 	unsigned int thisTileNum=0;
 	int haveMoreTiles=1;
 	while(haveMoreTiles)
 	{
-		
-		
 		thisTileNum++;
-		//tell user which image tile we are making from the total image
-		stream_cast(tmpStr,thisTileNum);
-		tmpStr = std::string(TRANS("Tile ")) + tmpStr + std::string(TRANS(" of ")) + tmpStrTwo + "...";
-		//Update progress bar, if required
-		if(showProgress)
-			wxD->Update(thisTileNum,wxStr(tmpStr));
 
 		//Manually set the camera
 		//--
@@ -983,36 +965,104 @@ bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 	
 		//Start the tile
 		trBeginTile(tr);
-		currentScene.draw(true);
+		currentScene->draw(true);
 
 		glPopMatrix();
 
+		//ending the tile copies
+		// data 
 		haveMoreTiles=trEndTile(tr);
+	
+		if(showProgress)	
+			wxD->Update(thisTileNum);
 
 	}
 
-	//re-set light coordinates
-	for(size_t ui=0;ui<3;ui++)
+	if(FLIP_LIGHT_HACK)
 	{
-		if (ui == UNTRANS_AXIS)
-			continue;
-		oldLightPos[ui]=-oldLightPos[ui];
+		//re-set light coordinates
+		currentScene->setLightPos(oldLightPos);
 	}
-	currentScene.setLightPos(oldLightPos);
 
-	if(showProgress)
-		wxD->Destroy();
 	trDelete(tr);
 
 	//Transfer pointer to image, which will perform free-ing of the buffer
 	image->SetData(imageBuffer);
-	
 	//HACK : Tiling function returns upside-down image. Fix in post-process
 	// argument is to set mirror axis such that x axis is unchanged
 	*image=image->Mirror(false);
-	
-	bool isOK=image->SaveFile(wxCStr(filename),wxBITMAP_TYPE_PNG);
+	//--------------	
 
+	//PASS 2
+	//--------------	
+
+	if(currentScene->hasOverlays())
+	{
+		//alllocate RGBA (4-channel) image
+		imageBuffer= (unsigned char*) malloc(4*(width)*height);
+		if(!imageBuffer)
+			return false;
+
+
+		tr=generateTileContext(width,height,imageBuffer,true);
+		trOrtho(tr,0.0f,aspect,
+				0.0f,1.0f,-1.0f,1.0f);
+		
+
+		haveMoreTiles=1;
+
+	
+		float rClear,gClear,bClear;
+		currentScene->getBackgroundColour(rClear,gClear,bClear);
+		glClearColor( rClear, gClear, 
+					bClear,0.0f);
+
+		//I am unclear why, but the faces are reversed
+		glDisable(GL_CULL_FACE);
+		while(haveMoreTiles)
+		{
+			thisTileNum++;
+			//Start the tile
+			trBeginTile(tr);
+			glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
+			currentScene->drawOverlays(true);
+			//ending the tile copies
+			// data 
+			haveMoreTiles=trEndTile(tr);
+		
+			if(showProgress)	
+				wxD->Update(thisTileNum);
+		}	
+		glEnable(GL_CULL_FACE);
+		//restore the GL clear colour
+		updateClearColour();
+
+		///unpack the tile buffer into a wx image
+		wxImage imageOverlay(width,height);
+		imageOverlay.InitAlpha();
+
+		//FIXME: HACK - using "blue screen" effect
+		//don't use background as mask colour.
+		// use depth buffer or gl alpha
+		const unsigned char mask[3] = {(unsigned char)rClear*255.0f,
+				(unsigned char)gClear*255.0f,(unsigned char)bClear*255.0f};
+		copyRGBAtoWXImage(width,height,imageBuffer,imageOverlay,mask);
+
+		free(imageBuffer);
+
+		combineWxImage(*image,imageOverlay);
+	}
+
+	//Free the tile buffer
+	trDelete(tr);
+	
+	//--------------	
+	bool isOK=image->SaveFile(filename,wxBITMAP_TYPE_PNG);
+	
+
+	if(showProgress)
+		wxD->Destroy();
+	
 	delete image;
 
 	if (needPostPaint) {
@@ -1036,16 +1086,16 @@ bool BasicGLPane::saveImageSequence(unsigned int resX, unsigned int resY, unsign
 	//
 
 
-	ASSERT(!currentScene.haveTempCam());
+	ASSERT(!currentScene->haveTempCam());
 	std::string outFile;
-	wxProgressDialog *wxD = new wxProgressDialog(wxTRANS("Animation progress"), 
-					wxTRANS("Rendering sequence..."), nFrames,this,wxPD_CAN_ABORT|wxPD_APP_MODAL );
+	wxProgressDialog *wxD = new wxProgressDialog(TRANS("Animation progress"), 
+					TRANS("Rendering sequence..."), nFrames,this,wxPD_CAN_ABORT|wxPD_APP_MODAL );
 
 	wxD->Show();
 	std::string tmpStr,tmpStrTwo;
 	stream_cast(tmpStrTwo,nFrames);
 
-	Camera *origCam=currentScene.getActiveCam()->clone();
+	Camera *origCam=currentScene->getActiveCam()->clone();
 	
 	
 	for(unsigned int ui=0;ui<nFrames;ui++)
@@ -1063,14 +1113,14 @@ bool BasicGLPane::saveImageSequence(unsigned int resX, unsigned int resY, unsign
 		Camera *modifiedCam;
 		modifiedCam=origCam->clone();
 		modifiedCam->move(angle,0);
-		currentScene.setActiveCam(modifiedCam);
+		currentScene->setActiveCam(modifiedCam);
 
 		//Save the result
 		outFile = string(stlStr(path))+ string("/") + 
 				string(stlStr(prefix))+digitStr+ string(".") + string(stlStr(ext));
 		if(!saveImage(resX,resY,outFile.c_str(),false, false))
 		{
-			currentScene.setActiveCam(origCam);
+			currentScene->setActiveCam(origCam);
 			return false;
 		}
 
@@ -1078,16 +1128,16 @@ bool BasicGLPane::saveImageSequence(unsigned int resX, unsigned int resY, unsign
 		stream_cast(tmpStr,ui+1);
 		//Tell user which image from the animation we are saving
 		tmpStr = std::string(TRANS("Saving Image ")) + tmpStr + std::string(TRANS(" of ")) + tmpStrTwo + "...";
-		if(!wxD->Update(ui,wxStr(tmpStr)))
+		if(!wxD->Update(ui,tmpStr))
 			break;
 
 		Refresh();
 	}
 
-	currentScene.setActiveCam(origCam);
+	currentScene->setActiveCam(origCam);
 
 	//Discard the current temp. cam to return the scene back to normal
-	currentScene.discardTempCam();
+	currentScene->discardTempCam();
 	wxD->Destroy();
 	
 	wxPaintEvent event;
