@@ -1,6 +1,6 @@
 /*
  *	gLPane.h - WxWidgets opengl Pane. 
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  
 
 #include "gl/scene.h"
+#include "gl/tr.h"
 
 #include <wx/glcanvas.h>
 #include <wx/timer.h>
@@ -29,9 +30,9 @@ class BasicGLPane : public wxGLCanvas
 {
 private:
 
-#if wxCHECK_VERSION(2,9,0)
 	wxGLContext *context;
-#endif
+
+	Scene *currentScene;
 
 	wxStatusBar *parentStatusBar;
 	wxTimer *parentStatusTimer;
@@ -76,15 +77,20 @@ private:
 	unsigned int lastKeyDoubleTap;
 
 	wxTimer *keyDoubleTapTimer;
+
+	//build a TR tile context to allow drawing the image in chunks.
+	// width and height are the output image size. 
+	// wantAlpha is used if we want to retrieve the alpha (transpar.) channel
+	TRcontext *generateTileContext(unsigned int width, unsigned int height, 
+			unsigned char *buffer, bool wantAlpha=false) const;
 public:
 	bool displaySupported() const;
 
+	void setScene(Scene *s) { currentScene=s;}
+
 	//Enable/Disable the scene interaction for user objects?
 	void setSceneInteractionAllowed(bool enabled=true);
 
-	//!The scene object, holds all info about 3D drawable components
-	Scene currentScene;
-
 	//!Must be called before user has a chance to perform interaction
 	void setParentStatus(wxStatusBar *statusBar,
 			wxTimer *timer,unsigned int statDelay) 
@@ -129,7 +135,7 @@ public:
 			wxString &path, wxString &prefix, wxString &extension);
 
 	//!Get the background colour
-	void getGlClearColour(float &r,float &g,float &b) { currentScene.getBackgroundColour(r,g,b);}
+	void getGlClearColour(float &r,float &g,float &b) { currentScene->getBackgroundColour(r,g,b);}
 	// events
 	void mouseMoved(wxMouseEvent& event);
 	void mouseDown(wxMouseEvent& event);
