@@ -1,6 +1,6 @@
 /*
  * 	3Depict.h - main program header
- * 	Copyright (C) 2013 D Haley
+ * 	Copyright (C) 2015 D Haley
  *
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -28,9 +28,8 @@
 #include <wx/dnd.h>
 #include <wx/grid.h>
 #include <wx/treectrl.h>
-#if wxCHECK_VERSION(2,9,0)
+#include <wx/propgrid/propgrid.h>
 #include <wx/filehistory.h>
-#endif
 // end wxGlade
 
 //Local stuff
@@ -43,23 +42,37 @@
 #include "backend/viscontrol.h"
 #include "backend/configFile.h"
 
+
+
 #ifndef THREEDEPICT_H 
 #define THREEDEPICT_H
 
-//Workaround for keypress not detected under MSW wx3.0 and apple
-#if wxCHECK_VERSION(2,9,0) && ( defined(__WIN32) || defined(__WIN64) || defined(__APPLE__)) || wxCHECK_VERSION(3,0,0)
-#define WX_TREE_WORKAROUND
-#endif
 
 class FileDropTarget;
+class RefreshThread;
 
 enum
 {
 	MESSAGE_ERROR=1,
 	MESSAGE_INFO,
-	MESSAGE_HINT,
-	MESSAGE_NONE_BUT_HINT,
-	MESSAGE_NONE
+	MESSAGE_HINT, //lowest priority message in the queue. Only one HINT can be in queue at a time
+	MESSAGE_NONE // pseudo-message to wipe all messages
+};
+
+
+//This is used to create and run a worker thread that will perform a refresh calculation
+class RefreshThread: public wxThread
+{
+	private:
+		RefreshController *refreshControl;
+		wxWindow *targetWindow;
+	public:
+		RefreshThread(wxWindow *target,RefreshController *rc); 
+		~RefreshThread();
+		//!Used internally by wxwidgets to launch thread
+		void *Entry();
+
+		void abort() {ASSERT(false);}
 };
 
 class MainWindowFrame: public wxFrame {
@@ -88,14 +101,30 @@ private:
     void set_properties();
     void do_layout();
     // end wxGlade
-   
-   	//!Give a message in the satus bar
+    //Force a re-layout of the filter property grid
+    void do_filtergrid_prop_layout();
+    //Force a re-layout of the camera property grid
+    void do_cameragrid_prop_layout();
+  
+	bool refreshThreadActive() { return refreshThread && refreshThread->IsRunning();};
+ 
+   	//!Queue up a status message for display
+    	void showStatusMessage(const char *message, unsigned int messageType=MESSAGE_ERROR); 
+   	
+	//!Queue up a status message for display
     	void statusMessage(const char *message, unsigned int messageType=MESSAGE_ERROR); 
 
 	//!Update the progress information in the status bar
 	void updateProgressStatus();
 	//!Perform an update to the 3D Scene. Returns false if refresh failed
 	bool doSceneUpdate();
+	
+	//!Complete the scene update. Returns false if failed
+	void finishSceneUpdate(unsigned int errCode);
+
+	//!Wrapper for viscontrol's update function, as we need to
+	// prevent wx from firing events during tree update
+	void updateWxTreeCtrl( wxTreeCtrl *t, const Filter *f=0);
 
 	//!Update the post-processing effects in the 3D scene. 
 	void updatePostEffects(); 
@@ -106,7 +135,7 @@ private:
 	void setFilterTreeAnalysisImages(); 
 
 	//!Update the effects UI from some effects vector
-	void updateFxUI(const vector<const Effect *> &fx);
+	void updateFxUI(const std::vector<const Effect *> &fx);
 
 	void setLockUI(bool amlocking,unsigned int lockMode);
 
@@ -115,6 +144,11 @@ private:
 	//!Scene - user interaction interface "visualisation control"
 	VisController visControl;
 
+	//!Refresh control thread
+	RefreshThread *refreshThread;
+	//!Refresh control object
+	RefreshController *refreshControl;
+
 	//!Program on-disk configuration class
 	ConfigFile configFile;
 
@@ -141,23 +175,24 @@ private:
 	//!Did the main frame's constructor complete OK?
 	bool initedOK;
 
-	//The type of status message last sent to user
-	unsigned int lastMessageType;
-
 	//Pointer to version check thread, occasionally initialised at startup to
 	// check online for new program updates
 	VersionCheckThread *verCheckThread;
 
 	//Map to convert filter drop down choices to IDs
-	map<std::string,size_t> filterMap;
+	std::map<std::string,size_t> filterMap;
    
 	//TODO: Refactor -  remove me.
 	// True if there are pending updates for the mahthgl window
 	bool plotUpdates;
+
+	//List of pending messages to show in status bar
+	// first int is priority (eg MESSAGE_ERROR), string is message
+	std::list<std::pair<unsigned int, std::string > > statusQueue;
 protected:
-    wxTimer *statusTimer;
-    wxTimer *progressTimer;
+    wxTimer *statusTimer; //One-shot timer that is used to clear the status bar
     wxTimer *updateTimer; //Periodically calls itself to check for updates from user interaction
+    wxTimer *progressTimer; //Periodically calls itself to refresh progress status
     wxTimer *autoSaveTimer; //Periodically calls itself to create an autosave state file
     wxMenuItem *checkMenuControlPane;
     wxMenuItem *checkMenuRawDataPane;
@@ -185,8 +220,6 @@ protected:
     wxStaticText* filteringLabel;
     wxComboBox* comboFilters;
     TextTreeCtrl* treeFilters;
-    wxStaticText* lastRefreshLabel;
-    wxListCtrl* listLastRefresh;
     wxCheckBox* checkAutoUpdate;
     wxButton* refreshButton;
     wxButton* btnFilterTreeExpand;
@@ -194,7 +227,8 @@ protected:
     wxBitmapButton* btnFilterTreeErrs;
     wxPanel* filterTreePane;
     wxStaticText* propGridLabel;
-    wxCustomPropGrid* gridFilterPropGroup;
+    wxPropertyGrid* gridFilterPropGroup;
+    wxPropertyGrid *backFilterPropGrid;
     wxPanel* filterPropertyPane;
     wxSplitterWindow* filterSplitter;
     wxPanel* noteData;
@@ -202,7 +236,8 @@ protected:
     wxComboBox* comboCamera;
     wxButton* buttonRemoveCam;
     wxStaticLine* cameraNamePropertySepStaticLine;
-    wxCustomPropGrid* gridCameraProperties;
+    wxPropertyGrid* gridCameraProperties;
+    wxPropertyGrid* backCameraPropGrid;
     wxScrolledWindow* noteCamera;
     wxCheckBox* checkPostProcessing;
     wxCheckBox* checkFxCrop;
@@ -287,11 +322,7 @@ public:
     virtual void OnComboStashEnter(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnComboStash(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnTreeEndDrag(wxTreeEvent &event); // wxGlade: <event_handler>
-#if defined(WX_TREE_WORKAROUND)
     virtual void OnTreeKeyDown(wxKeyEvent &event); // wxGlade: <event_handler>
-#else
-     virtual void OnTreeKeyDown(wxTreeEvent &event); // wxGlade: <event_handler>
-#endif
     virtual void OnTreeSelectionPreChange(wxTreeEvent &event); // wxGlade: <event_handler>
     virtual void OnTreeSelectionChange(wxTreeEvent &event); // wxGlade: <event_handler>
     virtual void OnTreeDeleteItem(wxTreeEvent &event); // wxGlade: <event_handler>
@@ -301,7 +332,8 @@ public:
     virtual void OnBtnFilterTreeErrs(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnComboCameraText(wxCommandEvent &event); // wxGlade: <event_handler>
 
-    virtual void OnGridFilterPropertyChange(wxGridEvent &event); // wxGlade: <event_handler>
+    virtual void OnGridFilterPropertyChange(wxPropertyGridEvent &event); // wxGlade: <event_handler>
+    virtual void OnGridFilterDClick(wxPropertyGridEvent &event); // wxGlade: <event_handler>
     virtual void OnComboCameraEnter(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnComboCamera(wxCommandEvent &event); // wxGlade: <event_handler>
     
@@ -334,12 +366,9 @@ public:
 
     virtual void OnComboFilterEnter(wxCommandEvent &event); // 
     virtual void OnComboFilter(wxCommandEvent &event); // 
+    virtual void OnComboFilterText(wxCommandEvent &event); // wxGlade: <event_handler>
     
     virtual void OnStatusBarTimer(wxTimerEvent &event); // 
-    virtual void OnFilterGridCellEditorShow(wxGridEvent &event); // 
-    virtual void OnFilterGridCellEditorHide(wxGridEvent &event); // 
-    virtual void OnCameraGridCellEditorShow(wxGridEvent &event); // 
-    virtual void OnCameraGridCellEditorHide(wxGridEvent &event); // 
     virtual void OnProgressTimer(wxTimerEvent &event);
     virtual void OnProgressAbort(wxCommandEvent &event);
     virtual void OnViewFullscreen(wxCommandEvent &event);
@@ -361,7 +390,7 @@ public:
     virtual void OnComboCameraSetFocus(wxFocusEvent &evt);
     virtual void OnComboStashSetFocus(wxFocusEvent &evt);
     virtual void OnNoteDataView(wxNotebookEvent &evt);
-    virtual void OnGridCameraPropertyChange(wxGridEvent &event); // wxGlade: <event_handler>
+    virtual void OnGridCameraPropertyChange(wxPropertyGridEvent &event); // wxGlade: <event_handler>
 
     virtual void OnFileExportVideo(wxCommandEvent &event);
     virtual void OnFileExportFilterVideo(wxCommandEvent &event);
@@ -375,13 +404,18 @@ public:
     virtual void OnAutosaveTimer(wxTimerEvent &evt);
 
     virtual void OnCheckUpdatesThread(wxCommandEvent &evt);
+    virtual void OnFinishRefreshThread(wxCommandEvent &evt);
+    virtual void OnIdle(wxIdleEvent &evt);
 
     virtual void SetCommandLineFiles(wxArrayString &files);
-    virtual void updateLastRefreshBox();
 
     //return type of file, based upon heuristic check
     static unsigned int guessFileType(const std::string &file);
 
+
+    //Check to see if the user wants a tip file
+    void checkShowTips();
+
     //See if the user wants to save the current state
     void checkAskSaveState();
     
@@ -397,6 +431,8 @@ public:
     void onPanelSpectraUpdate() {plotUpdates=true;} ;
 
     bool initOK() const {return initedOK;}
+    
+    void finaliseStartup();
 
     //This is isolated from the layout code, due to "bug" 4815 in wx. The splitter window
     //does not know how to choose a good size until the window is shown
