@@ -1,6 +1,6 @@
 /*
  *	mathglPane.cpp - mathgl-wx interface panel control
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -24,16 +24,16 @@
 
 #include "wx/wxcommon.h"
 #include "common/translation.h"
+#include "backend/plot.h"
 
 #ifndef pow10
 #define pow10(x) pow(10,x)
 #endif
 
-#ifdef USE_MGL2
-	#include <mgl2/canvas_wnd.h>
-#else
-	#include <mgl/mgl_eps.h>
-#endif
+#include <mgl2/canvas_wnd.h>
+
+using std::string;
+using std::vector;
 
 //Panning speed modifier
 const float MGL_PAN_SPEED=2.0f;
@@ -51,7 +51,7 @@ enum
 };
 
 //Do the particular enums require a redraw?
-const bool MOUSE_ACTION_NEEDS_REDRAW[] = { false,true,true};
+const bool MOUSE_ACTION_NEEDS_REDRAW[] = { false,true,true,false};
 
 enum
 {
@@ -97,10 +97,32 @@ enum
 };
 
 
+void zoomBounds(float minV,float maxV,  float centre, 
+		float zoomFactor,float &newMin, float &newMax)
+{
+	ASSERT(minV < maxV);
+	ASSERT(minV< centre && maxV > centre);
+	ASSERT(zoomFactor > 0);
+	
+	//find deltas, then multiply them out
+	float lowerDelta,upperDelta;
+	lowerDelta = (centre-minV);
+	upperDelta = (maxV-centre);
+	upperDelta*=zoomFactor;
+	lowerDelta*=zoomFactor;
+	ASSERT(upperDelta > 0 && lowerDelta > 0);
+
+	//compute new bounds
+	newMin= centre - lowerDelta;
+	newMax= centre + upperDelta;
+
+	ASSERT(newMin <=newMax);
+}
+
 MathGLPane::MathGLPane(wxWindow* parent, int id) :
 wxPanel(parent, id,  wxDefaultPosition, wxDefaultSize)
 {
-	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(MOUSE_ACTION_NEEDS_REDRAW) == MOUSE_MODE_ENUM_END);
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(MOUSE_ACTION_NEEDS_REDRAW) == MOUSE_MODE_ENUM_END + 1);
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(mglTextureFile) == PLOT_TEXTURE_ENUM_END);
 
 	hasResized=true;
@@ -109,7 +131,6 @@ wxPanel(parent, id,  wxDefaultPosition, wxDefaultSize)
 	leftWindow=true;
 	thePlot=0;	
 	gr=0;
-	ownPlotPtr=false;
 	lastEditedPlot=lastEditedRegion=-1;
 	regionSelfUpdate=false;
 	plotIsLogarithmic=false;
@@ -120,13 +141,33 @@ wxPanel(parent, id,  wxDefaultPosition, wxDefaultSize)
 
 MathGLPane::~MathGLPane()
 {
-	if(thePlot && ownPlotPtr)
-		delete thePlot;
 	if(gr)
 		delete gr;
 }
 
 
+void MathGLPane::setPanCoords() const
+{
+	float xMin,xMax,yMin,yMax;
+	thePlot->getBounds(xMin,xMax,yMin,yMax);
+
+	float pEndX, pStartX,dummy;
+	toPlotCoords(draggingCurrent.x,draggingCurrent.y,pEndX,dummy);
+	toPlotCoords(draggingStart.x,draggingStart.y,pStartX,dummy);
+	
+	float offX = pEndX-pStartX;
+
+	//This is not needed if re-using mgl object!
+	// - not sure why!
+	//offX*=xMax-xMin;
+
+	//Modify for speed
+	offX*=MGL_PAN_SPEED;
+
+	thePlot->setBounds(origPanMinX+offX/2,+origPanMaxX + offX/2.0,
+				yMin,yMax);
+}
+
 bool MathGLPane::readyForInput() const
 {
 	return (thePlot && gr && 
@@ -144,19 +185,11 @@ unsigned int MathGLPane::getAxisMask(int x, int y) const
 
 	unsigned int retVal=0;
 
-#ifdef USE_MGL2
 	if(mglCurX < gr->Self()->GetOrgX('x'))
 		retVal |=AXIS_POSITION_LOW_X;
 
 	if(mglCurY < gr->Self()->GetOrgY('y'))
 		retVal |=AXIS_POSITION_LOW_Y;
-#else
-	if(mglCurX < gr->Org.x)
-		retVal |=AXIS_POSITION_LOW_X;
-
-	if(mglCurY < gr->Org.y)
-		retVal |=AXIS_POSITION_LOW_Y;
-#endif
 
 	if(!retVal)
 		retVal=AXIS_POSITION_INTERIOR;
@@ -166,11 +199,7 @@ unsigned int MathGLPane::getAxisMask(int x, int y) const
 
 void MathGLPane::setPlotWrapper(PlotWrapper *newPlot,bool takeOwnPtr)
 {
-	if(thePlot && ownPlotPtr)
-		delete thePlot;
-
 	thePlot=newPlot;
-	ownPlotPtr=takeOwnPtr;
 
 	Refresh();
 }
@@ -227,7 +256,7 @@ void MathGLPane::render(wxPaintEvent &event)
 		int clientW,clientH;
 		GetClientSize(&clientW,&clientH);
 		
-		wxString str=wxTRANS("No plots selected.");
+		wxString str=TRANS("No plots selected.");
 		dc->GetMultiLineTextExtent(str,&w,&h);
 		dc->DrawText(str,(clientW-w)/2, (clientH-h)/2);
 
@@ -240,29 +269,28 @@ void MathGLPane::render(wxPaintEvent &event)
 
 	}
 
+#ifdef DEBUG
+	bool doTrap=getTrapfpe();
+	if(doTrap)
+		trapfpe(false);
+#endif
 	//If the plot has changed, been resized or is performing
 	// a mouse action that requires updating, we need to update it
 	//likewise if we don't have a plot, we need one.
 	if(!gr || hasChanged || hasResized || 
 		MOUSE_ACTION_NEEDS_REDRAW[mouseDragMode])
 	{
-		//TODO: There appears to be a bug in mathgl
-		// where attempting to clear the plot with ->SetSize()
-		// causes objects to become zero sized (eg ticks).
-		// using a brand "new" plot entity bypasses this
-		// at some computational cost.
-		// Need to make a minimal example.
-	
 		//clear the plot drawing entity
 		if(!gr)
 		{
-#ifdef USE_MGL2
-		gr = new mglGraph(0,w,h);
-#else
-		gr = new mglGraphZB(w,h);
+			gr = new mglGraph(0,w,h);
+#ifdef __APPLE__
+			//apparenty bug in mgl under osx - font wont load,
+			// use random string to force fallback
+			gr->LoadFont("asdfrandom");
 #endif
 		}
-		else 
+		else
 		{
 			gr->SetSize(w,h);
 		}
@@ -270,48 +298,31 @@ void MathGLPane::render(wxPaintEvent &event)
 		//change the plot by panningOneD it before we draw.
 		//if we need to 
 		if(mouseDragMode==MOUSE_MODE_DRAG_PAN)
-		{
-			float xMin,xMax,yMin,yMax;
-			thePlot->getBounds(xMin,xMax,yMin,yMax);
-		
-			float pEndX, pStartX,dummy;
-			toPlotCoords(draggingCurrent.x,draggingCurrent.y,pEndX,dummy);
-			toPlotCoords(draggingStart.x,draggingStart.y,pStartX,dummy);
-			
-			float offX = pEndX-pStartX;
-
-			//This is not needed if re-using mgl object!
-			// - not sure why!
-			//offX*=xMax-xMin;
-
-			//Modify for speed
-			offX*=MGL_PAN_SPEED;
-
-			thePlot->setBounds(origPanMinX+offX/2,+origPanMaxX + offX/2.0,
-						yMin,yMax);
-		}
+			setPanCoords();
 
 		//Draw the plot
-		thePlot->drawPlot(gr,plotIsLogarithmic);	
+		thePlot->drawPlot(gr,plotIsLogarithmic);
+#ifdef DEBUG
+		if(strlen(gr->Message()))
+		{
+			std::cerr << "Mathgl reports error:" << gr->Message() << std::endl;
+		}
+#endif
 		thePlot->resetChange();
 		hasResized=false;
 
 		//Copy the plot's memory buffer into a wxImage object, then draw it	
-	#ifdef USE_MGL2
 		char *rgbdata = (char*)malloc(w*h*3);
 		gr->GetRGB((char*)rgbdata,w*h*3);
 		
-		imageCacheBmp=wxImage(w,h,(unsigned char*)rgbdata,true);
+		imageCacheBmp=wxBitmap(wxImage(w,h,(unsigned char*)rgbdata,true));
 		free(rgbdata);
-	#else
-		unsigned char *tmp;
-		tmp=const_cast<unsigned char*>(gr->GetBits());
-		wxImage imTmp;
-		imTmp=wxImage(w,h,tmp,true);
-		imageCacheBmp=wxBitmap(imTmp);
-	#endif
 	}
 
+#ifdef DEBUG
+	if(doTrap)
+		trapfpe(true);
+#endif
 	dc->DrawBitmap(wxBitmap(imageCacheBmp),0,0);
 	//If we are engaged in a dragging operation
 	//draw the nice little bits we need
@@ -426,9 +437,11 @@ void MathGLPane::updateMouseCursor()
 		return;
 
 	//Set cursor to normal by default
-	SetCursor(wxNullCursor);
 	if(!readyForInput())
+	{
+		SetCursor(wxNullCursor);
 		return;
+	}
 
 	//Update mouse cursor
 	//---------------
@@ -460,9 +473,11 @@ void MathGLPane::updateMouseCursor()
 			SetCursor(wxCURSOR_SIZEWE);
 				break;
 			case AXIS_POSITION_INTERIOR:
-				SetCursor(wxCURSOR_MAGNIFIER);
+				//SetCursor(wxCURSOR_MAGNIFIER);
+				SetCursor(wxNullCursor);
 				break;
 			default:
+				SetCursor(wxNullCursor);
 				;
 		}
 	}
@@ -485,13 +500,9 @@ bool MathGLPane::getRegionUnderCursor(const wxPoint  &mousePos, unsigned int &pl
 		return false;
 
 	//Only allow  range interaction within the plot bb
-#ifdef USE_MGL2
 	if(pMouse.x > gr->Self()->Max.x || pMouse.x < gr->Self()->Min.x)
 		return false;
-#else
-	if(pMouse.x > gr->Max.x || pMouse.x < gr->Min.x)
-		return false;
-#endif
+	
 	//check if we actually have a region
 	if(!thePlot->getRegionIdAtPosition(pMouse.x,pMouse.y,plotId,regionId))
 		return false;
@@ -613,7 +624,7 @@ void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 			thePlot->getRegion(plotId,regionId,r);
 
 			//TODO: Implement a more generic region handler?
-			ASSERT(thePlot->plotType(plotId) == PLOT_TYPE_ONED);
+			ASSERT(thePlot->plotType(plotId) == PLOT_MODE_1D);
 
 			float mglStartX,mglStartY;
 			toPlotCoords(draggingStart.x, draggingStart.y,mglStartX,mglStartY);
@@ -643,6 +654,35 @@ void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 
 }
 
+void MathGLPane::twoDMouseDownAction(bool leftDown,bool middleDown,
+		 bool alternateDown, int dragX,int dragY)
+{
+	ASSERT(thePlot->getNumVisible());
+	
+	float xMin,xMax,yMin,yMax;
+	thePlot->getBounds(xMin,xMax,yMin,yMax);
+
+	//Set the interaction mode
+	if(leftDown && !alternateDown )
+	{
+		draggingStart = wxPoint(dragX,dragY);
+		mouseDragMode=MOUSE_MODE_DRAG;
+	}
+	
+	
+	if( (leftDown && alternateDown) || middleDown)
+	{
+		mouseDragMode=MOUSE_MODE_DRAG_PAN;
+		draggingStart = wxPoint(dragX,dragY);
+		
+		origPanMinX=xMin;
+		origPanMaxX=xMax;
+		origPanMinY=yMin;
+		origPanMaxY=yMax;
+	}
+
+}
+
 void MathGLPane::leftMouseDown(wxMouseEvent& event)
 {
 	if(!readyForInput())
@@ -659,16 +699,21 @@ void MathGLPane::leftMouseDown(wxMouseEvent& event)
 		event.GetPosition().x < 0 || event.GetPosition().y < 0)
 		return;
 
-	switch(thePlot->getVisibleType())
+	switch(thePlot->getVisibleMode())
 	{
-		case PLOT_TYPE_ONED:
+		case PLOT_MODE_1D:
 			oneDMouseDownAction(event.LeftDown(),false,
 						event.ShiftDown(),
 						event.GetPosition().x,
 						event.GetPosition().y);
 			break;
-		case PLOT_TYPE_ENUM_END:
+		case PLOT_MODE_2D:
+		case PLOT_MODE_ENUM_END:
 			//Do nothing
+			twoDMouseDownAction(event.LeftDown(),false,
+						event.ShiftDown(),
+						event.GetPosition().x,
+						event.GetPosition().y);
 			break;
 		default:
 			ASSERT(false);
@@ -689,15 +734,15 @@ void MathGLPane::middleMouseDown(wxMouseEvent &event)
 	if(!w || !h)
 		return;
 	
-	switch(thePlot->getVisibleType())
+	switch(thePlot->getVisibleMode())
 	{
-		case PLOT_TYPE_ONED:
+		case PLOT_MODE_1D:
 			oneDMouseDownAction(false,event.MiddleDown(),
 						event.ShiftDown(),
 						event.GetPosition().x,
 						event.GetPosition().y);
 			break;
-		case PLOT_TYPE_ENUM_END:
+		case PLOT_MODE_ENUM_END:
 			//Do nothing
 			break;
 		default:
@@ -721,23 +766,28 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 
 
 
-	//Bigger numbers mean faster
-	const float SCROLL_WHEEL_ZOOM_RATE=0.75;
+	//Bigger numbers mean faster. 
+	const float SCROLL_WHEEL_ZOOM_RATE=0.20;
+
 	float zoomRate=(float)event.GetWheelRotation()/(float)event.GetWheelDelta();
 	zoomRate=zoomRate*SCROLL_WHEEL_ZOOM_RATE;
 
 	//Convert from additive space to multiplicative
 	float zoomFactor;
-	if(zoomRate < 0.0f)
-		zoomFactor=-1.0f/zoomRate;
+	if(zoomRate > 0.0f)
+	{
+		zoomFactor=1.0/(1.0+zoomRate);
+		ASSERT(zoomFactor> 1.0f);
+	}
 	else
-		zoomFactor=zoomRate;
+	{
+		zoomFactor=(1.0-zoomRate);
+		ASSERT(zoomFactor < 1.0f);
+	}
 
 
 
-	ASSERT(zoomFactor >0.0f);
-
-
+	//retrieve the mouse position
 	mglPoint mousePos;
 	float mglX,mglY;
 	toPlotCoords(curMouse.x,curMouse.y,mglX,mglY);
@@ -757,11 +807,11 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 		//below x axis -> y zoom only
 		case AXIS_POSITION_LOW_X:
 		{
-			float newYMax,newYMin;
-			//Zoom along Y
-			newYMin= mousePos.y + (yMin-mousePos.y)*zoomFactor ;
-			newYMax= mousePos.y + (yMax-mousePos.y)*zoomFactor ;
-	
+			float newYMin,newYMax;
+			//work out existing bounds on zooming
+			zoomBounds(yMin,yMax,mousePos.y,
+				zoomFactor, newYMin,newYMax);
+			//clamp to plot
 			newYMin=std::max(yPlotMin,newYMin);
 			newYMax=std::min(yPlotMax,newYMax);
 		
@@ -772,11 +822,11 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 		//Below y axis -> x zoom only
 		case AXIS_POSITION_LOW_Y:
 		{
-			float newXMax,newXMin;
-			//Zoom along X
-			newXMin= mousePos.x + (xMin-mousePos.x)*zoomFactor ;
-			newXMax= mousePos.x + (xMax-mousePos.x)*zoomFactor ;
-	
+			float newXMin,newXMax;
+			//work out existing bounds on zooming
+			zoomBounds(xMin,xMax,mousePos.x,
+				zoomFactor, newXMin,newXMax);
+
 			newXMin=std::max(xPlotMin,newXMin);
 			newXMax=std::min(xPlotMax,newXMax);
 		
@@ -789,13 +839,12 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 		{
 			float newXMax,newXMin;
 			float newYMax,newYMin;
+			//work out existing bounds on zooming
+			zoomBounds(xMin,xMax,mousePos.x,
+				zoomFactor, newXMin,newXMax);
+			zoomBounds(yMin,yMax,mousePos.y,
+				zoomFactor, newYMin,newYMax);
 			
-			//Zoom along X
-			newXMin= mousePos.x + (xMin-mousePos.x)*zoomFactor ;
-			newXMax= mousePos.x + (xMax-mousePos.x)*zoomFactor ;
-			newYMin= mousePos.y + (yMin-mousePos.y)*zoomFactor ;
-			newYMax= mousePos.y + (yMax-mousePos.y)*zoomFactor ;
-	
 				
 			newXMin=std::max(xPlotMin,newXMin);
 			newXMax=std::min(xPlotMax,newXMax);
@@ -868,7 +917,7 @@ void MathGLPane::leftMouseReleased(wxMouseEvent& event)
 	{
 		for(size_t ui=0;ui<updateHandlers.size(); ui++)
 		{
-			pair<wxWindow*,UpdateHandler> u;
+			std::pair<wxWindow*,UpdateHandler> u;
 			u=updateHandlers[ui];
 
 			//Call the function
@@ -942,12 +991,9 @@ void MathGLPane::updateDragPos(const wxPoint &draggingEnd) const
 
 
 	mglPoint cA;
-#ifdef USE_MGL2
 	cA.x=gr->Self()->GetOrgX('x');
 	cA.y=gr->Self()->GetOrgY('y');
-#else
-	cA=gr->Org;
-#endif
+	
 	float currentAxisX,currentAxisY;
 	currentAxisX=cA.x;
 	currentAxisY=cA.y;
@@ -968,13 +1014,8 @@ void MathGLPane::updateDragPos(const wxPoint &draggingEnd) const
 			//left of X-Axis event
 			//Reset the axes such that the
 			//zoom is only along one dimension (y)
-#ifdef USE_MGL2
 			pStart.x = gr->Self()->Min.x;
 			pEnd.x = gr->Self()->Max.x;
-#else
-			pStart.x = gr->Min.x;
-			pEnd.x = gr->Max.x;
-#endif
 		}
 	}
 	else if(pStart.y < currentAxisY  && pEnd.y < currentAxisY )
@@ -985,13 +1026,8 @@ void MathGLPane::updateDragPos(const wxPoint &draggingEnd) const
 		//below Y axis event
 		//Reset the axes such that the
 		//zoom is only along one dimension (x)
-#ifdef USE_MGL2
 		pStart.y = gr->Self()->Min.y;
 		pEnd.y = gr->Self()->Max.y;
-#else				
-		pStart.y = gr->Min.y;
-		pEnd.y = gr->Max.y;
-#endif
 	}
 
 
@@ -1053,11 +1089,7 @@ unsigned int MathGLPane::savePNG(const std::string &filename,
 	ASSERT(filename.size());
 	try
 	{
-#ifdef USE_MGL2
 		gr = new mglGraph(0, width,height);
-#else
-		gr = new mglGraphZB(width,height);
-#endif
 	}
 	catch(std::bad_alloc)
 	{
@@ -1065,41 +1097,25 @@ unsigned int MathGLPane::savePNG(const std::string &filename,
 		return MGLPANE_ERR_BADALLOC;
 	}
 
-#ifdef USE_MGL2
 	gr->SetWarn(0,"");
-#else
-	char *mglWarnMsgBuf=new char[1024];
-	*mglWarnMsgBuf=0;
-	gr->SetWarn(0);
-	gr->Message=mglWarnMsgBuf;
-#endif
+	
 	bool dummy;
 	thePlot->drawPlot(gr,dummy);	
 
 	gr->WritePNG(filename.c_str());
 
 	bool doWarn;
-#ifdef USE_MGL2
 	doWarn=gr->GetWarn();
-#else
-	doWarn=gr->WarnCode;
-#endif
+	
 	if(doWarn)
 	{
-#ifdef USE_MGL2
 		lastMglErr= gr->Self()->Mess;
-#else
-		lastMglErr=mglWarnMsgBuf;
-		delete[] mglWarnMsgBuf;
-#endif
+		
 		delete gr;
 		gr=0;
 		return MGLPANE_ERR_MGLWARN;
 	}
 
-#ifndef USE_MGL2
-	delete[] mglWarnMsgBuf;
-#endif
 	delete gr;
 	gr=0;
 	//Hack. mathgl does not return an error value from its writer
@@ -1124,28 +1140,13 @@ unsigned int MathGLPane::saveSVG(const std::string &filename)
 	ASSERT(filename.size());
 
 
-#ifdef USE_MGL2
 	mglGraph *grS;
 	grS = new mglGraph();
-#else
-	mglGraphPS *grS;
-
-	//Width and height are not *really* important per se, 
-	//since this is scale-less data.
-	grS = new mglGraphPS(1024,768);
-#endif
 
 	bool dummy;
 	thePlot->drawPlot(grS,dummy);
 
-#ifdef USE_MGL2
 	grS->SetWarn(0,"");
-#else
-	char *mglWarnMsgBuf=new char[1024];
-	*mglWarnMsgBuf=0;
-	grS->SetWarn(0);
-	grS->Message=mglWarnMsgBuf;
-#endif	
 
 	//Mathgl does not set locale prior to writing SVG
 	// do this by hand
@@ -1155,20 +1156,11 @@ unsigned int MathGLPane::saveSVG(const std::string &filename)
 
 
 	bool doWarn;
-#ifdef USE_MGL2
 	doWarn=grS->GetWarn();
-#else
-	doWarn=grS->WarnCode;
-#endif
 
 	if(doWarn)
 	{
-#ifdef USE_MGL2
 		lastMglErr=grS->Self()->Mess;
-#else
-		lastMglErr=mglWarnMsgBuf;
-		delete[] mglWarnMsgBuf;
-#endif
 		delete grS;
 		grS=0;
 		return MGLPANE_ERR_MGLWARN;
@@ -1330,8 +1322,8 @@ void MathGLPane::drawInteractOverlay(wxDC *dc) const
 			//---------
 			string labelText;
 			labelText = r.getName();
-			wxSize textSize=dc->GetTextExtent(wxStr(labelText));
-			dc->DrawText(wxStr(labelText),curMouse.x-textSize.GetWidth()/2, 
+			wxSize textSize=dc->GetTextExtent((labelText));
+			dc->DrawText((labelText),curMouse.x-textSize.GetWidth()/2, 
 					h/2-(textSize.GetHeight() + 1.5*ARROW_SIZE));
 			//---------
 		}	
@@ -1354,35 +1346,43 @@ void MathGLPane::drawInteractOverlay(wxDC *dc) const
 		}
 
 		const float THUMB_FRACTION=0.1;
+		const unsigned int MIN_THUMB_SIZE=10;
 		unsigned int thumbSize=THUMB_FRACTION*std::min(h,w);
 
-		for(size_t ui=0;ui<textureIDs.size();ui++)
-		{
-			size_t textureID;
-			textureID = textureIDs[ui];
-
-			ASSERT(textureID < PLOT_TEXTURE_ENUM_END);
-			std::string filename;
-			filename=locateDataFile(mglTextureFile[textureID]);
 		
-			//Need to draw a picture
-			wxImage img;
-			if(wxFileExists(wxStr(filename)) && img.LoadFile(wxStr(filename) ))
+
+		if(thumbSize > MIN_THUMB_SIZE)
+		{
+
+			for(size_t ui=0;ui<textureIDs.size();ui++)
 			{
-				int position[2];
-				float tmp;
-				
-				img.Rescale(thumbSize,thumbSize,wxIMAGE_QUALITY_HIGH);
+				size_t textureID;
+				textureID = textureIDs[ui];
 
-				wxBitmap bmp(img);
-				//Draw in upper right, by one fraction
-				tmp= (1.0-1.5*THUMB_FRACTION);
-				position[0] = tmp*w;
-				
-				//Compute the vertical spacing for each icon 
-				position[1] = (1.0-(tmp - 2.0*(float)ui*THUMB_FRACTION))*h;
+				ASSERT(textureID < PLOT_TEXTURE_ENUM_END);
+				std::string filename;
+				filename=locateDataFile(mglTextureFile[textureID]);
+		
+					
+				//Need to draw a picture
+				wxImage img;
+				if(wxFileExists((filename)) && img.LoadFile((filename) ))
+				{
+					int position[2];
+					float tmp;
+					
+					img.Rescale(thumbSize,thumbSize,wxIMAGE_QUALITY_HIGH);
 
-				dc->DrawBitmap(img,position[0],position[1]);
+					wxBitmap bmp(img);
+					//Draw in upper right, by one fraction
+					tmp= (1.0-1.5*THUMB_FRACTION);
+					position[0] = tmp*w;
+					
+					//Compute the vertical spacing for each icon 
+					position[1] = (1.0-(tmp - 2.0*(float)ui*THUMB_FRACTION))*h;
+
+					dc->DrawBitmap(img,position[0],position[1]);
+				}
 			}
 		}
 	}
@@ -1408,13 +1408,8 @@ bool MathGLPane::toPlotCoords(int winX, int winY,float &resX, float &resY) const
 	if(plotIsLogarithmic)
 	{
 		float plotMinY,plotMaxY;
-#if USE_MGL2
 		plotMinY=gr->Self()->Min.y;
 		plotMaxY=gr->Self()->Max.y;
-#else
-		plotMinY=gr->Min.y;
-		plotMaxY=gr->Max.y;
-#endif
 		float proportion =(pt.y-plotMinY)/(plotMaxY-plotMinY);
 		float tmp = proportion*(log10(plotMaxY)-log10(plotMinY)) + log10(plotMinY); 
 		
@@ -1427,17 +1422,9 @@ bool MathGLPane::toPlotCoords(int winX, int winY,float &resX, float &resY) const
 }
 bool MathGLPane::toWinCoords(float plotX, float plotY, float &winX, float &winY) const
 {
-#ifdef USE_MGL2
 	mglPoint tmp;
 	tmp=gr->CalcScr(mglPoint(plotX,plotY));
 	winX=tmp.x; winY=tmp.y;
-#else
-	
-	int iWinX,iWinY;
-	iWinX=winX;
-	iWinY=winY;
-	gr->CalcScr(mglPoint(plotX,plotY),&iWinX,&iWinY);
-#endif
 
 	if(plotIsLogarithmic)
 	{
@@ -1467,7 +1454,7 @@ void MathGLPane::drawRegionDraggingOverlay(wxDC *dc) const
 		return;
 
 
-	ASSERT(thePlot->plotType(startMousePlot) == PLOT_TYPE_ONED);
+	ASSERT(thePlot->plotType(startMousePlot) == PLOT_MODE_1D);
 
 	//See where extending the region is allowed up to.
 	thePlot->findRegionLimit(startMousePlot,startMouseRegion,
@@ -1483,7 +1470,7 @@ void MathGLPane::drawRegionDraggingOverlay(wxDC *dc) const
 	std::string str;
 	stream_cast(str,regionLimitX);
 	wxString wxs;
-	wxs=wxStr(str);
+	wxs=(str);
 	wxCoord textW,textH;
 	dc->GetTextExtent(wxs,&textW,&textH);
 
@@ -1542,7 +1529,7 @@ void MathGLPane::drawRegionDraggingOverlay(wxDC *dc) const
 		{
 			//This needs to be extended to support more
 			//plot types.
-			ASSERT(thePlot->plotType(startMousePlot) == PLOT_TYPE_ONED);
+			ASSERT(thePlot->plotType(startMousePlot) == PLOT_MODE_1D);
 			
 			//Draw "ghost" limits markers for move,
 			//these appear as moving vertical bars to outline
