@@ -1,6 +1,6 @@
 /*
  *	testing.cpp - unit testing implementation
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <wx/dir.h>
 
 #include <fstream>
+#include <map>
 
 #include "wx/wxcommon.h"
 
@@ -31,12 +32,18 @@
 #include "backend/state.h"
 #include "backend/configFile.h"
 #include "backend/filters/algorithms/binomial.h"
+#include "backend/filters/algorithms/K3DTree-mk2.h"
+#include "backend/filters/algorithms/K3DTree.h"
+#include "backend/filters/algorithms/mass.h"
+
 #include "backend/APT/ionhit.h"
 #include "backend/APT/APTFileIO.h"
+#include "backend/APT/abundanceParser.h"
 
 #include "common/stringFuncs.h"
 #include "common/xmlHelper.h"
 
+#include "gl/isoSurface.h"
 
 const char *TESTING_RESOURCE_DIRS[] = {
 		"../test/",
@@ -45,6 +52,10 @@ const char *TESTING_RESOURCE_DIRS[] = {
 
 #include "filtertesting.cpp"
 
+using std::ifstream;
+using std::cerr;
+using std::endl;
+using std::map;
 
 //!Try loading each range file in the testing folder
 bool rangeFileLoadTests();
@@ -58,6 +69,11 @@ bool XMLTests();
 //!Check to see if manifest contents can be found
 bool locateDataTests();
 
+bool abundanceTests();
+
+//run the tests for algorithms/ 
+bool algorithmTests();
+
 bool basicFunctionTests()
 {
 	testStringFuncs();
@@ -107,19 +123,30 @@ bool basicFunctionTests()
 
 bool runUnitTests()
 {
+	//Set the abort pointer for the filter
+#ifdef HAVE_CPP_1X
+	ATOMIC_BOOL abortFlag(false);
+#else
+	ATOMIC_BOOL abortFlag=false;
+#endif
+	Filter::wantAbort=&abortFlag;
+	K3DTree::setAbortFlag(&abortFlag);
+	K3DTreeMk2::setAbortFlag(&abortFlag);
+
+	unsigned int progressVar=0;
+	K3DTree::setProgressPtr(&progressVar);
+	K3DTreeMk2::setProgressPtr(&progressVar);
 
 	cerr << "Running unit tests..." ;
 
+	if(!algorithmTests())
+		return false;
 
 	if(!testIonHit())
 		return false;
 
 	if(!filterTests())
 		return false;
-
-	if(!filterCloneTests())
-		return false;
-
 	if(!rangeFileLoadTests())
 		return false;
 
@@ -130,14 +157,9 @@ bool runUnitTests()
 	if(!XMLTests())
 		return false;
 
-	if(!filterTreeTests())
-		return false;
-
 	if(!runVoxelTests())
 		return false;
 
-	if(!testBinomial())
-		return false;
 
 	if(!runStateTests())
 		return false;
@@ -147,10 +169,16 @@ bool runUnitTests()
 
 	if(!testFileIO())
 		return false;
+	//MGL test is disabled, due to a bug in mathgl in debian testing
+	// which causes threading segfaults. This is fixed in recent versions
+//	if(!mglTest())
+//		return false;
 
-	if(!mglTest())
+	if(!abundanceTests())
 		return false;
 
+	if(!testIsoSurface())
+		return false;
 	cerr << " OK" << endl << endl;
 
 	return true;
@@ -162,9 +190,11 @@ bool rangeFileLoadTests()
 	//whichever is first. 
 	wxString testDir;
 	bool haveDir=false;
-	for(unsigned int ui=0;ui<THREEDEP_ARRAYSIZE(TESTING_RESOURCE_DIRS);ui++)
+	size_t n;
+	n = THREEDEP_ARRAYSIZE(TESTING_RESOURCE_DIRS);
+	for(unsigned int ui=0;ui<n;ui++)
 	{
-		testDir=wxCStr(TESTING_RESOURCE_DIRS[ui]);
+		testDir=(TESTING_RESOURCE_DIRS[ui]);
 		if(wxDirExists(testDir))
 		{
 			haveDir=true;
@@ -189,7 +219,7 @@ bool rangeFileLoadTests()
 		std::string tmp;
 		tmp = std::string("*.") + rangeExts[ui];
 		
-		wxDir::GetAllFiles(testDir,&tmpArr,wxStr(tmp));
+		wxDir::GetAllFiles(testDir,&tmpArr,(tmp));
 		for(unsigned int uj=0;uj<tmpArr.GetCount();uj++)
 			arrayStr.Add(tmpArr[uj]);
 		tmpArr.clear();
@@ -247,7 +277,7 @@ bool rangeFileLoadTests()
 		fileLongname=stlStr(arrayStr[ui]);
 
 		wxFileName filename;
-		filename=wxStr(fileLongname);
+		filename=(fileLongname);
 		//This returns the short name of the file. Yes, its badly named.
 		fileShortname=stlStr(filename.GetFullName());
 		{
@@ -299,7 +329,7 @@ bool rangeFileLoadTests()
 
 
 
-	map<string,int> typeMapping;
+	map<string,unsigned int> typeMapping;
 
 	typeMapping["test1.rng"]=RANGE_FORMAT_ORNL;
 	typeMapping["test2.rng"]=RANGE_FORMAT_ORNL; 
@@ -326,7 +356,7 @@ bool rangeFileLoadTests()
 		wxFileName filename;
 		
 		fileLongname=stlStr(arrayStr[ui]);
-		filename=wxStr(fileLongname);
+		filename=(fileLongname);
 		fileShortname=stlStr(filename.GetFullName());
 
 		//Check to see that the auto-parser correctly identifies the type
@@ -336,7 +366,7 @@ bool rangeFileLoadTests()
 			errString="Range type detection : ";
 			errString+=fileLongname;
 
-			if(!wxFileExists(wxStr(fileLongname)))
+			if(!wxFileExists((fileLongname)))
 			{
 				cerr << "File expected, but not found during test:" <<
 					fileLongname << endl;
@@ -436,7 +466,7 @@ bool locateDataTests()
 			check=true;
 		else
 		{
-			//Check only if compiled for thiarch arch
+			//Check only if compiled for a specific arch
 			if(arch == "win")
 			{
 #if defined(__WIN32__) || defined(__WIN64)
@@ -493,6 +523,44 @@ bool locateDataTests()
 		}
 	}
 
+
+
+	return true;
+}
+
+bool abundanceTests()
+{
+	const char *ABUNDANCE_FILE="../data/naturalAbundance.xml";
+	ifstream f(ABUNDANCE_FILE);
+
+	//Check that abundance file exists
+	if(f)
+	{
+		f.close();
+		//run abundance tests
+		if(!AbundanceData::runUnitTests(ABUNDANCE_FILE))
+			return false;
+	}
+	else
+	{
+		WARN(false,"Unable to locate natural abundance file, skipping");
+	}
+	
+	return true;
+}
+
+bool algorithmTests()
+{
+	if(!testAnderson())
+		return false;
+	if(!testBackgroundFit())
+		return false;
+
+	if(!K3DMk2Tests())
+		return false;
+	
+	if(!testBinomial())
+		return false;
 	return true;
 }
 
