@@ -1,6 +1,6 @@
 /*
  *	wxcommon.cpp - Comon wxwidgets functionality 
- *	Copyright (C) 2013, D Haley 
+ *	Copyright (C) 2015, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -44,8 +44,10 @@ const char *RSS_FEED_LOCATION="http://threedepict.sourceforge.net/rss.xml";
 
 //Auto update event for posting back to main thread upon completion
 wxEventType RemoteUpdateAvailEvent = wxNewEventType(); // You get to choose the name yourself
-		
 
+//Signalling event for main frame that it is completed
+wxEventType RefreshCompleteEvent= wxNewEventType(); // You get to choose the name yourself
+		
 //Maximum amount of content in RSS header is 1MB.
 const unsigned int MAX_RSS_CONTENT_SIZE=1024*1024;
 
@@ -87,39 +89,44 @@ std::string locateDataFile(const char *name)
 
 			s+=name;
 
-			if(wxFileExists(wxStr(s)))
+			if(wxFileExists((s)))
 				return s;
 		}
 	}
-	else if(wxFileExists(wxStr(s)))
-		return s;
+	
+	std::string s;
+	s =name; 
+	
+	if(s.size() && wxFileExists((s)))
+	{
+		return string(name);
+	}
 	else
 		return std::string("");
 #elif defined( __linux__)
 
 	//Possible search paths. Must have trailing slash. will
 	//be searched in sequence.
-	const unsigned int NUM_SEARCH_DIRS=6;
 	const char *possibleDirs[] = { "./",
 					"/usr/local/share/3Depict/",
 					"/usr/share/3Depict/",
 					"/usr/share/3depict/", //Under debian, we have to use lowercase according to the debian guidelines, so handle this case.
 					"../data/",
-					"./data/",
+					"./data/"
 					};
 
-	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(possibleDirs) == NUM_SEARCH_DIRS);
-	
+	const unsigned int NUM_SEARCH_DIRS=THREEDEP_ARRAYSIZE(possibleDirs);
+
 	std::string s;
 	for(unsigned int ui=0; ui<NUM_SEARCH_DIRS; ui++)
 	{
 		s=std::string(possibleDirs[ui]) + name;
 
-		if(wxFileExists(wxStr(s)))
+		if(wxFileExists((s)))
 			return s;
 	}
 
-	//Return empty string, as we cna't find it
+	//Return empty string, as we can't find it
 	return std::string("");
 #elif defined (__APPLE__)
     CFBundleRef mainBundle = CFBundleGetMainBundle();
@@ -132,8 +139,10 @@ std::string locateDataFile(const char *name)
     }
     CFRelease(resourcesURL);
 	std::string s=std::string(path) + "/" + name;
-		if(wxFileExists(wxStr(s)))
+		if(wxFileExists((s)))
 			return s;
+		else
+			return std::string("");
 #else
 
 	//	- Look in cwd
@@ -169,7 +178,7 @@ void *VersionCheckThread::Entry()
 	strUrl = std::string(RSS_FEED_LOCATION) + std::string("?progver=") + std::string(PROGRAM_VERSION) + 
 				std::string("&os=") + stlStr(::wxGetOsDescription());
 
-	wxURI uri(wxStr(strUrl));
+	wxURI uri((strUrl));
 	rssUrl = uri.BuildURI();
 
 	url.SetURL(rssUrl); 
@@ -289,8 +298,8 @@ void *VersionCheckThread::Entry()
 
 void wxErrMsg(wxWindow *win, const std::string &title, const std::string &mesg)
 {
-	wxMessageDialog *wxMesD  =new wxMessageDialog(win,wxStr(mesg)
-					,wxStr(title),wxOK|wxICON_ERROR);
+	wxMessageDialog *wxMesD  =new wxMessageDialog(win,(mesg)
+					,(title),wxOK|wxICON_ERROR);
 	wxMesD->ShowModal();
 	wxMesD->Destroy();
 }
@@ -305,11 +314,7 @@ bool processMatchesName(size_t processID, const std::string &procName)
 	
 	wxArrayString stdOut;
 	long res;
-#if wxCHECK_VERSION(2,9,0)
 	res=wxExecute(wxT("ps ax"),stdOut, wxEXEC_BLOCK);
-#else
-	res=wxExecute(wxT("ps ax"),stdOut);
-#endif
 
 	if(res !=0 )
 		return false;
@@ -338,7 +343,7 @@ bool processMatchesName(size_t processID, const std::string &procName)
 		procNameFound=pidFound=false;
 		for(unsigned int ui=0;ui<strVec.size(); ui++)
 		{
-			wxFileName fName(wxStr(strVec[ui]));
+			wxFileName fName((strVec[ui]));
 			std::string maybeProcName;
 			maybeProcName = stlStr(fName.GetFullName());
 			
@@ -465,6 +470,88 @@ bool processMatchesName(size_t processID, const std::string &procName)
 
 	}
 #endif
+
+
+
+void copyRGBAtoWXImage(unsigned int width, unsigned int height,
+		const unsigned char *rgbaBuf, wxImage &image, 
+		const unsigned char *mask)
+
+{
+	//wx image must have an alpha channel
+	ASSERT(image.HasAlpha());
+	ASSERT(image.GetWidth() == width && 
+		image.GetHeight() == height);
+
+	//FIXME: This will likely be very slow
+	const unsigned char *p=rgbaBuf;
+	for(unsigned int uj=0;uj<height;uj++)
+	{
+		for(unsigned int ui=0;ui<width;ui++)
+		{	
+			image.SetRGB(ui,uj,*p, *(p+1), *(p+2)); 
+
+			if( (*p == *mask) && 
+			    ( *(mask+1) == *(p+1)) &&
+			    ( *(mask+2) == *(p+2)) )
+				image.SetAlpha(ui,uj,0);
+			else
+				image.SetAlpha(ui,uj,255);
+			p+=4;
+		}
+	}
+
+}
+
+void combineWxImage(wxImage &base, const wxImage &overlay)
+{
+	ASSERT(base.GetWidth() == overlay.GetWidth());
+	ASSERT(base.GetHeight() == overlay.GetHeight());
+	ASSERT(overlay.HasAlpha());
+	ASSERT(base.IsOk() && overlay.IsOk());
+
+
+	unsigned int width=base.GetWidth();
+	unsigned int height=base.GetHeight();
+
+
+	wxImage debugIm(width,height);
+	//Now loop through each pixel and perform
+	// combine operation
+	#pragma omp parallel for
+	for(unsigned int uj=0;uj<height;uj++)
+	{
+		unsigned char rgbIm[3],rgbaOv[4];
+		for(unsigned int ui=0;ui<width;ui++)
+		{
+			rgbaOv[3] = overlay.GetAlpha(ui,uj);
+
+			debugIm.SetRGB(ui,uj,rgbaOv[3], rgbaOv[3],rgbaOv[3]);
+			if(rgbaOv[3])
+			{
+				//obtain src rgb
+				rgbIm[0]=base.GetRed(ui,uj);
+				rgbIm[1]=base.GetGreen(ui,uj);
+				rgbIm[2]=base.GetBlue(ui,uj);
+				//obtain overlay rgb	
+				rgbaOv[0]=overlay.GetRed(ui,uj);
+				rgbaOv[1]=overlay.GetGreen(ui,uj);
+				rgbaOv[2]=overlay.GetBlue(ui,uj);
+		
+		
+				for(unsigned int chan=0;chan<3;chan++)
+				{
+					rgbIm[chan] = (unsigned char) (float(255-rgbaOv[3])/255.0f*rgbIm[chan] + float(rgbaOv[3]/255.0f)*rgbaOv[chan]);
+				}
+				
+				base.SetRGB(ui,uj,rgbIm[0],rgbIm[1],rgbIm[2]);	
+			}
+		}
+	}	
+
+	debugIm.SaveFile("debug.png",wxBITMAP_TYPE_PNG);
+}
+
 	
 TreePersistNode::TreePersistNode(const wxTreeCtrl *treeCtrl,wxTreeItemId t)
 {
